/** ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * $Id: ThreadRecibirMensajesConversacion.java,v 1.1 2006/11/09 03:10:48 f-vela Exp $
 * Universidad de los Andes (Bogot - Colombia)
 * Departamento de Ingeniera de Sistemas y Computacin 
 * Licenciado bajo el esquema Academic Free License versin 2.1
 *
 * Proyecto Cupi2 (http://cupi2.uniandes.edu.co)
 * Ejercicio: n12_messengerAmigos
 * Autor: Mario Snchez - 21/04/2006
 * Modificado: Silvia Takahashi - 13/03/2009
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 */

package uniandes.cupi2.messengerAmigos.cliente;

import java.io.BufferedReader;
import java.io.IOException;


/**
 * Esta clase recibe los mensajes de una conversacin. <br>
 * Cuando un mensaje es recibido, la ejecucin de las acciones correspondientes es delegada a la Conversacion.
 */
public class ThreadRecibirMensajesConversacion extends Thread
{
    // -----------------------------------------------------------------
    // Atributos
    // -----------------------------------------------------------------

    /**
     * Es la conversacin para la cual se estn escuchando los mensajes
     */
    private Conversacion conversacion;

    /**
     * Es el stream a travs del cual se reciben los mensajes enviados por el amigo que participa en la conversacin
     */
    private BufferedReader inAmigo;

    // -----------------------------------------------------------------
    // Constructores
    // -----------------------------------------------------------------

    /**
     * Construye el nuevo hilo y lo deja listo para iniciar
     * @param conv La conversacin a la cual este hilo debe reportar los mensajes recibidos
     * @param in El stream a travs del cual se reciben los mensajes enviados por el amigo que participa en la conversacin
     */
    public ThreadRecibirMensajesConversacion( Conversacion conv, BufferedReader in )
    {
        conversacion = conv;
        inAmigo = in;
    }

    // -----------------------------------------------------------------
    // Mtodos
    // -----------------------------------------------------------------

    /**
     * Recibe los mensajes enviados por el amigo que participa en la conversacin, delega a la conversacin las tareas asociadas a los mensajes y espera nuevos mensajes. <br>
     * Este hilo debe continuar vivo hasta que se reciba un mensaje de CONVERSACION_TERMINADA o de TERMINAR
     */
    public void run( )
    {
        String mensaje = "";
        while( mensaje != null && !mensaje.startsWith( Conversacion.CONVERSACION_TERMINADA ) && !mensaje.startsWith( Conversacion.TERMINAR ) )
        {
            try
            {
                mensaje = inAmigo.readLine( );

                if( mensaje == null )
                {

                }
                else if( mensaje.startsWith( Conversacion.MENSAJE ) )
                {
                    String mensajeRecibido = mensaje.replaceFirst( Conversacion.MENSAJE + ":", "" );
                    conversacion.mostrarMensajeRecibido( mensajeRecibido );
                }
                else if( mensaje.startsWith( Conversacion.CONVERSACION_TERMINADA ) )
                {
                    conversacion.terminarConversacion( );
                }
                else if( mensaje.startsWith( Conversacion.TERMINAR ) )
                {
                    conversacion.enviarConfirmacionTerminarConversacion( );
                    conversacion.terminarConversacion( );
                }
                // Cambia el modo de chequeo/traduccion de los mensajes
                else if( mensaje.startsWith( Conversacion.CAMBIAR_CHECK ) )
                {
                    conversacion.cambiarCheckRemote( mensaje );
                }
            }
            catch( IOException e )
            {
                e.printStackTrace( );
            }
        }
    }
}
