/** ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * $Id: InterfazClienteMessengerAmigos.java,v 1.1 2006/11/09 03:11:30 f-vela Exp $
 * Universidad de los Andes (Bogot - Colombia)
 * Departamento de Ingeniera de Sistemas y Computacin 
 * Licenciado bajo el esquema Academic Free License version 2.1 
 *
 * Proyecto Cupi2 (http://cupi2.uniandes.edu.co)
 * Ejercicio: n12_messengerAmigos
 * Autor: Mario Snchez - 21-abr-2006
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 */

package uniandes.cupi2.messengerAmigos.interfazCliente;

import java.awt.BorderLayout;
import java.io.IOException;
import java.util.Collection;

import javax.swing.JFrame;
import javax.swing.JOptionPane;

import uniandes.cupi2.messengerAmigos.cliente.ClienteAmigos;
import uniandes.cupi2.messengerAmigos.cliente.Conversacion;
import uniandes.cupi2.messengerAmigos.cliente.IClienteMessenger;

/**
 * Esta es la ventana principal de la aplicacin.
 */
public class InterfazClienteMessengerAmigos extends JFrame implements IInterfazCliente
{
    // -----------------------------------------------------------------
    // Atributos
    // -----------------------------------------------------------------

    /**
     * Es una referencia a la clase principal del cliente
     */
    private IClienteMessenger clienteAmigos;

    /**
     * Indica que se inici el proceso para salir de la aplicacin
     */
    private boolean saliendo;

    // -----------------------------------------------------------------
    // Atributos de la interfaz
    // -----------------------------------------------------------------

    /**
     * El panel donde se muestran los amigos del usuario
     */
    private PanelAmigos panelAmigos;

    /**
     * Es el men donde estn las opciones para el cliente
     */
    private BarraMenu barraMenu;

    // -----------------------------------------------------------------
    // Constructores
    // -----------------------------------------------------------------

    /**
     * Construye la ventana e inicializa sus componentes
     * @param archivoConfiguracion Es el nombre del archivo que contiene la configuracin para el cliente
     * @throws Exception Se lanza esta excepcin si hay problemas con el archivo
     */
    public InterfazClienteMessengerAmigos( String archivoConfiguracion ) throws Exception
    {
        // Crea la clase principal
        clienteAmigos = new ClienteAmigos( this, archivoConfiguracion );

        // Construye la forma
        getContentPane( ).setLayout( new BorderLayout( ) );
        setSize( 280, 530 );
        setDefaultCloseOperation( JFrame.DISPOSE_ON_CLOSE );
        setTitle( "Messenger Amigos" );

        panelAmigos = new PanelAmigos( this );
        getContentPane( ).add( panelAmigos );

        barraMenu = new BarraMenu( this );
        setJMenuBar( barraMenu );
    }

    // -----------------------------------------------------------------
    // Mtodos
    // -----------------------------------------------------------------

    /**
     * Inicia la conexin del cliente con el servidor: <br>
     * pide al usuario que indique su nombre y luego se conecta al servidor.
     */
    public void iniciarConexion( )
    {
        String nombre = JOptionPane.showInputDialog( this, "Nombre del Usuario" );
        if( nombre != null && nombre.trim( ).length( ) > 0 )
        {
            try
            {
                clienteAmigos.conectar( nombre );
                actualizarEstadoInterfaz( );
            }
            catch( IOException e )
            {
                JOptionPane.showMessageDialog( this, "Hubo un error realizando la conexin:" + e.getMessage( ), "Error en la Conexin", JOptionPane.ERROR_MESSAGE );
                e.printStackTrace( );
            }
        }
    }

    /**
     * Cierra la conexin del cliente con el servidor
     */
    public void terminarConexion( )
    {
        clienteAmigos.enviarDesconexion( );
    }

    /**
     * Inicia el proceso de finalizacin de la aplicacin. <br>
     * El cliente se debe desconectar y todas las conversaciones se deben terminar. Cuando esto se haya hecho entonces se cierra la ventana y la aplicacin.<br>
     * Si el cliente no estaba conectado entonces la aplicacin simplemente se cierra.
     */
    public void iniciarFinalizacionAplicacion( )
    {
        if( clienteAmigos.estaConectado( ) )
        {
            saliendo = true;
            clienteAmigos.enviarDesconexion( );
        }
        else
        {
            saliendo = true;
            terminarAplicacion( );
        }
    }

    /**
     * Si se haba solicitado que se terminara el programa, se cierra la ventana y la aplicacin.<br>
     * <b>pre:</b>El cliente est desconectado del servidor y no hay conversaciones en curso.
     */
    public void terminarAplicacion( )
    {
        if( saliendo )
        {
            super.dispose( );
            System.exit( 0 );
        }
    }

    /**
     * Cuando se cierra la ventana, se inicia el proceso de finalizacin de la aplicacin
     */
    public void dispose( )
    {
        saliendo = true;
        iniciarFinalizacionAplicacion( );
    }

    /**
     * Actualiza la interfaz segn el estado de la conexin: <br>
     * Se actualiza el ttulo de la ventana y se activan o desactivan opciones de la barra de men
     */
    public void actualizarEstadoInterfaz( )
    {
        if( clienteAmigos.estaConectado( ) )
        {
            setTitle( "Messenger Amigos: " + clienteAmigos.darNombreUsuario( ) );
            barraMenu.activarDesconexion( );
        }
        else
        {
            setTitle( "Messenger Amigos (desconectado)" );
            barraMenu.activarConexion( );
        }
    }

    /**
     * Actualiza la lista de amigos mostrada
     * @param amigos Una coleccin con los amigos (Usuario) del usuario - amigos != null
     */
    public void actualizarAmigos( Collection amigos )
    {
        panelAmigos.actualizarAmigos( amigos );
        validate( );
    }

    /**
     * Agrega un amigo al usuario actual. Este mtodo debe pedirle al usuario que indique el nombre del amigo. <br>
     * <b>pre: </b>El cliente est conectado
     */
    public void agregarAmigo( )
    {
        String nombre = JOptionPane.showInputDialog( this, "Nombre del Nuevo Amigo" );
        if( nombre != null && nombre.trim( ).length( ) > 0 )
        {
            clienteAmigos.agregarAmigo( nombre );
        }
    }

    /**
     * Crea una nueva ventana para una conversacin
     * @param conv La conversacin asociada a la nueva ventana - conv != null
     */
    public void crearVentanaConversacion( Conversacion conv )
    {
        VentanaConversacion ventana = new VentanaConversacion( conv );
        ventana.setLocation( getLocation( ) );
        ventana.setVisible( true );
    }

    /**
     * Inicia una nueva conversacin con un amigo
     * @param nombreAmigo El nombre del amigo con el que se va a iniciar la conversacin - nombreAmigo != null
     */
    public void iniciarConversacion( String nombreAmigo )
    {
        try
        {
            Conversacion conversacion = clienteAmigos.crearConversacionLocal( nombreAmigo );
            crearVentanaConversacion( conversacion );
        }
        catch( IOException e )
        {
            JOptionPane.showMessageDialog( this, "Error creando la conversacion: " + e.getMessage( ) );
        }
    }

    // -----------------------------------------------------------------
    // Puntos de Extensin
    // -----------------------------------------------------------------

    /**
     * Mtodo para la extensin 1
     */
    public void reqFuncOpcion1( )
    {
        String resultado = clienteAmigos.metodo1( );
        JOptionPane.showMessageDialog( this, resultado, "Respuesta", JOptionPane.INFORMATION_MESSAGE );
    }

    /**
     * Mtodo para la extensin 2
     */
    public void reqFuncOpcion2( )
    {
        String resultado = clienteAmigos.metodo2( );
        JOptionPane.showMessageDialog( this, resultado, "Respuesta", JOptionPane.INFORMATION_MESSAGE );
    }

    // -----------------------------------------------------------------
    // Main
    // -----------------------------------------------------------------

    /**
     * Este mtodo ejecuta la aplicacin, creando una nueva interfaz
     * @param args Parmetros de ejecucin. No son necesarios, pero si se usan el primero debe ser la ruta al archivo de configuracin.
     */
    public static void main( String[] args )
    {
        try
        {
            String archivo = "./data/cliente.properties";
            if( args.length > 0 )
                archivo = args[ 0 ];

            InterfazClienteMessengerAmigos interfaz = new InterfazClienteMessengerAmigos( archivo );
            interfaz.setVisible( true );
        }
        catch( Exception e )
        {
            e.printStackTrace( );
        }
    }

}