/** ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * $Id: PanelAmigos.java,v 1.1 2006/11/09 03:11:30 f-vela Exp $
 * Universidad de los Andes (Bogot - Colombia)
 * Departamento de Ingeniera de Sistemas y Computacin 
 * Licenciado bajo el esquema Academic Free License versin 2.1
 *
 * Proyecto Cupi2 (http://cupi2.uniandes.edu.co)
 * Ejercicio: n12_messengerAmigos
 * Autor: Mario Snchez - 24/04/2006
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 */

package uniandes.cupi2.messengerAmigos.interfazCliente;
 
import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Collection;
 
import javax.swing.JButton;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.ListSelectionModel;

import uniandes.cupi2.messengerAmigos.cliente.Usuario;

/**
 * Es el panel donde se muestran los amigos del usuario
 */
public class PanelAmigos extends JPanel implements ActionListener
{

    // -----------------------------------------------------------------
    // Constantes
    // -----------------------------------------------------------------

    /**
     * Comando para el botn que abre una conversacin
     */
    private final static String CONVERSAR = "CONVERSAR";

    // -----------------------------------------------------------------
    // Atributos
    // -----------------------------------------------------------------

    /**
     * Es una referencia a la clase principal de la interfaz
     */
    private InterfazClienteMessengerAmigos ventanaPrincipal;

    // -----------------------------------------------------------------
    // Atributos de la interfaz
    // -----------------------------------------------------------------

    /**
     * Es la lista donde se muestran los amigos
     */
    private JList listaAmigos;

    /**
     * Es el botn en el que se hace click para abrir una conversacin
     */
    private JButton botonConversacion;

    // -----------------------------------------------------------------
    // Constructores
    // -----------------------------------------------------------------

    /**
     * Construye el panel
     * @param ventana La ventana dentro de la que se encuentra este panel - ventana != null
     */
    public PanelAmigos( InterfazClienteMessengerAmigos ventana )
    {
        ventanaPrincipal = ventana;
        setLayout( new BorderLayout( ) );

        JScrollPane scroll = new JScrollPane( );
        scroll.setVerticalScrollBarPolicy( JScrollPane.VERTICAL_SCROLLBAR_ALWAYS );

        listaAmigos = new JList( );
        listaAmigos.setSelectionMode( ListSelectionModel.SINGLE_SELECTION );

        scroll.getViewport( ).add( listaAmigos );
        add( scroll );

        botonConversacion = new JButton( "Abrir Conversacin" );
        botonConversacion.setActionCommand( CONVERSAR );
        botonConversacion.addActionListener( this );
        add( botonConversacion, BorderLayout.SOUTH );
    }

    // -----------------------------------------------------------------
    // Mtodos
    // -----------------------------------------------------------------

    /**
     * Actualiza la lista de amigos mostrada
     * @param amigos La coleccin de amigos (Usuario) que se deben mostrar - amigos != null
     */
    public void actualizarAmigos( Collection amigos )
    {
        listaAmigos.setListData( amigos.toArray( ) );
    }

    /**
     * El mtodo que se llama cuando se hace click sobre el botn
     * @param evento El evento del click sobre el botn
     */
    public void actionPerformed( ActionEvent evento )
    {
        String comando = evento.getActionCommand( );

        if( CONVERSAR.equals( comando ) )
        {
            Usuario amigo = ( Usuario )listaAmigos.getSelectedValue( );
            if( amigo != null )
            {
                ventanaPrincipal.iniciarConversacion( amigo.darNombre( ) );
            }
        }
    }
}
