/** ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * $Id: PanelEnviarMensajes.java,v 1.1 2006/11/09 03:11:30 f-vela Exp $
 * Universidad de los Andes (Bogot - Colombia)
 * Departamento de Ingeniera de Sistemas y Computacin 
 * Licenciado bajo el esquema Academic Free License versin 2.1
 *
 * Proyecto Cupi2 (http://cupi2.uniandes.edu.co)
 * Ejercicio: n12_messengerAmigos
 * Autor: Mario Snchez - 24/04/2006
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 */

package uniandes.cupi2.messengerAmigos.interfazCliente;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;

import javax.swing.JButton;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;


/**
 * Este es el panel desde donde se envan los mensajes de una conversacin
 */
public class PanelEnviarMensajes extends JPanel implements ActionListener
{

    // -----------------------------------------------------------------
    // Constantes
    // -----------------------------------------------------------------

    /**
     * Comando para el botn de Enviar un mensaje
     */
    private final static String ENVIAR = "ENVIAR";
    
    // -----------------------------------------------------------------
    // Atributos
    // -----------------------------------------------------------------

    /**
     * Es una referencia a la ventana de la cual hace parte este panel
     */
    private VentanaConversacion ventana;

    // -----------------------------------------------------------------
    // Atributos de la Interfaz
    // -----------------------------------------------------------------

    /**
     * Es la zona donde se escriben los mensajes para enviar
     */
    private JTextArea txtMensaje;

    /**
     * Es el botn usado para enviar un mensaje
     */
    private JButton botonEnviar;
    
    private JPanel panelBotones = new JPanel();

    // -----------------------------------------------------------------
    // Constructores
    // -----------------------------------------------------------------

    /**
     * Construye el panel e inicializa sus componentes
     * @param ventanaConv Es la ventana a la que pertenece este panel - ventanaConv != null
     */
    public PanelEnviarMensajes( VentanaConversacion ventanaConv )
    {
        ventana = ventanaConv;

        setLayout( new BorderLayout( ) );
        JScrollPane scroll = new JScrollPane( );
        txtMensaje = new JTextArea( );
        txtMensaje.setWrapStyleWord( true );
        txtMensaje.setLineWrap( true );
        scroll.getViewport( ).add( txtMensaje );

        add( scroll);  
        
        
        botonEnviar = new JButton( "Enviar" );
        botonEnviar.setActionCommand( ENVIAR );
        botonEnviar.addActionListener( this );
    
        panelBotones.add(botonEnviar);
        
              
        add( panelBotones,BorderLayout.SOUTH);
      }

    // -----------------------------------------------------------------
    // Mtodos
    // -----------------------------------------------------------------

    /**
     * Este mtodo se ejecuta cuando se hace click en el botn para enviar un mensaje
     * @param evento El evento de click sobre el botn
     */
    public void actionPerformed( ActionEvent evento )
    {
        String comando = evento.getActionCommand( );

        if( ENVIAR.equals( comando ) )
        {
            String mensaje = txtMensaje.getText( ).trim( );
            if( mensaje.length( ) > 0 )
            {
                ventana.enviarMensaje( mensaje );
                txtMensaje.setText( "" );
            }
        }
    
    }

    
    
}
