/** ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * $Id: PanelClientes.java,v 1.1 2006/11/09 03:11:30 f-vela Exp $
 * Universidad de los Andes (Bogot - Colombia)
 * Departamento de Ingeniera de Sistemas y Computacin 
 * Licenciado bajo el esquema Academic Free License versin 2.1
 *
 * Proyecto Cupi2 (http://cupi2.uniandes.edu.co)
 * Ejercicio: n12_messengerAmigos
 * Autor: Mario Snchez - 21-abr-2006
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 */

package uniandes.cupi2.messengerAmigos.interfazServidor;

import java.awt.BorderLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Collection;

import javax.swing.JButton;
import javax.swing.JList;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.border.TitledBorder;


/**
 * Es el panel donde se muestran los clientes que hay conectados actualmente al servidor
 */
public class PanelClientes extends JPanel implements ActionListener
{
    // -----------------------------------------------------------------
    // Constantes
    // -----------------------------------------------------------------

    /**
     * Comando para el botn Refrescar
     */
    private static final String REFRESCAR = "Refrescar";

    // -----------------------------------------------------------------
    // Atributos
    // -----------------------------------------------------------------

    /**
     * Es una referencia a la ventana principal de la aplicacin del servidor
     */
    private InterfazServidorMessengerAmigos principal;

    // -----------------------------------------------------------------
    // Atributos de la Interfaz
    // -----------------------------------------------------------------

    /**
     * Es la lista donde se muestran los clientes
     */
    private JList listaClientes;

    /**
     * Es el botn que se usa para refrescar la lista de clientes
     */
    private JButton botonRefrescar;

    // -----------------------------------------------------------------
    // Constructores
    // -----------------------------------------------------------------

    /**
     * Inicializa el panel
     * @param padre Es una referencia a la ventana principal del servidor
     */
    public PanelClientes( InterfazServidorMessengerAmigos padre )
    {
        principal = padre;
        inicializarPanel( );
    }

    // -----------------------------------------------------------------
    // Mtodos
    // -----------------------------------------------------------------

    /**
     * Inicializa los elementos dentro del panel
     */
    private void inicializarPanel( )
    {
        setLayout( new BorderLayout( ) );

        JScrollPane scroll = new JScrollPane( );
        listaClientes = new JList( );
        scroll.getViewport( ).add( listaClientes );
        add( scroll, BorderLayout.CENTER );

        botonRefrescar = new JButton( "Refrescar" );
        botonRefrescar.addActionListener( this );
        botonRefrescar.setActionCommand( REFRESCAR );
        add( botonRefrescar, BorderLayout.EAST );

        setBorder( new TitledBorder( "Clientes en lnea" ) );
    }

    /**
     * Actualiza la lista mostrada de clientes
     * @param clientes Es una coleccin con la informacin (String) de los clientes que hay conectados actualmente
     */
    public void actualizarClientes( Collection clientes )
    {
        listaClientes.setListData( clientes.toArray( ) );
    }

    /**
     * Es el mtodo llamado cuando se hace click sobre el botn refrescar
     * @param evento Es el evento de click sobre el botn
     */
    public void actionPerformed( ActionEvent evento )
    {
        String comando = evento.getActionCommand( );

        if( REFRESCAR.equals( comando ) )
        {
            principal.actualizarClientes( );
        }
    }

}
