/**~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * $Id: IServidorAmigos.java,v 1.3 2007/04/13 14:25:45 carl-veg Exp $ 
 * Universidad de los Andes (Bogot - Colombia)
 * Departamento de Ingeniera de Sistemas y Computacin 
 * Todos los derechos reservados 2005 
 *
 * Proyecto Cupi2 
 * Ejercicio: n12_messengerAmigos 
 * Autor: Mario Snchez - 5/05/2006 
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
 */

package uniandes.cupi2.messengerAmigos.servidor;


import java.io.IOException;
import java.sql.SQLException;
import java.util.Collection;

import uniandes.cupi2.messengerAmigos.servidor.Usuario;

/**
 * TODO documentar
 */
public interface IServidorAmigos
{
    // -----------------------------------------------------------------
    // Constantes
    // -----------------------------------------------------------------

    /**
     * Mensaje que se enva cuando un cliente se va a conectar
     */
    public static final String LOGIN = "LOGIN";

    /**
     * Mensaje que se enva cuando un cliente se va a desconectar
     */
    public static final String LOGOUT = "LOGOUT";

    /**
     * Mensaje que enva el servidor a un cliente cuando lo desconecta
     */
    public static final String DESCONEXION = "DESCONEXION";

    /**
     * Mensaje que se usa para indicar que un amigo est conectado
     */
    public static final String ONLINE = "ONLINE";

    /**
     * Mensaje que se usa para indicar que un amigo est desconectado
     */
    public static final String OFFLINE = "OFFLINE";

    /**
     * Mensaje que se usa para agregar un amigo
     */
    public static final String AGREGAR_AMIGO = "AGREGAR_AMIGO";

    /**
     * Mensaje que se usa para que el servidor le indique a un cliente que va a iniciarse una conversacin
     */
    public static final String INCIAR_CHARLA = "INCIO_CHARLA";

    /**
     * Mensaje que se usa para que un cliente le indique al servidor que va a iniciar una charla y que necesita ponerse en contacto con un amigo
     */
    public static final String CONVERSACION = "CONVERSACION";

    /**
     * Retorna el administrador de amigos usado por el servidor
     * @return adminAmigos
     */
    IAdministradorAmigos darAministradorAmigos( );

    /**
     * Termina la conexin a la base de datos y la cierra
     * @throws SQLException Se lanza esta excepcin si hay problemas terminando la conexin a la base de datos
     */
    void desconectarDB( ) throws SQLException;

    /**
     * Este mtodo se encarga de recibir todas las conexiones de los clientes. <br>
     * Este mtodo debe abrir el socket y para cada conexin entrante construir un ManejadorCliente asociado.
     */
    void recibirConexiones( );

    /**
     * Cierra el socket del servidor que espera clientes <br>
     * <b>pre: </b>No se est esperando una conexin en el socket.
     * @throws IOException Se lanza esta excepcin si hay problemas cerrando el socket
     */
    void cerrarConexion( ) throws IOException;

    /**
     * Retorna una coleccin de Strings con la informacin de los clientes que estn conectados actualmente
     * @return Collection de Strings
     */
    Collection darClientes( );

    /**
     * Enva un mensaje a un usuario indicado el estado de un amigo
     * @param usuario El nombre del usuario al que se le debe enviar el mensaje - usuario != null && usuario != ""
     * @param estadoAmigo El estado del amigo del que se va a enviar la notificacin - estadoAmigo != null
     */
    void enviarNotificacionAmigo( String usuario, Usuario estadoAmigo );

    /**
     * Enva un mensaje a un usuario para que participe en una conversacin
     * @param usuario El nombre del usuario al que se le va a enviar el mensaje para iniciar una conversacin - usuario != null
     * @param amigo El nombre del usuario con el que se va a establecer una conversacin - amigo != null
     * @param direccionIp La direccion ip del usuario con el que se va a establecer la conversacin - direccionIp != null
     * @param puerto El puerto al que se debe conectar el usuario
     */
    void iniciarConversacion( String usuario, String amigo, String direccionIp, int puerto );

    /**
     * Elimina el manejador del usuario indicado porque este se est desconectando
     * @param usuario El nombre del usuario que se est desconectado - usuario != null
     */
    void desconectarCliente( String usuario );

    /**
     * Mtodo para la extensin 1
     * @return respuesta1
     */
    String metodo1( );

    /**
     * Mtodo para la extensin2
     * @return respuesta2
     */
    String metodo2( );
}