/**
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * $Id: AyudanteConversacionEspera.java,v 1.1 2006/11/09 03:13:57 f-vela Exp $
 * Universidad de los Andes (Bogot - Colombia)
 * Departamento de Ingeniera de Sistemas y Computacin 
 * Licenciado bajo el esquema Academic Free License version 2.1 
 *
 * Proyecto Cupi2 (http://cupi2.uniandes.edu.co)
 * Ejercicio: n12_messengerAmigos
 * Autor: Mario Snchez - 21-abr-2006
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 */
package uniandes.cupi2.messengerAmigos.testCliente;

import java.io.IOException;

import uniandes.cupi2.messengerAmigos.cliente.Conversacion;

/**
 * Esta clase se usa para construir una conversacin que esperar a que alguien se conecte a ella.<br>
 * La conversacin esperar en un thread diferente al principal.
 */
public class AyudanteConversacionEspera extends Thread
{
    // -----------------------------------------------------------------
    // Constantes
    // -----------------------------------------------------------------

    /**
     * Es el nombre del usuario asociado a la conversacin
     */
    public static final String USUARIO_ESPERA = "usuarioEspera";

    /**
     * Es el puerto en el cual se esper la conexin
     */
    public static final int PUERTO_ESPERA = 9998;

    // -----------------------------------------------------------------
    // Atributos
    // -----------------------------------------------------------------

    /**
     * Es la conversacin que espera
     */
    private Conversacion conversacion;

    /**
     * Es el cliente asociado a la conversacin
     */
    private SimuladorClienteAmigos simuladorCliente;

    /**
     * Es la ventana asociada a la conversacin
     */
    private SimuladorVentanaConversacion simuladorVentana;

    /**
     * Es el nombre del amigo con el cual se supone que se est sosteniendo la conversacin
     */
    private String nombreAmigoActivo;

    // -----------------------------------------------------------------
    // Constructores
    // -----------------------------------------------------------------

    /**
     * Crea el ayudante, pero no hace nada an con la conversacin
     * @param nombreAmigo El nombre del amigo con el que se realizar la conversacin
     * @param ventana La ventana que estar asociada a la conversacin
     * @param cliente El cliente que estar asociado a la conversacin
     */
    public AyudanteConversacionEspera( String nombreAmigo, SimuladorVentanaConversacion ventana, SimuladorClienteAmigos cliente )
    {
        nombreAmigoActivo = nombreAmigo;
        simuladorVentana = ventana;
        simuladorCliente = cliente;
    }

    // -----------------------------------------------------------------
    // Mtodos
    // -----------------------------------------------------------------

    /**
     * Inicia el thread y crea la conversacin que espera a que alguien se conecte
     */
    public void run( )
    {
        try
        {
            conversacion = new Conversacion( simuladorCliente, PUERTO_ESPERA, nombreAmigoActivo );
            conversacion.cambiarVentanaConversacion( simuladorVentana );
        }
        catch( IOException e )
        {
            e.printStackTrace( );
        }
    }

    /**
     * Retorna la conversacin que espera
     * @return conversacion
     */
    public Conversacion darConversacion( )
    {
        return conversacion;
    }

}