/**
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * $Id: ClienteAmigosTest.java,v 1.1 2006/11/09 03:13:57 f-vela Exp $
 * Universidad de los Andes (Bogot - Colombia)
 * Departamento de Ingeniera de Sistemas y Computacin 
 * Licenciado bajo el esquema Academic Free License version 2.1 
 *
 * Proyecto Cupi2 (http://cupi2.uniandes.edu.co)
 * Ejercicio: n12_messengerAmigos
 * Autor: Mario Snchez - 21-abr-2006
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 */
package uniandes.cupi2.messengerAmigos.testCliente;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.PrintWriter;
import java.net.InetAddress;
import java.net.Socket;
import java.util.Collection;

import junit.framework.TestCase;
import uniandes.cupi2.messengerAmigos.cliente.ClienteAmigos;
import uniandes.cupi2.messengerAmigos.cliente.Conversacion;
import uniandes.cupi2.messengerAmigos.cliente.Usuario;

/**
 * Esta clase verifica que los mtodos de la clase ClienteAmigos estn correctamente implementados
 */
public class ClienteAmigosTest extends TestCase
{
    // -----------------------------------------------------------------
    // Constantes
    // -----------------------------------------------------------------

    /**
     * Es el nombre del usuario que usa el cliente que se prueba
     */
    public static final String NOMBRE_USUARIO = "nombreUsuario";

    // -----------------------------------------------------------------
    // Atributos
    // -----------------------------------------------------------------

    /**
     * Es el objeto sobre el cual se realizan las pruebas
     */
    private ClienteAmigos clienteAmigos;

    /**
     * Es el objeto que simula ser un servidor
     */
    private SimuladorServidor simuladorServidor;

    /**
     * Es el objeto que simula ser la interfaz de la aplicacin
     */
    private SimuladorInterfaz simuladorInterfaz;

    /**
     * Es el objeto que permite simular que hay una conversacin esperando a que alguien se conecte
     */
    private SimuladorConversacionEnEspera simuladorConversacion;

    // -----------------------------------------------------------------
    // Mtodos
    // -----------------------------------------------------------------

    /**
     * Construye un escenario en el cual hay un cliente que se conecta a un objeto que simula ser un servidor
     */
    private void setupEscenario1( )
    {
        try
        {
            simuladorServidor = new SimuladorServidor( );
            simuladorInterfaz = new SimuladorInterfaz( );

            clienteAmigos = new ClienteAmigos( simuladorInterfaz, "./test/data/cliente.properties" );

            simuladorServidor.start( );
            clienteAmigos.conectar( NOMBRE_USUARIO );

            String mensaje = simuladorServidor.recibirMensaje( );
            String ipLocal = InetAddress.getLocalHost( ).getHostAddress( );
            assertEquals( "El mensaje enviado por el cliente est equivocado", ClienteAmigos.LOGIN + ":" + NOMBRE_USUARIO + ";" + ipLocal + ";9998", mensaje );
        }
        catch( Exception e )
        {
            e.printStackTrace( );
            fail( "No debera haber problemas construyendo el escenario: " + e.getMessage( ) );
        }
    }

    /**
     * Elimina los objetos y las conexiones creadas para el escenario 1
     */
    private void terminarEscenario1( )
    {
        try
        {
            simuladorServidor.enviarMensajeDesconexion( );
            simuladorServidor.cerrarConexion( );
        }
        catch( IOException e )
        {
            e.printStackTrace( );
            fail( "No debera haber problemas destruyendo el escenario: " + e.getMessage( ) );
        }
    }

    /**
     * Construye un escenario en el cual hay un cliente que se conecta a un objeto que simula ser un servidor y un objeto que simula ser una conversacin a la cual se tiene
     * que unir el cliente
     */
    private void setupEscenario2( )
    {
        try
        {
            simuladorServidor = new SimuladorServidor( );
            simuladorInterfaz = new SimuladorInterfaz( );
            simuladorConversacion = new SimuladorConversacionEnEspera( );

            clienteAmigos = new ClienteAmigos( simuladorInterfaz, "./test/data/cliente.properties" );

            simuladorServidor.start( );
            clienteAmigos.conectar( NOMBRE_USUARIO );
            simuladorConversacion.start( );

            String mensaje = simuladorServidor.recibirMensaje( );
            String ipLocal = InetAddress.getLocalHost( ).getHostAddress( );
            assertEquals( "El mensaje enviado por el cliente est equivocado", ClienteAmigos.LOGIN + ":" + NOMBRE_USUARIO + ";" + ipLocal + ";9998", mensaje );
        }
        catch( Exception e )
        {
            e.printStackTrace( );
            fail( "No debera haber problemas construyendo el escenario: " + e.getMessage( ) );
        }
    }

    /**
     * Elimina los objetos y las conexiones creadas para el escenario 2
     */
    private void terminarEscenario2( )
    {
        try
        {
            simuladorConversacion.enviarMensajeTerminar( );
            simuladorConversacion.recibirMensaje( );
            simuladorConversacion.terminarConexion( );
            simuladorServidor.enviarMensajeDesconexion( );
            simuladorServidor.cerrarConexion( );
        }
        catch( IOException e )
        {
            e.printStackTrace( );
            fail( "No debera haber problemas destruyendo el escenario: " + e.getMessage( ) );
        }
    }

    /**
     * Verifica que la conexin del cliente se establezca de forma correcta
     */
    public void testClienteAmigos( )
    {
        setupEscenario1( );

        assertEquals( "El nombre del usuario es incorrecto", NOMBRE_USUARIO, clienteAmigos.darNombreUsuario( ) );

        terminarEscenario1( );
    }

    /**
     * Verifica que el mtodo actualizarEstado actualice la interfaz
     */
    public void testActualizarEstado( )
    {
        setupEscenario1( );

        Usuario alicia = new Usuario( "Alicia" );
        clienteAmigos.actualizarEstado( alicia );

        // Verificar que se haya actualizado la interfaz
        Collection estadoAmigos = simuladorInterfaz.darEstadoAmigos( );
        assertEquals( "Debera haber un amigo", 1, estadoAmigos.size( ) );
        Usuario amigo = ( Usuario )estadoAmigos.iterator( ).next( );
        assertEquals( "El estado del usuario est equivocado", Usuario.OFFLINE, amigo.darEstado( ) );
        assertEquals( "El nombre del usuario est equivocado", "Alicia", amigo.darNombre( ) );

        terminarEscenario1( );
    }

    /**
     * Verifica que el mtodo agregarAmigo actualice enve el mensaje correcto al servidor
     */
    public void testAgregarAmigo( )
    {
        setupEscenario1( );

        try
        {
            clienteAmigos.agregarAmigo( "Alicia" );

            String mensaje = simuladorServidor.recibirMensaje( );
            assertEquals( "El mensaje enviado por el cliente est equivocado", ClienteAmigos.AGREGAR_AMIGO + ":Alicia", mensaje );
        }
        catch( IOException e )
        {
            e.printStackTrace( );
            fail( "No debera haber problemas: " + e.getMessage( ) );
        }

        terminarEscenario1( );
    }

    /**
     * Verifica que el mtodo conectarAConversacion se conecte correctamente a una conversacin remota
     */
    public void testConectarAConversacion( )
    {
        setupEscenario2( );

        try
        {
            clienteAmigos.conectarAConversacion( "amigo", "localhost", SimuladorConversacionEnEspera.PUERTO_ESPERA );

            // Verificar que se le haya notificado a la interfaz de la nueva conversacion
            Conversacion conv = simuladorInterfaz.darConversacion( );
            assertNotNull( "La interfaz no fue notificada de la creacin de la conversacin", conv );

            // Obtener la conversacin creada y revisar que est efectivamente conectada al socket
            conv.enviarMensaje( "hola" );

            String mensajeConv = simuladorConversacion.recibirMensaje( );
            assertTrue( "El mensaje no fue el esperado", mensajeConv.startsWith( Conversacion.MENSAJE ) && mensajeConv.indexOf( "hola" ) != -1 );
        }
        catch( IOException e )
        {
            e.printStackTrace( );
            fail( "No debera haber problemas: " + e.getMessage( ) );
        }

        terminarEscenario2( );
    }

    /**
     * Verifica que el mtodo crearConversacionLocal cree correctamente la conversacin y permita la conexin remota
     */
    public void testCrearConversacionLocal( )
    {
        setupEscenario1( );

        try
        {
            AyudanteConversacionLocal ayudante = new AyudanteConversacionLocal( clienteAmigos );
            ayudante.start( );

            // Revisar que se haya enviado el mensaje correcto al servidor
            String mensaje = simuladorServidor.recibirMensaje( );
            assertEquals( "El mensaje que se envi al servidor no fue el esperado", ClienteAmigos.CONVERSACION + ":Alicia", mensaje );

            // Establecer la conexin con el cliente
            Socket socketConversacion = new Socket( "localhost", 9998 );
            PrintWriter outConversacion = new PrintWriter( socketConversacion.getOutputStream( ), true );
            BufferedReader inConversacion = new BufferedReader( new InputStreamReader( socketConversacion.getInputStream( ) ) );

            // Obtener la conversacin creada y revisar que est efectivamente conectada al socket
            Conversacion conv = null;
            while( conv == null )
                conv = ayudante.darConversacion( );

            conv.enviarMensaje( "hola" );

            String mensajeConv = inConversacion.readLine( );
            assertTrue( "El mensaje no fue el esperado", mensajeConv.startsWith( Conversacion.MENSAJE ) && mensajeConv.indexOf( "hola" ) != -1 );

            // Cerrar la conversacion
            outConversacion.println( Conversacion.TERMINAR );
            inConversacion.readLine( ); // Leer el menasje CONVERSACION_TERMINADA

            outConversacion.close( );
            inConversacion.close( );
            socketConversacion.close( );
        }
        catch( IOException e )
        {
            e.printStackTrace( );
            fail( "No debera haber problemas: " + e.getMessage( ) );
        }

        terminarEscenario1( );
    }

    /**
     * Verifica que el mtodo enviarDesconexion enve correctamente el mensaje al servidor
     */
    public void testEnviarDesconexion( )
    {
        setupEscenario1( );

        try
        {
            clienteAmigos.enviarDesconexion( );

            String mensaje = simuladorServidor.recibirMensaje( );
            assertEquals( "El mensaje enviado por el cliente est equivocado", ClienteAmigos.LOGOUT + ":" + NOMBRE_USUARIO, mensaje );
        }
        catch( IOException e )
        {
            e.printStackTrace( );
            fail( "No debera haber problemas: " + e.getMessage( ) );
        }

        terminarEscenario1( );
    }

}