/** ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * $Id: ConversacionTest.java,v 1.1 2006/11/09 03:13:57 f-vela Exp $
 * Universidad de los Andes (Bogot - Colombia)
 * Departamento de Ingeniera de Sistemas y Computacin 
 * Licenciado bajo el esquema Academic Free License versin 2.1
 *
 * Proyecto Cupi2 (http://cupi2.uniandes.edu.co)
 * Ejercicio: n12_messengerAmigos
 * Autor: Mario Snchez - 3/05/2006
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 */

package uniandes.cupi2.messengerAmigos.testCliente;

import java.util.LinkedList;

import junit.framework.TestCase;
import uniandes.cupi2.messengerAmigos.cliente.Conversacion;
import uniandes.cupi2.messengerAmigos.cliente.Usuario;

/**
 * Esta es la clase usada para verificar la implementacin de los mtodos de la clase Conversacin. <br>
 * Las pruebas requieren la ayuda de numerosas clases que se encuentran en el paquete uniandes.cupi2.messengerAmigos.test.conversacion <br>
 * Todas las pruebas realizadas se realizan tanto en el caso que la conversacin que se est verificando haya esperado una conexin como en el caso de que se haya conectado a
 * otra conversacin.
 */
public class ConversacionTest extends TestCase
{
    // -----------------------------------------------------------------
    // Constantes
    // -----------------------------------------------------------------

    /**
     * Indica el tiempo que se va a esperar cuando se realicen acciones en threads diferentes al principal
     */
    public static final long TIEMPO_ESPERA = 100;

    /**
     * Es el nombre del usuario que se supone establece una conexin activa
     */
    public static final String USUARIO_ACTIVO = "usuarioActivo";

    /**
     * Es el nombre del usuario que se supone establece una conversacin que espera
     */
    public static final String USUARIO_ESPERA = "usuarioEspera";

    /**
     * Es el nombre de la persona que se conecta a la conversacin que espera
     */
    public static final String AMIGO_ACTIVO = "amigoActivo";

    // -----------------------------------------------------------------
    // Atributos
    // -----------------------------------------------------------------

    /**
     * Es la conversacin que inicia una conexin hacia la otra conversacin
     */
    private Conversacion conversacionActiva;

    /**
     * Es la conversacin que espera a que alguien se conecte a ella
     */
    private Conversacion conversacionEspera;

    /**
     * Es el objeto que simula ser una conversacin que espera a que la conversacin activa se conecte
     */
    private SimuladorConversacionEspera simuladorEspera;

    /**
     * Es el objeto que simula ser una conversacin que se conecta a una conversacin que espera
     */
    private SimuladorConversacionActiva simuladorActiva;

    /**
     * Es el objeto que simula ser el clienteAmigos asociado a una conversacin
     */
    private SimuladorClienteAmigos simuladorClienteAmigos;

    /**
     * Es el objeto que simula ser la ventana asociada a una conversacin
     */
    private SimuladorVentanaConversacion simuladorVentana;

    /**
     * Es el objeto que permite que una conversacin quede esperando una conexin, desde un thread diferente al principal
     */
    private AyudanteConversacionEspera ayudanteConversacionEspera;

    // -----------------------------------------------------------------
    // Mtodos
    // -----------------------------------------------------------------

    /**
     * En este escenario una conversacin es creada para que se conecte a un objeto que simula ser una conversacin. <br>
     * Despus de establecer la conexin se verifica que la conversacin funcione correctamente. <br>
     * La conversacin ser llamada conversacionActiva.
     */
    private void setupEscenario1( )
    {
        try
        {
            // Crear el objeto que simular ser el cliente amigos que necesita la conversacin
            simuladorClienteAmigos = new SimuladorClienteAmigos( USUARIO_ACTIVO );

            // Crear el objeto que simular ser la ventana de la conversacin
            simuladorVentana = new SimuladorVentanaConversacion( );

            // Crear el objeto que esperar la conexin de la conversacin activa
            simuladorEspera = new SimuladorConversacionEspera( );
            simuladorEspera.start( );

            // Darle tiempo al socket para prepararse
            Thread.sleep( TIEMPO_ESPERA );

            // Crear la conversacion activa
            Usuario amigo = new Usuario( SimuladorConversacionEspera.AMIGO_ESPERA, "localhost", SimuladorConversacionEspera.PUERTO_ESPERA );
            conversacionActiva = new Conversacion( simuladorClienteAmigos, amigo );
            conversacionActiva.cambiarVentanaConversacion( simuladorVentana );
        }
        catch( Exception e )
        {
            e.printStackTrace( );
            fail( "No debera producirse una excepcin creando la conversacin activa: " + e.getMessage( ) );
        }
    }

    /**
     * En este escenario un objeto que simula ser una conversacin se conecta a una conversacin que estaba esperando una conexin. <br>
     * Despus de establecer la conexin se verifica que la conversacin funcione correctamente. <br>
     * La conversacin ser llamada conversacionEspera.
     */
    private void setupEscenario2( )
    {
        try
        {
            // Crear el objeto que simular ser el cliente amigos que necesita la conversacin
            simuladorClienteAmigos = new SimuladorClienteAmigos( USUARIO_ESPERA );

            // Crear el objeto que simular ser la ventana de la conversacin
            simuladorVentana = new SimuladorVentanaConversacion( );

            // Crear el objeto que crear la conversacin que esperar a que alguien se conecte
            ayudanteConversacionEspera = new AyudanteConversacionEspera( AMIGO_ACTIVO, simuladorVentana, simuladorClienteAmigos );
            ayudanteConversacionEspera.start( );

            // Conectarse a la conversacin en espera usando un simulador de una conversacion activa
            simuladorActiva = new SimuladorConversacionActiva( );
            simuladorActiva.conectar( );

            // Darle tiempo al socket para prepararse
            Thread.sleep( TIEMPO_ESPERA );

            // Obtener la conversacin en espera
            conversacionEspera = ayudanteConversacionEspera.darConversacion( );
        }
        catch( Exception e )
        {
            e.printStackTrace( );
            fail( "No debera producirse una excepcin creando la conversacin en espera: " + e.getMessage( ) );
        }
    }

    /**
     * Este mtodo se encarga de deshacer lo construdo al preparar el escenario 1 <br>
     * La conversacin activa debe ser terminada, al igual que el simulador de la conversacin en espera. <br>
     * El xito de esta tarea depende de que la conversacin termine correctamente cuando reciba el mensaje TERMINAR.
     */
    private void terminarEscenario1( )
    {
        try
        {
            // Enviar el mensaje TERMINAR
            simuladorEspera.enviarMensajeTerminar( );

            // Esperar el mensaje CONVERSACION_TERMINADA
            simuladorEspera.recibirMensaje( );
            LinkedList mensajes = simuladorEspera.darMensajes( );
            assertTrue( "Debera haber un mensaje de respuesta", mensajes.size( ) > 0 );
            String ultimoMensaje = ( String )mensajes.getLast( );
            assertTrue( "El mensaje debera ser CONVERSACION_TERMINADA, pero fue " + ultimoMensaje, ultimoMensaje.startsWith( Conversacion.CONVERSACION_TERMINADA ) );

            // Darle tiempo a la conversacion para cerrarse
            Thread.sleep( TIEMPO_ESPERA );

            // Verificar que la conversacin y su ventana se hayan cerrado
            assertTrue( "La ventana debera estar cerrada", simuladorVentana.estaCerrada( ) );
            assertTrue( "La conversacion debera haber terminado", conversacionActiva.conversacionTerminada( ) );

            // Cerrar el simulador de la conversacin en espera
            simuladorEspera.terminarConexion( );
        }
        catch( Exception e )
        {
            e.printStackTrace( );
            fail( "No debera producirse una excepcin terminando la conversacin activa: " + e.getMessage( ) );
        }
    }

    /**
     * Este mtodo se encarga de deshacer lo construdo al preparar el escenario 2 <br>
     * La conversacin que esperaba debe ser terminada, al igual que el simulador de la conversacin activa. <br>
     * El xito de esta tarea depende de que la conversacin termine correctamente cuando reciba el mensaje TERMINAR.
     */
    private void terminarEscenario2( )
    {
        try
        {
            // Enviar el mensaje TERMINAR
            simuladorActiva.enviarMensajeTerminar( );

            // Esperar el mensaje CONVERSACION_TERMINADA
            simuladorActiva.recibirMensaje( );
            LinkedList mensajes = simuladorActiva.darMensajes( );
            assertTrue( "Debera haber un mensaje de respuesta", mensajes.size( ) > 0 );
            String ultimoMensaje = ( String )mensajes.getLast( );
            assertTrue( "El mensaje debera ser CONVERSACION_TERMINADA pero fue " + ultimoMensaje, ultimoMensaje.startsWith( Conversacion.CONVERSACION_TERMINADA ) );

            // Darle tiempo a la conversacion para cerrarse
            Thread.sleep( TIEMPO_ESPERA );

            // Verificar que la conversacin y su ventana se hayan cerrado
            assertTrue( "La ventana debera estar cerrada", simuladorVentana.estaCerrada( ) );
            assertTrue( "La conversacion debera haber terminado", conversacionEspera.conversacionTerminada( ) );

            // Cerrar el simulador de la conversacin en espera
            simuladorActiva.terminarConexion( );
        }
        catch( Exception e )
        {
            e.printStackTrace( );
            fail( "No debera producirse una excepcin terminando la conversacin en espera: " + e.getMessage( ) );
        }
    }

    /**
     * Verifica que se haya podido crear una conversacin activa (que se conecta a un objeto que simula ser una conversacin en espera). <br>
     * Adems se verifica que la conversacin termine correctamente cuando se le enva el mensaje adecuado (TERMINAR).
     */
    public void testConversacionActiva( )
    {
        setupEscenario1( );

        assertFalse( "La conversacion NO debera haber terminado", conversacionActiva.conversacionTerminada( ) );

        terminarEscenario1( );
    }

    /**
     * Verifica que se haya podido crear una conversacin en espera (a la que se conecta a un objeto que simula ser una conversacin activa). <br>
     * Adems se verifica que la conversacin termine correctamente cuando se le enva el mensaje adecuado (TERMINAR).
     */
    public void testConversacionEspera( )
    {
        setupEscenario2( );

        assertFalse( "La conversacion NO debera haber terminado", conversacionEspera.conversacionTerminada( ) );

        terminarEscenario2( );
    }

    /**
     * Verifica el mtodo darNombreAmigo en el caso de la conversacin activa
     */
    public void testDarNombreAmigo1( )
    {
        setupEscenario1( );

        assertEquals( "El nombre del amigo en la conversacin no es correcto", SimuladorConversacionEspera.AMIGO_ESPERA, conversacionActiva.darNombreAmigo( ) );

        terminarEscenario1( );
    }

    /**
     * Verifica el mtodo darNombreAmigo en el caso de la conversacin en espera
     */
    public void testDarNombreAmigo2( )
    {
        setupEscenario2( );

        assertEquals( "El nombre del amigo en la conversacin no es correcto", AMIGO_ACTIVO, conversacionEspera.darNombreAmigo( ) );

        terminarEscenario2( );
    }

    /**
     * Verifica el mtodo enviarMensaje en el caso de la conversacin activa. <br>
     * El mensaje debera ser enviado a la conversacin remota y debera ser publicado en la ventana de la conversacin.
     */
    public void testEnviarMensaje1( )
    {
        setupEscenario1( );

        try
        {
            conversacionActiva.enviarMensaje( "prueba mensaje" );

            // Verificar que el mensaje haya sido publicado en la ventana local
            String mensajePublicado = simuladorVentana.darUltimoMensaje( );
            assertTrue( "El mensaje publicado no tiene el remitente correcto", mensajePublicado.startsWith( USUARIO_ACTIVO ) );
            assertTrue( "El mensaje publicado no tiene el mensaje correcto", mensajePublicado.indexOf( "prueba mensaje" ) != -1 );

            // Verificar que el mensaje haya llegado al otro cliente
            simuladorEspera.recibirMensaje( );
            LinkedList mensajes = simuladorEspera.darMensajes( );
            assertTrue( "Debera estar el mensaje enviado", mensajes.size( ) > 0 );
            String ultimoMensaje = ( String )mensajes.getLast( );
            assertTrue( "El mensaje debera ser 'prueba mensaje'", ultimoMensaje.indexOf( "prueba mensaje" ) != -1 );
            mensajes.clear( );

        }
        catch( Exception e )
        {
            e.printStackTrace( );
            fail( "No debera producirse una excepcin enviando o recibiendo un mensaje: " + e.getMessage( ) );
        }
        finally
        {
            terminarEscenario1( );
        }
    }

    /**
     * Verifica el mtodo enviarMensaje en el caso de la conversacin en espera. <br>
     * El mensaje debera ser enviado a la conversacin remota y debera ser publicado en la ventana de la conversacin.
     */
    public void testEnviarMensaje2( )
    {
        setupEscenario2( );

        try
        {
            conversacionEspera.enviarMensaje( "prueba mensaje" );

            // Verificar que el mensaje haya sido publicado en la ventana local
            String mensajePublicado = simuladorVentana.darUltimoMensaje( );
            assertTrue( "El mensaje publicado no tiene el remitente correcto", mensajePublicado.startsWith( USUARIO_ESPERA ) );
            assertTrue( "El mensaje publicado no tiene el mensaje correcto", mensajePublicado.indexOf( "prueba mensaje" ) != -1 );

            // Verificar que el mensaje haya llegado al otro cliente
            simuladorActiva.recibirMensaje( );
            LinkedList mensajes = simuladorActiva.darMensajes( );
            assertTrue( "Debera estar el mensaje enviado", mensajes.size( ) > 0 );
            String ultimoMensaje = ( String )mensajes.getLast( );
            assertTrue( "El mensaje debera ser 'prueba mensaje'", ultimoMensaje.indexOf( "prueba mensaje" ) != -1 );
            mensajes.clear( );

        }
        catch( Exception e )
        {
            e.printStackTrace( );
            fail( "No debera producirse una excepcin enviando o recibiendo un mensaje: " + e.getMessage( ) );
        }
        finally
        {
            terminarEscenario2( );
        }
    }

    /**
     * Verifica que la conversacin activa reciba correctamente los mensajes enviados desde la otra conversacin
     */
    public void testRecibirMensajes1( )
    {
        setupEscenario1( );

        try
        {
            // Enviar el mensaje desde la conversacin remota
            simuladorEspera.enviarMensaje( "prueba mensaje" );

            // Darle tiempo a la conversacion para recibir y publicar el mensaje
            Thread.sleep( TIEMPO_ESPERA );

            // Verificar que el mensaje haya sido publicado en la ventana local
            String mensajePublicado = simuladorVentana.darUltimoMensaje( );
            assertTrue( "El mensaje publicado no tiene el remitente correcto", mensajePublicado.startsWith( SimuladorConversacionEspera.AMIGO_ESPERA ) );
            assertTrue( "El mensaje publicado no tiene el mensaje correcto", mensajePublicado.indexOf( "prueba mensaje" ) != -1 );
        }
        catch( Exception e )
        {
            e.printStackTrace( );
            fail( "No debera producirse una excepcin enviando o recibiendo un mensaje: " + e.getMessage( ) );
        }
        finally
        {
            terminarEscenario1( );
        }
    }

    /**
     * Verifica que la conversacin que espera reciba correctamente los mensajes enviados desde la otra conversacin
     */
    public void testRecibirMensajes2( )
    {
        setupEscenario2( );

        try
        {
            // Enviar el mensaje desde la conversacin remota
            simuladorActiva.enviarMensaje( "prueba mensaje" );

            // Darle tiempo a la conversacion para recibir y publicar el mensaje
            Thread.sleep( TIEMPO_ESPERA );

            // Verificar que el mensaje haya sido publicado en la ventana local
            String mensajePublicado = simuladorVentana.darUltimoMensaje( );
            assertTrue( "El mensaje publicado no tiene el remitente correcto", mensajePublicado.startsWith( AMIGO_ACTIVO ) );
            assertTrue( "El mensaje publicado no tiene el mensaje correcto", mensajePublicado.indexOf( "prueba mensaje" ) != -1 );
        }
        catch( Exception e )
        {
            e.printStackTrace( );
            fail( "No debera producirse una excepcin enviando o recibiendo un mensaje: " + e.getMessage( ) );
        }
        finally
        {
            terminarEscenario2( );
        }
    }

    /**
     * Verifica que el mtodo mostrarMensajeRecibido de la conversacin activa publique los mensajes recibidos en la ventana
     */
    public void testMostrarMensajeRecibido1( )
    {
        setupEscenario1( );

        try
        {
            // Enviar el mensaje directamente a la conversacin
            conversacionActiva.mostrarMensajeRecibido( "prueba mensaje" );

            // Verificar que el mensaje haya sido publicado en la ventana local
            String mensajePublicado = simuladorVentana.darUltimoMensaje( );
            assertTrue( "El mensaje publicado no tiene el remitente correcto", mensajePublicado.startsWith( SimuladorConversacionEspera.AMIGO_ESPERA ) );
            assertTrue( "El mensaje publicado no tiene el mensaje correcto", mensajePublicado.indexOf( "prueba mensaje" ) != -1 );
        }
        catch( Exception e )
        {
            e.printStackTrace( );
            fail( "No debera producirse una excepcin enviando o recibiendo un mensaje: " + e.getMessage( ) );
        }
        finally
        {
            terminarEscenario1( );
        }
    }

    /**
     * Verifica que el mtodo mostrarMensajeRecibido de la conversacin que espera publique los mensajes recibidos en la ventana
     */
    public void testMostrarMensajeRecibido2( )
    {
        setupEscenario2( );

        try
        {
            // Enviar el mensaje directamente a la conversacin
            conversacionEspera.mostrarMensajeRecibido( "prueba mensaje" );

            // Verificar que el mensaje haya sido publicado en la ventana local
            String mensajePublicado = simuladorVentana.darUltimoMensaje( );
            assertTrue( "El mensaje publicado no tiene el remitente correcto", mensajePublicado.startsWith( AMIGO_ACTIVO ) );
            assertTrue( "El mensaje publicado no tiene el mensaje correcto", mensajePublicado.indexOf( "prueba mensaje" ) != -1 );
        }
        catch( Exception e )
        {
            e.printStackTrace( );
            fail( "No debera producirse una excepcin enviando o recibiendo un mensaje: " + e.getMessage( ) );
        }
        finally
        {
            terminarEscenario2( );
        }
    }

    /**
     * Verifica que el mtodo terminar de la conversacin activa enve el mensaje TERMINAR a la otra conversacin
     */
    public void testTerminar1( )
    {
        setupEscenario1( );

        try
        {
            // Enviar el mensaje TERMINAR
            conversacionActiva.terminar( );

            // Verificar que el mensaje haya llegado al otro cliente
            simuladorEspera.recibirMensaje( );
            LinkedList mensajes = simuladorEspera.darMensajes( );
            assertTrue( "Debera estar el mensaje TERMINAR ", mensajes.size( ) > 0 );
            String ultimoMensaje = ( String )mensajes.getLast( );
            assertTrue( "El mensaje debera ser 'TERMINAR'", ultimoMensaje.startsWith( Conversacion.TERMINAR ) );
            mensajes.clear( );
        }
        catch( Exception e )
        {
            e.printStackTrace( );
            fail( "No debera producirse una excepcin enviando o recibiendo un mensaje: " + e.getMessage( ) );
        }
        finally
        {
            terminarEscenario1( );
        }
    }

    /**
     * Verifica que el mtodo terminar de la conversacin que espera enve el mensaje TERMINAR a la otra conversacin
     */
    public void testTerminar2( )
    {
        setupEscenario2( );

        try
        {
            // Enviar el mensaje TERMINAR
            conversacionEspera.terminar( );

            // Verificar que el mensaje haya llegado al otro cliente
            simuladorActiva.recibirMensaje( );
            LinkedList mensajes = simuladorActiva.darMensajes( );
            assertTrue( "Debera estar el mensaje TERMINAR ", mensajes.size( ) > 0 );
            String ultimoMensaje = ( String )mensajes.getLast( );
            assertTrue( "El mensaje debera ser 'TERMINAR'", ultimoMensaje.startsWith( Conversacion.TERMINAR ) );
            mensajes.clear( );
        }
        catch( Exception e )
        {
            e.printStackTrace( );
            fail( "No debera producirse una excepcin enviando o recibiendo un mensaje: " + e.getMessage( ) );
        }
        finally
        {
            terminarEscenario2( );
        }
    }

    /**
     * Verifica que el mtodo enviarConfirmacionTerminarConversacion de la conversacin activa enve el mensaje CONVERSACION_TERMINADA a la otra conversacin
     */
    public void testEnviarConfirmacionTerminarConversacion1( )
    {
        setupEscenario1( );

        try
        {
            // Enviar el mensaje CONVERSACION_TERMINADA
            conversacionActiva.enviarConfirmacionTerminarConversacion( );

            // Verificar que el mensaje haya llegado al otro cliente
            simuladorEspera.recibirMensaje( );
            LinkedList mensajes = simuladorEspera.darMensajes( );
            assertTrue( "Debera estar el mensaje CONVERSACION_TERMINADA ", mensajes.size( ) > 0 );
            String ultimoMensaje = ( String )mensajes.getLast( );
            assertTrue( "El mensaje debera ser 'CONVERSACION_TERMINADA'", ultimoMensaje.startsWith( Conversacion.CONVERSACION_TERMINADA ) );
            mensajes.clear( );
        }
        catch( Exception e )
        {
            e.printStackTrace( );
            fail( "No debera producirse una excepcin enviando o recibiendo un mensaje: " + e.getMessage( ) );
        }
        finally
        {
            terminarEscenario1( );
        }
    }

    /**
     * Verifica que el mtodo enviarConfirmacionTerminarConversacion de la conversacin que espera enve el mensaje CONVERSACION_TERMINADA a la otra conversacin
     */
    public void testEnviarConfirmacionTerminarConversacion2( )
    {
        setupEscenario2( );

        try
        {
            // Enviar el mensaje CONVERSACION_TERMINADA
            conversacionEspera.enviarConfirmacionTerminarConversacion( );

            // Verificar que el mensaje haya llegado al otro cliente
            simuladorActiva.recibirMensaje( );
            LinkedList mensajes = simuladorActiva.darMensajes( );
            assertTrue( "Debera estar el mensaje CONVERSACION_TERMINADA ", mensajes.size( ) > 0 );
            String ultimoMensaje = ( String )mensajes.getLast( );
            assertTrue( "El mensaje debera ser 'CONVERSACION_TERMINADA'", ultimoMensaje.startsWith( Conversacion.CONVERSACION_TERMINADA ) );
            mensajes.clear( );
        }
        catch( Exception e )
        {
            e.printStackTrace( );
            fail( "No debera producirse una excepcin enviando o recibiendo un mensaje: " + e.getMessage( ) );
        }
        finally
        {
            terminarEscenario2( );
        }
    }
}
