/** ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * $Id: SimuladorConversacionActiva.java,v 1.1 2006/11/09 03:13:57 f-vela Exp $
 * Universidad de los Andes (Bogot - Colombia)
 * Departamento de Ingeniera de Sistemas y Computacin 
 * Licenciado bajo el esquema Academic Free License versin 2.1
 *
 * Proyecto Cupi2 (http://cupi2.uniandes.edu.co)
 * Ejercicio: n12_messengerAmigos
 * Autor: Mario Snchez - 3/05/2006
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 */

package uniandes.cupi2.messengerAmigos.testCliente;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.PrintWriter;
import java.net.Socket;
import java.util.LinkedList;

import uniandes.cupi2.messengerAmigos.cliente.Conversacion;

/**
 * Esta clase sirve para simular el comportamiento de una conversacin activa que se conecta a una conversacin que espera
 */
public class SimuladorConversacionActiva
{
    // -----------------------------------------------------------------
    // Atributos
    // -----------------------------------------------------------------

    /**
     * Es el socket a travs del cual se lleva a cabo la comunicacin con la conversacion
     */
    private Socket socketConversacion;

    /**
     * Es el stream usuado para leer los mensajes enviados por la conversacion
     */
    private BufferedReader inConversacion;

    /**
     * Es el stream usado para enviar mensajes la conversacion
     */
    private PrintWriter outConversacion;

    /**
     * Es la lista donde se mantienen los mensajes recibidos
     */
    private LinkedList mensajes;

    // -----------------------------------------------------------------
    // Constructores
    // -----------------------------------------------------------------

    /**
     * Construye el simulador pero no hace nada con la conversacin por el momento
     */
    public SimuladorConversacionActiva( )
    {
        mensajes = new LinkedList( );
    }

    // -----------------------------------------------------------------
    // Mtodos
    // -----------------------------------------------------------------

    /**
     * Establece una conexin con una conversacin que esperaba a que alguien se conectara
     * @throws IOException Se lanza esta excepcin si hay problemas en la comunicacin
     */
    public void conectar( ) throws IOException
    {
        socketConversacion = new Socket( "localhost", AyudanteConversacionEspera.PUERTO_ESPERA );
        outConversacion = new PrintWriter( socketConversacion.getOutputStream( ), true );
        inConversacion = new BufferedReader( new InputStreamReader( socketConversacion.getInputStream( ) ) );
    }

    /**
     * Enva un mensaje a la conversacin
     * @param mensaje el mensaje enviado
     * @throws IOException Se lanza esta excepcin si hay problemas en la comunicacin
     */
    public void enviarMensaje( String mensaje ) throws IOException
    {
        outConversacion.println( Conversacion.MENSAJE + ":" + mensaje );
    }

    /**
     * Enva un mensaje para terminar la conversacin (TERMINAR)
     */
    public void enviarMensajeTerminar( )
    {
        outConversacion.println( Conversacion.TERMINAR );
    }

    /**
     * Recibe un mensaje enviado por la conversacin
     * @throws IOException Se lanza esta excepcin si hay problemas en la comunicacin
     */
    public void recibirMensaje( ) throws IOException
    {
        String mensajeRecibido = inConversacion.readLine( );
        mensajes.add( mensajeRecibido );
    }

    /**
     * Retorna la lista de mensajes recibidos por el simulador
     * @return mensajes
     */
    public LinkedList darMensajes( )
    {
        return mensajes;
    }

    /**
     * Termina la conexin del simulador con la conversacin
     * @throws IOException Se lanza esta excepcin si hay problemas en la comunicacin
     */
    public void terminarConexion( ) throws IOException
    {
        inConversacion.close( );
        outConversacion.close( );
        socketConversacion.close( );
    }
}
