/** ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * $Id: SimuladorConversacionEnEspera.java,v 1.1 2006/11/09 03:13:57 f-vela Exp $
 * Universidad de los Andes (Bogot - Colombia)
 * Departamento de Ingeniera de Sistemas y Computacin 
 * Licenciado bajo el esquema Academic Free License versin 2.1
 *
 * Proyecto Cupi2 (http://cupi2.uniandes.edu.co)
 * Ejercicio: n12_messengerAmigos
 * Autor: Mario Snchez - 3/05/2006
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 */

package uniandes.cupi2.messengerAmigos.testCliente;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.PrintWriter;
import java.net.ServerSocket;
import java.net.Socket;

import junit.framework.TestCase;
import uniandes.cupi2.messengerAmigos.cliente.Conversacion;

/**
 * Esta clase sirve para simular que hay una conversacin que est esperando a que un cliente se conecte
 */
public class SimuladorConversacionEnEspera extends Thread
{
    // -----------------------------------------------------------------
    // Constantes
    // -----------------------------------------------------------------

    /**
     * Es el puerto en el que se espera la conexin
     */
    public static final int PUERTO_ESPERA = 9998;

    // -----------------------------------------------------------------
    // Atributos
    // -----------------------------------------------------------------

    /**
     * Es el socket a travs del cual se lleva a cabo la comunicacin con la conversacion
     */
    private Socket socketConversacion;

    /**
     * Es el stream usuado para leer los mensajes enviados por la conversacion
     */
    private BufferedReader inConversacion;

    /**
     * Es el stream usado para enviar mensajes la conversacion
     */
    private PrintWriter outConversacion;

    /**
     * Indica si el socket ya se pudo conectar
     */
    private boolean conectado;

    // -----------------------------------------------------------------
    // Mtodos
    // -----------------------------------------------------------------

    /**
     * Inicia el thread y crea un socket que espera a que una conversacin se conecte.
     */
    public void run( )
    {
        try
        {
            ServerSocket ssocket = new ServerSocket( PUERTO_ESPERA );

            socketConversacion = ssocket.accept( );
            outConversacion = new PrintWriter( socketConversacion.getOutputStream( ), true );
            inConversacion = new BufferedReader( new InputStreamReader( socketConversacion.getInputStream( ) ) );

            ssocket.close( );

            conectado = true;
        }
        catch( IOException e )
        {
            e.printStackTrace( );
            TestCase.fail( "Hubo un problema esperando la conexin de la conversacin: " + e.getMessage( ) );
        }
    }

    /**
     * Indica si la conversacin del simulador ya se conect
     * @return conectado
     */
    public boolean estaConectado( )
    {
        return conectado;
    }

    /**
     * Enva un mensaje para terminar la conversacin (TERMINAR)
     */
    public void enviarMensajeTerminar( )
    {
        outConversacion.println( Conversacion.TERMINAR );
    }

    /**
     * Recibe un mensaje enviado por el cliente y lo retorna
     * @return mensaje
     * @throws IOException Se lanza esta excepcin si hay problemas en la comunicacin
     */
    public String recibirMensaje( ) throws IOException
    {
        return inConversacion.readLine( );
    }

    /**
     * Termina la conexin del simulador con la conversacin
     * @throws IOException Se lanza esta excepcin si hay problemas en la comunicacin
     */
    public void terminarConexion( ) throws IOException
    {
        inConversacion.close( );
        outConversacion.close( );
        socketConversacion.close( );
    }
}
