/** ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * $Id: SimuladorInterfaz.java,v 1.1 2006/11/09 03:13:57 f-vela Exp $
 * Universidad de los Andes (Bogot - Colombia)
 * Departamento de Ingeniera de Sistemas y Computacin 
 * Licenciado bajo el esquema Academic Free License versin 2.1
 *
 * Proyecto Cupi2 (http://cupi2.uniandes.edu.co)
 * Ejercicio: n12_messengerAmigos
 * Autor: Mario Snchez - 7/05/2006
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 */

package uniandes.cupi2.messengerAmigos.testCliente;

import java.util.Collection;

import uniandes.cupi2.messengerAmigos.cliente.Conversacion;
import uniandes.cupi2.messengerAmigos.interfazCliente.IInterfazCliente;

/**
 * Esta clase sirve para simular la interfaz de la aplicacin, a la cual el cliente tiene que informarle de varios sucesos durante la ejecucin de la aplicacin
 */
public class SimuladorInterfaz implements IInterfazCliente
{
    // -----------------------------------------------------------------
    // Atributos
    // -----------------------------------------------------------------

    /**
     * Es la informacin sobre el estado de los amigos que enva el cliente a la interfaz
     */
    private Collection estadoAmigos;

    /**
     * Es la conversacin que el cliente crea y para la cual la interfaz debera crear una ventana
     */
    private Conversacion conversacion;

    // -----------------------------------------------------------------
    // Mtodos
    // -----------------------------------------------------------------

    /**
     * La implementacin de este mtodo permite conocer la informacin que el cliente le envi a la interfaz para actualizar el estado de los amigos.
     * @param amigos Es la informacin sobre el estado de los amigos del usuario
     */
    public void actualizarAmigos( Collection amigos )
    {
        estadoAmigos = amigos;
    }

    /**
     * Retorna la informacin que el cliente envi a la interfaz sobre el estado de los amigos
     * @return estadoAmigos
     */
    public Collection darEstadoAmigos( )
    {
        return estadoAmigos;
    }

    /**
     * La implementacin de este mtodo permite saber si el cliente le notific a la interfaz la necesidad de crear una ventana para una nueva conversacin
     */
    public void crearVentanaConversacion( Conversacion conv )
    {
        conversacion = conv;
    }

    /**
     * Retorna la conversacin que fue enviada por el cliente
     * @return conversacion
     */
    public Conversacion darConversacion( )
    {
        return conversacion;
    }

    /**
     * Este mtodo no es necesario para las pruebas as que no es implementado
     */
    public void terminarAplicacion( )
    {
    }

    /**
     * Este mtodo no es necesario para las pruebas as que no es implementado
     */
    public void actualizarEstadoInterfaz( )
    {
    }
}