/** ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * $Id: SimuladorServidor.java,v 1.1 2006/11/09 03:13:57 f-vela Exp $
 * Universidad de los Andes (Bogot - Colombia)
 * Departamento de Ingeniera de Sistemas y Computacin 
 * Licenciado bajo el esquema Academic Free License versin 2.1
 *
 * Proyecto Cupi2 (http://cupi2.uniandes.edu.co)
 * Ejercicio: n12_messengerAmigos
 * Autor: Mario Snchez - 7/05/2006
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 */

package uniandes.cupi2.messengerAmigos.testCliente;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.PrintWriter;
import java.net.ServerSocket;
import java.net.Socket;

import uniandes.cupi2.messengerAmigos.cliente.ClienteAmigos;

/**
 * Esta clase simula el comportamiento de un servidor para las pruebas de la clase ClienteAmigos
 */
public class SimuladorServidor extends Thread
{
    // -----------------------------------------------------------------
    // Atributos
    // -----------------------------------------------------------------

    /**
     * Es el socket a travs del cual se lleva a cabo la comunicacin con el cliente
     */
    private Socket socketCliente;

    /**
     * Es el stream usuado para leer los mensajes enviados por el cliente
     */
    private BufferedReader inCliente;

    /**
     * Es el stream usado para enviar mensajes al cliente
     */
    private PrintWriter outCliente;

    // -----------------------------------------------------------------
    // Mtodos
    // -----------------------------------------------------------------

    /**
     * Espera una conexin para que el cliente se pueda conectar al servidor
     */
    public void run( )
    {
        try
        {
            ServerSocket ssocket = new ServerSocket( 9999 );
            socketCliente = ssocket.accept( );
            outCliente = new PrintWriter( socketCliente.getOutputStream( ), true );
            inCliente = new BufferedReader( new InputStreamReader( socketCliente.getInputStream( ) ) );

            ssocket.close( );
        }
        catch( IOException e )
        {
            e.printStackTrace( );
        }
    }

    /**
     * Cierra la conexin del servidor con el cliente
     * @throws IOException Se lanza esta excepcin si hay problemas en la comunicacin
     */
    public void cerrarConexion( ) throws IOException
    {
        inCliente.close( );
        outCliente.close( );
        socketCliente.close( );
    }

    /**
     * Lee un mensaje enviado por el cliente y lo retorna
     * @return mensaje
     * @throws IOException Se lanza esta excepcin si hay problemas en la comunicacin
     */
    public String recibirMensaje( ) throws IOException
    {
        return inCliente.readLine( );
    }

    /**
     * Enva al cliente un mensaje que indica que se puede desconectar (DESCONEXION)
     */
    public void enviarMensajeDesconexion( )
    {
        outCliente.println( ClienteAmigos.DESCONEXION );
    }

}