/**~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
 * $Id: SimuladorServidor.java,v 1.2 2006/12/11 07:31:42 man-muno Exp $ 
 * Universidad de los Andes (Bogot - Colombia)
 * Departamento de Ingeniera de Sistemas y Computacin 
 * Todos los derechos reservados 2005 
 *
 * Proyecto Cupi2 
 * Ejercicio: n12_messengerAmigos 
 * Autor: Mario Snchez - 5/05/2006 
 * ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ 
 */

package uniandes.cupi2.messengerAmigos.testServidor;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.PrintWriter;
import java.net.Socket;
import java.sql.SQLException;
import java.util.Collection;

import uniandes.cupi2.messengerAmigos.servidor.Usuario;
import uniandes.cupi2.messengerAmigos.servidor.IAdministradorAmigos;
import uniandes.cupi2.messengerAmigos.servidor.IServidorAmigos;

/**
 * Esta clase simula ser un servidor para facilitar las pruebas a los mtodos de la clase ManejadorCliente
 */
public class SimuladorServidor implements IServidorAmigos
{
    // -----------------------------------------------------------------
    // Constantes
    // -----------------------------------------------------------------

    /**
     * Es el puerto en el que el servidor recibe las conexiones
     */
    public static final int PUERTO_SERVIDOR = 9999;

    /**
     * Es el nombre del cliente
     */
    public static final String NOMBRE_CLIENTE = "Alicia";

    /**
     * Es la direccin ip del cliente
     */
    public static final String IP_CLIENTE = "127.0.0.1";

    /**
     * Es el puerto en el que el cliente recibe las conexiones entrantes
     */
    public static final int PUERTO_CLIENTE = 9997;

    // -----------------------------------------------------------------
    // Atributos
    // -----------------------------------------------------------------

    /**
     * Es el objeto que simula ser el AdministradorAmigos
     */
    private SimuladorAdministradorAmigos simuladorAdministrador;

    /**
     * Es el objeto que sirve para crear una conexin en un thread diferente al principal
     */
    private AyudanteConexion ayudante;

    /**
     * Es el socket que se supone est del lado del servidor y que ser usado por el manejadorCliente
     */
    private Socket socketServidor;

    /**
     * Es el socket que comunica al servidor con el cliente
     */
    private Socket socketCliente;

    /**
     * Es el stream usuado para leer los mensajes enviados al cliente
     */
    private BufferedReader inCliente;

    /**
     * Es el stream usado para enviar mensajes al servidor desde el supuesto cliente
     */
    private PrintWriter outCliente;

    /**
     * Es el ltimo mensaje que fue recibido por el cliente
     */
    private String ultimoMensaje;

    /**
     * Indica si se envi una notificacin de desconexin a los que conocen al usuario
     */
    private boolean enviarNotificacionDesconexion;

    /**
     * Indica si se envi una notificacin de conexin a los que conocen al usuario
     */
    private boolean enviarNotificacionConexion;

    /**
     * Indica si se envi una notificacin para iniciar una conversacin
     */
    private boolean enviarConversacionAmigo;

    // -----------------------------------------------------------------
    // Constructores
    // -----------------------------------------------------------------

    /**
     * Crea el servidor simulado y lo deja esperando una conexin en un thread aparte
     */
    public SimuladorServidor( )
    {
        simuladorAdministrador = new SimuladorAdministradorAmigos( );
        ayudante = new AyudanteConexion( );
        ayudante.start( );
    }

    // -----------------------------------------------------------------
    // Mtodos
    // -----------------------------------------------------------------

    /**
     * Conecta el socketCliente al servidor
     * @return socketServidor
     * @throws IOException Se lanza esta excepcin si hay problemas en la comunicacin
     */
    public Socket conectar( ) throws IOException
    {
        socketCliente = new Socket( "localhost", PUERTO_SERVIDOR );
        outCliente = new PrintWriter( socketCliente.getOutputStream( ), true );
        inCliente = new BufferedReader( new InputStreamReader( socketCliente.getInputStream( ) ) );

        while( socketServidor == null )
            socketServidor = ayudante.darSocket( );

        outCliente.println( IServidorAmigos.LOGIN + ":" + NOMBRE_CLIENTE + ";" + IP_CLIENTE + ";" + PUERTO_CLIENTE );

        return socketServidor;
    }

    /**
     * Cierra la conexin desde el lado del cliente
     * @throws IOException Se lanza esta excepcin si hay problemas en la comunicacin
     */
    public void terminarConexion( ) throws IOException
    {
        outCliente.close( );
        inCliente.close( );
        socketCliente.close( );
    }

    /**
     * Simula el envo desde el cliente de un mensaje de LOGOUT
     */
    public void enviarLogout( )
    {
        outCliente.println( IServidorAmigos.LOGOUT );
    }

    /**
     * Lee un mensaje enviado hacia el cliente
     * @throws IOException Se lanza esta excepcin si hay problemas en la comunicacin
     */
    public void recibirMensaje( ) throws IOException
    {
        ultimoMensaje = inCliente.readLine( );
    }

    /**
     * Retorna el ltimo mensaje recibido
     * @return ultimoMensaje
     */
    public String darUltimoMensaje( )
    {
        return ultimoMensaje;
    }

    /**
     * Retorna el objeto que simula ser el AdministradorAmigos
     */
    public IAdministradorAmigos darAministradorAmigos( )
    {
        return simuladorAdministrador;
    }

    /**
     * Este mtodo no es relevante para las pruebas as que no se implementa
     */
    public void desconectarDB( ) throws SQLException
    {
    }

    /**
     * Inicia la recepcin de conexiones
     */
    public void recibirConexiones( )
    {
        ayudante.start( );
    }

    /**
     * Este mtodo no es relevante para las pruebas as que no se implementa
     */
    public void cerrarConexion( ) throws IOException
    {
    }

    /**
     * Este mtodo no es relevante para las pruebas as que no se implementa
     */
    public Collection darClientes( )
    {
        return null;
    }

    /**
     * La implementacin de este mtodo registra que se envi una notificacin con el estado del usuario
     * @param usuario El nombre del usuario al que se le va a enviar la notificacin
     * @param estadoAmigo El estado que va a ser informado
     */
    public void enviarNotificacionAmigo( String usuario, Usuario estadoAmigo )
    {
        if( estadoAmigo.darEstado( ) == Usuario.ONLINE )
            enviarNotificacionConexion = true;
        else
            enviarNotificacionDesconexion = true;
    }

    /**
     * Indica si se envi la notificacin de desconexin
     * @return enviarNotificacionDesconexion
     */
    public boolean envioNotificacionDesconexion( )
    {
        return enviarNotificacionDesconexion;
    }

    /**
     * Indica si se envi la notificacin de conexin
     * @return enviarNotificacionConexion
     */
    public boolean envioNotificacionConexion( )
    {
        return enviarNotificacionConexion;
    }

    /**
     * La implementacin de este mtodo verifica que se haya enviado al amigo una notificacin para crear una conversacin
     * @param usuario El nombre del usuario al que se le va a enviar la notificacin
     * @param amigo El nombre del amigo que invit al usuario a la conversacin
     * @param direccionIp La direccin ip a la que debera conectarse el usuario
     * @param puerto El puerto al que debera conectarse el usuario
     */
    public void iniciarConversacion( String usuario, String amigo, String direccionIp, int puerto )
    {
        if( amigo.equals( SimuladorServidor.NOMBRE_CLIENTE ) && usuario.equals( "Alicia" ) )
            enviarConversacionAmigo = true;
    }

    /**
     * Indica si se envi el mensaje al amigo para iniciar la convesacin
     * @return enviarConversacionAmigo
     */
    public boolean envioConversacionAmigo( )
    {
        return enviarConversacionAmigo;
    }

    /**
     * Este mtodo no es relevante para las pruebas as que no se implementa
     */
    public void desconectarCliente( String usuario )
    {
    }

    /**
     * Este mtodo no es relevante para las pruebas as que no se implementa
     */
    public String metodo1( )
    {
        return null;
    }

    /**
     * Este mtodo no es relevante para las pruebas as que no se implementa
     */
    public String metodo2( )
    {
        return null;
    }

}
