package gold.structures.automaton;

import java.util.*;
import gold.structures.collection.*;
import gold.util.*;

public class GAlphabet implements IAlphabet {
  
  private static final long serialVersionUID=267146564254188978L;
  
  protected final char[] symbols;
  
  public GAlphabet(Object pSymbols) {
    Set<Character> set=new TreeSet<Character>();
    for (Object object:GCollections.unmodifiableCollection(pSymbols)) {
      if (object==null) throw new IllegalArgumentException("The alphabet cannot contain null elements.");
      if (!(object instanceof Character)) throw new IllegalArgumentException("The alphabet cannot contain elements of type "+object.getClass().getCanonicalName()+".");
      set.add((Character)object);
    }
    if (set.contains('\0')) throw new IllegalArgumentException("The alphabet cannot contain the null character ('\\0').");
    symbols=GArrays.toPrimitiveArray(set.toArray(new Character[0]));
  }
  
  @Override
  public char[] getSymbols() {
    return symbols;
  }
  
  @Override
  public Iterator<Character> iterator() {
    return new GAbstractIterator<Character>() {
      private int index=0;
      @Override
      public boolean hasNext() {
        return index<symbols.length;
      }
      @Override
      public Character next() {
        return symbols[index++];
      }
    };
  }
  
  @Override
  public boolean isEmpty() {
    return symbols.length==0;
  }
  
  @Override
  public int size() {
    return symbols.length;
  }
  
  @Override
  public char getSymbol(int pIndex) {
    return symbols[pIndex];
  }
  
  @Override
  public boolean contains(char pSymbol) {
    return indexOf(pSymbol)!=-1;
  }
  
  @Override
  public boolean containsAll(String pSymbols) {
    for (char symbol:pSymbols.toCharArray()) {
      if (!contains(symbol)) return false;
    }
    return true;
  }
  
  @Override
  public int indexOf(char pSymbol) {
    int index=symbols.length<3?GArrays.linearSearch(symbols,pSymbol):GArrays.binarySearch(symbols,pSymbol);
    return index>=0?index:-1;
  }
  
  @Override
  public boolean equals(Object pObject) {
    return (pObject instanceof IAlphabet)&&GArrays.equals(symbols,((IAlphabet)pObject).getSymbols());
  }
  
  @Override
  public int hashCode() {
    return GArrays.hashCode(symbols);
  }
  
  @Override
  public String toString() {
    return GArrays.toString(symbols);
  }
  
  @Override
  public GAlphabet clone() {
    return new GAlphabet(this);
  }
  
}
