package gold.structures.automaton;

import gold.structures.set.*;
import gold.util.*;

public class GDeterministicAutomaton<V> extends GNondeterministicAutomaton<V> {
  
  private static final long serialVersionUID=2571419168443243917L;
  
  public GDeterministicAutomaton(Iterable<? extends V> pStates, Object pAlphabet, V pInitialState, Iterable<? extends V> pAcceptStates, IMethod pTransitionFunction) {
    super(pStates,pAlphabet,pInitialState,pAcceptStates,pTransitionFunction);
  }
  
  public GDeterministicAutomaton(Iterable<? extends V> pStates, Object pAlphabet, V pInitialState, Iterable<? extends V> pAcceptStates, Object[] pTransitionFunctionTable) {
    super(pStates,pAlphabet,pInitialState,pAcceptStates,pTransitionFunctionTable);
  }
  
  public GDeterministicAutomaton(Iterable<? extends V> pStates, Object pAlphabet, V pInitialState, Iterable<? extends V> pAcceptStates) {
    super(pStates,pAlphabet,pInitialState,pAcceptStates);
  }
  
  public GDeterministicAutomaton(Iterable<? extends V> pStates, Object pAlphabet) {
    super(pStates,pAlphabet);
  }
  
  public <W extends V> GDeterministicAutomaton(IAutomaton<W> pAutomaton) {
    super(pAutomaton);
    if (!pAutomaton.isDeterministic()) throw new IllegalArgumentException("The given automaton isn't deterministic.");
  }
  
  @Override
  protected char checkAction(char pAction) {
    if (pAction==IAutomaton.LAMBDA) throw new IllegalArgumentException("The character '"+pAction+"' isn't a symbol of the automaton's alphabet.");
    return super.checkAction(pAction);
  }
  
  @Override
  public Iterable<Character> getActions() {
    return getAlphabet();
  }
  
  @Override
  @SuppressWarnings("unchecked")
  public ISet<V> getDeltaSet(V pInput, char pAction) {
    try {
      if (pAction==IAutomaton.LAMBDA) {
        return GCollections.emptySet();
      }
      else {
        ISet<V> outputs=transitionFunction!=null?GCollections.singletonSet(super.checkState((V)(super.invokeTransitionFunction(pInput,pAction)))):super.getDeltaSet(pInput,pAction);
        if (outputs.size()==0) throw new IllegalStateException("The automaton doesn't have a transition for the input '"+pInput+"' and action '"+pAction+"'.");
        if (outputs.size()!=1) throw new IllegalStateException("The automaton has multiple transitions for the input '"+pInput+"' and action '"+pAction+"'.");
        return outputs;
      }
    }
    catch (RuntimeException ex) {
      throw new RuntimeException("There was an error evaluating the transition function for the state '"+pInput+"' and symbol '"+pAction+"'.",ex);
    }
  }
  
  @Override
  public void setDeltaSet(V pInput, char pAction, Iterable<? extends V> pOutputs) {
    int size=GCollections.sizeOf(pOutputs);
    if (pAction==IAutomaton.LAMBDA&&size!=0) throw new IllegalArgumentException("The outputs parameter must be an empty set.");
    if (pAction!=IAutomaton.LAMBDA&&size!=1) throw new IllegalArgumentException("The outputs parameter must be a singleton set.");
    super.setDeltaSet(pInput,pAction,pOutputs);
  }
  
  @Override
  public GDeterministicAutomaton<V> clone() {
    return new GDeterministicAutomaton<V>(this);
  }
  
  private Object writeReplace() {
    return clone();
  }
  
}
