package gold.structures.automaton;

import java.io.*;
import gold.structures.graph.*;
import gold.structures.set.*;

public interface IAutomaton<V> extends Cloneable,Serializable {
  
  public static final char LAMBDA='\u03BB';
  
  public ISet<V> getStates();
  
  public IAlphabet getAlphabet();
  
  public V getInitialState();
  
  public ISet<V> getAcceptStates();
  
  public V getDelta(V pInput, char pAction);
  
  public ISet<V> getDeltaSet(V pInput, char pAction);
  
  public Iterable<Character> getActions();
  
  public boolean isAcceptState(V pState);
  
  public boolean isDeterministic();
  
  public ISet<V> applyLambdasRecursively(Iterable<? extends V> pStates);
  
  public ISet<V> applyAction(Iterable<? extends V> pStates, char pAction);
  
  public ISet<V> getReachableStates();
  
  public ISet<V> getLiveStates();
  
  public ISet<V> getUsefulStates();
  
  public ISet<String> getAcceptedStrings(int pMaximumLength);
  
  public ISet<String> getRejectedStrings(int pMaximumLength);
  
  public boolean acceptsNoStrings();
  
  public boolean acceptsAllStrings();
  
  public boolean acceptsEmptyString();
  
  public boolean acceptsString(String pInputString);
  
  public IAutomaton<?> simplify();
  
  public IAutomaton<?> determinize();
  
  public IAutomaton<?> minimize();
  
  public IAutomaton<?> union(IAutomaton<?> pAutomaton);
  
  public IAutomaton<?> intersection(IAutomaton<?> pAutomaton);
  
  public IAutomaton<?> concatenation(IAutomaton<?> pAutomaton);
  
  public IAutomaton<?> difference(IAutomaton<?> pAutomaton);
  
  public IAutomaton<?> transpose();
  
  public IAutomaton<?> complement();
  
  public IAutomaton<?> optionality();
  
  public IAutomaton<?> kleenePlus();
  
  public IAutomaton<?> kleeneStar();
  
  public boolean isEquivalentTo(IAutomaton<?> pAutomaton);
  
  public boolean isDisjointFrom(IAutomaton<?> pAutomaton);
  
  public boolean isSubset(IAutomaton<?> pAutomaton);
  
  public IGraph<V,ISet<?>> toGraphCopy();
  
  public IAutomaton<V> clone();
  
  @Override
  public boolean equals(Object pObject);
  
  @Override
  public int hashCode();
  
  @Override
  public String toString();
  
}
