package gold.structures.collection;

import java.util.*;
import gold.structures.bag.*;
import gold.structures.graph.*;
import gold.structures.set.*;
import gold.util.*;

// Implementación basada en java.util.AbstractCollection y en java.util.AbstractList

public abstract class GAbstractCollection<E> implements ICollection<E> {
  
  private static final long serialVersionUID=5718932625615604962L;
  
  @Override
  public abstract int size();
  
  @Override
  public boolean isEmpty() {
    return size()==0;
  }
  
  @Override
  public boolean isFinite() {
    return true;
  }
  
  @Override
  public abstract void clear();
  
  @Override
  public E pick() {
    if (isEmpty()) throw new NoSuchElementException();
    return iterator().next();
  }
  
  @Override
  public Iterator<E> iterator() {
    return toJavaView().iterator();
  }
  
  @Override
  public boolean contains(Object pElement) {
    for (Iterator<E> it=iterator(); it.hasNext();) {
      if (GToolkit.equals(it.next(),pElement)) return true;
    }
    return false;
  }
  
  @Override
  public boolean equals(Object pObject) {
    if (pObject==this) return true;
    if (!(this instanceof IEdge)&&(pObject instanceof IEdge)) return ((IEdge<?,?>)pObject).equals(this);
    if (!(pObject instanceof Iterable)) return false;
    if (size()!=GCollections.sizeOf((Iterable<?>)pObject)) return false;
    Iterator<?> it1=iterator(),it2=((Iterable<?>)pObject).iterator();
    while (it1.hasNext()&&it2.hasNext()) {
      if (!GToolkit.equals(it1.next(),it2.next())) return false;
    }
    return it1.hasNext()==it2.hasNext();
  }
  
  @Override
  public int hashCode() {
    int hash=1;
    for (Iterator<E> it=iterator(); it.hasNext();) {
      E e=it.next();
      hash=31*hash+(e==null?0:e.hashCode());
    }
    return hash;
  }
  
  @Override
  public String toString() {
    boolean isSet=(this instanceof ISet);
    boolean isBag=(this instanceof IBag);
    if (isEmpty()) return isSet||isBag?"\u00D8":"[]";
    boolean isFirst=true;
    StringBuilder sb=new StringBuilder(30);
    sb.append(isBag?"{|":(isSet?"{":"["));
    for (Iterator<E> it=iterator(); it.hasNext(); isFirst=false) {
      if (!isFirst) sb.append(",");
      E e=it.next();
      sb.append(e==null?"null":(e==this?"(this)":e.toString()));
    }
    sb.append(isBag?"|}":(isSet?"}":"]"));
    return sb.toString();
  }
  
  @Override
  public Object[] toArray() {
    return toJavaView().toArray();
  }
  
  @Override
  public <T> T[] toArray(T[] pArray) {
    return toJavaView().toArray(pArray);
  }
  
  @Override
  public abstract Collection<E> toJavaView();
  
  @Override
  public abstract ICollection<E> clone();
  
}
