package gold.structures.disjointset;

import gold.structures.map.*;
import gold.util.*;

// Basado en el libro "Introduction to Algorithms" de Thomas Cormen.

public class GForestDisjointSets<E> implements IDisjointSets<E> {
  
  private static final long serialVersionUID=4455813228949804100L;
  
  protected final IMap<E,GForestDisjointSet<E>> sets;
  
  public GForestDisjointSets() {
    this(1024);
  }
  
  public GForestDisjointSets(int pCapacity) {
    sets=new GHashTableMap<E,GForestDisjointSet<E>>(pCapacity);
  }
  
  public GForestDisjointSets(Iterable<? extends E> pElements) {
    this(GCollections.sizeOf(pElements));
    for (E element:pElements) {
      makeSet(element);
    }
  }
  
  @Override
  public GForestDisjointSet<E> makeSet(E pData) {
    GForestDisjointSet<E> set=new GForestDisjointSet<E>(pData);
    sets.put(pData,set);
    return set;
  }
  
  @Override
  public void union(E pData1, E pData2) {
    union(sets.get(pData1),sets.get(pData2));
  }
  
  protected void union(GForestDisjointSet<E> pSet1, GForestDisjointSet<E> pSet2) {
    GForestDisjointSet<E> x=findSet(pSet1),y=findSet(pSet2);
    if (x.getRank()>y.getRank()) {
      y.setParent(x);
    }
    else if (x.getRank()<y.getRank()) {
      x.setParent(y);
    }
    else if (x!=y) {
      x.setParent(y);
      y.setRank(y.getRank()+1);
    }
  }
  
  @Override
  public GForestDisjointSet<E> findSet(E pData) {
    return findSet(sets.get(pData));
  }
  
  protected GForestDisjointSet<E> findSet(GForestDisjointSet<E> pSet) {
    GForestDisjointSet<E> x=pSet;
    if (x.getParent()!=x) x.setParent(findSet(x.getParent()));
    return x.getParent();
  }
  
}
