package gold.structures.disjointset;

import gold.structures.list.*;
import gold.structures.map.*;
import gold.util.*;

public class GListDisjointSets<E> implements IDisjointSets<E> {
  
  private static final long serialVersionUID=6119264890065998553L;
  
  protected final IMap<E,GListDisjointSet<E>> sets;
  
  public GListDisjointSets() {
    this(1024);
  }
  
  public GListDisjointSets(int pCapacity) {
    sets=new GHashTableMap<E,GListDisjointSet<E>>(pCapacity);
  }
  
  public GListDisjointSets(Iterable<? extends E> pElements) {
    this(GCollections.sizeOf(pElements));
    for (E element:pElements) {
      makeSet(element);
    }
  }
  
  @Override
  public GListDisjointSet<E> makeSet(E pData) {
    GListDisjointSet<E> set=new GListDisjointSet<E>(pData);
    sets.put(pData,set);
    return set;
  }
  
  @Override
  public void union(E pData1, E pData2) {
    union(sets.get(pData1),sets.get(pData2));
  }
  
  protected void union(GListDisjointSet<E> pSet1, GListDisjointSet<E> pSet2) {
    GListDisjointSet<E> x=pSet1,y=pSet2;
    IList<GListDisjointSet<E>> listX=x.getNeighbors(),listY=y.getNeighbors();
    if (listX!=listY) {
      listX.concatenate(listY);
      for (GListDisjointSet<E> z:listX) {
        z.setNeighbors(listX);
      }
    }
  }
  
  @Override
  public GListDisjointSet<E> findSet(E pData) {
    return findSet(sets.get(pData));
  }
  
  protected GListDisjointSet<E> findSet(GListDisjointSet<E> pSet) {
    return pSet.getNeighbors().get(0);
  }
  
  @Override
  public int hashCode() {
    return super.hashCode();
  }
  
  @Override
  public boolean equals(Object pObject) {
    return this==pObject;
  }
  
}
