package gold.structures.graph;

import gold.structures.set.*;
import gold.util.*;
import edu.uci.ics.jung.graph.*;

public class GAdaptorJungGraph<V,E> extends GAbstractGraph<V,E> {
  
  private static final long serialVersionUID=-2358987932875889551L;
  
  protected final Graph<V,E> adaptee;
  
  public GAdaptorJungGraph(Graph<V,E> pAdaptee) {
    adaptee=pAdaptee;
  }
  
  @Override
  public boolean isEmpty() {
    return adaptee.getVertexCount()==0;
  }
  
  @Override
  public boolean isUndirected() {
    return (adaptee instanceof UndirectedGraph)||super.isUndirected();
  }
  
  @Override
  public int getVertexCount() {
    return adaptee.getVertexCount();
  }
  
  @Override
  public ISet<V> getVertices() {
    return GCollections.unmodifiableSet(adaptee.getVertices());
  }
  
  @Override
  public int getEdgeCount() {
    return adaptee.getEdgeCount();
  }
  
  @Override
  public double getCost(V pSource, V pTarget) {
    E data=getData(pSource,pTarget);
    if (data==null) {
      return IEdge.INFINITE_COST;
    }
    else if (data instanceof Double) {
      return (Double)data;
    }
    else {
      return 1.0;
    }
  }
  
  @Override
  public E getData(V pSource, V pTarget) {
    return adaptee.findEdge(pSource,pTarget);
  }
  
  @Override
  public boolean containsVertex(V pVertex) {
    return adaptee.containsVertex(pVertex);
  }
  
  @Override
  public boolean containsEdge(V pSource, V pTarget) {
    return adaptee.findEdge(pSource,pTarget)!=null;
  }
  
  @Override
  public void addEdge(V pSource, V pTarget, double pCost, E pData) {
    adaptee.addEdge(pData,pSource,pTarget,adaptee.getDefaultEdgeType());
  }
  
  @Override
  public void removeEdge(V pSource, V pTarget) {
    E edge=adaptee.findEdge(pSource,pTarget);
    if (edge!=null) adaptee.removeEdge(edge);
  }
  
  @Override
  public ISet<V> getSuccessors(V pVertex) {
    return GCollections.unmodifiableSet(adaptee.getSuccessors(pVertex));
  }
  
  @Override
  public ISet<V> getPredecessors(V pVertex) {
    return GCollections.unmodifiableSet(adaptee.getPredecessors(pVertex));
  }
  
  @Override
  public int getOutDegree(V pVertex) {
    return adaptee.outDegree(pVertex);
  }
  
  @Override
  public int getInDegree(V pVertex) {
    return adaptee.inDegree(pVertex);
  }
  
  public Graph<V,E> getAdaptee() {
    return adaptee;
  }
  
  @Override
  public Graph<V,IEdge<V,E>> toJungView() {
    return super.toJungView(adaptee.getDefaultEdgeType());
  }
  
  @Override
  public Graph<V,IEdge<V,E>> toJungCopy() {
    return super.toJungCopy(adaptee.getDefaultEdgeType());
  }
  
  @Override
  @SuppressWarnings("unchecked")
  public GAdaptorJungGraph<V,E> clone() {
    try {
      return new GAdaptorJungGraph<V,E>((Graph<V,E>)(GToolkit.clone(adaptee)));
    }
    catch (Throwable th) {
      throw new InternalError("The class "+adaptee.getClass().getCanonicalName()+" is not cloneable.");
    }
  }
  
}
