package gold.structures.graph;

import java.util.*;

import gold.structures.collection.*;
import gold.structures.tuple.*;
import gold.util.*;

public class GEdge<V,E> implements IEdge<V,E> {
  
  private static final long serialVersionUID=7687739669330167859L;
  
  protected final V source;
  protected final V target;
  protected final double cost;
  protected final E data;
  
  public GEdge(V pSource, V pTarget) {
    this(pSource,pTarget,0.0);
  }
  
  public GEdge(V pSource, V pTarget, double pCost) {
    this(pSource,pTarget,pCost,null);
  }
  
  public GEdge(V pSource, V pTarget, double pCost, E pData) {
    if (pCost==IEdge.INFINITE_COST) throw new IllegalArgumentException("The cost of the edge cannot be infinite.");
    source=pSource;
    target=pTarget;
    cost=pCost;
    data=pData;
  }
  
  @Override
  public V getSource() {
    return source;
  }
  
  @Override
  public V getTarget() {
    return target;
  }
  
  @Override
  public double getCost() {
    return cost;
  }
  
  @Override
  public E getData() {
    return data;
  }
  
  @Override
  public Iterator<Object> iterator() {
    return new GAbstractIterator<Object>() {
      private int index=0;
      @Override
      public boolean hasNext() {
        return index<4;
      }
      @Override
      public Object next() {
        switch (index++) {
          case 0:
            return source;
          case 1:
            return target;
          case 2:
            return cost;
          case 3:
            return data;
          default:
            return null;
        }
      }
    };
  }
  
  @Override
  public int compareTo(IEdge<V,E> pEdge) {
    return ((Double)cost).compareTo(pEdge.getCost());
  }
  
  @Override
  public boolean equals(Object pObject) {
    if (pObject instanceof IEdge) {
      IEdge<?,?> edge=(IEdge<?,?>)pObject;
      return GToolkit.equals(source,edge.getSource())&&GToolkit.equals(target,edge.getTarget());
    }
    else if (pObject instanceof ITuple) {
      ITuple<?> tuple=(ITuple<?>)pObject;
      return tuple.size()==2&&GToolkit.equals(source,tuple.get(0))&&GToolkit.equals(target,tuple.get(1));
    }
    else {
      return false;
    }
  }
  
  @Override
  @SuppressWarnings("unchecked")
  public int hashCode() {
    return GCollections.asCollection(source,target).hashCode();
  }
  
  @Override
  public String toString() {
    return "["+source+"->"+target+"]@"+cost;
  }
  
  @Override
  public GEdge<V,E> clone() {
    return new GEdge<V,E>(source,target,cost,data);
  }
  
}
