package gold.structures.graph;

import gold.structures.set.*;
import gold.util.*;

public class GImplicitDirectedGraph<V,E> extends GAbstractGraph<V,E> {
  
  private static final long serialVersionUID=569990114270251570L;
  
  protected transient final ISet<V> vertices;
  protected transient IMethod successorsFunction=null;
  protected transient IMethod costFunction=null;
  protected transient IMethod dataFunction=null;
  
  public GImplicitDirectedGraph(Iterable<? extends V> pVertices) {
    if (!GCollections.isFinite(pVertices)) throw new IllegalArgumentException("The set of vertices of the graph cannot be infinite.");
    vertices=new GHashTableSet<V>(GCollections.unmodifiableCollection(pVertices));
  }
  
  public GImplicitDirectedGraph(Iterable<? extends V> pVertices, IMethod pSuccessorsFunction, IMethod pCostFunction) {
    this(pVertices);
    setSuccessorsFunction(pSuccessorsFunction);
    setCostFunction(pCostFunction);
  }
  
  protected GImplicitDirectedGraph(GImplicitDirectedGraph<? extends V,? extends E> pGraph) {
    this(pGraph.getVertices());
    successorsFunction=pGraph.successorsFunction;
    costFunction=pGraph.costFunction;
    dataFunction=pGraph.dataFunction;
  }
  
  @Override
  public ISet<V> getVertices() {
    return GCollections.unmodifiableSet(vertices);
  }
  
  @Override
  @SuppressWarnings("unchecked")
  public ISet<V> getSuccessors(V pVertex) {
    if (successorsFunction==null) throw new IllegalStateException("The successors function of the graph is not defined.");
    return GCollections.unmodifiableSet((Iterable<V>)(successorsFunction.invoke(pVertex)));
  }
  
  @Override
  public double getCost(V pSource, V pTarget) {
    if (costFunction==null) throw new IllegalStateException("The cost function of the graph is not defined.");
    return ((Number)(costFunction.invoke(pSource,pTarget))).doubleValue();
  }
  
  @Override
  @SuppressWarnings("unchecked")
  public E getData(V pSource, V pTarget) {
    if (dataFunction==null) return null;
    return (E)(dataFunction.invoke(pSource,pTarget));
  }
  
  public void setSuccessorsFunction(IMethod pSuccessorsFunction) {
    successorsFunction=pSuccessorsFunction;
  }
  
  public void setCostFunction(IMethod pCostFunction) {
    costFunction=pCostFunction;
  }
  
  public void setDataFunction(IMethod pDataFunction) {
    dataFunction=pDataFunction;
  }
  
  @Override
  public GImplicitDirectedGraph<V,E> clone() {
    return new GImplicitDirectedGraph<V,E>(this);
  }
  
  private Object writeReplace() {
    return new GDirectedGraph<V,E>(this);
  }
  
}
