package gold.structures.graph;

import java.util.*;
import edu.uci.ics.jung.graph.*;
import edu.uci.ics.jung.graph.util.*;

public class GViewJungGraph<V,E> extends AbstractTypedGraph<V,IEdge<V,E>> {
  
  private static final long serialVersionUID=5783697037815186245L;
  
  protected final IGraph<V,E> adaptee;
  
  public GViewJungGraph(IGraph<V,E> pAdaptee, EdgeType pEdgeType) {
    super(pEdgeType);
    adaptee=pAdaptee;
  }
  
  @Override
  public V getDest(IEdge<V,E> pEdge) {
    return pEdge.getTarget();
  }
  
  @Override
  public V getSource(IEdge<V,E> pEdge) {
    return pEdge.getSource();
  }
  
  @Override
  public Pair<V> getEndpoints(IEdge<V,E> pEdge) {
    return new Pair<V>(pEdge.getSource(),pEdge.getTarget());
  }
  
  @Override
  public Collection<IEdge<V,E>> getOutEdges(V pVertex) {
    return adaptee.getOutEdges(pVertex).toJavaView();
  }
  
  @Override
  public Collection<IEdge<V,E>> getInEdges(V pVertex) {
    return adaptee.getInEdges(pVertex).toJavaView();
  }
  
  @Override
  public Collection<V> getSuccessors(V pVertex) {
    return adaptee.getSuccessors(pVertex).toJavaView();
  }
  
  @Override
  public Collection<V> getPredecessors(V pVertex) {
    return adaptee.getPredecessors(pVertex).toJavaView();
  }
  
  @Override
  public Collection<IEdge<V,E>> getIncidentEdges(V pVertex) {
    if (super.getDefaultEdgeType().equals(EdgeType.DIRECTED)) {
      Set<IEdge<V,E>> result=new HashSet<IEdge<V,E>>(getVertexCount());
      result.addAll(getOutEdges(pVertex));
      result.addAll(getInEdges(pVertex));
      return Collections.unmodifiableSet(result);
    }
    else {
      return getOutEdges(pVertex);
    }
  }
  
  @Override
  public Collection<V> getNeighbors(V pVertex) {
    if (super.getDefaultEdgeType().equals(EdgeType.DIRECTED)) {
      Set<V> result=new HashSet<V>();
      result.addAll(getSuccessors(pVertex));
      result.addAll(getPredecessors(pVertex));
      return Collections.unmodifiableSet(result);
    }
    else {
      return getSuccessors(pVertex);
    }
  }
  
  @Override
  public boolean isDest(V pVertex, IEdge<V,E> pEdge) {
    return pEdge.getTarget().equals(pVertex);
  }
  
  @Override
  public boolean isSource(V pVertex, IEdge<V,E> pEdge) {
    return pEdge.getSource().equals(pVertex);
  }
  
  @Override
  public int getVertexCount() {
    return adaptee.getVertexCount();
  }
  
  @Override
  public Collection<V> getVertices() {
    return adaptee.getVertices().toJavaView();
  }
  
  @Override
  public int getEdgeCount() {
    return adaptee.getEdgeCount();
  }
  
  @Override
  public Collection<IEdge<V,E>> getEdges() {
    return adaptee.getEdges().toJavaView();
  }
  
  @Override
  public boolean containsVertex(V pVertex) {
    return adaptee.containsVertex(pVertex);
  }
  
  @Override
  public boolean containsEdge(IEdge<V,E> pEdge) {
    return adaptee.containsEdge(pEdge.getSource(),pEdge.getTarget());
  }
  
  @Override
  public boolean addVertex(V pVertex) {
    throw new UnsupportedOperationException();
  }
  
  @Override
  public boolean removeVertex(V pVertex) {
    throw new UnsupportedOperationException();
  }
  
  @Override
  public boolean addEdge(IEdge<V,E> pEdge, Pair<? extends V> pPair, EdgeType pEdgeType) {
    super.validateEdgeType(pEdgeType);
    adaptee.addEdge(pPair.getFirst(),pPair.getSecond(),pEdge.getCost(),pEdge.getData());
    if (super.getDefaultEdgeType().equals(EdgeType.UNDIRECTED)) {
      adaptee.addEdge(pPair.getSecond(),pPair.getFirst(),pEdge.getCost(),pEdge.getData());
    }
    return true;
  }
  
  @Override
  public boolean removeEdge(IEdge<V,E> pEdge) {
    if (adaptee.containsEdge(pEdge.getSource(),pEdge.getTarget())) {
      adaptee.removeEdge(pEdge.getSource(),pEdge.getTarget());
      if (super.getDefaultEdgeType().equals(EdgeType.UNDIRECTED)) {
        adaptee.removeEdge(pEdge.getTarget(),pEdge.getSource());
      }
      return true;
    }
    else {
      return false;
    }
  }
  
  public IGraph<V,E> getAdaptee() {
    return adaptee;
  }
  
}
