package gold.structures.graph;

import java.io.*;
import gold.structures.set.*;
import edu.uci.ics.jung.graph.*;

public interface IGraph<V,E> extends Cloneable,Serializable {
  
  public boolean isEmpty();
  
  public boolean isUndirected();
  
  public int getVertexCount();
  
  public ISet<V> getVertices();
  
  public int getEdgeCount();
  
  public ISet<IEdge<V,E>> getEdges();
  
  public IEdge<V,E> getEdge(V pSource, V pTarget);
  
  public double getCost(V pSource, V pTarget);
  
  public E getData(V pSource, V pTarget);
  
  public boolean containsVertex(V pVertex);
  
  public boolean containsEdge(V pSource, V pTarget);
  
  public void addEdge(IEdge<? extends V,? extends E> pEdge);
  
  public void addEdge(V pSource, V pTarget);
  
  public void addEdge(V pSource, V pTarget, double pCost);
  
  public void addEdge(V pSource, V pTarget, double pCost, E pData);
  
  public void addEdges(Iterable<?> pEdges);
  
  public void removeEdge(IEdge<? extends V,? extends E> pEdge);
  
  public void removeEdge(V pSource, V pTarget);
  
  public void removeEdges(Iterable<?> pEdges);
  
  public ISet<V> getSuccessors(V pVertex);
  
  public ISet<V> getPredecessors(V pVertex);
  
  public ISet<V> getNeighbors(V pVertex);
  
  public ISet<IEdge<V,E>> getOutEdges(V pVertex);
  
  public ISet<IEdge<V,E>> getInEdges(V pVertex);
  
  public int getOutDegree(V pVertex);
  
  public int getInDegree(V pVertex);
  
  public double[][] getCostMatrix();
  
  public double[][] getCostMatrix(Iterable<? extends V> pVertexOrdering);
  
  public boolean[][] getAdjacencyMatrix();
  
  public boolean[][] getAdjacencyMatrix(Iterable<? extends V> pVertexOrdering);
  
  @Override
  public boolean equals(Object pObject);
  
  @Override
  public int hashCode();
  
  @Override
  public String toString();
  
  public Graph<V,IEdge<V,E>> toJungView();
  
  public Graph<V,IEdge<V,E>> toJungCopy();
  
  public IGraph<V,E> clone();
  
}
