package gold.structures.heap;

import java.util.*;
import gold.structures.collection.*;
import gold.structures.map.*;
import gold.util.*;

public class GBinaryHeap<E> extends GAbstractCollection<E> implements IHeap<E> {
  
  private static final long serialVersionUID=3406503717405616893L;
  
  protected final PriorityQueue<BinaryHeapNode<E>> adaptee;
  
  public GBinaryHeap() {
    this(1024);
  }
  
  public GBinaryHeap(int pInitialCapacity) {
    adaptee=new PriorityQueue<BinaryHeapNode<E>>(pInitialCapacity);
  }
  
  protected GBinaryHeap(GBinaryHeap<E> pHeap) {
    this(pHeap.adaptee.size());
    for (BinaryHeapNode<E> node:pHeap.adaptee) {
      insert(node.getData(),node.getKey());
    }
  }
  
  public GBinaryHeap(Map<E,?> pElements) {
    this(pElements.size());
    for (Map.Entry<E,?> entry:pElements.entrySet()) {
      insert(entry.getKey(),((Number)(entry.getValue())).doubleValue());
    }
  }
  
  public GBinaryHeap(IMap<E,?> pElements) {
    this(pElements.toJavaView());
  }
  
  @Override
  public void insert(E pElement, double pKey) {
    adaptee.offer(new BinaryHeapNode<E>(pElement,pKey));
  }
  
  @Override
  public void decreaseKey(E pElement, double pKey) {
    throw new UnsupportedOperationException();
  }
  
  @Override
  public void delete(E pElement) {
    adaptee.remove(new BinaryHeapMask(pElement));
  }
  
  @Override
  public boolean contains(Object pElement) {
    return adaptee.contains(new BinaryHeapMask(pElement));
  }
  
  @Override
  public E minimum() {
    if (isEmpty()) throw new NoSuchElementException();
    return adaptee.peek().getData();
  }
  
  @Override
  public double minimumKey() {
    if (isEmpty()) throw new NoSuchElementException();
    return adaptee.peek().getKey();
  }
  
  @Override
  public E extractMinimum() {
    if (isEmpty()) throw new NoSuchElementException();
    return adaptee.poll().getData();
  }
  
  @Override
  public int size() {
    return adaptee.size();
  }
  
  @Override
  public boolean isEmpty() {
    return adaptee.isEmpty();
  }
  
  @Override
  public void clear() {
    adaptee.clear();
  }
  
  @Override
  public Iterator<E> iterator() {
    return new GAbstractIterator<E>() {
      private final Iterator<BinaryHeapNode<E>> iterator=adaptee.iterator();
      @Override
      public boolean hasNext() {
        return iterator.hasNext();
      }
      @Override
      public E next() {
        return iterator.next().getData();
      }
      @Override
      public void remove() {
        iterator.remove();
      }
    };
  }
  
  @Override
  public Collection<E> toJavaView() {
    return new AbstractCollection<E>() {
      @Override
      public Iterator<E> iterator() {
        return GBinaryHeap.this.iterator();
      }
      @Override
      public int size() {
        return GBinaryHeap.this.size();
      }
    };
  }
  
  @Override
  public IHeap<E> clone() {
    return new GBinaryHeap<E>(this);
  }
  
  protected static class BinaryHeapNode<E> implements Comparable<BinaryHeapNode<E>> {
    
    protected final E data;
    protected final double key;
    
    public BinaryHeapNode(E pData, double pKey) {
      data=pData;
      key=pKey;
    }
    
    public E getData() {
      return data;
    }
    
    public double getKey() {
      return key;
    }
    
    @Override
    public int compareTo(BinaryHeapNode<E> pEntry) {
      return ((Double)key).compareTo(pEntry.key);
    }
    
  }
  
  protected static class BinaryHeapMask {
    
    protected final Object data;
    
    public BinaryHeapMask(Object pData) {
      data=pData;
    }
    
    @Override
    public int hashCode() {
      return data!=null?data.hashCode():0;
    }
    
    @Override
    public boolean equals(Object pObject) {
      return GToolkit.equals(data,((BinaryHeapNode<?>)pObject).getData());
    }
    
  }
  
}
