package gold.structures.heap;

import java.util.*;
import gold.structures.collection.*;
import gold.structures.map.*;
import com.jgrapht.util.*;

public class GFibonacciHeap<E> extends GAbstractCollection<E> implements IHeap<E> {
  
  private static final long serialVersionUID=3939706199633497775L;
  
  protected final FibonacciHeap<E> adaptee;
  protected final Map<E,FibonacciHeapNode<E>> nodes;
  
  public GFibonacciHeap() {
    this(1024);
  }
  
  public GFibonacciHeap(int pInitialCapacity) {
    adaptee=new FibonacciHeap<E>();
    nodes=new HashMap<E,FibonacciHeapNode<E>>(pInitialCapacity);
  }
  
  protected GFibonacciHeap(GFibonacciHeap<E> pHeap) {
    this(pHeap.nodes.size());
    for (FibonacciHeapNode<E> node:pHeap.nodes.values()) {
      insert(node.getData(),node.getKey());
    }
  }
  
  public GFibonacciHeap(Map<E,?> pElements) {
    this(pElements.size());
    for (Map.Entry<E,?> entry:pElements.entrySet()) {
      insert(entry.getKey(),((Number)(entry.getValue())).doubleValue());
    }
  }
  
  public GFibonacciHeap(IMap<E,?> pElements) {
    this(pElements.toJavaView());
  }
  
  @Override
  public void insert(E pElement, double pKey) {
    FibonacciHeapNode<E> node=nodes.get(pElement);
    if (node==null) {
      node=new FibonacciHeapNode<E>(pElement);
      adaptee.insert(node,pKey);
      nodes.put(pElement,node);
    }
    else if (pKey>node.getKey()) {
      throw new IllegalArgumentException("Cannot increase the key of the element '"+pElement+"'.");
    }
    else if (pKey<node.getKey()) {
      adaptee.decreaseKey(node,pKey);
    }
  }
  
  @Override
  public void decreaseKey(E pElement, double pKey) {
    FibonacciHeapNode<E> node=nodes.get(pElement);
    if (node==null) {
      throw new IllegalArgumentException("The element '"+pElement+"' doesn't belong to the heap.");
    }
    else if (pKey>node.getKey()) {
      throw new IllegalArgumentException("Cannot increase the key of the element '"+pElement+"'.");
    }
    else if (pKey<node.getKey()) {
      adaptee.decreaseKey(node,pKey);
    }
  }
  
  @Override
  public void delete(E pElement) {
    FibonacciHeapNode<E> node=nodes.remove(pElement);
    if (node==null) {
      throw new IllegalArgumentException("The element '"+pElement+"' doesn't belong to the heap.");
    }
    else {
      adaptee.delete(node);
    }
  }
  
  @Override
  public boolean contains(Object pElement) {
    return nodes.containsKey(pElement);
  }
  
  @Override
  public E minimum() {
    if (isEmpty()) throw new NoSuchElementException();
    return adaptee.min().getData();
  }
  
  @Override
  public double minimumKey() {
    if (isEmpty()) throw new NoSuchElementException();
    return adaptee.min().getKey();
  }
  
  @Override
  public E extractMinimum() {
    if (isEmpty()) throw new NoSuchElementException();
    FibonacciHeapNode<E> node=adaptee.removeMin();
    nodes.remove(node.getData());
    return node.getData();
  }
  
  @Override
  public int size() {
    return adaptee.size();
  }
  
  @Override
  public boolean isEmpty() {
    return adaptee.isEmpty();
  }
  
  @Override
  public void clear() {
    adaptee.clear();
    nodes.clear();
  }
  
  @Override
  public Iterator<E> iterator() {
    return nodes.keySet().iterator();
  }
  
  @Override
  public Collection<E> toJavaView() {
    return nodes.keySet();
  }
  
  @Override
  public IHeap<E> clone() {
    return new GFibonacciHeap<E>(this);
  }
  
}
