package gold.structures.heap;

import java.util.*;
import gold.structures.collection.*;
import gold.structures.map.*;

public class GRedBlackTreeHeap<E> extends GAbstractCollection<E> implements IHeap<E> {
  
  private static final long serialVersionUID=8661517818420589001L;
  
  protected final TreeMap<Double,List<E>> adaptee;
  
  public GRedBlackTreeHeap() {
    adaptee=new TreeMap<Double,List<E>>();
  }
  
  protected GRedBlackTreeHeap(GRedBlackTreeHeap<E> pHeap) {
    this();
    for (Map.Entry<Double,List<E>> entry:pHeap.adaptee.entrySet()) {
      for (E element:entry.getValue()) {
        insert(element,entry.getKey());
      }
    }
  }
  
  public GRedBlackTreeHeap(Map<E,?> pElements) {
    this();
    for (Map.Entry<E,?> entry:pElements.entrySet()) {
      insert(entry.getKey(),((Number)(entry.getValue())).doubleValue());
    }
  }
  
  public GRedBlackTreeHeap(IMap<E,?> pElements) {
    this(pElements.toJavaView());
  }
  
  @Override
  public void insert(E pElement, double pKey) {
    List<E> list=adaptee.get(pKey);
    if (list==null) adaptee.put(pKey,list=new ArrayList<E>(4));
    list.add(pElement);
  }
  
  @Override
  public void decreaseKey(E pElement, double pKey) {
    throw new UnsupportedOperationException();
  }
  
  @Override
  public void delete(E pElement) {
    for (Iterator<List<E>> it=adaptee.values().iterator(); it.hasNext();) {
      List<E> list=it.next();
      list.remove(pElement);
      if (list.isEmpty()) it.remove();
    }
  }
  
  @Override
  public boolean contains(Object pElement) {
    for (List<E> list:adaptee.values()) {
      if (list.contains(pElement)) return true;
    }
    return false;
  }
  
  @Override
  public E minimum() {
    if (isEmpty()) throw new NoSuchElementException();
    return adaptee.firstEntry().getValue().iterator().next();
  }
  
  @Override
  public double minimumKey() {
    if (isEmpty()) throw new NoSuchElementException();
    return adaptee.firstKey();
  }
  
  @Override
  public E extractMinimum() {
    if (isEmpty()) throw new NoSuchElementException();
    List<E> list=adaptee.firstEntry().getValue();
    E element=list.remove(0);
    if (list.isEmpty()) adaptee.pollFirstEntry();
    return element;
  }
  
  @Override
  public int size() {
    int count=0;
    for (List<E> list:adaptee.values()) {
      count+=list.size();
    }
    return count;
  }
  
  @Override
  public boolean isEmpty() {
    return adaptee.isEmpty();
  }
  
  @Override
  public void clear() {
    adaptee.clear();
  }
  
  @Override
  public Iterator<E> iterator() {
    return new GAbstractIterator<E>() {
      private final Iterator<List<E>> iterator=adaptee.values().iterator();
      private Iterator<E> current=null;
      @Override
      public boolean hasNext() {
        return iterator.hasNext()||(current!=null&&current.hasNext());
      }
      @Override
      public E next() {
        if (current.hasNext()) {
          return current.next();
        }
        else if (iterator.hasNext()) {
          current=iterator.next().iterator();
          return next();
        }
        else {
          return null;
        }
      }
    };
  }
  
  @Override
  public Collection<E> toJavaView() {
    return new AbstractCollection<E>() {
      @Override
      public Iterator<E> iterator() {
        return GRedBlackTreeHeap.this.iterator();
      }
      @Override
      public int size() {
        return GRedBlackTreeHeap.this.size();
      }
    };
  }
  
  @Override
  public IHeap<E> clone() {
    return new GRedBlackTreeHeap<E>(this);
  }
  
}
