package gold.structures.list;

import java.util.*;
import gold.structures.collection.*;
import gold.util.*;

public class GAdaptorList<E> extends GAbstractCollection<E> implements IList<E> {
  
  private static final long serialVersionUID=3664483069133764331L;
  
  protected final List<E> adaptee;
  
  public GAdaptorList(List<E> pAdaptee) {
    adaptee=pAdaptee;
  }
  
  @Override
  public int size() {
    return adaptee.size();
  }
  
  @Override
  public void clear() {
    adaptee.clear();
  }
  
  @Override
  public Iterator<E> iterator() {
    return adaptee.iterator();
  }
  
  @Override
  public List<E> toJavaView() {
    return adaptee;
  }
  
  @Override
  @SuppressWarnings("unchecked")
  public IList<E> clone() {
    try {
      return new GAdaptorList<E>((List<E>)(GToolkit.clone(adaptee)));
    }
    catch (Throwable th) {
      throw new InternalError("The class "+adaptee.getClass().getCanonicalName()+" is not cloneable.");
    }
  }
  
  @Override
  public E getFirst() {
    return get(0);
  }
  
  @Override
  public E getLast() {
    return get(size()-1);
  }
  
  @Override
  public E get(int pIndex) {
    return adaptee.get(pIndex);
  }
  
  @Override
  public void addFirst(E pElement) {
    adaptee.add(0,pElement);
  }
  
  @Override
  public void addLast(E pElement) {
    adaptee.add(pElement);
  }
  
  @Override
  public void add(int pIndex, E pElement) {
    adaptee.add(pIndex,pElement);
  }
  
  @Override
  public void add(E pElement) {
    adaptee.add(pElement);
  }
  
  @Override
  public E removeFirst() {
    return remove(0);
  }
  
  @Override
  public E removeLast() {
    return remove(size()-1);
  }
  
  @Override
  public E remove(int pIndex) {
    return adaptee.remove(pIndex);
  }
  
  @Override
  public void setFirst(E pElement) {
    set(0,pElement);
  }
  
  @Override
  public void setLast(E pElement) {
    set(size()-1,pElement);
  }
  
  @Override
  public void set(int pIndex, E pElement) {
    adaptee.set(pIndex,pElement);
  }
  
  @Override
  public int count(Object pElement) {
    int c=0;
    for (Iterator<E> it=adaptee.iterator(); it.hasNext();) {
      if (GToolkit.equals(it.next(),pElement)) c++;
    }
    return c;
  }
  
  @Override
  public int firstIndexOf(E pElement) {
    int index=0;
    for (Iterator<E> it=adaptee.iterator(); it.hasNext(); index++) {
      if (GToolkit.equals(it.next(),pElement)) return index;
    }
    return -1;
  }
  
  @Override
  public int lastIndexOf(E pElement) {
    int index=adaptee.size()-1;
    for (ListIterator<E> it=adaptee.listIterator(index+1); it.hasPrevious(); index--) {
      if (GToolkit.equals(it.previous(),pElement)) return index;
    }
    return -1;
  }
  
  @Override
  public void concatenate(IList<? extends E> pList) {
    adaptee.addAll(pList.toJavaView());
  }
  
  @Override
  public void reverse() {
    Collections.reverse(adaptee);
  }
  
  @Override
  public IList<E> sublist(int pStartIndex, int pEndIndex) {
    int start=Math.max(0,Math.min(adaptee.size(),pStartIndex));
    int end=Math.max(start,Math.min(adaptee.size(),pEndIndex));
    return new GAdaptorList<E>(adaptee.subList(start,end));
  }
  
}
