package gold.structures.map;

import java.util.*;
import gold.structures.collection.*;
import gold.structures.set.*;
import gold.util.*;

public class GAdaptorMap<K,V> implements IMap<K,V> {
  
  private static final long serialVersionUID=291403377601712776L;
  
  protected final Map<K,V> adaptee;
  
  public GAdaptorMap(Map<K,V> pAdaptee) {
    adaptee=pAdaptee;
  }
  
  @Override
  public int size() {
    return adaptee.size();
  }
  
  @Override
  public boolean isEmpty() {
    return adaptee.isEmpty();
  }
  
  @Override
  public void clear() {
    adaptee.clear();
  }
  
  @Override
  public Iterator<V> iterator() {
    return adaptee.values().iterator();
  }
  
  @Override
  public ISet<K> keys() {
    return new GAdaptorSet<K>(adaptee.keySet());
  }
  
  @Override
  public ICollection<V> values() {
    return new GAdaptorCollection<V>(adaptee.values());
  }
  
  @Override
  public boolean containsKey(Object pKey) {
    return adaptee.containsKey(pKey);
  }
  
  @Override
  public boolean containsValue(V pValue) {
    return adaptee.containsValue(pValue);
  }
  
  @Override
  public V get(Object pKey) {
    return adaptee.get(pKey);
  }
  
  @Override
  public void put(K pKey, V pValue) {
    adaptee.put(pKey,pValue);
  }
  
  @Override
  public V remove(Object pKey) {
    return adaptee.remove(pKey);
  }
  
  @Override
  public boolean equals(Object pObject) {
    if (pObject instanceof IMap) {
      return adaptee.equals(((IMap<?,?>)pObject).toJavaView());
    }
    else {
      return adaptee.equals(pObject);
    }
  }
  
  @Override
  public int hashCode() {
    return adaptee.hashCode();
  }
  
  @Override
  public String toString() {
    return adaptee.toString();
  }
  
  @Override
  public Map<K,V> toJavaView() {
    return adaptee;
  }
  
  @Override
  @SuppressWarnings("unchecked")
  public IMap<K,V> clone() {
    try {
      return new GAdaptorMap<K,V>((Map<K,V>)(GToolkit.clone(adaptee)));
    }
    catch (Throwable th) {
      throw new InternalError("The class "+adaptee.getClass().getCanonicalName()+" is not cloneable.");
    }
  }
  
}
