package gold.structures.set;

import java.util.*;
import gold.structures.collection.*;
import gold.util.*;

public class GAdaptorSet<E> extends GAbstractCollection<E> implements ISet<E> {
  
  private static final long serialVersionUID=3343102848561874843L;
  
  protected final Set<E> adaptee;
  protected boolean invert=false;
  
  public GAdaptorSet(Set<E> pAdaptee) {
    this(pAdaptee,false);
  }
  
  public GAdaptorSet(Set<E> pAdaptee, boolean pInvert) {
    adaptee=pAdaptee;
    invert=pInvert;
  }
  
  @Override
  public int size() {
    if (!isFinite()) {
      return -1-adaptee.size();
    }
    else {
      return adaptee.size();
    }
  }
  
  @Override
  public boolean isEmpty() {
    return isFinite()&&adaptee.isEmpty();
  }
  
  @Override
  public void clear() {
    adaptee.clear();
    invert=false;
  }
  
  @Override
  public Iterator<E> iterator() {
    return adaptee.iterator();
  }
  
  @Override
  public boolean contains(Object pElement) {
    return isFinite()==adaptee.contains(pElement);
  }
  
  @Override
  public boolean equals(Object pObject) {
    if (pObject instanceof ISet) {
      ISet<?> set=(ISet<?>)pObject;
      Set<?> coll=set.toJavaView();
      return isFinite()==set.isFinite()&&adaptee.size()==coll.size()&&adaptee.equals(coll);
    }
    else if (pObject instanceof ICollection) {
      return isFinite()&&adaptee.equals(((ICollection<?>)pObject).toJavaView());
    }
    else if (pObject instanceof Collection) {
      return isFinite()&&adaptee.equals(pObject);
    }
    else {
      return false;
    }
  }
  
  @Override
  public int hashCode() {
    int hash=0;
    for (Iterator<E> it=iterator(); it.hasNext();) {
      E e=it.next();
      if (e!=null) hash+=e.hashCode();
    }
    return isFinite()?hash:~hash;
  }
  
  @Override
  public String toString() {
    return isFinite()?super.toString():"~"+super.toString();
  }
  
  @Override
  public Set<E> toJavaView() {
    return adaptee;
  }
  
  @Override
  @SuppressWarnings("unchecked")
  public ISet<E> clone() {
    try {
      return new GAdaptorSet<E>((Set<E>)(GToolkit.clone(adaptee)),invert);
    }
    catch (Throwable th) {
      throw new InternalError("The class "+adaptee.getClass().getCanonicalName()+" is not cloneable.");
    }
  }
  
  @Override
  public void add(E pElement) {
    if (isFinite()) {
      adaptee.add(pElement);
    }
    else {
      adaptee.remove(pElement);
    }
  }
  
  @Override
  public void remove(E pElement) {
    if (isFinite()) {
      adaptee.remove(pElement);
    }
    else {
      adaptee.add(pElement);
    }
  }
  
  protected void makeMirroredDifference(Set<? extends E> pSet) {
    List<E> copy=new ArrayList<E>(adaptee);
    adaptee.clear();
    adaptee.addAll(pSet);
    adaptee.removeAll(copy);
    invert=!invert;
  }
  
  protected static boolean containsAny(Collection<?> pCollection1, Collection<?> pCollection2) {
    for (Object element:pCollection2) {
      if (pCollection1.contains(element)) return true;
    }
    return false;
  }
  
  protected static boolean containsAll(Collection<?> pCollection1, Collection<?> pCollection2) {
    return pCollection1.containsAll(pCollection2);
  }
  
  @Override
  public void union(ISet<? extends E> pSet) {
    Set<? extends E> coll=pSet.toJavaView();
    if (isFinite()&&pSet.isFinite()) {
      adaptee.addAll(coll);
    }
    else if (isFinite()&&pSet.isCofinite()) {
      makeMirroredDifference(coll);
    }
    else if (isCofinite()&&pSet.isFinite()) {
      adaptee.removeAll(coll);
    }
    else if (isCofinite()&&pSet.isCofinite()) {
      adaptee.retainAll(coll);
    }
  }
  
  @Override
  public void intersection(ISet<? extends E> pSet) {
    Set<? extends E> coll=pSet.toJavaView();
    if (isFinite()&&pSet.isFinite()) {
      adaptee.retainAll(coll);
    }
    else if (isFinite()&&pSet.isCofinite()) {
      adaptee.removeAll(coll);
    }
    else if (isCofinite()&&pSet.isFinite()) {
      makeMirroredDifference(coll);
    }
    else if (isCofinite()&&pSet.isCofinite()) {
      adaptee.addAll(coll);
    }
  }
  
  @Override
  public void difference(ISet<? extends E> pSet) {
    Set<? extends E> coll=pSet.toJavaView();
    if (isFinite()&&pSet.isFinite()) {
      adaptee.removeAll(coll);
    }
    else if (isFinite()&&pSet.isCofinite()) {
      adaptee.retainAll(coll);
    }
    else if (isCofinite()&&pSet.isFinite()) {
      adaptee.addAll(coll);
    }
    else if (isCofinite()&&pSet.isCofinite()) {
      makeMirroredDifference(coll);
    }
  }
  
  @Override
  public void symmetricDifference(ISet<? extends E> pSet) {
    Set<? extends E> coll=pSet.toJavaView();
    List<E> copy=new ArrayList<E>(adaptee);
    adaptee.retainAll(coll);
    List<E> inter=new ArrayList<E>(adaptee);
    adaptee.clear();
    adaptee.addAll(copy);
    adaptee.addAll(coll);
    adaptee.removeAll(inter);
    invert=(isFinite()!=pSet.isFinite());
  }
  
  @Override
  public void complement() {
    invert=!invert;
  }
  
  @Override
  public boolean isSubset(ISet<?> pSet) {
    Set<?> coll1=adaptee,coll2=pSet.toJavaView();
    if (isFinite()&&pSet.isFinite()) {
      return GAdaptorSet.containsAll(coll2,coll1);
    }
    else if (isFinite()&&pSet.isCofinite()) {
      return !GAdaptorSet.containsAny(coll1,coll2);
    }
    else if (isCofinite()&&pSet.isFinite()) {
      return false;
    }
    else if (isCofinite()&&pSet.isCofinite()) {
      return GAdaptorSet.containsAll(coll1,coll2);
    }
    else {
      return false;
    }
  }
  
  @Override
  public boolean isSuperset(ISet<?> pSet) {
    return pSet.isSubset(this);
  }
  
  @Override
  public boolean isProperSubset(ISet<?> pSet) {
    return isSubset(pSet)&&!equals(pSet);
  }
  
  @Override
  public boolean isProperSuperset(ISet<?> pSet) {
    return pSet.isProperSubset(this);
  }
  
  @Override
  public boolean isDisjointFrom(ISet<?> pSet) {
    Set<?> coll1=adaptee,coll2=pSet.toJavaView();
    if (isFinite()&&pSet.isFinite()) {
      return !GAdaptorSet.containsAny(coll1,coll2);
    }
    else if (isFinite()&&pSet.isCofinite()) {
      return GAdaptorSet.containsAll(coll2,coll1);
    }
    else if (isCofinite()&&pSet.isFinite()) {
      return GAdaptorSet.containsAll(coll1,coll2);
    }
    else if (isCofinite()&&pSet.isCofinite()) {
      return false;
    }
    else {
      return false;
    }
  }
  
  @Override
  public boolean isFinite() {
    return !invert;
  }
  
  @Override
  public boolean isCofinite() {
    return invert;
  }
  
  @Override
  public boolean isUniverse() {
    return !isFinite()&&adaptee.isEmpty();
  }
  
}
