package gold.structures.tree;

import java.util.*;
import gold.structures.collection.*;
import gold.structures.list.*;
import gold.structures.queue.*;
import gold.structures.tuple.*;
import edu.uci.ics.jung.graph.*;
import org.apache.commons.collections15.*;

public abstract class GAbstractTree<E> extends GAbstractCollection<E> implements ITree<E> {
  
  private static final long serialVersionUID=-1967101097028872401L;
  
  @Override
  public int size() {
    return getWeight();
  }
  
  @Override
  public boolean equals(Object pObject) {
    return this==pObject;
  }
  
  @Override
  public Tree<ITree<E>,IPair<ITree<E>>> toJungCopy() {
    GDelegateTree result=new GDelegateTree();
    IQueue<ITree<E>> queue=new GArrayQueue<ITree<E>>(size());
    if (!isEmpty()) {
      queue.enqueue(this);
    }
    while (!queue.isEmpty()) {
      ITree<E> parent=queue.dequeue();
      if (parent==this) result.addVertex(parent);
      for (ITree<E> child:parent.getSubtrees()) {
        if (!child.isEmpty()) {
          result.addChild(new GPair<ITree<E>>(parent,child),parent,child);
          queue.enqueue(child);
        }
      }
    }
    return result;
  }
  
  protected class GDelegateTree extends DelegateTree<ITree<E>,IPair<ITree<E>>> {
    
    private static final long serialVersionUID=6399762981553553339L;
    
    public GDelegateTree() {
      super(new Factory<DirectedGraph<ITree<E>,IPair<ITree<E>>>>() {
        @Override
        public DirectedGraph<ITree<E>,IPair<ITree<E>>> create() {
          return new DirectedSparseMultigraph<ITree<E>,IPair<ITree<E>>>();
        }
      });
    }
    
    @Override
    public Collection<ITree<E>> getSuccessors(ITree<E> pNode) {
      IList<? extends ITree<E>> list=pNode!=null?pNode.getSubtrees():null;
      if (list==null) return new ArrayList<ITree<E>>();
      return new GArrayList<ITree<E>>(list).toJavaView();
    }
    
  }
  
}
