package gold.structures.tree.binary;

import java.util.*;
import gold.structures.list.*;
import gold.structures.queue.*;
import gold.structures.stack.*;
import gold.structures.tree.*;
import gold.util.*;

public abstract class GAbstractBinaryTree<E> extends GAbstractTree<E> implements IBinaryTree<E> {
  
  private static final long serialVersionUID=-8684025660191103395L;
  
  @Override
  public abstract boolean isEmpty();
  
  @Override
  public abstract void clear();
  
  @Override
  public boolean contains(Object pElement) {
    if (isEmpty()) {
      return false;
    }
    else {
      return GToolkit.equals(getRoot(),pElement)||getLeftSubtree().contains(pElement)||getRightSubtree().contains(pElement);
    }
  }
  
  @Override
  public String toString() {
    if (isEmpty()) {
      return "\u00D8";
    }
    else {
      return "["+getRoot()+":"+getLeftSubtree()+","+getRightSubtree()+"]";
    }
  }
  
  @Override
  public Collection<E> toJavaView() {
    return inOrderTraversal().toJavaView();
  }
  
  @Override
  public abstract IBinaryTree<E> clone();
  
  @Override
  public abstract E getRoot();
  
  @Override
  public abstract IBinaryTree<E> getLeftSubtree();
  
  @Override
  public abstract IBinaryTree<E> getRightSubtree();
  
  @Override
  public abstract void setRoot(E pRoot);
  
  @Override
  public abstract void setLeftSubtree(IBinaryTree<E> pLeftSubtree);
  
  @Override
  public abstract void setRightSubtree(IBinaryTree<E> pRightSubtree);
  
  @Override
  @SuppressWarnings("unchecked")
  public IList<IBinaryTree<E>> getSubtrees() {
    if (isEmpty()) {
      return null;
    }
    else {
      return GCollections.<IBinaryTree<E>>asList(getLeftSubtree(),getRightSubtree());
    }
  }
  
  @Override
  public IBinaryTree<E> getSubtree(int pIndex) {
    if (isEmpty()) {
      return null;
    }
    else {
      if (pIndex==0) {
        return getLeftSubtree();
      }
      else if (pIndex==1) {
        return getRightSubtree();
      }
      else {
        throw new IndexOutOfBoundsException();
      }
    }
  }
  
  @Override
  public IList<E> getChildren() {
    if (isEmpty()) {
      return null;
    }
    else {
      ITree<E> left=getLeftSubtree(),right=getRightSubtree();
      IList<E> result=new GArrayList<E>(2);
      if (!left.isEmpty()) result.addLast(left.getRoot());
      if (!right.isEmpty()) result.addLast(right.getRoot());
      return GCollections.unmodifiableList(result);
    }
  }
  
  @Override
  public boolean isLeaf() {
    return !isEmpty()&&getLeftSubtree().isEmpty()&&getRightSubtree().isEmpty();
  }
  
  @Override
  public int getOrder() {
    return 2;
  }
  
  @Override
  public int getWeight() {
    if (isEmpty()) {
      return 0;
    }
    else {
      return 1+getLeftSubtree().getWeight()+getRightSubtree().getWeight();
    }
  }
  
  @Override
  public int getHeight() {
    if (isEmpty()) {
      return 0;
    }
    else {
      return 1+Math.max(getLeftSubtree().getHeight(),getRightSubtree().getHeight());
    }
  }
  
  @Override
  public int count(E pElement) {
    if (isEmpty()) {
      return 0;
    }
    else {
      return (GToolkit.equals(getRoot(),pElement)?1:0)+getLeftSubtree().count(pElement)+getRightSubtree().count(pElement);
    }
  }
  
  @Override
  public IList<E> preOrderTraversal() {
    IList<E> result=new GArrayList<E>(size());
    IStack<IBinaryTree<E>> stack=new GArrayStack<IBinaryTree<E>>(result.size());
    stack.push(this);
    while (!stack.isEmpty()) {
      IBinaryTree<E> tree=stack.pop();
      if (!tree.isEmpty()) {
        result.addLast(tree.getRoot());
        stack.push(tree.getRightSubtree());
        stack.push(tree.getLeftSubtree());
      }
    }
    return GCollections.unmodifiableList(result);
  }
  
  @Override
  public IList<E> inOrderTraversal() {
    IList<E> result=new GArrayList<E>(size());
    if (isEmpty()) return result;
    IStack<IBinaryTree<E>> stack=new GArrayStack<IBinaryTree<E>>(result.size());
    IBinaryTree<E> tree=getLeftSubtree();
    stack.push(this);
    while (!tree.isEmpty()||!stack.isEmpty()) {
      if (!tree.isEmpty()) {
        stack.push(tree);
        tree=tree.getLeftSubtree();
      }
      else {
        tree=stack.pop();
        result.addLast(tree.getRoot());
        tree=tree.getRightSubtree();
      }
    }
    return GCollections.unmodifiableList(result);
  }
  
  @Override
  public IList<E> postOrderTraversal() {
    IList<E> result=new GArrayList<E>(size());
    IStack<IBinaryTree<E>> stack=new GArrayStack<IBinaryTree<E>>(result.size());
    stack.push(this);
    while (!stack.isEmpty()) {
      IBinaryTree<E> tree=stack.pop();
      if (!tree.isEmpty()) {
        result.addLast(tree.getRoot());
        stack.push(tree.getLeftSubtree());
        stack.push(tree.getRightSubtree());
      }
    }
    result.reverse();
    return GCollections.unmodifiableList(result);
  }
  
  @Override
  public IList<E> levelOrderTraversal() {
    IList<E> result=new GArrayList<E>(size());
    IQueue<IBinaryTree<E>> queue=new GArrayQueue<IBinaryTree<E>>(result.size());
    queue.enqueue(this);
    while (!queue.isEmpty()) {
      IBinaryTree<E> tree=queue.dequeue();
      if (!tree.isEmpty()) {
        result.addLast(tree.getRoot());
        queue.enqueue(tree.getLeftSubtree());
        queue.enqueue(tree.getRightSubtree());
      }
    }
    return GCollections.unmodifiableList(result);
  }
  
}
