package gold.structures.tree.binary;

import gold.structures.list.*;

public class GRecursiveBinaryTree<E> extends GAbstractBinaryTree<E> {
  
  private static final long serialVersionUID=-2172589954064190973L;
  
  protected E root=null;
  protected IBinaryTree<E> leftSubtree=null;
  protected IBinaryTree<E> rightSubtree=null;
  
  public GRecursiveBinaryTree() {
    clear();
  }
  
  public GRecursiveBinaryTree(E pRoot, IBinaryTree<E> pLeftSubtree, IBinaryTree<E> pRightSubtree) {
    setRoot(pRoot);
    setLeftSubtree(pLeftSubtree);
    setRightSubtree(pRightSubtree);
  }
  
  public GRecursiveBinaryTree(IBinaryTree<? extends E> pTree) {
    if (pTree.isEmpty()) {
      clear();
    }
    else {
      setRoot(pTree.getRoot());
      setLeftSubtree(new GRecursiveBinaryTree<E>(pTree.getLeftSubtree()));
      setRightSubtree(new GRecursiveBinaryTree<E>(pTree.getRightSubtree()));
    }
  }
  
  @Override
  public boolean isEmpty() {
    return root==null||leftSubtree==null||rightSubtree==null;
  }
  
  @Override
  public void clear() {
    root=null;
    leftSubtree=null;
    rightSubtree=null;
  }
  
  @Override
  public GRecursiveBinaryTree<E> clone() {
    return new GRecursiveBinaryTree<E>(this);
  }
  
  @Override
  public E getRoot() {
    return root;
  }
  
  @Override
  public IBinaryTree<E> getLeftSubtree() {
    return leftSubtree;
  }
  
  @Override
  public IBinaryTree<E> getRightSubtree() {
    return rightSubtree;
  }
  
  @Override
  public void setRoot(E pRoot) {
    if (pRoot==null) throw new IllegalArgumentException("The root cannot be null.");
    root=pRoot;
  }
  
  @Override
  public void setLeftSubtree(IBinaryTree<E> pLeftSubtree) {
    if (pLeftSubtree==null) throw new IllegalArgumentException("The left subtree cannot be null.");
    leftSubtree=pLeftSubtree;
  }
  
  @Override
  public void setRightSubtree(IBinaryTree<E> pRightSubtree) {
    if (pRightSubtree==null) throw new IllegalArgumentException("The right subtree cannot be null.");
    rightSubtree=pRightSubtree;
  }
  
  public static <E> GRecursiveBinaryTree<E> reconstruct(IList<E> pPreOrder, IList<E> pInOrder) {
    if (pInOrder.isEmpty()) {
      return new GRecursiveBinaryTree<E>();
    }
    else {
      int length=pInOrder.size();
      if (pPreOrder.size()!=length) throw new IllegalArgumentException("The inorder traversal must have the same size than the preorder traversal.");
      E root=pPreOrder.getFirst();
      int count=pInOrder.count(root);
      if (count!=1) throw new IllegalArgumentException("The inorder traversal must be a permutation of preorder traversal, without duplicated elements.");
      int position=pInOrder.firstIndexOf(root);
      IBinaryTree<E> leftSubtree=reconstruct(pPreOrder.sublist(1,position+1),pInOrder.sublist(0,position));
      IBinaryTree<E> rightSubtree=reconstruct(pPreOrder.sublist(position+1,length),pInOrder.sublist(position+1,length));
      return new GRecursiveBinaryTree<E>(root,leftSubtree,rightSubtree);
    }
  }
  
}
