package gold.structures.tree.nary;

import java.util.*;
import gold.structures.collection.*;
import gold.structures.list.*;
import gold.structures.queue.*;
import gold.structures.tree.*;
import gold.util.*;

public abstract class GAbstractNaryTree<E> extends GAbstractTree<E> implements INaryTree<E> {
  
  private static final long serialVersionUID=512698671252106867L;
  
  @Override
  public abstract boolean isEmpty();
  
  @Override
  public abstract void clear();
  
  @Override
  public boolean contains(Object pElement) {
    if (isEmpty()) {
      return false;
    }
    else {
      for (INaryTree<E> subtree:getSubtrees()) {
        if (subtree.contains(pElement)) return true;
      }
      return false;
    }
  }
  
  @Override
  public String toString() {
    if (isEmpty()) {
      return "\u00D8";
    }
    else {
      int index=0;
      StringBuilder sb=new StringBuilder();
      sb.append("[");
      sb.append(getRoot());
      for (INaryTree<E> subtree:getSubtrees()) {
        sb.append(index++>0?",":":");
        sb.append(subtree);
      }
      sb.append("]");
      return sb.toString();
    }
  }
  
  @Override
  public Collection<E> toJavaView() {
    return new AbstractCollection<E>() {
      @Override
      public int size() {
        return GAbstractNaryTree.this.size();
      }
      @Override
      public Iterator<E> iterator() {
        return new GAbstractIterator<E>() {
          private final Iterator<INaryTree<E>> iteratorS=getSubtrees().iterator();
          private Iterator<E> iteratorE=null;
          private int state=0;
          @Override
          public boolean hasNext() {
            if (state==0) {
              return !GAbstractNaryTree.this.isEmpty();
            }
            else {
              while ((iteratorE==null||!iteratorE.hasNext())&&iteratorS.hasNext()) {
                iteratorE=iteratorS.next().iterator();
              }
              return iteratorE!=null&&iteratorE.hasNext();
            }
          }
          @Override
          public E next() {
            if (state==0) {
              state=1;
              return GAbstractNaryTree.this.getRoot();
            }
            else {
              return iteratorE.next();
            }
          }
        };
      }
    };
  }
  
  @Override
  public abstract INaryTree<E> clone();
  
  @Override
  public abstract E getRoot();
  
  @Override
  public abstract IList<INaryTree<E>> getSubtrees();
  
  @Override
  public abstract void setRoot(E pRoot);
  
  @Override
  public abstract void setSubtrees(IList<INaryTree<E>> pSubtrees);
  
  @Override
  public abstract void addSubtree(INaryTree<E> pSubtree);
  
  @Override
  public abstract void addSubtree(int pIndex, INaryTree<E> pSubtree);
  
  @Override
  public abstract void removeSubtree(int pIndex);
  
  @Override
  public abstract INaryTree<E> getSubtree(int pIndex);
  
  @Override
  public IList<E> getChildren() {
    if (isEmpty()) {
      return null;
    }
    else {
      IList<E> result=new GArrayList<E>();
      for (INaryTree<E> subtree:getSubtrees()) {
        if (!subtree.isEmpty()) result.addLast(subtree.getRoot());
      }
      return GCollections.unmodifiableList(result);
    }
  }
  
  @Override
  public boolean isLeaf() {
    if (isEmpty()) {
      return false;
    }
    else {
      for (INaryTree<E> subtree:getSubtrees()) {
        if (!subtree.isEmpty()) return false;
      }
      return true;
    }
  }
  
  @Override
  public int getOrder() {
    if (isEmpty()) {
      return 0;
    }
    else {
      int result=Math.max(1,getChildren().size());
      for (INaryTree<E> subtree:getSubtrees()) {
        result=Math.max(result,subtree.getOrder());
      }
      return result;
    }
  }
  
  @Override
  public int getWeight() {
    if (isEmpty()) {
      return 0;
    }
    else {
      int result=0;
      for (INaryTree<E> subtree:getSubtrees()) {
        result+=subtree.getWeight();
      }
      return 1+result;
    }
  }
  
  @Override
  public int getHeight() {
    if (isEmpty()) {
      return 0;
    }
    else {
      int result=0;
      for (INaryTree<E> subtree:getSubtrees()) {
        result=Math.max(result,subtree.getHeight());
      }
      return 1+result;
    }
  }
  
  @Override
  public int count(E pElement) {
    if (isEmpty()) {
      return 0;
    }
    else {
      int result=GToolkit.equals(getRoot(),pElement)?1:0;
      for (INaryTree<E> subtree:getSubtrees()) {
        result+=subtree.count(pElement);
      }
      return result;
    }
  }
  
  @Override
  public IList<E> preOrderTraversal() {
    IList<E> result=new GArrayList<E>(size());
    GAbstractNaryTree.preOrderTraversal(this,result);
    return GCollections.unmodifiableList(result);
  }
  
  protected static <E> void preOrderTraversal(INaryTree<E> pTree, IList<E> pResult) {
    if (!pTree.isEmpty()) {
      pResult.addLast(pTree.getRoot());
      for (INaryTree<E> subtree:pTree.getSubtrees()) {
        GAbstractNaryTree.preOrderTraversal(subtree,pResult);
      }
    }
  }
  
  @Override
  public IList<E> inOrderTraversal() {
    IList<E> result=new GArrayList<E>(size());
    GAbstractNaryTree.inOrderTraversal(this,result);
    return GCollections.unmodifiableList(result);
  }
  
  protected static <E> void inOrderTraversal(INaryTree<E> pTree, IList<E> pResult) {
    if (!pTree.isEmpty()) {
      Iterator<INaryTree<E>> iterator=pTree.getSubtrees().iterator();
      if (iterator.hasNext()) {
        GAbstractNaryTree.inOrderTraversal(iterator.next(),pResult);
      }
      pResult.addLast(pTree.getRoot());
      while (iterator.hasNext()) {
        GAbstractNaryTree.inOrderTraversal(iterator.next(),pResult);
      }
    }
  }
  
  @Override
  public IList<E> postOrderTraversal() {
    IList<E> result=new GArrayList<E>(size());
    GAbstractNaryTree.postOrderTraversal(this,result);
    return GCollections.unmodifiableList(result);
  }
  
  protected static <E> void postOrderTraversal(INaryTree<E> pTree, IList<E> pResult) {
    if (!pTree.isEmpty()) {
      for (INaryTree<E> subtree:pTree.getSubtrees()) {
        GAbstractNaryTree.postOrderTraversal(subtree,pResult);
      }
      pResult.addLast(pTree.getRoot());
    }
  }
  
  @Override
  public IList<E> levelOrderTraversal() {
    IList<E> result=new GArrayList<E>(size());
    IQueue<INaryTree<E>> queue=new GArrayQueue<INaryTree<E>>(result.size());
    queue.enqueue(this);
    while (!queue.isEmpty()) {
      INaryTree<E> tree=queue.dequeue();
      if (!tree.isEmpty()) {
        result.addLast(tree.getRoot());
        for (INaryTree<E> subtree:tree.getSubtrees()) {
          queue.enqueue(subtree);
        }
      }
    }
    return GCollections.unmodifiableList(result);
  }
  
}
