package gold.structures.tree.nary;

import java.util.*;
import gold.structures.collection.*;
import gold.structures.list.*;
import gold.util.*;

public class GLinkedNaryTree<E> extends GAbstractNaryTree<E> {
  
  private static final long serialVersionUID=-389471778630162905L;
  
  protected E root=null;
  protected GLinkedNaryTree<E> firstSubtree=null;
  protected GLinkedNaryTree<E> rightBrother=null;
  
  public GLinkedNaryTree() {
    clear();
  }
  
  public GLinkedNaryTree(E pRoot) {
    this(pRoot,GCollections.<INaryTree<E>>emptyList());
  }
  
  public GLinkedNaryTree(E pRoot, IList<INaryTree<E>> pSubtrees) {
    setRoot(pRoot);
    setSubtrees(pSubtrees);
  }
  
  public GLinkedNaryTree(INaryTree<E> pTree) {
    if (pTree.isEmpty()) {
      clear();
    }
    else {
      setRoot(pTree.getRoot());
      setSubtrees(pTree.getSubtrees());
    }
  }
  
  @Override
  public boolean isEmpty() {
    return root==null;
  }
  
  @Override
  public void clear() {
    root=null;
    firstSubtree=null;
  }
  
  @Override
  public GLinkedNaryTree<E> clone() {
    return new GLinkedNaryTree<E>(this);
  }
  
  @Override
  public E getRoot() {
    return root;
  }
  
  @Override
  public IList<INaryTree<E>> getSubtrees() {
    return new GAdaptorList<INaryTree<E>>(new AbstractList<INaryTree<E>>() {
      @Override
      public Iterator<INaryTree<E>> iterator() {
        return new GAbstractIterator<INaryTree<E>>() {
          private GLinkedNaryTree<E> current=firstSubtree;
          @Override
          public boolean hasNext() {
            return current!=null;
          }
          @Override
          public INaryTree<E> next() {
            GLinkedNaryTree<E> result=current;
            current=current.rightBrother;
            return result;
          }
        };
      }
      @Override
      public INaryTree<E> get(int pIndex) {
        int index=0;
        for (Iterator<INaryTree<E>> iterator=iterator(); iterator.hasNext();) {
          INaryTree<E> subtree=iterator.next();
          if (index++==pIndex) return subtree;
        }
        return null;
      }
      @Override
      public int size() {
        int result=0;
        for (Iterator<INaryTree<E>> iterator=iterator(); iterator.hasNext();) {
          iterator.next();
          result++;
        }
        return result;
      }
    });
  }
  
  @Override
  public void setRoot(E pRoot) {
    if (pRoot==null) throw new IllegalArgumentException("The root cannot be null.");
    root=pRoot;
  }
  
  @Override
  public void setSubtrees(IList<INaryTree<E>> pSubtrees) {
    if (pSubtrees==null) throw new IllegalArgumentException("The list of subtrees cannot be null.");
    firstSubtree=null;
    GLinkedNaryTree<E> previous=null;
    for (INaryTree<E> subtree:pSubtrees) {
      GLinkedNaryTree<E> current=new GLinkedNaryTree<E>(subtree);
      if (previous==null) {
        firstSubtree=current;
      }
      else {
        previous.rightBrother=current;
      }
      previous=current;
    }
  }
  
  @Override
  public void addSubtree(INaryTree<E> pSubtree) {
    addSubtree(getSubtrees().size(),pSubtree);
  }
  
  @Override
  public void addSubtree(int pIndex, INaryTree<E> pSubtree) {
    if (pIndex<0||pIndex>getSubtrees().size()) throw new IndexOutOfBoundsException();
    GLinkedNaryTree<E> subtree=(pSubtree instanceof GLinkedNaryTree)?(GLinkedNaryTree<E>)pSubtree:new GLinkedNaryTree<E>(pSubtree);
    if (pIndex==0) {
      subtree.rightBrother=firstSubtree;
      firstSubtree=subtree;
    }
    else {
      GLinkedNaryTree<E> current=getSubtree(pIndex-1);
      subtree.rightBrother=current.rightBrother;
      current.rightBrother=subtree;
    }
  }
  
  @Override
  public void removeSubtree(int pIndex) {
    if (pIndex<0||pIndex>=getSubtrees().size()) throw new IndexOutOfBoundsException();
    if (pIndex==0) {
      GLinkedNaryTree<E> next=firstSubtree.rightBrother;
      firstSubtree.rightBrother=null;
      firstSubtree=next;
    }
    else {
      GLinkedNaryTree<E> current=getSubtree(pIndex-1);
      GLinkedNaryTree<E> next=current.rightBrother;
      current.rightBrother=next.rightBrother;
      next.rightBrother=null;
    }
  }
  
  @Override
  public GLinkedNaryTree<E> getSubtree(int pIndex) {
    GLinkedNaryTree<E> current=firstSubtree;
    for (int i=0; i<pIndex-1; i++) {
      current=current.rightBrother;
    }
    return current;
  }
  
}
