package gold.structures.tree.nary;

import gold.structures.list.*;
import gold.structures.point.*;
import gold.structures.stack.*;
import gold.util.*;

public class GQuadtree extends GAbstractNaryTree<GQuadtreeColor> {
  
  private static final long serialVersionUID=2696223936721435046L;
  
  protected GQuadtreeColor color=null;
  protected GQuadtree[] subtrees=null;
  
  protected GQuadtree(GQuadtreeColor pColor) {
    color=pColor;
    subtrees=new GQuadtree[0];
  }
  
  public GQuadtree(String pPostOrder) {
    GQuadtree quadtree=GQuadtree.reconstruct(pPostOrder);
    color=quadtree.color;
    subtrees=quadtree.subtrees;
  }
  
  @Override
  public boolean isEmpty() {
    return false;
  }
  
  @Override
  public void clear() {
    clear(GQuadtreeColor.BLACK);
  }
  
  protected void clear(GQuadtreeColor pColor) {
    color=pColor;
    subtrees=new GQuadtree[0];
  }
  
  @Override
  public GQuadtree clone() {
    return GQuadtree.reconstruct(postOrderTraversalAsString());
  }
  
  @Override
  public GQuadtreeColor getRoot() {
    return color;
  }
  
  @Override
  public IList<INaryTree<GQuadtreeColor>> getSubtrees() {
    return GCollections.<INaryTree<GQuadtreeColor>>asList(subtrees);
  }
  
  @Override
  public void setRoot(GQuadtreeColor pRoot) {
    throw new UnsupportedOperationException();
  }
  
  @Override
  public void setSubtrees(IList<INaryTree<GQuadtreeColor>> pSubtrees) {
    throw new UnsupportedOperationException();
  }
  
  @Override
  public void addSubtree(INaryTree<GQuadtreeColor> pSubtree) {
    throw new UnsupportedOperationException();
  }
  
  @Override
  public void addSubtree(int pIndex, INaryTree<GQuadtreeColor> pSubtree) {
    throw new UnsupportedOperationException();
  }
  
  @Override
  public void removeSubtree(int pIndex) {
    throw new UnsupportedOperationException();
  }
  
  @Override
  public GQuadtree getSubtree(int pIndex) {
    return subtrees[pIndex];
  }
  
  public String postOrderTraversalAsString() {
    int index=0;
    char[] result=new char[getWeight()];
    for (GQuadtreeColor color:postOrderTraversal()) {
      result[index++]=color.getIdentifier();
    }
    return new String(result);
  }
  
  protected int getPixelIndex(int pHalf, ILatticePoint pPoint) {
    return pPoint.getCoordinateY()<pHalf?(pPoint.getCoordinateX()<pHalf?0:1):(pPoint.getCoordinateX()<pHalf?2:3);
  }
  
  public GQuadtreeColor getPixelColor(int pSize, ILatticePoint pPoint) {
    if (pSize!=Integer.highestOneBit(pSize)) {
      throw new IllegalArgumentException("The size must be a power of two.");
    }
    else if (!color.isGray()) {
      return color;
    }
    else {
      int half=pSize>>>1;
      return subtrees[getPixelIndex(half,pPoint)].getPixelColor(half,new GLatticePoint(pPoint.getCoordinateX()%half,pPoint.getCoordinateY()%half));
    }
  }
  
  public void setPixelColor(int pSize, ILatticePoint pPoint, GQuadtreeColor pColor) {
    int half=pSize>>>1;
    if (pSize!=Integer.highestOneBit(pSize)) {
      throw new IllegalArgumentException("The size must be a power of two.");
    }
    else if (pColor.isGray()) {
      throw new IllegalArgumentException("The color cannot be gray.");
    }
    else if (pSize==1) {
      clear(pColor);
    }
    else if (color!=pColor) {
      if (!color.isGray()) {
        subtrees=new GQuadtree[4];
        for (int index=0; index<4; index++) {
          subtrees[index]=new GQuadtree(color);
        }
        color=GQuadtreeColor.GRAY;
      }
      subtrees[getPixelIndex(half,pPoint)].setPixelColor(half,new GLatticePoint(pPoint.getCoordinateX()%half,pPoint.getCoordinateY()%half),pColor);
      int count=0;
      for (GQuadtree subtree:subtrees) {
        if (subtree.color==pColor) count++;
      }
      if (count==4) clear(pColor);
    }
  }
  
  public static GQuadtree reconstruct(String pPostOrder) {
    IStack<GQuadtree> stack=new GArrayStack<GQuadtree>();
    for (char character:pPostOrder.toCharArray()) {
      GQuadtree quadtree=new GQuadtree(GQuadtreeColor.getColor(character));
      if (quadtree.color.isGray()) {
        quadtree.subtrees=new GQuadtree[4];
        for (int index=3; index>=0; index--) {
          quadtree.subtrees[index]=stack.pop();
        }
      }
      stack.push(quadtree);
    }
    return stack.pop();
  }
  
}
