package gold.structures.tree.nary;

import gold.structures.list.*;
import gold.util.*;

public class GRecursiveNaryTree<E> extends GAbstractNaryTree<E> {
  
  private static final long serialVersionUID=-3030324352139613413L;
  
  protected E root=null;
  protected IList<INaryTree<E>> subtrees=null;
  
  public GRecursiveNaryTree() {
    clear();
  }
  
  public GRecursiveNaryTree(E pRoot) {
    this(pRoot,new GArrayList<INaryTree<E>>(4));
  }
  
  public GRecursiveNaryTree(E pRoot, IList<INaryTree<E>> pSubtrees) {
    setRoot(pRoot);
    setSubtrees(pSubtrees);
  }
  
  public GRecursiveNaryTree(INaryTree<E> pTree) {
    if (pTree.isEmpty()) {
      clear();
    }
    else {
      setRoot(pTree.getRoot());
      for (INaryTree<E> subtree:pTree.getSubtrees()) {
        addSubtree(new GRecursiveNaryTree<E>(subtree));
      }
    }
  }
  
  @Override
  public boolean isEmpty() {
    return root==null||subtrees==null;
  }
  
  @Override
  public void clear() {
    root=null;
    subtrees=null;
  }
  
  @Override
  public GRecursiveNaryTree<E> clone() {
    return new GRecursiveNaryTree<E>(this);
  }
  
  @Override
  public E getRoot() {
    return root;
  }
  
  @Override
  public IList<INaryTree<E>> getSubtrees() {
    return GCollections.unmodifiableList(subtrees);
  }
  
  @Override
  public void setRoot(E pRoot) {
    if (pRoot==null) throw new IllegalArgumentException("The root cannot be null.");
    root=pRoot;
  }
  
  @Override
  public void setSubtrees(IList<INaryTree<E>> pSubtrees) {
    if (pSubtrees==null) throw new IllegalArgumentException("The list of subtrees cannot be null.");
    subtrees=pSubtrees;
  }
  
  @Override
  public void addSubtree(INaryTree<E> pSubtree) {
    subtrees.addLast(pSubtree);
  }
  
  @Override
  public void addSubtree(int pIndex, INaryTree<E> pSubtree) {
    subtrees.add(pIndex,pSubtree);
  }
  
  @Override
  public void removeSubtree(int pIndex) {
    subtrees.remove(pIndex);
  }
  
  @Override
  public INaryTree<E> getSubtree(int pIndex) {
    return subtrees.get(pIndex);
  }
  
}
