package gold.structures.tuple;

import java.util.*;
import gold.structures.collection.*;
import gold.util.*;

// Basado en java.util.TreeMap.Entry

public class GCouple<T,U> extends GAbstractCollection<Object> implements ICouple<T,U> {
  
  private static final long serialVersionUID=667826763865406616L;
  
  protected T first=null;
  protected U second=null;
  
  public GCouple(T pFirst, U pSecond) {
    first=pFirst;
    second=pSecond;
  }
  
  @Override
  public Object get(int pIndex) {
    if (pIndex==0) {
      return first;
    }
    else if (pIndex==1) {
      return second;
    }
    else {
      throw new IndexOutOfBoundsException();
    }
  }
  
  @Override
  @SuppressWarnings("unchecked")
  public void set(int pIndex, Object pElement) {
    if (pIndex==0) {
      first=(T)pElement;
    }
    else if (pIndex==1) {
      second=(U)pElement;
    }
    else {
      throw new IndexOutOfBoundsException();
    }
  }
  
  @Override
  public T getFirst() {
    return first;
  }
  
  @Override
  public U getSecond() {
    return second;
  }
  
  @Override
  public void setFirst(T pFirst) {
    first=pFirst;
  }
  
  @Override
  public void setSecond(U pSecond) {
    second=pSecond;
  }
  
  @Override
  public int size() {
    return 2;
  }
  
  @Override
  public void clear() {
    throw new UnsupportedOperationException();
  }
  
  @Override
  public boolean equals(Object pObject) {
    if (pObject instanceof ICouple) {
      return GToolkit.equals(first,((ICouple<?,?>)pObject).getFirst())&&GToolkit.equals(second,((ICouple<?,?>)pObject).getSecond());
    }
    else if (pObject instanceof IPair) {
      return GToolkit.equals(first,((IPair<?>)pObject).getFirst())&&GToolkit.equals(second,((IPair<?>)pObject).getSecond());
    }
    else if (pObject instanceof ITuple) {
      ITuple<?> tuple=(ITuple<?>)pObject;
      return tuple.size()==2&&GToolkit.equals(first,tuple.get(0))&&GToolkit.equals(second,tuple.get(1));
    }
    else {
      return super.equals(pObject);
    }
  }
  
  @Override
  public List<Object> toJavaView() {
    return new AbstractList<Object>() {
      @Override
      public Object get(int pIndex) {
        return GCouple.this.get(pIndex);
      }
      @Override
      public Object set(int pIndex, Object pElement) {
        Object oldElement=GCouple.this.get(pIndex);
        GCouple.this.set(pIndex,pElement);
        return oldElement;
      }
      @Override
      public int size() {
        return GCouple.this.size();
      }
    };
  }
  
  @Override
  public Map.Entry<T,U> toJavaEntry() {
    return new Map.Entry<T,U>() {
      @Override
      public T getKey() {
        return GCouple.this.getFirst();
      }
      @Override
      public U getValue() {
        return GCouple.this.getSecond();
      }
      @Override
      public U setValue(U pValue) {
        U oldValue=getValue();
        GCouple.this.setSecond(pValue);
        return oldValue;
      }
    };
  }
  
  @Override
  public GCouple<T,U> clone() {
    return new GCouple<T,U>(first,second);
  }
  
}
