package gold.structures.tuple;

import java.util.*;
import gold.structures.collection.*;

public class GNullTuple<E> extends GAbstractCollection<E> implements INullTuple<E> {
  
  private static final long serialVersionUID=7654860926398850311L;
  
  public GNullTuple() {
  }
  
  @Override
  public E get(int pIndex) {
    throw new IndexOutOfBoundsException();
  }
  
  @Override
  public void set(int pIndex, E pElement) {
    throw new IndexOutOfBoundsException();
  }
  
  @Override
  public int size() {
    return 0;
  }
  
  @Override
  public void clear() {
    throw new UnsupportedOperationException();
  }
  
  @Override
  public boolean equals(Object pObject) {
    if (pObject instanceof INullTuple) {
      return true;
    }
    else if (pObject instanceof ITuple) {
      ITuple<?> tuple=(ITuple<?>)pObject;
      return tuple.size()==0;
    }
    else {
      return super.equals(pObject);
    }
  }
  
  @Override
  public List<E> toJavaView() {
    return new AbstractList<E>() {
      @Override
      public E get(int pIndex) {
        throw new IndexOutOfBoundsException();
      }
      @Override
      public E set(int pIndex, E pElement) {
        throw new IndexOutOfBoundsException();
      }
      @Override
      public int size() {
        return 0;
      }
    };
  }
  
  @Override
  public GNullTuple<E> clone() {
    return new GNullTuple<E>();
  }
  
}
