package gold.swing.look;

import java.awt.*;
import java.awt.event.*;
import java.awt.geom.*;
import javax.swing.*;
import javax.swing.plaf.*;
import javax.swing.plaf.metal.*;

public class GButtonLook extends MetalButtonUI {
  
  public static ComponentUI createUI(JComponent pComponent) {
    return new GButtonLook((JButton)pComponent);
  }
  
  private boolean hover=false;
  private final JButton button;
  private final boolean special;
  
  private GButtonLook(JButton pButton) {
    button=pButton;
    String text=button.getText();
    special=text!=null&&!text.isEmpty();
    button.addMouseListener(new GButtonLookMouseListener());
  }
  
  @Override
  public void installUI(JComponent pComponent) {
    super.installUI(pComponent);
    JButton button=(JButton)pComponent;
    button.setCursor(Cursor.getPredefinedCursor(Cursor.HAND_CURSOR));
    if (special) {
      button.setPreferredSize(new Dimension(button.getPreferredSize().width,26));
      button.setMinimumSize(button.getPreferredSize());
      button.setMaximumSize(button.getPreferredSize());
      button.setSize(button.getPreferredSize());
      button.setBorder(BorderFactory.createEmptyBorder());
      button.setBorderPainted(false);
      button.setFocusPainted(false);
      button.setContentAreaFilled(false);
      button.setOpaque(false);
    }
    else {
      button.setBackground(new Color(225,225,225));
    }
  }
  
  private Paint getGoldBackgroundColor(boolean pEnabled, boolean pPressed, int pHeight) {
    Color color1=null,color2=null;
    if (pEnabled&&pPressed) {
      color1=new Color(0xD2D2D2);
      color2=new Color(0xA6A6A6);
    }
    else if (pEnabled) {
      color1=new Color(0xF2F2F2);
      color2=new Color(0xC6C6C6);
    }
    else {
      color1=new Color(0xC4C4C4);
      color2=new Color(0xA9A9A9);
    }
    return new GradientPaint(0,0,color1,0,pHeight,color2);
  }
  
  private Paint getGoldBorderColor(boolean pEnabled, boolean pHover) {
    return new Color(pHover&&pEnabled?0x666666:0xCCCCCC);
  }
  
  private Paint getGoldTextColor(boolean pEnabled, boolean pHover) {
    return new Color(pHover&&pEnabled?0x000000:0x444444);
  }
  
  @Override
  public void paint(Graphics pGraphics, JComponent pComponent) {
    JButton button=(JButton)pComponent;
    if (special) {
      boolean enabled=button.isEnabled();
      boolean pressed=button.getModel().isPressed();
      String text=button.getText();
      Graphics2D graphics=(Graphics2D)pGraphics;
      graphics.setRenderingHint(RenderingHints.KEY_TEXT_ANTIALIASING,RenderingHints.VALUE_TEXT_ANTIALIAS_LCD_HRGB);
      graphics.setRenderingHint(RenderingHints.KEY_ANTIALIASING,RenderingHints.VALUE_ANTIALIAS_ON);
      Rectangle2D r=graphics.getFontMetrics().getStringBounds(text,graphics);
      int w=button.getWidth(),h=button.getHeight(),b=12;
      graphics.setPaint(new Color(0,0,0,0));
      graphics.fill(new Rectangle2D.Double(0,0,w,h));
      graphics.setPaint(getGoldBackgroundColor(enabled,pressed,h));
      graphics.fill(new RoundRectangle2D.Double(0,0,w,h,b,b));
      graphics.setPaint(getGoldBorderColor(enabled,hover));
      graphics.draw(new RoundRectangle2D.Double(0,0,w-1,h-1,b,b));
      graphics.setPaint(getGoldTextColor(enabled,hover));
      graphics.setRenderingHint(RenderingHints.KEY_ANTIALIASING,RenderingHints.VALUE_ANTIALIAS_OFF);
      graphics.drawString(text,(float)((w-r.getWidth())/2),(float)((h-r.getHeight())/2+r.getHeight()-3));
    }
    else {
      super.paint(pGraphics,pComponent);
    }
  }
  
  private final class GButtonLookMouseListener implements MouseListener {
    
    private void setHover(boolean pHover) {
      if (hover!=pHover) {
        hover=pHover;
        button.repaint();
      }
    }
    
    @Override
    public void mouseClicked(MouseEvent pEvent) {
    }
    
    @Override
    public void mouseEntered(MouseEvent pEvent) {
      setHover(true);
    }
    
    @Override
    public void mouseExited(MouseEvent pEvent) {
      setHover(false);
    }
    
    @Override
    public void mousePressed(MouseEvent pEvent) {
    }
    
    @Override
    public void mouseReleased(MouseEvent pEvent) {
      setHover(false);
    }
    
  }
  
}
