package gold.swing.look;

import java.awt.*;
import java.awt.geom.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.plaf.metal.*;
import gold.swing.util.*;

@SuppressWarnings("serial")
public final class GLookAndFeel extends MetalLookAndFeel {
  
  private static final int RADIUS=15;
  
  public GLookAndFeel() {
  }
  
  @Override
  public UIDefaults getDefaults() {
    UIDefaults table=super.getDefaults();
    Font font=new Font("Verdana",Font.PLAIN,12);
    // Background
    for (String id:new String[]{"Button","CheckBox","ColorChooser","ComboBox","Desktop","OptionPane","Label","Panel","RadioButton","Slider","SplitPane","TabbedPane","TableHeader","ToggleButton"}) {
      table.put(id+".background",GColor.BACKGROUND);
    }
    for (String id:new String[]{"ComboBox","FormattedTextField"}) {
      table.put(id+".background",Color.WHITE);
    }
    // Font
    for (String id:new String[]{"Button","CheckBox","CheckBoxMenuItem","ColorChooser","ComboBox","DesktopIcon","EditorPane","FormattedTextField","Label","List","Menu","MenuBar","MenuItem","OptionPane","Panel","PasswordField","PopupMenu","ProgressBar","RadioButton","RadioButtonMenuItem","ScrollPane","Slider","Spinner","TabbedPane","Table","TableHeader","TextArea","TextField","TextPane","TitledBorder","ToggleButton","ToolBar","ToolTip","Tree","Viewport"}) {
      table.put(id+".font",font);
    }
    // boldMetal
    table.put("swing.boldMetal",Boolean.FALSE);
    // SplitPane
    table.put("SplitPane.dividerSize",6);
    // ScrollBar
    table.put("ScrollBar.width",14);
    table.put("ScrollBar.shadow",table.get("ScrollBar.background"));
    table.put("ScrollPane.border",BorderFactory.createLineBorder(GColor.BORDER));
    // TitledBorder
    table.put("TitledBorder.border",BorderFactory.createLineBorder(GColor.BORDER));
    // Spinner
    table.put("Spinner.border",BorderFactory.createEmptyBorder());
    // Tree
    table.put("Tree.hash",GColor.BORDER);
    // ToolTip
    table.put("ToolTip.border",new GCurvedBorder(5,new Color(118,118,118)));
    table.put("ToolTip.background",new Color(245,245,245));
    // TabbedPane
    table.put("TabbedPane.contentBorderInsets",new Insets(1,1,1,1));
    table.put("TabbedPane.tabAreaInsets",new Insets(0,0,0,0));
    table.put("TabbedPane.tabInsets",new Insets(0,0,0,0));
    table.put("TabbedPane.borderHightlightColor",GColor.BORDER);
    table.put("TabbedPane.contentAreaColor",GColor.BORDER);
    table.put("TabbedPane.darkShadow",GColor.BORDER);
    table.put("TabbedPane.focus",new Color(0,0,0,0));
    table.put("TabbedPane.foreground",Color.BLACK);
    // OptionPane
    table.put("OptionPane.warningDialog.titlePane.background",new Color(255,204,153));
    table.put("inactiveCaptionBorder",GColor.TRANSPARENT);
    table.put("activeCaptionBorder",GColor.TRANSPARENT);
    // Menu
    table.put("Menu.borderPainted",false);
    table.put("MenuItem.borderPainted",false);
    table.put("CheckBoxMenuItem.borderPainted",false);
    table.put("RadioButtonMenuItem.borderPainted",false);
    // PopupMenu
    table.put("PopupMenu.border",BorderFactory.createCompoundBorder(BorderFactory.createLineBorder(GColor.BORDER),BorderFactory.createEmptyBorder(0,0,0,0)));
    // SelectionBackground
    table.put("ComboBox.selectionBackground",GColor.SELECTED);
    table.put("List.selectionBackground",GColor.SELECTED);
    table.put("Table.selectionBackground",GColor.SELECTED);
    table.put("Tree.selectionBackground",GColor.SELECTED);
    // UI
    table.put("ComboBoxUI","gold.swing.look.GComboBoxLook");
    table.put("FormattedTextFieldUI","gold.swing.look.GFormattedTextFieldLook");
    table.put("TextFieldUI","gold.swing.look.GTextFieldLook");
    table.put("MenuItemUI","gold.swing.look.GMenuItemLook");
    table.put("RadioButtonMenuItemUI","gold.swing.look.GRadioButtonMenuItemLook");
    table.put("RadioButtonUI","gold.swing.look.GRadioButtonLook");
    table.put("CheckBoxMenuItemUI","gold.swing.look.GCheckBoxMenuItemLook");
    table.put("CheckBoxUI","gold.swing.look.GCheckBoxLook");
    table.put("MenuUI","gold.swing.look.GMenuLook");
    table.put("MenuBarUI","gold.swing.look.GMenuBarLook");
    table.put("PopupMenuSeparatorUI","gold.swing.look.GPopupMenuSeparatorLook");
    table.put("TabbedPaneUI","gold.swing.look.GTabbedPaneLook");
    table.put("ButtonUI","gold.swing.look.GButtonLook");
    table.put("ToggleButtonUI","gold.swing.look.GToggleButtonLook");
    table.put("ScrollBarUI","gold.swing.look.GScrollBarLook");
    return table;
  }
  
  @Override
  public String getID() {
    return "GoldLookAndFeel";
  }
  
  @Override
  public String getName() {
    return "GoldLookAndFeel";
  }
  
  @Override
  public String getDescription() {
    return "GOLD Look and Feel";
  }
  
  @Override
  public boolean isNativeLookAndFeel() {
    return false;
  }
  
  @Override
  public boolean isSupportedLookAndFeel() {
    return true;
  }
  
  public static void configureSize(JComponent pComponent) {
    pComponent.setPreferredSize(new Dimension(100,21));
  }
  
  public static void configureBorder(JComponent pComponent) {
    pComponent.setBorder(BorderFactory.createCompoundBorder(new GCurvedBorder(RADIUS,GColor.BORDER),BorderFactory.createEmptyBorder(0,3,0,3)));
  }
  
  public static void paintBackground(JComponent pComponent, Graphics pGraphics) {
    Graphics2D graphics=(Graphics2D)pGraphics;
    Object valAntialiasing=graphics.getRenderingHint(RenderingHints.KEY_ANTIALIASING);
    int width=pComponent.getWidth(),height=pComponent.getHeight();
    graphics.setRenderingHint(RenderingHints.KEY_ANTIALIASING,RenderingHints.VALUE_ANTIALIAS_ON);
    graphics.setColor(pComponent.getParent().getBackground());
    graphics.fillRect(0,0,width,height);
    graphics.setColor(pComponent.getBackground());
    graphics.fillRoundRect(0,0,width-1,height-1,RADIUS,RADIUS);
    graphics.setRenderingHint(RenderingHints.KEY_ANTIALIASING,valAntialiasing);
  }
  
  private static final class GCurvedBorder extends AbstractBorder {
    
    private final int radius;
    private final Color color;
    
    public GCurvedBorder(int pRadius, Color pColor) {
      radius=pRadius;
      color=pColor;
    }
    
    @Override
    public void paintBorder(Component pComponent, Graphics pGraphics, int pX, int pY, int pW, int pH) {
      Graphics2D graphics=(Graphics2D)pGraphics;
      graphics.setRenderingHint(RenderingHints.KEY_ANTIALIASING,RenderingHints.VALUE_ANTIALIAS_ON);
      graphics.setColor(color);
      graphics.draw(new RoundRectangle2D.Double(pX,pY,pW-1,pH-1,radius,radius));
    }
    
    @Override
    public Insets getBorderInsets(Component pComponent) {
      return new Insets(2,2,2,2);
    }
    
    @Override
    public boolean isBorderOpaque() {
      return true;
    }
    
  }
  
}
