package gold.swing.look;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.plaf.*;
import javax.swing.plaf.basic.*;
import gold.swing.util.*;

public class GMenuItemLook extends BasicMenuItemUI {
  
  public static ComponentUI createUI(JComponent pComponent) {
    return new GMenuItemLook();
  }
  
  @Override
  protected void paintBackground(Graphics pGraphics, JMenuItem pMenuItem, Color pColor) {
    GMenuItemLook.paintBackground(pGraphics,pMenuItem);
  }
  
  @Override
  protected Dimension getPreferredMenuItemSize(JComponent pComponent, Icon pCheckIcon, Icon pArrowIcon, int pDefaultTextIconGap) {
    return super.getPreferredMenuItemSize(pComponent,pCheckIcon,pArrowIcon,GMenuItemLook.getTextIconGap(pComponent,pCheckIcon,pDefaultTextIconGap));
  }
  
  @Override
  protected void paintText(Graphics pGraphics, JMenuItem pMenuItem, Rectangle pTextRect, String pText) {
    super.paintText(pGraphics,pMenuItem,GMenuItemLook.getTextRectangle(pGraphics,pMenuItem,pTextRect),pText);
  }
  
  @Override
  protected void paintMenuItem(Graphics pGraphics, JComponent pComponent, Icon pCheckIcon, Icon pArrowIcon, Color pBackground, Color pForeground, int pDefaultTextIconGap) {
    JMenuItem menuItem=(JMenuItem)pComponent;
    int textHeight=pGraphics.getFontMetrics().getHeight();
    GMenuItemLook.paint(pGraphics,pComponent,menuItem.isEnabled()?menuItem.getIcon():menuItem.getDisabledIcon(),pCheckIcon,pArrowIcon,pBackground,pForeground,pDefaultTextIconGap);
    paintText(pGraphics,menuItem,new Rectangle(30,(menuItem.getHeight()-textHeight)/2,menuItem.getWidth()-30,textHeight),menuItem.getText());
  }
  
  public static void paint(Graphics pGraphics, JComponent pComponent, Icon pIcon, Icon pCheckIcon, Icon pArrowIcon, Color pBackground, Color pForeground, int pDefaultTextIconGap) {
    JMenuItem menuItem=(JMenuItem)pComponent;
    Graphics2D graphics=(Graphics2D)pGraphics;
    graphics.setRenderingHint(RenderingHints.KEY_TEXT_ANTIALIASING,RenderingHints.VALUE_TEXT_ANTIALIAS_ON);
    Icon icon=pIcon;
    GMenuItemLook.paintBackground(graphics,menuItem);
    if (icon!=null) icon.paintIcon(menuItem,graphics,6,(menuItem.getHeight()-icon.getIconHeight())/2);
    KeyStroke keyStroke=menuItem.getAccelerator();
    if (keyStroke!=null) {
      String textKeyStroke=KeyEvent.getKeyModifiersText(keyStroke.getModifiers())+"-"+KeyEvent.getKeyText(keyStroke.getKeyCode());
      Font font=UIManager.getFont("MenuItem.acceleratorFont"),oldFont=graphics.getFont();
      Color color=menuItem.isEnabled()?new Color(100,130,190):UIManager.getColor("MenuItem.disabledForeground"),oldColor=graphics.getColor();
      if (font!=null) graphics.setFont(font);
      if (color!=null) graphics.setColor(color);
      int ksTextWidth=graphics.getFontMetrics().stringWidth(textKeyStroke),ksTextHeight=graphics.getFontMetrics().getHeight();
      graphics.drawString(textKeyStroke,menuItem.getWidth()-ksTextWidth-5,0.5f*(menuItem.getHeight()+ksTextHeight)-4);
      graphics.setColor(oldColor);
      graphics.setFont(oldFont);
    }
  }
  
  public static void paintBackground(Graphics pGraphics, JMenuItem pMenuItem) {
    Graphics2D graphics=(Graphics2D)pGraphics;
    int width=pMenuItem.getWidth(),height=pMenuItem.getHeight(),left=25;
    Color oldColor=graphics.getColor();
    ButtonModel model=pMenuItem.getModel();
    boolean isSelected=model.isArmed()||((pMenuItem instanceof JMenu)&&model.isSelected());
    boolean isMenu=(pMenuItem.getParent() instanceof JMenuBar);
    graphics.setColor(isSelected?GColor.SELECTED:(isMenu?GColor.TRANSPARENT:GColor.BACKGROUND));
    graphics.fillRect(0,0,width,height);
    if (!isMenu) {
      graphics.setColor(new Color(205,149,12));
      graphics.drawLine(left,0,left,height);
    }
    if (isSelected) {
      graphics.setColor(GColor.BORDER);
      graphics.drawRect(0,0,width-1,height-1);
    }
    graphics.setColor(oldColor);
  }
  
  public static int getTextIconGap(JComponent pComponent, Icon pCheckIcon, int pDefaultTextIconGap) {
    JMenuItem menuItem=((JMenuItem)pComponent);
    if (menuItem.getParent() instanceof JMenuBar) {
      return pDefaultTextIconGap;
    }
    else {
      Icon icon=pCheckIcon!=null?pCheckIcon:menuItem.getIcon();
      int iconWidth=icon!=null?icon.getIconWidth():0;
      return Math.max(pDefaultTextIconGap,20-iconWidth);
    }
  }
  
  public static Rectangle getTextRectangle(Graphics pGraphics, JMenuItem pMenuItem, Rectangle pRectangle) {
    if (pMenuItem.getParent() instanceof JMenuBar) {
      int x=(pMenuItem.getWidth()-pGraphics.getFontMetrics().stringWidth(pMenuItem.getText()))/2;
      return new Rectangle(x,pRectangle.y,pRectangle.width,pRectangle.height);
    }
    else {
      return pRectangle;
    }
  }
  
}
