package gold.swing.look;

import java.awt.*;
import java.awt.geom.*;
import javax.swing.*;
import javax.swing.plaf.*;
import javax.swing.plaf.metal.*;
import gold.swing.util.*;

public class GScrollBarLook extends MetalScrollBarUI {
  
  public static ComponentUI createUI(JComponent pComponent) {
    return new GScrollBarLook();
  }
  
  @Override
  protected void paintThumb(Graphics pGraphics, JComponent pComponent, Rectangle pThumbBounds) {
    Rectangle rect=pThumbBounds;
    if (!rect.isEmpty()&&super.scrollbar.isEnabled()) {
      boolean a=super.scrollbar.getValueIsAdjusting()||super.scrollbar.getMousePosition()!=null;
      boolean b=super.scrollbar.getOrientation()==SwingConstants.HORIZONTAL;
      Color colors[]={getColor(255,245,235,a),getColor(255,235,215,a),getColor(250,220,170,a),getColor(230,200,150,a)};
      double rx=rect.x,ry=rect.y,rw=rect.width+(b?-1.0:0.0),rh=rect.height+(b?0.0:-1.0);
      Graphics2D graphics=GUtilities.setRenderingHints(pGraphics);
      Shape shape=new RoundRectangle2D.Double(rx+1,ry+1,rw-1,rh-1,10,10);
      graphics.setPaint(new LinearGradientPaint((float)rx,(float)ry,(float)(rx+(b?0:rw)),(float)(ry+(b?rh:0)),new float[]{0.0f,0.49f,0.51f,1.0f},colors));
      graphics.fill(shape);
      graphics.setPaint(getColor(177,127,60,a));
      graphics.draw(shape);
      for (int k=0,kT=(b?rw:rh)<15?2:3; k<kT; k++) {
        double ls=0.5*(b?rh:rw),lx=rx+0.5*(rw-(b?kT*3:ls))+(b?k*3+2:0.0),ly=ry+0.5*(rh-(b?ls:kT*3))+(b?0.0:k*3+2);
        graphics.setColor(getColor(190,160,90,a));
        graphics.draw(new Line2D.Double(lx+(b?0.0:0.0),ly+(b?0.0:0.0),lx+(b?0.0:ls),ly+(b?ls:0.0)));
        graphics.setColor(getColor(210,210,210,a));
        graphics.draw(new Line2D.Double(lx+(b?1.0:0.0),ly+(b?0.0:1.0),lx+(b?1.0:ls),ly+(b?ls:1.0)));
      }
    }
  }
  
  @Override
  protected void paintTrack(Graphics pGraphics, JComponent pComponent, Rectangle pTrackBounds) {
    Graphics2D graphics=GUtilities.setRenderingHints(pGraphics);
    graphics.setColor(pComponent.getBackground());
    graphics.fillRect(pTrackBounds.x,pTrackBounds.y,pTrackBounds.width,pTrackBounds.height);
    graphics.setColor(GColor.BORDER);
    if (super.scrollbar.getOrientation()==SwingConstants.HORIZONTAL) {
      graphics.drawLine(pTrackBounds.x,pTrackBounds.y,pTrackBounds.x+pTrackBounds.width,pTrackBounds.y);
    }
    else {
      graphics.drawLine(pTrackBounds.x,pTrackBounds.y,pTrackBounds.x,pTrackBounds.y+pTrackBounds.height);
    }
  }
  
  @Override
  protected JButton createDecreaseButton(int pOrientation) {
    return createGoldButton(pOrientation);
  }
  
  @Override
  protected JButton createIncreaseButton(int pOrientation) {
    return createGoldButton(pOrientation);
  }
  
  private JButton createGoldButton(int pOrientation) {
    boolean b=super.scrollbar.getOrientation()==SwingConstants.HORIZONTAL;
    int index=1;
    switch (pOrientation) {
      case SwingConstants.NORTH:
        index=1;
        break;
      case SwingConstants.EAST:
        index=2;
        break;
      case SwingConstants.SOUTH:
        index=3;
        break;
      case SwingConstants.WEST:
        index=4;
        break;
    }
    Dimension d=super.scrollbar.getPreferredSize();
    int s=Math.min(d.width,d.height);
    JButton button=new JButton(GUtilities.getIcon("gold/scrollArrow"+index+".png"));
    button.setCursor(Cursor.getPredefinedCursor(Cursor.HAND_CURSOR));
    button.setBorder(BorderFactory.createMatteBorder(1,1,b?0:1,b?1:0,GColor.BORDER));
    button.setFocusPainted(false);
    button.setPreferredSize(new Dimension(s,s));
    return button;
  }
  
  private static Color getColor(int pRed, int pGreen, int pBlue, boolean pIsActive) {
    if (pIsActive) {
      return new Color(pRed,pGreen,pBlue);
    }
    else {
      int c=(pRed+pGreen+pBlue)/3;
      return new Color(c,c,c);
    }
  }
  
}
