package gold.swing.util;

import java.awt.*;
import java.beans.*;
import java.util.*;
import javax.swing.*;

@SuppressWarnings("serial")
public final class GDesktop extends JDesktopPane {
  
  private int index=0;
  
  public GDesktop() {
    setDragMode(JDesktopPane.LIVE_DRAG_MODE);
  }
  
  public int getOpenedFrameCount() {
    return index;
  }
  
  private void setCascadeLocation(JInternalFrame pInternalFrame, int pIndex) {
    Dimension d=pInternalFrame.getPreferredSize();
    pInternalFrame.reshape((30*pIndex)%(getWidth()-d.width),(30*pIndex)%(getHeight()-d.height),d.width,d.height);
  }
  
  public void cascade() {
    JInternalFrame[] internalFrames=getAllFrames();
    for (int i=0; i<internalFrames.length; i++) {
      restoreInternalFrame(internalFrames[i]);
      setCascadeLocation(internalFrames[i],i);
      internalFrames[i].toFront();
    }
  }
  
  // Basado en http://www.thatsjava.com/java-swing/4349/
  public void tiles() {
    JInternalFrame[] internalFrames=getAllFrames();
    if (internalFrames.length==0) return;
    int tam=internalFrames.length,rows=(int)(Math.sqrt(tam)),cols=tam/rows,extra=tam%rows,width=getWidth()/cols,height=getHeight()/rows,r=0,c=0;
    for (int i=0; i<tam; i++) {
      restoreInternalFrame(internalFrames[i]);
      internalFrames[i].reshape(c*width,r*height,width,height);
      if (++r==rows) {
        r=0;
        if (++c==cols-extra) height=getHeight()/(++rows);
      }
    }
  }
  
  public void addInternalFrame(JInternalFrame pInternalFrame) {
    if (!Arrays.asList(getAllFrames()).contains(pInternalFrame)) {
      setCascadeLocation(pInternalFrame,index++);
      super.add(pInternalFrame);
      pInternalFrame.setVisible(true);
      selectInternalFrame(pInternalFrame);
    }
  }
  
  public void selectInternalFrame(JInternalFrame pInternalFrame) {
    if (pInternalFrame.isIcon()) restoreInternalFrame(pInternalFrame);
    pInternalFrame.toFront();
    try {
      pInternalFrame.setSelected(true);
    }
    catch (PropertyVetoException e) {
    }
  }
  
  public void maximizeInternalFrame(JInternalFrame pInternalFrame) {
    if (pInternalFrame.isMaximum()) return;
    try {
      if (pInternalFrame.isIcon()) pInternalFrame.setIcon(false);
      pInternalFrame.setMaximum(true);
    }
    catch (PropertyVetoException e) {
    }
    selectInternalFrame(pInternalFrame);
  }
  
  public void minimizeInternalFrame(JInternalFrame pInternalFrame) {
    if (pInternalFrame.isIcon()) return;
    try {
      if (pInternalFrame.isMaximum()) pInternalFrame.setMaximum(false);
      pInternalFrame.setIcon(true);
    }
    catch (PropertyVetoException e) {
    }
  }
  
  public void restoreInternalFrame(JInternalFrame pInternalFrame) {
    try {
      if (pInternalFrame.isIcon()) pInternalFrame.setIcon(false);
      if (pInternalFrame.isMaximum()) pInternalFrame.setMaximum(false);
    }
    catch (PropertyVetoException e) {
    }
    selectInternalFrame(pInternalFrame);
  }
  
}
