package gold.swing.util;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import javax.swing.*;
import javax.swing.filechooser.*;
import javax.swing.plaf.metal.*;

@SuppressWarnings("serial")
public abstract class GFileChooser extends JFileChooser {
  
  protected static final GFileView FILE_VIEW=new GFileView();
  protected static final FileSystemView FILE_SYSTEM_VIEW=FileSystemView.getFileSystemView();
  protected static File CURRENT_DIRECTORY=FILE_SYSTEM_VIEW.getHomeDirectory();
  
  protected final JButton acceptButton=new JButton("Accept");
  protected final JButton cancelButton=new JButton("Cancel");
  protected boolean accepted=false;
  protected JDialog dialog=null;
  
  protected GFileChooser(int pDialogType) {
    setBorder(BorderFactory.createEmptyBorder(3,0,8,0));
    setAcceptAllFileFilterUsed(false);
    setDialogType(pDialogType);
    setFileHidingEnabled(false);
    setMultiSelectionEnabled(false);
    setFileSelectionMode(JFileChooser.FILES_ONLY);
    setControlButtonsAreShown(false);
    setFileSystemView(FILE_SYSTEM_VIEW);
    setFileView(FILE_VIEW);
    acceptButton.addActionListener(new ActionListener() {
      @Override
      public void actionPerformed(ActionEvent pEvent) {
        ((MetalFileChooserUI)getUI()).getApproveSelectionAction().actionPerformed(pEvent);
      }
    });
    cancelButton.addActionListener(new ActionListener() {
      @Override
      public void actionPerformed(ActionEvent pEvent) {
        ((MetalFileChooserUI)getUI()).getCancelSelectionAction().actionPerformed(pEvent);
      }
    });
    addActionListener(new AbstractAction() {
      @Override
      public void actionPerformed(ActionEvent pEvent) {
        if (JFileChooser.APPROVE_SELECTION.equals(pEvent.getActionCommand())) {
          if (dialog!=null) dialog.dispose();
        }
        else if (JFileChooser.CANCEL_SELECTION.equals(pEvent.getActionCommand())) {
          if (dialog!=null) dialog.dispose();
        }
      }
    });
  }
  
  @Override
  public void approveSelection() {
    accepted=true;
    super.approveSelection();
  }
  
  @Override
  public void cancelSelection() {
    accepted=false;
    super.cancelSelection();
  }
  
  protected int showFileDialog(Component pParentComponent, String pTitle) {
    accepted=false;
    if (CURRENT_DIRECTORY!=null) setCurrentDirectory(CURRENT_DIRECTORY);
    dialog=new JDialog(GUtilities.getWindowForComponent(pParentComponent),pTitle,JDialog.ModalityType.TOOLKIT_MODAL);
    dialog.addWindowListener(new WindowAdapter() {
      @Override
      public void windowClosing(WindowEvent pEvent) {
        GFileChooser.this.cancelSelection();
        if (dialog.isVisible()) dialog.dispose();
      }
    });
    rescanCurrentDirectory();
    GUtilities.configure(dialog,GUtilities.getBorderPane(0,7,0,7,null,null,GUtilities.getBoxPaneY(0,this),null,GUtilities.getFlowPane(FlowLayout.RIGHT,acceptButton,Box.createHorizontalStrut(5),cancelButton)));
    GUtilities.show(dialog,null);
    CURRENT_DIRECTORY=getCurrentDirectory();
    return accepted?JFileChooser.APPROVE_OPTION:JFileChooser.CANCEL_OPTION;
  }
  
  protected File verifyFileForSaving(Component pParentComponent, String pExtension) {
    File file=getSelectedFile();
    if (file==null) throw new IllegalStateException("You must select a file.");
    if (file.isDirectory()) throw new IllegalStateException("You must select a file.");
    String name=file.getName();
    if (!name.endsWith(pExtension)) {
      name+=pExtension;
      file=new File(file.getParentFile(),name);
    }
    if (file.exists()) {
      if (file.isFile()&&GOptionDialog.showQuestion(pParentComponent,"The file with name '"+GUtilities.escapeHTML(file.getName())+"' already exists.<p><p>¿You want overwrite it?")!=0) return null;
      if (file.isDirectory()) throw new IllegalStateException("You must select a file.");
      if (!file.canWrite()) throw new IllegalStateException("The file cannot be written.");
    }
    else {
      FileOutputStream fos=null;
      try {
        fos=new FileOutputStream(file);
      }
      catch (Throwable th) {
        throw new IllegalStateException("The file cannot be written because it has an invalid name.");
      }
      finally {
        try {
          if (fos!=null) fos.close();
        }
        catch (Throwable th) {
        }
      }
    }
    return file;
  }
  
  private static final class GFileView extends FileView {
    
    @Override
    public String getName(File pFile) {
      return FILE_SYSTEM_VIEW.getSystemDisplayName(pFile);
    }
    
    @Override
    public String getDescription(File pFile) {
      return super.getDescription(pFile);
    }
    
    @Override
    public String getTypeDescription(File pFile) {
      return FILE_SYSTEM_VIEW.getSystemTypeDescription(pFile);
    }
    
    @Override
    public Icon getIcon(File pFile) {
      String fileName=pFile!=null?pFile.getName().toLowerCase():"";
      if (pFile==null||!pFile.exists()) {
        return super.getIcon(pFile);
      }
      else if (pFile.isFile()&&(fileName.endsWith(".gaf")||fileName.endsWith(".ggf"))) {
        return GUtilities.getIcon("famfamfam/coins.png");
      }
      else {
        return FILE_SYSTEM_VIEW.getSystemIcon(pFile);
      }
    }
    
    @Override
    public Boolean isTraversable(File pFile) {
      return FILE_SYSTEM_VIEW.isTraversable(pFile);
    }
    
  }
  
}
