package gold.swing.util;

import java.awt.*;
import java.awt.event.*;
import javax.swing.*;

@SuppressWarnings("serial")
public final class GOptionDialog extends JDialog {
  
  private static enum Type {
    ERROR("windows/error.png","Error"),INFO("windows/info.png","Information"),WARNING("windows/warning.png","Warning"),QUESTION("windows/question.png","Question");
    private ImageIcon icon;
    private String title;
    private Type(String pIcon, String pTitle) {
      icon=GUtilities.getIcon(pIcon);
      title=pTitle;
    }
  }
  
  private int selectedIndex=-1;
  
  private GOptionDialog(Component pParentComponent, Object pMessage, Type pType, String...pButtons) {
    super(GUtilities.getWindowForComponent(pParentComponent),pType.title,JDialog.ModalityType.TOOLKIT_MODAL);
    final JButton[] buttons=new JButton[pButtons.length];
    for (int i=0; i<buttons.length; i++) {
      final int buttonIndex=i;
      buttons[i]=new JButton(pButtons[i]);
      buttons[i].addActionListener(new ActionListener() {
        @Override
        public void actionPerformed(ActionEvent pEvent) {
          GOptionDialog.this.selectedIndex=buttonIndex;
          GOptionDialog.this.dispose();
        }
      });
    }
    Object message=pMessage;
    if (message instanceof Throwable) message=GUtilities.escapeHTML(((Throwable)message).getMessage());
    Component messageComponent=(message instanceof Component)?(Component)message:new GEditorPane("<table cellspacing='0' cellpadding='0' width='500'><tr><td align='left'>"+message.toString()+"</td></tr>",false);
    JPanel imagePanel=GUtilities.getBorderPane(0,10,buttons.length==0?0:18+26,25,null,null,GUtilities.getBoxPaneY(0,Box.createVerticalGlue(),new JLabel(pType.icon),Box.createVerticalGlue()),null,null);
    JPanel buttonsPanel=buttons.length==0?null:GUtilities.getBoxPaneY(0,Box.createVerticalStrut(8),GUtilities.getFlowPane(FlowLayout.CENTER,GUtilities.getBoxPaneX(12,buttons)),Box.createVerticalStrut(4));
    JPanel messagePanel=GUtilities.getBorderPane(null,null,messageComponent,null,buttonsPanel);
    GUtilities.configure(this,GUtilities.getBorderPane(null,imagePanel,messagePanel,null,buttons.length==0?GUtilities.getBoxPaneY(0,Box.createVerticalStrut(12)):null));
  }
  
  public static void showError(Component pParentComponent, Object pMessage) {
    GUtilities.show(new GOptionDialog(pParentComponent,pMessage,Type.ERROR),pParentComponent);
  }
  
  public static void showInfo(Component pParentComponent, Object pMessage) {
    GUtilities.show(new GOptionDialog(pParentComponent,pMessage,Type.INFO),pParentComponent);
  }
  
  public static void showWarning(Component pParentComponent, Object pMessage) {
    GUtilities.show(new GOptionDialog(pParentComponent,pMessage,Type.WARNING),pParentComponent);
  }
  
  public static int showQuestion(Component pParentComponent, Object pMessage, String...pButtons) {
    GOptionDialog dialog=new GOptionDialog(pParentComponent,pMessage,Type.QUESTION,pButtons.length>0?pButtons:new String[]{"Yes","No"});
    GUtilities.show(dialog,pParentComponent);
    return dialog.selectedIndex;
  }
  
}
