package gold.test.graph;

import java.util.*;
import gold.structures.graph.*;
import gold.structures.list.*;
import gold.structures.tuple.*;
import gold.util.*;
import static gold.util.GCollections.*;
import static java.lang.Math.*;

/**
 * <pre>
 * *******************************************************************
 * 3652 - Lazy Jumping Frog
 * http://acm.uva.es/archive/nuevoportal/data/problem.php?p=3652
 * *******************************************************************
 * dijkstraFibonacciHeap
 * Total execution time: 53722 milliseconds.
 * ------------------------------------------------------------------
 * dijkstraBinomialHeap
 * Total execution time: 51781 milliseconds.
 * ------------------------------------------------------------------
 * dijkstraBinaryHeap
 * Total execution time: 59768 milliseconds.
 * ------------------------------------------------------------------
 * bucketShortestPath
 * Total execution time: 47956 milliseconds.
 * *******************************************************************
 * </pre>
 */

@SuppressWarnings("unchecked")
public class TestFrogs {
  private static int C,R;
  private static boolean[][] water;
  public static void main(String[] args) {
    long initialTime=System.currentTimeMillis();
    Scanner sc=new Scanner(TestFrogs.class.getResourceAsStream("frog.in"));
    while (true) {
      C=sc.nextInt();
      R=sc.nextInt();
      if (C==0&&R==0) break;
      water=new boolean[C][R];
      int Cf=sc.nextInt(),Rf=sc.nextInt(),Ct=sc.nextInt(),Rt=sc.nextInt();
      for (int w=0,W=sc.nextInt(); w<W; w++) {
        int C1=sc.nextInt(),R1=sc.nextInt(),C2=sc.nextInt(),R2=sc.nextInt();
        for (ITuple<Integer> a:cartesianProduct(integerInterval(C1,C2),integerInterval(R1,R2))) {
          water[a.get(0)-1][a.get(1)-1]=true;
        }
      }
      if (water[Cf-1][Rf-1]||water[Ct-1][Rt-1]) {
        System.out.println("impossible");
      }
      else {
        int frog=(Cf-1)+(Rf-1)*C;
        int toad=(Ct-1)+(Rt-1)*C;
        IList<Integer> vertices=new GArrayList<Integer>(1024);
        for (int i=0; i<C; i++) {
          for (int j=0; j<R; j++) {
            if (!water[i][j]) vertices.addLast(i+j*C);
          }
        }
        GImplicitUndirectedGraph<Integer,Void> graph=new GImplicitUndirectedGraph<Integer,Void>(vertices);
        graph.setSuccessorsFunction(new GMethod(TestFrogs.class,"successors"));
        graph.setCostFunction(new GMethod(TestFrogs.class,"cost"));
        double result=GGraphs.bucketShortestPath(graph,frog,toad,7);
        System.out.println(result==IEdge.INFINITE_COST?"impossible":round(result));
      }
    }
    System.err.println("Total execution time: "+(System.currentTimeMillis()-initialTime)+" milliseconds.");
  }
  public static Iterable<Integer> successors(Integer a) {
    IList<Integer> list=new GArrayList<Integer>(8);
    int c=a%C,r=a/C;
    for (int i=c-2; i<=c+2; i++) {
      for (int j=r-2; j<=r+2; j++) {
        if (i>=0&&i<C&&j>=0&&j<R&&!(i==c&&j==r)&&!water[i][j]) list.addLast(i+j*C);
      }
    }
    return list;
  }
  public static double cost(Integer a, Integer b) {
    int c1=a%C,r1=a/C,c2=b%C,r2=b/C;
    int dc=abs(c2-c1),dr=abs(r2-r1);
    if (dc>2||dr>2||water[c1][r1]||water[c2][r2]) {
      return IEdge.INFINITE_COST;
    }
    else if (dc==0&&dr==0) {
      return 0;
    }
    else if (dc==2&&dr==2) {
      return 7;
    }
    else {
      return dc*dc+dr*dr+1;
    }
  }
}
