package gold.util;

import java.io.*;
import java.math.*;
import java.util.*;
import java.util.regex.*;
import gold.structures.bag.*;
import gold.structures.collection.*;
import gold.structures.list.*;
import gold.structures.map.*;
import gold.structures.set.*;
import gold.structures.tuple.*;

@SuppressWarnings("unchecked")
public final class GCollections {
  
  private GCollections() {
  }
  
  /*
   * Empty collections
   */
  
  public static <E> ICollection<E> emptyCollection() {
    return emptySet();
  }
  
  public static <E> IList<E> emptyList() {
    return new GArrayList<E>(0);
  }
  
  public static <E> ISet<E> emptySet() {
    return new GLinkedHashTableSet<E>(0);
  }
  
  public static <E> IBag<E> emptyBag() {
    return new GLinkedHashTableBag<E>(0);
  }
  
  public static <K,V> IMap<K,V> emptyMap() {
    return new GHashTableMap<K,V>(0);
  }
  
  /*
   * Universe collections
   */
  
  public static <E> ISet<E> universeSet() {
    ISet<E> set=emptySet();
    set.complement();
    return set;
  }
  
  /*
   * Unmodifiable collections
   */
  
  public static ICollection<?> unmodifiableCollection(Object pObject) {
    if (pObject==null) {
      return GCollections.unmodifiableCollection(Collections.emptySet());
    }
    else if (pObject instanceof Iterable) {
      return GCollections.unmodifiableCollection((Iterable<?>)pObject);
    }
    else if (pObject instanceof Enumeration) {
      return GCollections.unmodifiableCollection(Collections.list((Enumeration<?>)pObject));
    }
    else if (pObject instanceof CharSequence) {
      final CharSequence sequence=(CharSequence)pObject;
      return GCollections.unmodifiableCollection(new Iterable<Character>() {
        @Override
        public Iterator<Character> iterator() {
          return new GAbstractIterator<Character>() {
            private int index=0;
            @Override
            public boolean hasNext() {
              return index<sequence.length();
            }
            @Override
            public Character next() {
              return sequence.charAt(index++);
            }
          };
        }
      });
    }
    else if (pObject instanceof long[]) {
      final long[] array=(long[])pObject;
      return GCollections.unmodifiableCollection(new Iterable<Long>() {
        @Override
        public Iterator<Long> iterator() {
          return new GAbstractIterator<Long>() {
            private int index=0;
            @Override
            public boolean hasNext() {
              return index<array.length;
            }
            @Override
            public Long next() {
              return array[index++];
            }
          };
        }
      });
    }
    else if (pObject instanceof int[]) {
      final int[] array=(int[])pObject;
      return GCollections.unmodifiableCollection(new Iterable<Integer>() {
        @Override
        public Iterator<Integer> iterator() {
          return new GAbstractIterator<Integer>() {
            private int index=0;
            @Override
            public boolean hasNext() {
              return index<array.length;
            }
            @Override
            public Integer next() {
              return array[index++];
            }
          };
        }
      });
    }
    else if (pObject instanceof short[]) {
      final short[] array=(short[])pObject;
      return GCollections.unmodifiableCollection(new Iterable<Short>() {
        @Override
        public Iterator<Short> iterator() {
          return new GAbstractIterator<Short>() {
            private int index=0;
            @Override
            public boolean hasNext() {
              return index<array.length;
            }
            @Override
            public Short next() {
              return array[index++];
            }
          };
        }
      });
    }
    else if (pObject instanceof byte[]) {
      final byte[] array=(byte[])pObject;
      return GCollections.unmodifiableCollection(new Iterable<Byte>() {
        @Override
        public Iterator<Byte> iterator() {
          return new GAbstractIterator<Byte>() {
            private int index=0;
            @Override
            public boolean hasNext() {
              return index<array.length;
            }
            @Override
            public Byte next() {
              return array[index++];
            }
          };
        }
      });
    }
    else if (pObject instanceof double[]) {
      final double[] array=(double[])pObject;
      return GCollections.unmodifiableCollection(new Iterable<Double>() {
        @Override
        public Iterator<Double> iterator() {
          return new GAbstractIterator<Double>() {
            private int index=0;
            @Override
            public boolean hasNext() {
              return index<array.length;
            }
            @Override
            public Double next() {
              return array[index++];
            }
          };
        }
      });
    }
    else if (pObject instanceof float[]) {
      final float[] array=(float[])pObject;
      return GCollections.unmodifiableCollection(new Iterable<Float>() {
        @Override
        public Iterator<Float> iterator() {
          return new GAbstractIterator<Float>() {
            private int index=0;
            @Override
            public boolean hasNext() {
              return index<array.length;
            }
            @Override
            public Float next() {
              return array[index++];
            }
          };
        }
      });
    }
    else if (pObject instanceof char[]) {
      final char[] array=(char[])pObject;
      return GCollections.unmodifiableCollection(new Iterable<Character>() {
        @Override
        public Iterator<Character> iterator() {
          return new GAbstractIterator<Character>() {
            private int index=0;
            @Override
            public boolean hasNext() {
              return index<array.length;
            }
            @Override
            public Character next() {
              return array[index++];
            }
          };
        }
      });
    }
    else if (pObject instanceof boolean[]) {
      final boolean[] array=(boolean[])pObject;
      return GCollections.unmodifiableCollection(new Iterable<Boolean>() {
        @Override
        public Iterator<Boolean> iterator() {
          return new GAbstractIterator<Boolean>() {
            private int index=0;
            @Override
            public boolean hasNext() {
              return index<array.length;
            }
            @Override
            public Boolean next() {
              return array[index++];
            }
          };
        }
      });
    }
    else if (pObject instanceof Object[]) {
      final Object[] array=(Object[])pObject;
      return GCollections.unmodifiableCollection(new Iterable<Object>() {
        @Override
        public Iterator<Object> iterator() {
          return new GAbstractIterator<Object>() {
            private int index=0;
            @Override
            public boolean hasNext() {
              return index<array.length;
            }
            @Override
            public Object next() {
              return array[index++];
            }
          };
        }
      });
    }
    else {
      return GCollections.singletonCollection(pObject);
    }
  }
  
  public static <E> ICollection<E> unmodifiableCollection(Iterable<? extends E> pIterable) {
    if (pIterable instanceof Collection) {
      return new GAdaptorCollection<E>(Collections.unmodifiableCollection((Collection<E>)pIterable));
    }
    else if (pIterable instanceof ICollection) {
      return GCollections.unmodifiableCollection(((ICollection<E>)pIterable).toJavaView());
    }
    else {
      final Iterable<? extends E> iterable=pIterable;
      return new GAdaptorCollection<E>(Collections.unmodifiableCollection(new AbstractSerializableCollection<E>() {
        private static final long serialVersionUID=-1016543775956109072L;
        @Override
        public Iterator<E> iterator() {
          return new GAbstractIterator<E>() {
            private final Iterator<? extends E> iteratorE=iterable.iterator();
            @Override
            public boolean hasNext() {
              return iteratorE.hasNext();
            }
            @Override
            public E next() {
              return iteratorE.next();
            }
          };
        }
        @Override
        public int size() {
          return GCollections.sizeOf(iterable);
        }
      }));
    }
  }
  
  public static <E> IList<E> unmodifiableList(Iterable<? extends E> pIterable) {
    if (pIterable instanceof List) {
      return new GAdaptorList<E>(Collections.unmodifiableList((List<E>)pIterable));
    }
    else if (pIterable instanceof IList) {
      return GCollections.unmodifiableList(((IList<E>)pIterable).toJavaView());
    }
    else {
      final Collection<E> collection=GCollections.unmodifiableCollection(pIterable).toJavaView();
      return new GAdaptorList<E>(Collections.unmodifiableList(new AbstractSerializableList<E>() {
        private static final long serialVersionUID=-1225817957821756046L;
        @Override
        public E get(int pIndex) {
          if (pIndex<0||pIndex>=collection.size()) throw new IndexOutOfBoundsException();
          Iterator<E> iteratorE=collection.iterator();
          for (int indexE=0; indexE<pIndex; indexE++) {
            iteratorE.next();
          }
          return iteratorE.next();
        }
        @Override
        public Iterator<E> iterator() {
          return collection.iterator();
        }
        @Override
        public int size() {
          return collection.size();
        }
      }));
    }
  }
  
  public static <E> ISet<E> unmodifiableSet(Iterable<? extends E> pIterable) {
    if (pIterable instanceof Set) {
      return new GAdaptorSet<E>(Collections.unmodifiableSet((Set<E>)pIterable));
    }
    else if (pIterable instanceof ISet) {
      ISet<E> set=GCollections.unmodifiableSet(((ISet<E>)pIterable).toJavaView());
      if (!GCollections.isFinite(pIterable)) set.complement();
      return set;
    }
    else {
      final Iterable<? extends E> iterable=pIterable;
      return new GAdaptorSet<E>(Collections.unmodifiableSet(new AbstractSerializableSet<E>() {
        private static final long serialVersionUID=-493413343034926219L;
        @Override
        public Iterator<E> iterator() {
          return new GAbstractIterator<E>() {
            private final Iterator<? extends E> it=iterable.iterator();
            @Override
            public boolean hasNext() {
              return it.hasNext();
            }
            @Override
            public E next() {
              return it.next();
            }
          };
        }
        @Override
        public int size() {
          return GCollections.sizeOf(iterable);
        }
      }));
    }
  }
  
  public static <E> IBag<E> unmodifiableBag(Map<? extends E,Integer> pMap) {
    return new GAdaptorBag<E>(Collections.unmodifiableMap(pMap));
  }
  
  public static <E> IBag<E> unmodifiableBag(Iterable<? extends E> pIterable) {
    if (pIterable instanceof Collection) {
      return GCollections.unmodifiableBag(new GHashTableBag<E>((Collection<E>)pIterable).toJavaMap());
    }
    else if (pIterable instanceof IBag) {
      return GCollections.unmodifiableBag(((IBag<E>)pIterable).toJavaMap());
    }
    else if (pIterable instanceof ICollection) {
      return GCollections.unmodifiableBag(((ICollection<E>)pIterable).toJavaView());
    }
    else {
      return GCollections.unmodifiableBag(GCollections.unmodifiableCollection(pIterable));
    }
  }
  
  public static <K,V> IMap<K,V> unmodifiableMap(Map<? extends K,? extends V> pMap) {
    return new GAdaptorMap<K,V>(Collections.unmodifiableMap(pMap));
  }
  
  public static <K,V> IMap<K,V> unmodifiableMap(IMap<? extends K,? extends V> pMap) {
    return GCollections.unmodifiableMap(pMap.toJavaView());
  }
  
  /*
   * Conversion from GOLD collections to Java collections
   */
  
  public static Collection<?> asJavaCollection(Object pObject) {
    if (pObject instanceof Iterable) {
      return GCollections.asJavaCollection((Iterable<?>)pObject);
    }
    else {
      return GCollections.unmodifiableCollection(pObject).toJavaView();
    }
  }
  
  public static <E> Collection<E> asJavaCollection(Iterable<? extends E> pIterable) {
    if (pIterable instanceof Collection) {
      return (Collection<E>)pIterable;
    }
    else if (pIterable instanceof ICollection) {
      return ((ICollection<E>)pIterable).toJavaView();
    }
    else {
      return GCollections.unmodifiableCollection(pIterable).toJavaView();
    }
  }
  
  /*
   * Conversion from Java collections to GOLD collections
   */
  
  public static ICollection<?> asGoldCollection(Object pObject) {
    if (pObject instanceof Iterable) {
      return GCollections.asGoldCollection((Iterable<?>)pObject);
    }
    else {
      return GCollections.unmodifiableCollection(pObject);
    }
  }
  
  public static <E> ICollection<E> asGoldCollection(Iterable<? extends E> pIterable) {
    if (pIterable instanceof Collection) {
      return new GAdaptorCollection<E>((Collection<E>)pIterable);
    }
    else if (pIterable instanceof ICollection) {
      return (ICollection<E>)pIterable;
    }
    else {
      return GCollections.unmodifiableCollection(pIterable);
    }
  }
  
  /*
   * Conversion from array to unmodifiable collection
   */
  
  public static <E> ICollection<E> asCollection(E...pElements) {
    return new GAdaptorCollection<E>(Collections.unmodifiableCollection(Arrays.<E>asList(pElements)));
  }
  
  public static <E> IList<E> asList(E...pElements) {
    return new GAdaptorList<E>(Collections.unmodifiableList(Arrays.<E>asList(pElements)));
  }
  
  public static <E> ISet<E> asSet(E...pElements) {
    return new GAdaptorSet<E>(Collections.unmodifiableSet(new LinkedHashSet<E>(Arrays.<E>asList(pElements))));
  }
  
  public static <E> IBag<E> asBag(E...pElements) {
    return GCollections.unmodifiableBag(Arrays.asList(pElements));
  }
  
  public static <E> ITuple<E> asTuple(E...pElements) {
    return new GTuple<E>(Collections.unmodifiableList(Arrays.<E>asList(pElements)));
  }
  
  /*
   * Singleton collections
   */
  
  public static <E> ICollection<E> singletonCollection(E pElement) {
    return GCollections.asCollection(pElement);
  }
  
  public static <E> IList<E> singletonList(E pElement) {
    return GCollections.asList(pElement);
  }
  
  public static <E> ISet<E> singletonSet(E pElement) {
    return GCollections.asSet(pElement);
  }
  
  public static <E> IBag<E> singletonBag(E pElement) {
    return GCollections.asBag(pElement);
  }
  
  /*
   * Special collections
   */
  
  public static <E> IList<E> nCopies(int pCopies, E pElement) {
    return GCollections.unmodifiableList(Collections.nCopies(Math.max(0,pCopies),pElement));
  }
  
  /*
   * Intervals
   */
  
  public static ISet<Integer> integerInterval(int pMinimum, int pMaximum) {
    return integerInterval(pMinimum,pMaximum,1);
  }
  
  public static ISet<Integer> integerInterval(int pMinimum, int pMaximum, int pStep) {
    if (pStep==0) throw new IllegalArgumentException("The step cannot be zero.");
    final int minimum=pMinimum,maximum=pMaximum,step=pStep;
    return GCollections.unmodifiableSet(new AbstractSerializableList<Integer>() {
      private static final long serialVersionUID=-2572133510115736968L;
      @Override
      public Integer get(int pIndex) {
        return minimum+pIndex*step;
      }
      @Override
      public int size() {
        int sgn=step<0?-1:1,num=sgn*(maximum-minimum)+1,den=sgn*step;
        return num>=0?(num+den-1)/den:0;
      }
    });
  }
  
  public static ISet<Double> realInterval(double pMinimum, double pMaximum, double pStep) {
    if (Math.abs(pStep)<1E-10) throw new IllegalArgumentException("The step cannot be zero.");
    final double minimum=pMinimum,maximum=pMaximum,step=pStep;
    return GCollections.unmodifiableSet(new AbstractSerializableList<Double>() {
      private static final long serialVersionUID=8600474355495155494L;
      @Override
      public Double get(int pIndex) {
        return minimum+pIndex*step;
      }
      @Override
      public int size() {
        double sgn=step<0?-1:1,num=sgn*(maximum-minimum)+1E-10,den=sgn*step;
        return num>=0?(int)(Math.ceil(num/den)):0;
      }
    });
  }
  
  public static ISet<Character> characterInterval(char pMinimum, char pMaximum) {
    final int minimum=pMinimum&0xFFFF,maximum=pMaximum&0xFFFF;
    return GCollections.unmodifiableSet(new AbstractSerializableList<Character>() {
      private static final long serialVersionUID=3145217114973878254L;
      @Override
      public Character get(int pIndex) {
        return (char)(minimum+pIndex);
      }
      @Override
      public int size() {
        return maximum>=minimum?maximum-minimum+1:0;
      }
    });
  }
  
  public static ISet<Character> characterInterval(String pRegularExpression) {
    Pattern pattern=Pattern.compile(pRegularExpression);
    IList<Character> result=new GArrayList<Character>(256);
    for (int index=0; index<65536; index++) {
      if (pattern.matcher(String.valueOf((char)index)).matches()) result.addLast((char)index);
    }
    return GCollections.unmodifiableSet(result);
  }
  
  public static IPair<Integer> endPointsOfIntegerInterval(Iterable<?> pInterval) {
    Integer last=null,min=null;
    for (Object object:pInterval) {
      Integer current=(object instanceof Integer)?(Integer)object:null;
      if (current==null||(last!=null&&current.intValue()!=last.intValue()+1)) return null;
      if (min==null) min=current;
      last=current;
    }
    return last!=null&&min!=null?new GPair<Integer>(min,last):new GPair<Integer>(0,-1);
  }
  
  /*
   * Sorting
   */
  
  public static <E extends Comparable<? super E>> IList<E> sort(IList<E> pList) {
    Collections.sort(pList.toJavaView());
    return pList;
  }
  
  public static <E> IList<E> sort(IList<E> pList, Comparator<? super E> pComparator) {
    Collections.sort(pList.toJavaView(),pComparator);
    return pList;
  }
  
  public static <E> IList<E> sort(IList<E> pList, IMethod pComparator) {
    return GCollections.sort(pList,GComparators.<E>comparator(pComparator));
  }
  
  /*
   * Searching
   */
  
  public static <E> int binarySearch(IList<? extends Comparable<? super E>> pList, E pKey) {
    return Collections.binarySearch(pList.toJavaView(),pKey);
  }
  
  public static <E> int binarySearch(IList<? extends E> pList, E pKey, Comparator<? super E> pComparator) {
    return Collections.binarySearch(pList.toJavaView(),pKey,pComparator);
  }
  
  public static <E> int binarySearch(IList<? extends E> pList, E pKey, IMethod pComparator) {
    return GCollections.binarySearch(pList,pKey,GComparators.<E>comparator(pComparator));
  }
  
  /*
   * Collection cardinality
   */
  
  public static int sizeOf(Iterable<?> pIterable) {
    if (pIterable instanceof Collection) {
      return ((Collection<?>)pIterable).size();
    }
    else if (pIterable instanceof ICollection) {
      return ((ICollection<?>)pIterable).size();
    }
    else {
      int count=0;
      for (Iterator<?> it=pIterable.iterator(); it.hasNext(); count++) {
        it.next();
      }
      return count;
    }
  }
  
  public static boolean isEmpty(Iterable<?> pIterable) {
    if (pIterable instanceof Collection) {
      return ((Collection<?>)pIterable).isEmpty();
    }
    else if (pIterable instanceof ICollection) {
      return ((ICollection<?>)pIterable).isEmpty();
    }
    else {
      return !pIterable.iterator().hasNext();
    }
  }
  
  public static boolean isFinite(Iterable<?> pIterable) {
    if (pIterable instanceof ICollection) {
      return ((ICollection<?>)pIterable).isFinite();
    }
    else {
      return true;
    }
  }
  
  /*
   * Collection operations
   */
  
  public static <E,F> Iterable<F> filter(Iterable<E> pIterable, Class<F> pClass) {
    final Iterable<E> iterable=pIterable;
    final Class<F> type=pClass;
    return new AbstractSerializableCollection<F>() {
      private static final long serialVersionUID=5771556728148096072L;
      @Override
      public Iterator<F> iterator() {
        return new GAbstractIterator<F>() {
          private final Iterator<E> it=iterable.iterator();
          private E next=getNext();
          private E getNext() {
            while (it.hasNext()) {
              E e=it.next();
              if (e!=null&&type.isInstance(e)) return e;
            }
            return null;
          }
          @Override
          public boolean hasNext() {
            return next!=null;
          }
          @Override
          public F next() {
            E e=next;
            next=getNext();
            return e!=null?type.cast(e):null;
          }
        };
      }
      @Override
      public int size() {
        int count=0;
        for (E e:iterable) {
          if (e!=null&&type.isInstance(e)) count++;
        }
        return count;
      }
    };
  }
  
  public static <E> ISet<ITuple<E>> cartesianPower(ISet<? extends E> pSet, int pExponent) {
    return GCollections.cartesianProduct(GCollections.<ISet<? extends E>>nCopies(pExponent,pSet));
  }
  
  public static <E> ISet<ITuple<E>> cartesianProduct(ISet<? extends E>...pSets) {
    return GCollections.cartesianProduct(GCollections.asList(pSets));
  }
  
  public static <E> ISet<ITuple<E>> cartesianProduct(IList<ISet<? extends E>> pSets) {
    final int length=pSets.size();
    if (length==0) {
      return GCollections.<ITuple<E>>emptySet();
    }
    else if (length==1) {
      final IList<E> list=new GArrayList<E>(pSets.pick());
      return unmodifiableSet(new AbstractSerializableList<ITuple<E>>() {
        private static final long serialVersionUID=3170613024141840477L;
        @Override
        public ITuple<E> get(int pIndex) {
          return new GSingleton<E>(list.get(pIndex));
        }
        @Override
        public int size() {
          return list.size();
        }
      });
    }
    else if (length==2) {
      final IList<E> list1=new GArrayList<E>(pSets.getFirst()),list2=new GArrayList<E>(pSets.getLast());
      final int size1=list1.size(),size2=list2.size();
      return unmodifiableSet(new AbstractSerializableList<ITuple<E>>() {
        private static final long serialVersionUID=-4305914402879848594L;
        @Override
        public ITuple<E> get(int pIndex) {
          return new GPair<E>(list1.get(pIndex%size1),list2.get(pIndex/size1));
        }
        @Override
        public int size() {
          return size1*size2;
        }
      });
    }
    else {
      BigInteger count=BigInteger.ONE,limit=BigInteger.valueOf(1<<23);
      for (ISet<? extends E> set:pSets) {
        if (!set.isFinite()) throw new IllegalArgumentException("One of the operands of the cartesian product is an infinite set.");
        count=count.multiply(BigInteger.valueOf(set.size()));
      }
      if (count.compareTo(limit)>0) throw new IllegalArgumentException("The cardinality of the cartesian product ("+count+") exceeds the maximum limit ("+limit+").");
      final IList<IList<E>> source=new GArrayList<IList<E>>(pSets.size());
      final int cardinality=count.intValue();
      for (ICollection<? extends E> collection:pSets) {
        source.addLast(new GArrayList<E>(collection));
      }
      return GCollections.unmodifiableSet(new AbstractSerializableList<ITuple<E>>() {
        private static final long serialVersionUID=791478222857809678L;
        @Override
        public ITuple<E> get(int pIndex) {
          final int code=pIndex;
          return new GTuple<E>(new AbstractSerializableList<E>() {
            private static final long serialVersionUID=-1146400886606665748L;
            @Override
            public E get(int pIndex) {
              int positionE=code;
              for (int indexE=0; indexE<pIndex; indexE++) {
                positionE/=source.get(indexE).size();
              }
              return source.get(pIndex).get(positionE%source.get(pIndex).size());
            }
            @Override
            public Iterator<E> iterator() {
              return new GAbstractIterator<E>() {
                private int positionE=code,indexE=0;
                @Override
                public boolean hasNext() {
                  return indexE<length;
                }
                @Override
                public E next() {
                  IList<E> current=source.get(indexE++);
                  E result=current.get(positionE%current.size());
                  positionE/=current.size();
                  return result;
                }
              };
            }
            @Override
            public int size() {
              return length;
            }
          });
        }
        @Override
        public int size() {
          return cardinality;
        }
      });
    }
  }
  
  public static <E,F> ISet<ICouple<E,F>> binaryCartesianProduct(ISet<? extends E> pSet1, ISet<? extends F> pSet2) {
    final ISet<ITuple<Object>> product=GCollections.cartesianProduct(pSet1,pSet2);
    return new GAdaptorSet<ICouple<E,F>>(new AbstractSerializableSet<ICouple<E,F>>() {
      private static final long serialVersionUID=8050875148996244496L;
      @Override
      public Iterator<ICouple<E,F>> iterator() {
        return new GAbstractIterator<ICouple<E,F>>() {
          private final Iterator<ITuple<Object>> iteratorE=product.iterator();
          @Override
          public boolean hasNext() {
            return iteratorE.hasNext();
          }
          @Override
          public ICouple<E,F> next() {
            ITuple<Object> pair=iteratorE.next();
            return new GCouple<E,F>((E)(pair.get(0)),(F)(pair.get(1)));
          }
        };
      }
      @Override
      public int size() {
        return product.size();
      }
    });
  }
  
  public static <E> ISet<ISet<E>> singletons(ICollection<? extends E> pCollection) {
    final Collection<E> collection=Collections.<E>unmodifiableCollection(pCollection.toJavaView());
    return new GAdaptorSet<ISet<E>>(new AbstractSerializableSet<ISet<E>>() {
      private static final long serialVersionUID=692849177218019562L;
      @Override
      public Iterator<ISet<E>> iterator() {
        return new GAbstractIterator<ISet<E>>() {
          private final Iterator<E> iteratorE=collection.iterator();
          @Override
          public boolean hasNext() {
            return iteratorE.hasNext();
          }
          @Override
          public ISet<E> next() {
            return GCollections.singletonSet(iteratorE.next());
          }
        };
      }
      @Override
      public int size() {
        return collection.size();
      }
    });
  }
  
  public static <E> ISet<ISet<E>> powerSet(ISet<? extends E> pSet) {
    int limit=23;
    if (!pSet.isFinite()) throw new IllegalArgumentException("The operand of the power set is an infinite set.");
    if (pSet.size()>limit) throw new IllegalArgumentException("The cardinality of the power set (2^"+pSet.size()+") exceeds the maximum limit (2^"+limit+").");
    final IList<E> source=new GArrayList<E>(pSet);
    final int cardinality=1<<source.size();
    return new GAdaptorSet<ISet<E>>(new AbstractSerializableSet<ISet<E>>() {
      private static final long serialVersionUID=-7406098415536304852L;
      @Override
      public Iterator<ISet<E>> iterator() {
        return new GAbstractIterator<ISet<E>>() {
          private int indexS=0;
          @Override
          public boolean hasNext() {
            return indexS<cardinality;
          }
          @Override
          public ISet<E> next() {
            final int code=indexS++;
            return new GAdaptorSet<E>(new AbstractSerializableSet<E>() {
              private static final long serialVersionUID=1026951784401133005L;
              @Override
              public Iterator<E> iterator() {
                return new GAbstractIterator<E>() {
                  private int bitsetE=code,indexE=0;
                  @Override
                  public boolean hasNext() {
                    return bitsetE!=0;
                  }
                  @Override
                  public E next() {
                    int p=Integer.numberOfTrailingZeros(bitsetE)+1;
                    bitsetE>>>=p;
                    indexE+=p;
                    return source.get(indexE-1);
                  }
                };
              }
              @Override
              public int size() {
                return Integer.bitCount(code);
              }
            });
          }
        };
      }
      @Override
      public int size() {
        return cardinality;
      }
    });
  }
  
  /*
   * Set operations
   */
  
  public static <E> ISet<E> union(ISet<? extends E> pSet1, ISet<? extends E> pSet2) {
    ISet<E> result=new GHashTableSet<E>(pSet1);
    result.union(pSet2);
    return result;
  }
  
  public static <E> ISet<E> intersection(ISet<? extends E> pSet1, ISet<? extends E> pSet2) {
    ISet<E> result=new GHashTableSet<E>(pSet1);
    result.intersection(pSet2);
    return result;
  }
  
  public static <E> ISet<E> difference(ISet<? extends E> pSet1, ISet<? extends E> pSet2) {
    ISet<E> result=new GHashTableSet<E>(pSet1);
    result.difference(pSet2);
    return result;
  }
  
  public static <E> ISet<E> symmetricDifference(ISet<? extends E> pSet1, ISet<? extends E> pSet2) {
    ISet<E> result=new GHashTableSet<E>(pSet1);
    result.symmetricDifference(pSet2);
    return result;
  }
  
  public static <E> ISet<E> complement(ISet<? extends E> pSet) {
    ISet<E> result=new GHashTableSet<E>(pSet);
    result.complement();
    return result;
  }
  
  /*
   * Auxiliary classes
   */
  
  public static abstract class AbstractSerializableCollection<E> extends AbstractCollection<E> implements Serializable {
    private static final long serialVersionUID=9125028337230516648L;
  }
  
  public static abstract class AbstractSerializableList<E> extends AbstractList<E> implements Serializable {
    private static final long serialVersionUID=933230867421239010L;
  }
  
  public static abstract class AbstractSerializableSet<E> extends AbstractSet<E> implements Serializable {
    private static final long serialVersionUID=8935062244037654251L;
  }
  
  public static abstract class AbstractSerializableMap<K,V> extends AbstractMap<K,V> implements Serializable {
    private static final long serialVersionUID=8735189345730878929L;
  }
  
}
