package gold.util;

import java.util.*;

public final class GComparators {
  
  private GComparators() {
  }
  
  public static <T> Comparator<T> comparator(Class<T> pClass, IMethod pMethod) {
    return GComparators.<T>comparator(pMethod);
  }
  
  public static <T> Comparator<T> comparator(IMethod pMethod) {
    final IMethod method=pMethod;
    return new Comparator<T>() {
      @Override
      public int compare(T pObject1, T pObject2) {
        return GReflection.$int(method.invoke(pObject1,pObject2));
      }
    };
  }
  
  public static Comparator<String> shortlexStringComparator() {
    return new Comparator<String>() {
      @Override
      public int compare(String pString1, String pString2) {
        int c=pString1.length()-pString2.length();
        return c!=0?c:pString1.compareTo(pString2);
      }
    };
  }
  
  @SuppressWarnings("unchecked")
  public static <E> Comparator<E> genericComparator() {
    return new Comparator<E>() {
      @Override
      public int compare(E pObject1, E pObject2) {
        if (pObject1==null&&pObject2==null) {
          return 0;
        }
        else if (pObject1==null&&pObject2!=null) {
          return -1;
        }
        else if (pObject1!=null&&pObject2==null) {
          return 1;
        }
        else if (pObject1!=null&&pObject2!=null) {
          if (pObject1.equals(pObject2)) {
            return 0;
          }
          else if (pObject1.getClass()!=pObject2.getClass()) {
            return pObject1.getClass().getCanonicalName().compareTo(pObject2.getClass().getCanonicalName());
          }
          else if ((pObject1 instanceof Iterable)&&(pObject2 instanceof Iterable)) {
            Iterable<?> iterable1=(Iterable<?>)pObject1,iterable2=(Iterable<?>)pObject2;
            int size1=GCollections.sizeOf(iterable1),size2=GCollections.sizeOf(iterable2);
            if (size1!=size2) return size1<size2?-1:1;
            Comparator<Object> comparator=GComparators.genericComparator();
            for (Iterator<?> it1=iterable1.iterator(),it2=iterable2.iterator(); it1.hasNext()&&it2.hasNext();) {
              int comparison=comparator.compare(it1.next(),it2.next());
              if (comparison!=0) return comparison;
            }
            return 0;
          }
          else {
            return ((Comparable<? super E>)pObject1).compareTo(pObject2);
          }
        }
        else { // Caso imposible
          return 0;
        }
      }
    };
  }
  
}
