package gold.util;

import gold.structures.graph.*;
import gold.structures.heap.*;
import gold.structures.map.*;
import gold.structures.multimap.*;
import gold.structures.queue.*;
import gold.structures.set.*;

public final class GGraphs {
  
  private GGraphs() {
  }
  
  public static <V,E> double edmondsKarp(IGraph<V,E> G, V s, V t) {
    int n=G.getVertexCount();
    IMap<V,Integer> indices=new GHashTableMap<V,Integer>(G.getVertexCount());
    for (V v:G.getVertices()) {
      indices.put(v,indices.size());
    }
    double d=0.0,flow[][]=new double[n][n],capacity[][]=G.getCostMatrix();
    for (int i=0; i<n; i++) {
      for (int j=0; j<n; j++) {
        if (capacity[i][j]==IEdge.INFINITE_COST) capacity[i][j]=0;
      }
    }
    IMap<V,Double> w=new GHashTableMap<V,Double>();
    IMultiMap<V,V> neighbors=new GHashTableMultiMap<V,V>();
    for (IEdge<V,E> edge:G.getEdges()) {
      neighbors.add(edge.getTarget(),edge.getSource());
      neighbors.add(edge.getSource(),edge.getTarget());
    }
    loop:while (true) {
      IMap<V,V> p=new GHashTableMap<V,V>();
      IQueue<V> q=new GArrayQueue<V>();
      w.put(s,IEdge.INFINITE_COST);
      q.enqueue(s);
      while (!q.isEmpty()) {
        V u=q.dequeue();
        for (V v:neighbors.get(u)) {
          double z=capacity[indices.get(u)][indices.get(v)]-flow[indices.get(u)][indices.get(v)];
          if (z<=0||p.containsKey(v)) continue;
          double x=Math.min(w.get(u),z);
          p.put(v,u);
          w.put(v,x);
          if (v==t) {
            d+=x;
            for (v=t; v!=s; v=u) {
              u=p.get(v);
              flow[indices.get(u)][indices.get(v)]+=x;
              flow[indices.get(v)][indices.get(u)]-=x;
            }
            continue loop;
          }
          q.enqueue(v);
        }
      }
      break;
    }
    return d;
  }
  
  public static <V,E> IMap<V,Double> dijkstra(IGraph<V,E> G, V s) {
    return dijkstraFibonacciHeap(G,s);
  }
  
  public static <V,E> double dijkstra(IGraph<V,E> G, V s, V t) {
    return dijkstraFibonacciHeap(G,s,t);
  }
  
  public static <V,E> IMap<V,Double> dijkstraFibonacciHeap(IGraph<V,E> G, V s) {
    return dijkstraFibonacciHeapAuxiliary(G,s,null);
  }
  
  public static <V,E> double dijkstraFibonacciHeap(IGraph<V,E> G, V s, V t) {
    Double cost=dijkstraFibonacciHeapAuxiliary(G,s,t).get(t);
    return cost!=null?cost:IEdge.INFINITE_COST;
  }
  
  private static <V,E> IMap<V,Double> dijkstraFibonacciHeapAuxiliary(IGraph<V,E> G, V s, V t) {
    IMap<V,Double> d=new GHashTableMap<V,Double>();
    d.put(s,0.0);
    IHeap<V> Q=new GFibonacciHeap<V>(d);
    while (!Q.isEmpty()) {
      V u=Q.extractMinimum();
      if (t!=null&&u.equals(t)) return d;
      double costU=d.get(u);
      for (V v:G.getSuccessors(u)) {
        Double costV=d.get(v);
        double cost=costU+G.getCost(u,v);
        if (costV==null) {
          d.put(v,cost);
          Q.insert(v,cost);
        }
        else if (cost<costV) {
          d.put(v,cost);
          Q.decreaseKey(v,cost);
        }
      }
    }
    return d;
  }
  
  public static <V,E> IMap<V,Double> dijkstraBinomialHeap(IGraph<V,E> G, V s) {
    return dijkstraBinomialHeapAuxiliary(G,s,null);
  }
  
  public static <V,E> double dijkstraBinomialHeap(IGraph<V,E> G, V s, V t) {
    Double cost=dijkstraBinomialHeapAuxiliary(G,s,t).get(t);
    return cost!=null?cost:IEdge.INFINITE_COST;
  }
  
  private static <V,E> IMap<V,Double> dijkstraBinomialHeapAuxiliary(IGraph<V,E> G, V s, V t) {
    IMap<V,Double> d=new GHashTableMap<V,Double>();
    d.put(s,0.0);
    IHeap<V> Q=new GBinomialHeap<V>(d);
    while (!Q.isEmpty()) {
      V u=Q.extractMinimum();
      if (t!=null&&u.equals(t)) return d;
      double costU=d.get(u);
      for (V v:G.getSuccessors(u)) {
        Double costV=d.get(v);
        double cost=costU+G.getCost(u,v);
        if (costV==null) {
          d.put(v,cost);
          Q.insert(v,cost);
        }
        else if (cost<costV) {
          d.put(v,cost);
          Q.decreaseKey(v,cost);
        }
      }
    }
    return d;
  }
  
  /**
   * <code>
   * private static <V,E> IMap<V,Double> dijkstraBinomialHeapAuxiliary(IGraph<V,E> G, V s, V t) {
   *   IMap<V,Double> d=new GHashTableMap<V,Double>(G.getVertexCount());
   *   for (V v:G.getVertices()) {
   *     d.put(v,IEdge.INFINITE_COST);
   *   }
   *   d.put(s,0.0);
   *   IHeap<V> Q=new GBinomialHeap<V>(d);
   *   while (!Q.isEmpty()&&Q.minimumKey()<IEdge.INFINITE_COST) {
   *     V u=Q.extractMinimum();
   *     if (t!=null&&u.equals(t)) return d;
   *     double costU=d.get(u);
   *     for (V v:G.getSuccessors(u)) {
   *       if (Q.contains(v)) {
   *         double costV=d.get(v),cost=costU+G.getCost(u,v);
   *         if (cost<costV) {
   *           d.put(v,cost);
   *           Q.decreaseKey(v,cost);
   *         }
   *       }
   *     }
   *   }
   *   return d;
   * }
   * </code>
   */
  
  public static <V,E> IMap<V,Double> dijkstraBinaryHeap(IGraph<V,E> G, V s) {
    return dijkstraBinaryHeapAuxiliary(G,s,null);
  }
  
  public static <V,E> double dijkstraBinaryHeap(IGraph<V,E> G, V s, V t) {
    Double cost=dijkstraBinaryHeapAuxiliary(G,s,t).get(t);
    return cost!=null?cost:IEdge.INFINITE_COST;
  }
  
  private static <V,E> IMap<V,Double> dijkstraBinaryHeapAuxiliary(IGraph<V,E> G, V s, V t) {
    IMap<V,Double> d=new GHashTableMap<V,Double>();
    ISet<V> S=new GHashTableSet<V>();
    IHeap<V> Q=new GBinaryHeap<V>();
    d.put(s,0.0);
    Q.insert(s,0);
    while (!Q.isEmpty()) {
      V u=Q.extractMinimum();
      if (!S.contains(u)) {
        S.add(u);
        if (t!=null&&u.equals(t)) return d;
        double costU=d.get(u);
        for (V v:G.getSuccessors(u)) {
          Double costV=d.get(v);
          double cost=costU+G.getCost(u,v);
          if (costV==null||cost<costV) {
            d.put(v,cost);
            Q.insert(v,cost);
          }
        }
      }
    }
    return d;
  }
  
  public static <V,E> IMap<V,Double> bucketShortestPath(IGraph<V,E> G, V s, int L) {
    return bucketShortestPathAuxiliary(G,s,null,L);
  }
  
  public static <V,E> double bucketShortestPath(IGraph<V,E> G, V s, V t, int L) {
    Double cost=bucketShortestPathAuxiliary(G,s,t,L).get(t);
    return cost!=null?cost:IEdge.INFINITE_COST;
  }
  
  private static <V,E> IMap<V,Double> bucketShortestPathAuxiliary(IGraph<V,E> G, V s, V t, int L) {
    @SuppressWarnings("unchecked")
    IQueue<V>[] buckets=new IQueue[L+1];
    for (int i=0; i<=L; i++) {
      buckets[i]=new GArrayQueue<V>();
    }
    IMap<V,Double> d=new GHashTableMap<V,Double>();
    buckets[0].enqueue(s);
    for (int cost=0; true; cost++) {
      IQueue<V> first=buckets[0];
      while (!first.isEmpty()) {
        V u=first.dequeue();
        if (!d.containsKey(u)) {
          d.put(u,(double)cost);
          if (t!=null&&u.equals(t)) return d;
          for (V v:G.getSuccessors(u)) {
            buckets[(int)(G.getCost(u,v)+0.5)].enqueue(v);
          }
        }
      }
      boolean go=false;
      for (int i=0; i<L; i++) {
        buckets[i]=buckets[i+1];
        if (!go&&!buckets[i].isEmpty()) go=true;
      }
      buckets[L]=first;
      if (!go) break;
    }
    return d;
  }
  
  public static void floydWarshall(double[][] m) {
    int n=m.length;
    for (int k=0; k<n; k++) {
      for (int i=0; i<n; i++) {
        for (int j=0; j<n; j++) {
          m[i][j]=Math.min(m[i][j],m[i][k]+m[k][j]);
        }
      }
    }
  }
  
}
