package gold.util;

import java.lang.reflect.*;

public final class GMethod implements IMethod {
  
  private final Class<?> declaringClass;
  private final Object declaringObject;
  private final String methodName;
  private final Method unambiguousMethod;
  
  public GMethod(Class<?> pDeclaringClass, String pMethodName) {
    if (pDeclaringClass==null) throw new IllegalArgumentException("The declaring class cannot be null.");
    declaringClass=pDeclaringClass;
    declaringObject=null;
    methodName=pMethodName;
    unambiguousMethod=GReflection.$searchMethod(declaringClass,methodName,declaringObject==null,-1);
  }
  
  public GMethod(Object pDeclaringObject, String pMethodName) {
    if (pDeclaringObject==null) throw new IllegalArgumentException("The declaring object cannot be null.");
    declaringClass=pDeclaringObject.getClass();
    declaringObject=pDeclaringObject;
    methodName=pMethodName;
    unambiguousMethod=GReflection.$searchMethod(declaringClass,methodName,declaringObject==null,-1);
  }
  
  @Override
  public Class<?> getDeclaringClass() {
    return declaringClass;
  }
  
  @Override
  public Object getDeclaringObject() {
    return declaringObject;
  }
  
  @Override
  public String getMethodName() {
    return methodName;
  }
  
  @Override
  public Method getJavaMethod(Class<?>...pParameterTypes) {
    if (unambiguousMethod!=null) return unambiguousMethod;
    return GReflection.$searchMethod(declaringClass,methodName,declaringObject==null,pParameterTypes);
  }
  
  @Override
  public Object invoke(Object...pArguments) {
    if (unambiguousMethod!=null) return GReflection.$invokeMethod(unambiguousMethod,declaringObject,pArguments);
    return GReflection.$invokeMethod(declaringClass,methodName,declaringObject==null,declaringObject,pArguments);
  }
  
  @Override
  public int hashCode() {
    return declaringClass.hashCode()^(declaringObject!=null?declaringObject.hashCode():0)^methodName.hashCode();
  }
  
  @Override
  public boolean equals(Object pObject) {
    if (!(pObject instanceof IMethod)) return false;
    IMethod method=(IMethod)pObject;
    return declaringClass==method.getDeclaringClass()&&declaringObject==method.getDeclaringObject()&&methodName.equals(method.getMethodName());
  }
  
}
