package gold.util;

import java.lang.reflect.*;
import java.math.*;
import java.util.*;
import gold.structures.bag.*;
import gold.structures.collection.*;
import gold.structures.list.*;
import gold.structures.map.*;
import gold.structures.set.*;
import gold.structures.tuple.*;
import org.apfloat.*;

@SuppressWarnings("unchecked")
public final class GReflection {
  
  private static final Map<GAddress1,AccessibleObject> CACHE_LRU1=Collections.synchronizedMap(new GLRUCache<GAddress1,AccessibleObject>(16384));
  private static final Map<GAddress2,AccessibleObject> CACHE_LRU2=Collections.synchronizedMap(new GLRUCache<GAddress2,AccessibleObject>(16384));
  public static final int PRECISION=128;
  public static final MathContext MATH_CONTEXT=new MathContext(PRECISION);
  
  private GReflection() {
  }
  
  /*
   * General routines
   */
  
  private static Apcomplex $precision(Apcomplex pNumber) {
    return pNumber.precision()==Apcomplex.INFINITE?pNumber.precision(PRECISION):pNumber;
  }
  
  private static Apfloat $precision(Apfloat pNumber) {
    return pNumber.precision()==Apcomplex.INFINITE?pNumber.precision(PRECISION):pNumber;
  }
  
  public static Number $number(Object pObject) {
    if (pObject instanceof Character) return (int)(((Character)pObject).charValue());
    return (Number)pObject;
  }
  
  public static boolean $bool(boolean pObject) {
    return pObject;
  }
  
  public static boolean $bool(Boolean pObject) {
    return pObject;
  }
  
  public static boolean $bool(Object pObject) {
    return (Boolean)pObject;
  }
  
  public static int $int(byte pObject) {
    return pObject;
  }
  
  public static int $int(Byte pObject) {
    return pObject;
  }
  
  public static int $int(short pObject) {
    return pObject;
  }
  
  public static int $int(Short pObject) {
    return pObject;
  }
  
  public static int $int(int pObject) {
    return pObject;
  }
  
  public static int $int(Integer pObject) {
    return pObject;
  }
  
  public static int $int(long pObject) {
    return (int)pObject;
  }
  
  public static int $int(Long pObject) {
    return pObject.intValue();
  }
  
  public static int $int(float pObject) {
    return (int)pObject;
  }
  
  public static int $int(Float pObject) {
    return pObject.intValue();
  }
  
  public static int $int(double pObject) {
    return (int)pObject;
  }
  
  public static int $int(Double pObject) {
    return pObject.intValue();
  }
  
  public static int $int(char pObject) {
    return pObject;
  }
  
  public static int $int(Character pObject) {
    return pObject;
  }
  
  public static int $int(Object pObject) {
    if (pObject instanceof Character) return (Character)pObject;
    if (pObject instanceof Integer) return (Integer)pObject;
    return ((Number)pObject).intValue();
  }
  
  public static long $long(byte pObject) {
    return pObject;
  }
  
  public static long $long(Byte pObject) {
    return pObject;
  }
  
  public static long $long(short pObject) {
    return pObject;
  }
  
  public static long $long(Short pObject) {
    return pObject;
  }
  
  public static long $long(int pObject) {
    return pObject;
  }
  
  public static long $long(Integer pObject) {
    return pObject;
  }
  
  public static long $long(long pObject) {
    return pObject;
  }
  
  public static long $long(Long pObject) {
    return pObject;
  }
  
  public static long $long(float pObject) {
    return (long)pObject;
  }
  
  public static long $long(Float pObject) {
    return pObject.longValue();
  }
  
  public static long $long(double pObject) {
    return (long)pObject;
  }
  
  public static long $long(Double pObject) {
    return pObject.longValue();
  }
  
  public static long $long(char pObject) {
    return pObject;
  }
  
  public static long $long(Character pObject) {
    return pObject;
  }
  
  public static long $long(Object pObject) {
    if (pObject instanceof Character) return (Character)pObject;
    if (pObject instanceof Long) return (Long)pObject;
    return ((Number)pObject).longValue();
  }
  
  public static double $double(byte pObject) {
    return pObject;
  }
  
  public static double $double(Byte pObject) {
    return pObject;
  }
  
  public static double $double(short pObject) {
    return pObject;
  }
  
  public static double $double(Short pObject) {
    return pObject;
  }
  
  public static double $double(int pObject) {
    return pObject;
  }
  
  public static double $double(Integer pObject) {
    return pObject;
  }
  
  public static double $double(long pObject) {
    return pObject;
  }
  
  public static double $double(Long pObject) {
    return pObject;
  }
  
  public static double $double(float pObject) {
    return pObject;
  }
  
  public static double $double(Float pObject) {
    return pObject;
  }
  
  public static double $double(double pObject) {
    return pObject;
  }
  
  public static double $double(Double pObject) {
    return pObject;
  }
  
  public static double $double(char pObject) {
    return pObject;
  }
  
  public static double $double(Character pObject) {
    return pObject;
  }
  
  public static double $double(Object pObject) {
    if (pObject instanceof Character) return (Character)pObject;
    if (pObject instanceof Double) return (Double)pObject;
    return ((Number)pObject).doubleValue();
  }
  
  public static <T> T $cast(Class<T> pClass, Object pObject) {
    if (pObject==null) return null;
    if (pClass==null) throw new NullPointerException();
    Class<?> c=$getNonprimitiveType(pClass),objectClass=pObject.getClass();
    if (c==objectClass||c==Object.class||c.isInstance(pObject)) {
      return (T)pObject;
    }
    else if (c==String.class) {
      return (T)(pObject.toString());
    }
    else if (c.isArray()&&objectClass.isArray()) {
      int length=Array.getLength(pObject);
      Class<?> type=c.getComponentType();
      Object array=Array.newInstance(type,length);
      for (int index=0; index<length; index++) {
        Array.set(array,index,$cast(type,Array.get(pObject,index)));
      }
      return (T)array;
    }
    else if (objectClass==String.class) {
      String string=(String)pObject;
      if (c==Boolean.class) return (T)(new Boolean(Boolean.parseBoolean(string)));
      if (c==Character.class) return (T)(new Character(string.isEmpty()?'\0':string.charAt(0)));
      string=string.replace(',','.');
      if (string.startsWith(".")) string="0"+string;
      if (string.isEmpty()) string="0";
      if (!string.contains(".")&&Apcomplex.class.isAssignableFrom(c)) c=Apint.class;
      if (c==Byte.class) return (T)(new Byte(Byte.parseByte(string)));
      if (c==Short.class) return (T)(new Short(Short.parseShort(string)));
      if (c==Integer.class) return (T)(new Integer(Integer.parseInt(string)));
      if (c==Long.class) return (T)(new Long(Long.parseLong(string)));
      if (c==Float.class) return (T)(new Float(Float.parseFloat(string)));
      if (c==Double.class) return (T)(new Double(Double.parseDouble(string)));
      if (c==BigInteger.class) return (T)(new BigInteger(string));
      if (c==BigDecimal.class) return (T)(new BigDecimal(string,MATH_CONTEXT));
      if (c==Apint.class) return (T)(new Apint(string));
      if (c==Aprational.class) return (T)(new Aprational(string));
      if (c==Apfloat.class) return (T)(new Apfloat(string,PRECISION));
      if (c==Apcomplex.class) return (T)(new Apcomplex(string));
      return (T)pObject;
    }
    else {
      try {
        if ((c==Character.class)&&(pObject instanceof Number)) {
          return (T)(new Character((char)((Number)pObject).intValue()));
        }
        else if (Number.class.isAssignableFrom(c)&&(objectClass==Character.class)) {
          return $cast(pClass,new Integer(((Character)pObject)&0xFFFF));
        }
        else if (Number.class.isAssignableFrom(c)&&(pObject instanceof Number)) {
          Number number=(Number)pObject;
          if (c==Byte.class) return (T)(new Byte(number.byteValue()));
          if (c==Short.class) return (T)(new Short(number.shortValue()));
          if (c==Integer.class) return (T)(new Integer(number.intValue()));
          if (c==Long.class) return (T)(new Long(number.longValue()));
          if (c==Float.class) return (T)(new Float(number.floatValue()));
          if (c==Double.class) return (T)(new Double(number.doubleValue()));
          boolean mustBeInteger=c==BigInteger.class||c==Apint.class||c==Aprational.class;
          String string=(number instanceof Apcomplex)?((Apcomplex)number).real().toString(true):number.toString();
          if ((number instanceof Aprational)&&string.contains("/")) {
            string=new Apfloat(((Aprational)number).numerator().toBigInteger(),PRECISION).divide(new Apfloat(((Aprational)number).denominator().toBigInteger(),PRECISION)).toString(true);
          }
          string=string.replace(',','.').toLowerCase();
          if (string.startsWith(".")) string="0"+string;
          if (mustBeInteger&&(string.indexOf('.')!=-1||string.indexOf('e')!=-1)) string=new Apfloat(string).floor().toString(true);
          if (!string.contains(".")&&Apcomplex.class.isAssignableFrom(c)) c=Apint.class;
          if (c==BigInteger.class) return (T)(new BigInteger(string));
          if (c==BigDecimal.class) return (T)(new BigDecimal(string,MATH_CONTEXT));
          if (c==Apint.class) return (T)(new Apint(string));
          if (c==Aprational.class) return (T)(new Aprational(string));
          if (c==Apfloat.class) return (T)(new Apfloat(string,PRECISION));
          if (c==Apcomplex.class) return (T)(new Apcomplex(string));
        }
        return (T)pObject;
      }
      catch (Throwable th) {
        throw new ClassCastException("Cannot convert from "+objectClass.getCanonicalName()+" to "+c.getCanonicalName()+".");
      }
    }
  }
  
  public static byte $fix(byte pObject) {
    return pObject;
  }
  
  public static short $fix(short pObject) {
    return pObject;
  }
  
  public static int $fix(int pObject) {
    return pObject;
  }
  
  public static long $fix(long pObject) {
    return pObject;
  }
  
  public static float $fix(float pObject) {
    return pObject;
  }
  
  public static double $fix(double pObject) {
    return pObject;
  }
  
  public static char $fix(char pObject) {
    return pObject;
  }
  
  public static boolean $fix(boolean pObject) {
    return pObject;
  }
  
  public static Number $fix(Number pObject) {
    return pObject;
  }
  
  public static Object $fix(Object pObject) {
    if (pObject instanceof GMutableList) return new GArrayList<Object>((GMutableList)pObject);
    if (pObject instanceof GMutableSet) return new GLinkedHashTableSet<Object>((GMutableSet)pObject);
    if (pObject instanceof GMutableBag) return new GLinkedHashTableBag<Object>((GMutableBag)pObject);
    return pObject;
  }
  
  public static String $message(Object[] pMessages) {
    return GToolkit.concatenate(pMessages);
  }
  
  private static RuntimeException $processException(Throwable pThrowable) {
    Throwable th=pThrowable;
    if (th instanceof GRuntimeException) {
      while (th.getCause()!=null) {
        th=th.getCause();
      }
    }
    return (th instanceof RuntimeException)?(RuntimeException)th:new GRuntimeException(th);
  }
  
  public static void $rethrow(Throwable pThrowable) {
    throw $processException(pThrowable);
  }
  
  public static void $rethrow(Throwable pThrowable, Class<?> pClass, String pMethodName, int pLineNumber) {
    RuntimeException exception=$processException(pThrowable);
    String className=pClass.getName(),methodName=pMethodName!=null?pMethodName:"<clinit>",fileName=pClass.getSimpleName();
    StackTraceElement stackTrace[]=exception.getStackTrace();
    List<StackTraceElement> list=new ArrayList<StackTraceElement>(stackTrace.length);
    for (StackTraceElement element:stackTrace) {
      String elementClassName=element.getClassName(),elementFileName=element.getFileName();
      if (elementClassName!=null&&elementClassName.equals(className)&&elementFileName!=null&&elementFileName.equals(fileName+".java")) {
        // El error estuvo en una clase de Java traducida desde GOLD.
      }
      else if (elementClassName!=null&&(elementClassName.startsWith("sun.reflect.")||elementClassName.startsWith("java.lang.reflect."))) {
        // Error de reflexión de Java
      }
      else if (elementClassName!=null&&(elementClassName.equals("gold.util.GReflection")||elementClassName.equals("gold.util.GMethod"))) {
        // Error de reflexión de GOLD
      }
      else {
        list.add(element);
      }
    }
    list.add(new StackTraceElement(className,methodName,fileName+".gold",pLineNumber));
    exception.setStackTrace(list.toArray(new StackTraceElement[0]));
    throw exception;
  }
  
  public static <T> T $defaultValue(Class<T> pClass) {
    Class<?> type=$getPrimitiveType(pClass);
    if (type==boolean.class) return (T)(new Boolean(false));
    if (type==char.class) return (T)(new Character('\0'));
    if (type==byte.class) return (T)(new Byte((byte)0));
    if (type==short.class) return (T)(new Short((short)0));
    if (type==int.class) return (T)(new Integer(0));
    if (type==long.class) return (T)(new Long(0L));
    if (type==float.class) return (T)(new Float(0.0f));
    if (type==double.class) return (T)(new Double(0.0));
    if (type==Apint.class||type==Aprational.class||type==Apfloat.class||type==Apcomplex.class) return (T)(Apcomplex.ZERO);
    return null;
  }
  
  /*
   * Number routines
   */
  
  private static int $getRank(Class<?> pClass) {
    if (pClass==Byte.class) return 1;
    if (pClass==Short.class) return 2;
    if (pClass==Integer.class) return 3;
    if (pClass==Long.class) return 4;
    if (pClass==Float.class) return -5;
    if (pClass==Double.class) return -6;
    if (pClass==BigInteger.class) return 7;
    if (pClass==BigDecimal.class) return -8;
    if (pClass==Apint.class) return 9;
    if (pClass==Aprational.class) return 10;
    if (pClass==Apfloat.class) return -11;
    if (pClass==Apcomplex.class) return -12;
    return 0;
  }
  
  private static int $getResultTypeId(Number pOperand) {
    return $getResultTypeId(pOperand.getClass());
  }
  
  private static int $getResultTypeId(Class<?> pClass) {
    int r=$getRank(pClass);
    return r<0?-r:r;
  }
  
  private static int $getResultTypeId(Number pOperand1, Number pOperand2) {
    return $getResultTypeId(pOperand1.getClass(),pOperand2.getClass());
  }
  
  private static int $getResultTypeId(Class<?> pClass1, Class<?> pClass2) {
    int r1=$getRank(pClass1),r2=$getRank(pClass2);
    if (r1>0&&r2>0) return r1>r2?r1:r2;
    if (r1<0&&r2<0) return r1<r2?-r1:-r2;
    int s1=r1<0?-r1:r1,s2=r2<0?-r2:r2,m=s1>s2?s1:s2;
    return m<7?m:(m<9?8:(m<11?11:m));
  }
  
  /*
   * Number operations
   */
  
  private static long $gcd(long pNumber1, long pNumber2) {
    long a=pNumber1,b=pNumber2;
    while (b!=0) {
      long t=b;
      b=a%b;
      a=t;
    }
    return Math.abs(a);
  }
  
  private static long $lcm(long pNumber1, long pNumber2) {
    if (Math.max(Math.abs(pNumber1),Math.abs(pNumber2))>=(1L<<31)) {
      return ApintMath.lcm(new Apint(pNumber1),new Apint(pNumber2)).longValue();
    }
    else {
      return pNumber1*pNumber2/$gcd(pNumber1,pNumber2);
    }
  }
  
  private static long $pow(long pNumber1, long pNumber2) {
    long a=pNumber1,b=pNumber2;
    if (b<0) {
      return 0;
    }
    else if (b==0) {
      return 1;
    }
    else if (b==1) {
      return a;
    }
    else {
      long r=$pow(a,b/2);
      return (b%2)==0?r*r:r*r*a;
    }
  }
  
  /*
   * Evaluation methods
   */
  
  // Equality
  public static boolean $opEqualY(Byte pOperand1, Byte pOperand2) {
    return pOperand1.byteValue()==pOperand2.byteValue();
  }
  
  // Equality
  public static boolean $opEqualY(Short pOperand1, Short pOperand2) {
    return pOperand1.shortValue()==pOperand2.shortValue();
  }
  
  // Equality
  public static boolean $opEqualY(Integer pOperand1, Integer pOperand2) {
    return pOperand1.intValue()==pOperand2.intValue();
  }
  
  // Equality
  public static boolean $opEqualY(Long pOperand1, Long pOperand2) {
    return pOperand1.longValue()==pOperand2.longValue();
  }
  
  // Equality
  public static boolean $opEqualY(Float pOperand1, Float pOperand2) {
    return pOperand1.floatValue()==pOperand2.floatValue();
  }
  
  // Equality
  public static boolean $opEqualY(Double pOperand1, Double pOperand2) {
    return pOperand1.doubleValue()==pOperand2.doubleValue();
  }
  
  // Equality
  public static boolean $opEqualY(Character pOperand1, Character pOperand2) {
    return pOperand1.charValue()==pOperand2.charValue();
  }
  
  // Equality
  public static boolean $opEqualY(Boolean pOperand1, Boolean pOperand2) {
    return pOperand1.booleanValue()==pOperand2.booleanValue();
  }
  
  // Equality
  public static boolean $opEqualY(Object pOperand1, Object pOperand2) {
    if ((pOperand1 instanceof Number)&&(pOperand2 instanceof Number)) {
      Number a=(Number)pOperand1,b=(Number)pOperand2;
      switch ($getResultTypeId(a,b)) {
        case 1:
        case 2:
        case 3:
          return a.intValue()==b.intValue();
        case 4:
          return a.longValue()==b.longValue();
        case 5:
          return a.floatValue()==b.floatValue();
        case 6:
          return a.doubleValue()==b.doubleValue();
        case 7:
          return $cast(BigInteger.class,a).equals($cast(BigInteger.class,b));
        case 8:
          return $cast(BigDecimal.class,a).equals($cast(BigDecimal.class,b));
        case 9:
          return $cast(Apint.class,a).equals($cast(Apint.class,b));
        case 10:
          return $cast(Aprational.class,a).equals($cast(Aprational.class,b));
        case 11:
          return $cast(Apfloat.class,a).equals($cast(Apfloat.class,b));
        case 12:
          return $cast(Apcomplex.class,a).equals($cast(Apcomplex.class,b));
        default:
          return a.equals(b);
      }
    }
    else {
      return GToolkit.equals(pOperand1,pOperand2);
    }
  }
  
  // Less than
  public static boolean $opLesstY(Byte pOperand1, Byte pOperand2) {
    return pOperand1<pOperand2;
  }
  
  // Less than
  public static boolean $opLesstY(Short pOperand1, Short pOperand2) {
    return pOperand1<pOperand2;
  }
  
  // Less than
  public static boolean $opLesstY(Integer pOperand1, Integer pOperand2) {
    return pOperand1<pOperand2;
  }
  
  // Less than
  public static boolean $opLesstY(Long pOperand1, Long pOperand2) {
    return pOperand1<pOperand2;
  }
  
  // Less than
  public static boolean $opLesstY(Float pOperand1, Float pOperand2) {
    return pOperand1<pOperand2;
  }
  
  // Less than
  public static boolean $opLesstY(Double pOperand1, Double pOperand2) {
    return pOperand1<pOperand2;
  }
  
  // Less than
  public static boolean $opLesstY(Character pOperand1, Character pOperand2) {
    return pOperand1<pOperand2;
  }
  
  // Less than
  public static boolean $opLesstY(Object pOperand1, Object pOperand2) {
    Number a=$number(pOperand1),b=$number(pOperand2);
    Class<?> u=a.getClass(),v=b.getClass();
    if (u==v) {
      if (u==Integer.class) return a.intValue()<b.intValue();
      if (u==Double.class) return a.doubleValue()<b.doubleValue();
      if (u==Long.class) return a.longValue()<b.longValue();
      if (u==Float.class) return a.floatValue()<b.floatValue();
      if (u==Byte.class) return a.intValue()<b.intValue();
      if (u==Short.class) return a.intValue()<b.intValue();
    }
    switch ($getResultTypeId(u,v)) {
      case 1:
      case 2:
      case 3:
        return a.intValue()<b.intValue();
      case 4:
        return a.longValue()<b.longValue();
      case 5:
        return a.floatValue()<b.floatValue();
      case 6:
        return a.doubleValue()<b.doubleValue();
      case 7:
        return $cast(BigInteger.class,a).compareTo($cast(BigInteger.class,b))<0;
      case 8:
        return $cast(BigDecimal.class,a).compareTo($cast(BigDecimal.class,b))<0;
      case 9:
        return $cast(Apint.class,a).compareTo($cast(Apint.class,b))<0;
      case 10:
        return $cast(Aprational.class,a).compareTo($cast(Aprational.class,b))<0;
      case 11:
        return $cast(Apfloat.class,a).compareTo($cast(Apfloat.class,b))<0;
      case 12:
        throw new IllegalArgumentException("The operator 'less than' cannot be applied to complex numbers.");
      default:
        throw new IllegalStateException();
    }
  }
  
  // Less than or equal to
  public static boolean $opLessqY(Byte pOperand1, Byte pOperand2) {
    return pOperand1<=pOperand2;
  }
  
  // Less than or equal to
  public static boolean $opLessqY(Short pOperand1, Short pOperand2) {
    return pOperand1<=pOperand2;
  }
  
  // Less than or equal to
  public static boolean $opLessqY(Integer pOperand1, Integer pOperand2) {
    return pOperand1<=pOperand2;
  }
  
  // Less than or equal to
  public static boolean $opLessqY(Long pOperand1, Long pOperand2) {
    return pOperand1<=pOperand2;
  }
  
  // Less than or equal to
  public static boolean $opLessqY(Float pOperand1, Float pOperand2) {
    return pOperand1<=pOperand2;
  }
  
  // Less than or equal to
  public static boolean $opLessqY(Double pOperand1, Double pOperand2) {
    return pOperand1<=pOperand2;
  }
  
  // Less than or equal to
  public static boolean $opLessqY(Character pOperand1, Character pOperand2) {
    return pOperand1<=pOperand2;
  }
  
  // Less than or equal to
  public static boolean $opLessqY(Object pOperand1, Object pOperand2) {
    Number a=$number(pOperand1),b=$number(pOperand2);
    Class<?> u=a.getClass(),v=b.getClass();
    if (u==v) {
      if (u==Integer.class) return a.intValue()<=b.intValue();
      if (u==Double.class) return a.doubleValue()<=b.doubleValue();
      if (u==Long.class) return a.longValue()<=b.longValue();
      if (u==Float.class) return a.floatValue()<=b.floatValue();
      if (u==Byte.class) return a.intValue()<=b.intValue();
      if (u==Short.class) return a.intValue()<=b.intValue();
    }
    switch ($getResultTypeId(u,v)) {
      case 1:
      case 2:
      case 3:
        return a.intValue()<=b.intValue();
      case 4:
        return a.longValue()<=b.longValue();
      case 5:
        return a.floatValue()<=b.floatValue();
      case 6:
        return a.doubleValue()<=b.doubleValue();
      case 7:
        return $cast(BigInteger.class,a).compareTo($cast(BigInteger.class,b))<=0;
      case 8:
        return $cast(BigDecimal.class,a).compareTo($cast(BigDecimal.class,b))<=0;
      case 9:
        return $cast(Apint.class,a).compareTo($cast(Apint.class,b))<=0;
      case 10:
        return $cast(Aprational.class,a).compareTo($cast(Aprational.class,b))<=0;
      case 11:
        return $cast(Apfloat.class,a).compareTo($cast(Apfloat.class,b))<=0;
      case 12:
        throw new IllegalArgumentException("The operator 'less than or equal to' cannot be applied to complex numbers.");
      default:
        throw new IllegalStateException();
    }
  }
  
  // Greater than
  public static boolean $opGreatY(Byte pOperand1, Byte pOperand2) {
    return pOperand1>pOperand2;
  }
  
  // Greater than
  public static boolean $opGreatY(Short pOperand1, Short pOperand2) {
    return pOperand1>pOperand2;
  }
  
  // Greater than
  public static boolean $opGreatY(Integer pOperand1, Integer pOperand2) {
    return pOperand1>pOperand2;
  }
  
  // Greater than
  public static boolean $opGreatY(Long pOperand1, Long pOperand2) {
    return pOperand1>pOperand2;
  }
  
  // Greater than
  public static boolean $opGreatY(Float pOperand1, Float pOperand2) {
    return pOperand1>pOperand2;
  }
  
  // Greater than
  public static boolean $opGreatY(Double pOperand1, Double pOperand2) {
    return pOperand1>pOperand2;
  }
  
  // Greater than
  public static boolean $opGreatY(Character pOperand1, Character pOperand2) {
    return pOperand1>pOperand2;
  }
  
  // Greater than
  public static boolean $opGreatY(Object pOperand1, Object pOperand2) {
    Number a=$number(pOperand1),b=$number(pOperand2);
    Class<?> u=a.getClass(),v=b.getClass();
    if (u==v) {
      if (u==Integer.class) return a.intValue()>b.intValue();
      if (u==Double.class) return a.doubleValue()>b.doubleValue();
      if (u==Long.class) return a.longValue()>b.longValue();
      if (u==Float.class) return a.floatValue()>b.floatValue();
      if (u==Byte.class) return a.intValue()>b.intValue();
      if (u==Short.class) return a.intValue()>b.intValue();
    }
    switch ($getResultTypeId(u,v)) {
      case 1:
      case 2:
      case 3:
        return a.intValue()>b.intValue();
      case 4:
        return a.longValue()>b.longValue();
      case 5:
        return a.floatValue()>b.floatValue();
      case 6:
        return a.doubleValue()>b.doubleValue();
      case 7:
        return $cast(BigInteger.class,a).compareTo($cast(BigInteger.class,b))>0;
      case 8:
        return $cast(BigDecimal.class,a).compareTo($cast(BigDecimal.class,b))>0;
      case 9:
        return $cast(Apint.class,a).compareTo($cast(Apint.class,b))>0;
      case 10:
        return $cast(Aprational.class,a).compareTo($cast(Aprational.class,b))>0;
      case 11:
        return $cast(Apfloat.class,a).compareTo($cast(Apfloat.class,b))>0;
      case 12:
        throw new IllegalArgumentException("The operator 'greater than' cannot be applied to complex numbers.");
      default:
        throw new IllegalStateException();
    }
  }
  
  // Greater than or equal to
  public static boolean $opGreaqY(Byte pOperand1, Byte pOperand2) {
    return pOperand1>=pOperand2;
  }
  
  // Greater than or equal to
  public static boolean $opGreaqY(Short pOperand1, Short pOperand2) {
    return pOperand1>=pOperand2;
  }
  
  // Greater than or equal to
  public static boolean $opGreaqY(Integer pOperand1, Integer pOperand2) {
    return pOperand1>=pOperand2;
  }
  
  // Greater than or equal to
  public static boolean $opGreaqY(Long pOperand1, Long pOperand2) {
    return pOperand1>=pOperand2;
  }
  
  // Greater than or equal to
  public static boolean $opGreaqY(Float pOperand1, Float pOperand2) {
    return pOperand1>=pOperand2;
  }
  
  // Greater than or equal to
  public static boolean $opGreaqY(Double pOperand1, Double pOperand2) {
    return pOperand1>=pOperand2;
  }
  
  // Greater than or equal to
  public static boolean $opGreaqY(Character pOperand1, Character pOperand2) {
    return pOperand1>=pOperand2;
  }
  
  // Greater than or equal to
  public static boolean $opGreaqY(Object pOperand1, Object pOperand2) {
    Number a=$number(pOperand1),b=$number(pOperand2);
    Class<?> u=a.getClass(),v=b.getClass();
    if (u==v) {
      if (u==Integer.class) return a.intValue()>=b.intValue();
      if (u==Double.class) return a.doubleValue()>=b.doubleValue();
      if (u==Long.class) return a.longValue()>=b.longValue();
      if (u==Float.class) return a.floatValue()>=b.floatValue();
      if (u==Byte.class) return a.intValue()>=b.intValue();
      if (u==Short.class) return a.intValue()>=b.intValue();
    }
    switch ($getResultTypeId(u,v)) {
      case 1:
      case 2:
      case 3:
        return a.intValue()>=b.intValue();
      case 4:
        return a.longValue()>=b.longValue();
      case 5:
        return a.floatValue()>=b.floatValue();
      case 6:
        return a.doubleValue()>=b.doubleValue();
      case 7:
        return $cast(BigInteger.class,a).compareTo($cast(BigInteger.class,b))>=0;
      case 8:
        return $cast(BigDecimal.class,a).compareTo($cast(BigDecimal.class,b))>=0;
      case 9:
        return $cast(Apint.class,a).compareTo($cast(Apint.class,b))>=0;
      case 10:
        return $cast(Aprational.class,a).compareTo($cast(Aprational.class,b))>=0;
      case 11:
        return $cast(Apfloat.class,a).compareTo($cast(Apfloat.class,b))>=0;
      case 12:
        throw new IllegalArgumentException("The operator 'greater than or equal to' cannot be applied to complex numbers.");
      default:
        throw new IllegalStateException();
    }
  }
  
  // Divisibility
  public static boolean $opDivisY(Object pOperand1, Object pOperand2) {
    Number a=$number(pOperand1),b=$number(pOperand2);
    switch ($getResultTypeId(a,b)) {
      case 1:
      case 2:
      case 3: {
        int x=a.intValue(),y=b.intValue();
        return x==0?y==0:(y%x)==0;
      }
      case 4: {
        long x=a.longValue(),y=b.longValue();
        return x==0?y==0:(y%x)==0;
      }
      case 7: {
        BigInteger x=$cast(BigInteger.class,a),y=$cast(BigInteger.class,b);
        return x.equals(BigInteger.ZERO)?y.equals(BigInteger.ZERO):y.mod(x).equals(BigInteger.ZERO);
      }
      case 9: {
        Apint x=$cast(Apint.class,a),y=$cast(Apint.class,b);
        return x.equals(Apint.ZERO)?y.equals(Apint.ZERO):y.mod(x).equals(Apint.ZERO);
      }
      default:
        throw new IllegalArgumentException("The operator 'divisibility' only can be applied to integer numbers.");
    }
  }
  
  // Membership
  public static boolean $opMembrY(Object pOperand1, Object pOperand2) {
    if (pOperand2 instanceof ICollection) {
      return ((ICollection<?>)pOperand2).contains(pOperand1);
    }
    else if (pOperand2 instanceof Collection) {
      return ((Collection<?>)pOperand2).contains(pOperand1);
    }
    else {
      throw new IllegalArgumentException("The operator 'membership' only can be applied to collections.");
    }
  }
  
  // Disjoint
  public static boolean $opDisjoY(Object pOperand1, Object pOperand2) {
    if (((pOperand1 instanceof Set)||(pOperand1 instanceof ISet))&&((pOperand2 instanceof Set)||(pOperand2 instanceof ISet))) {
      return $set((Iterable<?>)pOperand1).isDisjointFrom($set((Iterable<?>)pOperand2));
    }
    else if ((pOperand1 instanceof IBag)&&(pOperand2 instanceof IBag)) {
      return $bag((Iterable<?>)pOperand1).isDisjointFrom($bag((Iterable<?>)pOperand2));
    }
    else {
      throw new IllegalArgumentException("The operator 'disjoint' must be applied to two sets or two bags.");
    }
  }
  
  // Subset
  public static boolean $opSbsetY(Object pOperand1, Object pOperand2) {
    if (((pOperand1 instanceof Set)||(pOperand1 instanceof ISet))&&((pOperand2 instanceof Set)||(pOperand2 instanceof ISet))) {
      return $set((Iterable<?>)pOperand1).isSubset($set((Iterable<?>)pOperand2));
    }
    else if ((pOperand1 instanceof IBag)&&(pOperand2 instanceof IBag)) {
      return $bag((Iterable<?>)pOperand1).isSubbag($bag((Iterable<?>)pOperand2));
    }
    else {
      throw new IllegalArgumentException("The operator 'subset/subbag' must be applied to two sets or two bags.");
    }
  }
  
  // Superset
  public static boolean $opSpsetY(Object pOperand1, Object pOperand2) {
    if (((pOperand1 instanceof Set)||(pOperand1 instanceof ISet))&&((pOperand2 instanceof Set)||(pOperand2 instanceof ISet))) {
      return $set((Iterable<?>)pOperand1).isSuperset($set((Iterable<?>)pOperand2));
    }
    else if ((pOperand1 instanceof IBag)&&(pOperand2 instanceof IBag)) {
      return $bag((Iterable<?>)pOperand1).isSuperbag($bag((Iterable<?>)pOperand2));
    }
    else {
      throw new IllegalArgumentException("The operator 'superset/superbag' must be applied to two sets or two bags.");
    }
  }
  
  // Proper subset
  public static boolean $opPssetY(Object pOperand1, Object pOperand2) {
    if (((pOperand1 instanceof Set)||(pOperand1 instanceof ISet))&&((pOperand2 instanceof Set)||(pOperand2 instanceof ISet))) {
      return $set((Iterable<?>)pOperand1).isProperSubset($set((Iterable<?>)pOperand2));
    }
    else if ((pOperand1 instanceof IBag)&&(pOperand2 instanceof IBag)) {
      return $bag((Iterable<?>)pOperand1).isProperSubbag($bag((Iterable<?>)pOperand2));
    }
    else {
      throw new IllegalArgumentException("The operator 'proper subset / proper subbag' must be applied to two sets or two bags.");
    }
  }
  
  // Proper superset
  public static boolean $opPpsetY(Object pOperand1, Object pOperand2) {
    if (((pOperand1 instanceof Set)||(pOperand1 instanceof ISet))&&((pOperand2 instanceof Set)||(pOperand2 instanceof ISet))) {
      return $set((Iterable<?>)pOperand1).isProperSuperset($set((Iterable<?>)pOperand2));
    }
    else if ((pOperand1 instanceof IBag)&&(pOperand2 instanceof IBag)) {
      return $bag((Iterable<?>)pOperand1).isProperSuperbag($bag((Iterable<?>)pOperand2));
    }
    else {
      throw new IllegalArgumentException("The operator 'proper superset / proper superbag' must be applied to two sets or two bags.");
    }
  }
  
  // Prepend
  public static IList<?> $opPrepdY(Object pOperand1, Object pOperand2) {
    if ((pOperand2 instanceof List)||(pOperand2 instanceof IList)) {
      IList<Object> result=$newList(pOperand2);
      result.addFirst(pOperand1);
      return result;
    }
    else {
      throw new IllegalArgumentException("The operator 'prepend' must be applied to an object and a list.");
    }
  }
  
  // Concatenation
  public static IList<?> $opConctY(Object pOperand1, Object pOperand2) {
    if (((pOperand1 instanceof List)||(pOperand1 instanceof IList))&&((pOperand2 instanceof List)||(pOperand2 instanceof IList))) {
      IList<Object> result=$newList(pOperand1);
      result.concatenate($list((Iterable<?>)pOperand2));
      return result;
    }
    else {
      throw new IllegalArgumentException("The operator 'concatenation' must be applied to two lists.");
    }
  }
  
  // Append
  public static IList<?> $opAppedY(Object pOperand1, Object pOperand2) {
    if ((pOperand1 instanceof List)||(pOperand1 instanceof IList)) {
      IList<Object> result=$newList(pOperand1);
      result.addLast(pOperand2);
      return result;
    }
    else {
      throw new IllegalArgumentException("The operator 'append' must be applied to a list and an object.");
    }
  }
  
  // Number of occurrences
  public static Integer $opOccurY(Object pOperand1, Object pOperand2) {
    if (pOperand2 instanceof IList) {
      return ((IList<?>)pOperand2).count(pOperand1);
    }
    else if (pOperand2 instanceof IBag) {
      return ((IBag<?>)pOperand2).count(pOperand1);
    }
    else if (pOperand2 instanceof ISet) {
      return ((ISet<?>)pOperand2).contains(pOperand1)?1:0;
    }
    else if (pOperand2!=null) {
      return GCollections.unmodifiableList((pOperand2 instanceof Iterable)?(Iterable<?>)pOperand2:GCollections.asJavaCollection(pOperand2)).count(pOperand1);
    }
    else {
      throw new IllegalArgumentException("The operator 'number of occurrences' must be applied to an object and a collection.");
    }
  }
  
  // Interval range
  public static ISet<?> $opIntvlY(Object pOperand1, Object pOperand2) {
    if ((pOperand1 instanceof Character)||(pOperand2 instanceof Character)) {
      return GCollections.characterInterval((char)($int(pOperand1)),(char)($int(pOperand2)));
    }
    else {
      return GCollections.integerInterval($int(pOperand1),$int(pOperand2));
    }
  }
  
  // Maximum
  public static byte $opMaximY(Byte pOperand1, Byte pOperand2) {
    return pOperand1>pOperand2?pOperand1:pOperand2;
  }
  
  // Maximum
  public static short $opMaximY(Short pOperand1, Short pOperand2) {
    return pOperand1>pOperand2?pOperand1:pOperand2;
  }
  
  // Maximum
  public static int $opMaximY(Integer pOperand1, Integer pOperand2) {
    return pOperand1>pOperand2?pOperand1:pOperand2;
  }
  
  // Maximum
  public static long $opMaximY(Long pOperand1, Long pOperand2) {
    return pOperand1>pOperand2?pOperand1:pOperand2;
  }
  
  // Maximum
  public static float $opMaximY(Float pOperand1, Float pOperand2) {
    return pOperand1>pOperand2?pOperand1:pOperand2;
  }
  
  // Maximum
  public static double $opMaximY(Double pOperand1, Double pOperand2) {
    return pOperand1>pOperand2?pOperand1:pOperand2;
  }
  
  // Maximum
  public static char $opMaximY(Character pOperand1, Character pOperand2) {
    return pOperand1>pOperand2?pOperand1:pOperand2;
  }
  
  // Maximum
  public static Number $opMaximY(Object pOperand1, Object pOperand2) {
    Number a=$number(pOperand1),b=$number(pOperand2);
    Class<?> u=a.getClass(),v=b.getClass();
    if (u==Double.class&&a.doubleValue()==Double.NEGATIVE_INFINITY) return b;
    if (v==Double.class&&b.doubleValue()==Double.NEGATIVE_INFINITY) return a;
    if (u==v) {
      if (u==Integer.class) return a.intValue()>b.intValue()?a:b;
      if (u==Double.class) return a.doubleValue()>b.doubleValue()?a:b;
      if (u==Long.class) return a.longValue()>b.longValue()?a:b;
      if (u==Float.class) return a.floatValue()>b.floatValue()?a:b;
      if (u==Byte.class) return a.intValue()>b.intValue()?a:b;
      if (u==Short.class) return a.intValue()>b.intValue()?a:b;
    }
    switch ($getResultTypeId(u,v)) {
      case 1:
        return (byte)(Math.max(a.intValue(),b.intValue()));
      case 2:
        return (short)(Math.max(a.intValue(),b.intValue()));
      case 3:
        return (int)(Math.max(a.intValue(),b.intValue()));
      case 4:
        return (long)(Math.max(a.longValue(),b.longValue()));
      case 5:
        return (float)(Math.max(a.floatValue(),b.floatValue()));
      case 6:
        return (double)(Math.max(a.doubleValue(),b.doubleValue()));
      case 7:
        return $cast(BigInteger.class,a).max($cast(BigInteger.class,b));
      case 8:
        return $cast(BigDecimal.class,a).max($cast(BigDecimal.class,b));
      case 9:
        return $cast(Apint.class,a).compareTo($cast(Apint.class,b))>0?a:b;
      case 10:
        return $cast(Aprational.class,a).compareTo($cast(Aprational.class,b))>0?a:b;
      case 11:
        return $cast(Apfloat.class,a).compareTo($cast(Apfloat.class,b))>0?a:b;
      case 12:
        throw new IllegalArgumentException("The operator 'maximum' cannot be applied to complex numbers.");
      default:
        throw new IllegalStateException();
    }
  }
  
  // Minimum
  public static byte $opMinimY(Byte pOperand1, Byte pOperand2) {
    return pOperand1<pOperand2?pOperand1:pOperand2;
  }
  
  // Minimum
  public static short $opMinimY(Short pOperand1, Short pOperand2) {
    return pOperand1<pOperand2?pOperand1:pOperand2;
  }
  
  // Minimum
  public static int $opMinimY(Integer pOperand1, Integer pOperand2) {
    return pOperand1<pOperand2?pOperand1:pOperand2;
  }
  
  // Minimum
  public static long $opMinimY(Long pOperand1, Long pOperand2) {
    return pOperand1<pOperand2?pOperand1:pOperand2;
  }
  
  // Minimum
  public static float $opMinimY(Float pOperand1, Float pOperand2) {
    return pOperand1<pOperand2?pOperand1:pOperand2;
  }
  
  // Minimum
  public static double $opMinimY(Double pOperand1, Double pOperand2) {
    return pOperand1<pOperand2?pOperand1:pOperand2;
  }
  
  // Minimum
  public static char $opMinimY(Character pOperand1, Character pOperand2) {
    return pOperand1<pOperand2?pOperand1:pOperand2;
  }
  
  // Minimum
  public static Number $opMinimY(Object pOperand1, Object pOperand2) {
    Number a=$number(pOperand1),b=$number(pOperand2);
    Class<?> u=a.getClass(),v=b.getClass();
    if (u==Double.class&&a.doubleValue()==Double.POSITIVE_INFINITY) return b;
    if (v==Double.class&&b.doubleValue()==Double.POSITIVE_INFINITY) return a;
    if (u==v) {
      if (u==Integer.class) return a.intValue()<b.intValue()?a:b;
      if (u==Double.class) return a.doubleValue()<b.doubleValue()?a:b;
      if (u==Long.class) return a.longValue()<b.longValue()?a:b;
      if (u==Float.class) return a.floatValue()<b.floatValue()?a:b;
      if (u==Byte.class) return a.intValue()<b.intValue()?a:b;
      if (u==Short.class) return a.intValue()<b.intValue()?a:b;
    }
    switch ($getResultTypeId(u,v)) {
      case 1:
        return (byte)(Math.min(a.intValue(),b.intValue()));
      case 2:
        return (short)(Math.min(a.intValue(),b.intValue()));
      case 3:
        return (int)(Math.min(a.intValue(),b.intValue()));
      case 4:
        return (long)(Math.min(a.longValue(),b.longValue()));
      case 5:
        return (float)(Math.min(a.floatValue(),b.floatValue()));
      case 6:
        return (double)(Math.min(a.doubleValue(),b.doubleValue()));
      case 7:
        return $cast(BigInteger.class,a).min($cast(BigInteger.class,b));
      case 8:
        return $cast(BigDecimal.class,a).min($cast(BigDecimal.class,b));
      case 9:
        return $cast(Apint.class,a).compareTo($cast(Apint.class,b))<0?a:b;
      case 10:
        return $cast(Aprational.class,a).compareTo($cast(Aprational.class,b))<0?a:b;
      case 11:
        return $cast(Apfloat.class,a).compareTo($cast(Apfloat.class,b))<0?a:b;
      case 12:
        throw new IllegalArgumentException("The operator 'minimum' cannot be applied to complex numbers.");
      default:
        throw new IllegalStateException();
    }
  }
  
  // Addition
  public static String $opAdditY(String pOperand1, String pOperand2) {
    return GToolkit.toString(pOperand1)+GToolkit.toString(pOperand2);
  }
  
  // Addition
  public static byte $opAdditY(Byte pOperand1, Byte pOperand2) {
    return (byte)(pOperand1+pOperand2);
  }
  
  // Addition
  public static short $opAdditY(Short pOperand1, Short pOperand2) {
    return (short)(pOperand1+pOperand2);
  }
  
  // Addition
  public static int $opAdditY(Integer pOperand1, Integer pOperand2) {
    return pOperand1+pOperand2;
  }
  
  // Addition
  public static long $opAdditY(Long pOperand1, Long pOperand2) {
    return pOperand1+pOperand2;
  }
  
  // Addition
  public static float $opAdditY(Float pOperand1, Float pOperand2) {
    return pOperand1+pOperand2;
  }
  
  // Addition
  public static double $opAdditY(Double pOperand1, Double pOperand2) {
    return pOperand1+pOperand2;
  }
  
  // Addition
  public static char $opAdditY(Character pOperand1, Character pOperand2) {
    return (char)(pOperand1+pOperand2);
  }
  
  // Addition
  public static Object $opAdditY(Object pOperand1, Object pOperand2) {
    if ((pOperand1 instanceof String)||(pOperand2 instanceof String)) {
      return GToolkit.toString(pOperand1)+GToolkit.toString(pOperand2);
    }
    else {
      Number a=$number(pOperand1),b=$number(pOperand2);
      Class<?> u=a.getClass(),v=b.getClass();
      if (u==v) {
        if (u==Integer.class) return a.intValue()+b.intValue();
        if (u==Double.class) return a.doubleValue()+b.doubleValue();
        if (u==Long.class) return a.longValue()+b.longValue();
        if (u==Float.class) return a.floatValue()+b.floatValue();
        if (u==Byte.class) return (byte)(a.intValue()+b.intValue());
        if (u==Short.class) return (short)(a.intValue()+b.intValue());
      }
      switch ($getResultTypeId(u,v)) {
        case 1:
          return (byte)(a.intValue()+b.intValue());
        case 2:
          return (short)(a.intValue()+b.intValue());
        case 3:
          return (int)(a.intValue()+b.intValue());
        case 4:
          return (long)(a.longValue()+b.longValue());
        case 5:
          return (float)(a.floatValue()+b.floatValue());
        case 6:
          return (double)(a.doubleValue()+b.doubleValue());
        case 7:
          return $cast(BigInteger.class,a).add($cast(BigInteger.class,b));
        case 8:
          return $cast(BigDecimal.class,a).add($cast(BigDecimal.class,b),MATH_CONTEXT);
        case 9:
          return $cast(Apint.class,a).add($cast(Apint.class,b));
        case 10:
          return $cast(Aprational.class,a).add($cast(Aprational.class,b));
        case 11:
          return $cast(Apfloat.class,a).add($cast(Apfloat.class,b));
        case 12:
          return $cast(Apcomplex.class,a).add($cast(Apcomplex.class,b));
        default:
          throw new IllegalStateException();
      }
    }
  }
  
  // Subtraction
  public static byte $opSubtrY(Byte pOperand1, Byte pOperand2) {
    return (byte)(pOperand1-pOperand2);
  }
  
  // Subtraction
  public static short $opSubtrY(Short pOperand1, Short pOperand2) {
    return (short)(pOperand1-pOperand2);
  }
  
  // Subtraction
  public static int $opSubtrY(Integer pOperand1, Integer pOperand2) {
    return pOperand1-pOperand2;
  }
  
  // Subtraction
  public static long $opSubtrY(Long pOperand1, Long pOperand2) {
    return pOperand1-pOperand2;
  }
  
  // Subtraction
  public static float $opSubtrY(Float pOperand1, Float pOperand2) {
    return pOperand1-pOperand2;
  }
  
  // Subtraction
  public static double $opSubtrY(Double pOperand1, Double pOperand2) {
    return pOperand1-pOperand2;
  }
  
  // Subtraction
  public static char $opSubtrY(Character pOperand1, Character pOperand2) {
    return (char)(pOperand1-pOperand2);
  }
  
  // Subtraction
  public static Object $opSubtrY(Object pOperand1, Object pOperand2) {
    Number a=$number(pOperand1),b=$number(pOperand2);
    Class<?> u=a.getClass(),v=b.getClass();
    if (u==v) {
      if (u==Integer.class) return a.intValue()-b.intValue();
      if (u==Double.class) return a.doubleValue()-b.doubleValue();
      if (u==Long.class) return a.longValue()-b.longValue();
      if (u==Float.class) return a.floatValue()-b.floatValue();
      if (u==Byte.class) return (byte)(a.intValue()-b.intValue());
      if (u==Short.class) return (short)(a.intValue()-b.intValue());
    }
    switch ($getResultTypeId(u,v)) {
      case 1:
        return (byte)(a.intValue()-b.intValue());
      case 2:
        return (short)(a.intValue()-b.intValue());
      case 3:
        return (int)(a.intValue()-b.intValue());
      case 4:
        return (long)(a.longValue()-b.longValue());
      case 5:
        return (float)(a.floatValue()-b.floatValue());
      case 6:
        return (double)(a.doubleValue()-b.doubleValue());
      case 7:
        return $cast(BigInteger.class,a).subtract($cast(BigInteger.class,b));
      case 8:
        return $cast(BigDecimal.class,a).subtract($cast(BigDecimal.class,b),MATH_CONTEXT);
      case 9:
        return $cast(Apint.class,a).subtract($cast(Apint.class,b));
      case 10:
        return $cast(Aprational.class,a).subtract($cast(Aprational.class,b));
      case 11:
        return $cast(Apfloat.class,a).subtract($cast(Apfloat.class,b));
      case 12:
        return $cast(Apcomplex.class,a).subtract($cast(Apcomplex.class,b));
      default:
        throw new IllegalStateException();
    }
  }
  
  // Multiplication
  public static byte $opMultpY(Byte pOperand1, Byte pOperand2) {
    return (byte)(pOperand1*pOperand2);
  }
  
  // Multiplication
  public static short $opMultpY(Short pOperand1, Short pOperand2) {
    return (short)(pOperand1*pOperand2);
  }
  
  // Multiplication
  public static int $opMultpY(Integer pOperand1, Integer pOperand2) {
    return pOperand1*pOperand2;
  }
  
  // Multiplication
  public static long $opMultpY(Long pOperand1, Long pOperand2) {
    return pOperand1*pOperand2;
  }
  
  // Multiplication
  public static float $opMultpY(Float pOperand1, Float pOperand2) {
    return pOperand1*pOperand2;
  }
  
  // Multiplication
  public static double $opMultpY(Double pOperand1, Double pOperand2) {
    return pOperand1*pOperand2;
  }
  
  // Multiplication
  public static Number $opMultpY(Object pOperand1, Object pOperand2) {
    Number a=$number(pOperand1),b=$number(pOperand2);
    Class<?> u=a.getClass(),v=b.getClass();
    if (u==v) {
      if (u==Integer.class) return a.intValue()*b.intValue();
      if (u==Double.class) return a.doubleValue()*b.doubleValue();
      if (u==Long.class) return a.longValue()*b.longValue();
      if (u==Float.class) return a.floatValue()*b.floatValue();
      if (u==Byte.class) return (byte)(a.intValue()*b.intValue());
      if (u==Short.class) return (short)(a.intValue()*b.intValue());
    }
    switch ($getResultTypeId(u,v)) {
      case 1:
        return (byte)(a.intValue()*b.intValue());
      case 2:
        return (short)(a.intValue()*b.intValue());
      case 3:
        return (int)(a.intValue()*b.intValue());
      case 4:
        return (long)(a.longValue()*b.longValue());
      case 5:
        return (float)(a.floatValue()*b.floatValue());
      case 6:
        return (double)(a.doubleValue()*b.doubleValue());
      case 7:
        return $cast(BigInteger.class,a).multiply($cast(BigInteger.class,b));
      case 8:
        return $cast(BigDecimal.class,a).multiply($cast(BigDecimal.class,b),MATH_CONTEXT);
      case 9:
        return $cast(Apint.class,a).multiply($cast(Apint.class,b));
      case 10:
        return $cast(Aprational.class,a).multiply($cast(Aprational.class,b));
      case 11:
        return $cast(Apfloat.class,a).multiply($cast(Apfloat.class,b));
      case 12:
        return $cast(Apcomplex.class,a).multiply($cast(Apcomplex.class,b));
      default:
        throw new IllegalStateException();
    }
  }
  
  // Division
  public static Number $opDividY(Object pOperand1, Object pOperand2) {
    Number a=$number(pOperand1),b=$number(pOperand2);
    Class<?> u=a.getClass(),v=b.getClass();
    int type=$getResultTypeId(u,v);
    if (type>=1&&type<=2) {
      if (!$opEqualY(b,0)&&!$opDivisY(b,a)) type=5;
    }
    else if (type>=3&&type<=4) {
      if (!$opEqualY(b,0)&&!$opDivisY(b,a)) type=6;
    }
    else if (type==7) {
      if (!$opEqualY(b,0)&&!$opDivisY(b,a)) type=8;
    }
    else if (type==9) {
      if (!$opEqualY(b,0)&&!$opDivisY(b,a)) type=10;
    }
    switch (type) {
      case 1:
        return (byte)(a.intValue()/b.intValue());
      case 2:
        return (short)(a.intValue()/b.intValue());
      case 3:
        return (int)(a.intValue()/b.intValue());
      case 4:
        return (long)(a.longValue()/b.longValue());
      case 5:
        return (float)(a.floatValue()/b.floatValue());
      case 6:
        return (double)(a.doubleValue()/b.doubleValue());
      case 7:
        return $cast(BigInteger.class,a).divide($cast(BigInteger.class,b));
      case 8:
        return $cast(BigDecimal.class,a).divide($cast(BigDecimal.class,b),MATH_CONTEXT);
      case 9:
        return $cast(Apint.class,a).divide($cast(Apint.class,b));
      case 10:
        return $cast(Aprational.class,a).divide($cast(Aprational.class,b));
      case 11:
        return $cast(Apfloat.class,a).divide($cast(Apfloat.class,b));
      case 12:
        return $cast(Apcomplex.class,a).divide($cast(Apcomplex.class,b));
      default:
        throw new IllegalStateException();
    }
  }
  
  // Integer residue
  public static byte $opModulY(Byte pOperand1, Byte pOperand2) {
    int x=pOperand1.intValue(),y=pOperand2.intValue(),r=x%y;
    return (byte)(r>=0?r:(y<0?r-y:r+y));
  }
  
  // Integer residue
  public static short $opModulY(Short pOperand1, Short pOperand2) {
    int x=pOperand1.intValue(),y=pOperand2.intValue(),r=x%y;
    return (short)(r>=0?r:(y<0?r-y:r+y));
  }
  
  // Integer residue
  public static int $opModulY(Integer pOperand1, Integer pOperand2) {
    int x=pOperand1.intValue(),y=pOperand2.intValue(),r=x%y;
    return r>=0?r:(y<0?r-y:r+y);
  }
  
  // Integer residue
  public static long $opModulY(Long pOperand1, Long pOperand2) {
    long x=pOperand1.longValue(),y=pOperand2.longValue(),r=x%y;
    return r>=0?r:(y<0?r-y:r+y);
  }
  
  // Integer residue
  public static Number $opModulY(Object pOperand1, Object pOperand2) {
    Number a=$number(pOperand1),b=$number(pOperand2);
    switch ($getResultTypeId(a,b)) {
      case 1: {
        int x=a.intValue(),y=b.intValue(),r=x%y;
        return (byte)(r>=0?r:(y<0?r-y:r+y));
      }
      case 2: {
        int x=a.intValue(),y=b.intValue(),r=x%y;
        return (short)(r>=0?r:(y<0?r-y:r+y));
      }
      case 3: {
        int x=a.intValue(),y=b.intValue(),r=x%y;
        return r>=0?r:(y<0?r-y:r+y);
      }
      case 4: {
        long x=a.longValue(),y=b.longValue(),r=x%y;
        return r>=0?r:(y<0?r-y:r+y);
      }
      case 7: {
        BigInteger x=$cast(BigInteger.class,a),y=$cast(BigInteger.class,b),r=x.mod(y);
        return r.compareTo(BigInteger.ZERO)>=0?r:r.add(y.abs());
      }
      case 9: {
        Apint x=$cast(Apint.class,a),y=$cast(Apint.class,b),r=x.mod(y);
        return r.compareTo(Apint.ZERO)>=0?r:r.add(ApintMath.abs(y));
      }
      default:
        throw new IllegalArgumentException("The operator 'remainder of integer division' only can be applied to integer numbers.");
    }
  }
  
  // Integer quotient
  public static byte $opQuotnY(Byte pOperand1, Byte pOperand2) {
    return (byte)(pOperand1/pOperand2);
  }
  
  // Integer quotient
  public static short $opQuotnY(Short pOperand1, Short pOperand2) {
    return (short)(pOperand1/pOperand2);
  }
  
  // Integer quotient
  public static int $opQuotnY(Integer pOperand1, Integer pOperand2) {
    return pOperand1/pOperand2;
  }
  
  // Integer quotient
  public static long $opQuotnY(Long pOperand1, Long pOperand2) {
    return pOperand1/pOperand2;
  }
  
  // Integer quotient
  public static Number $opQuotnY(Object pOperand1, Object pOperand2) {
    Number a=$number(pOperand1),b=$number(pOperand2);
    switch ($getResultTypeId(a,b)) {
      case 1:
        return (byte)(a.intValue()/b.intValue());
      case 2:
        return (short)(a.intValue()/b.intValue());
      case 3:
        return (int)(a.intValue()/b.intValue());
      case 4:
        return (long)(a.longValue()/b.longValue());
      case 7:
        return $cast(BigInteger.class,a).divide($cast(BigInteger.class,b));
      case 9:
        return $cast(Apint.class,a).divide($cast(Apint.class,b));
      default:
        throw new IllegalArgumentException("The operator 'quotient of integer division' only can be applied to integer numbers.");
    }
  }
  
  // Greatest common divisor
  public static Number $opGrtcdY(Object pOperand1, Object pOperand2) {
    Number a=$number(pOperand1),b=$number(pOperand2);
    switch ($getResultTypeId(a,b)) {
      case 1:
        return (byte)($gcd(a.longValue(),b.longValue()));
      case 2:
        return (short)($gcd(a.longValue(),b.longValue()));
      case 3:
        return (int)($gcd(a.longValue(),b.longValue()));
      case 4:
        return (long)($gcd(a.longValue(),b.longValue()));
      case 7:
        return $cast(BigInteger.class,a).gcd($cast(BigInteger.class,b));
      case 9:
        return ApintMath.gcd($cast(Apint.class,a),$cast(Apint.class,b));
      default:
        throw new IllegalArgumentException("The operator 'greatest common divisor' only can be applied to integer numbers.");
    }
  }
  
  // Least common multiple
  public static Number $opLstcmY(Object pOperand1, Object pOperand2) {
    Number a=$number(pOperand1),b=$number(pOperand2);
    switch ($getResultTypeId(a,b)) {
      case 1:
        return (byte)($lcm(a.longValue(),b.longValue()));
      case 2:
        return (short)($lcm(a.longValue(),b.longValue()));
      case 3:
        return (int)($lcm(a.longValue(),b.longValue()));
      case 4:
        return (long)($lcm(a.longValue(),b.longValue()));
      case 7: {
        BigInteger x=$cast(BigInteger.class,a),y=$cast(BigInteger.class,b);
        return x.multiply(y).divide(x.gcd(y));
      }
      case 9:
        return ApintMath.lcm($cast(Apint.class,a),$cast(Apint.class,b));
      default:
        throw new IllegalArgumentException("The operator 'least common multiple' only can be applied to integer numbers.");
    }
  }
  
  // Exponentiation
  public static Object $opExponY(Object pOperand1, Object pOperand2) {
    if (pOperand1 instanceof Iterable) {
      int exponent=$int(pOperand2);
      if (exponent<0) throw new IllegalArgumentException("The exponent of a cartesian power must be a non negative integer.");
      return GCollections.cartesianPower($set((Iterable<?>)pOperand1),exponent);
    }
    else {
      Number a=$number(pOperand1),b=$number(pOperand2);
      int type=$getResultTypeId(a,b);
      if (type>=1&&type<=2) {
        if ($opLesstY(b,0)) type=5;
      }
      else if (type>=3&&type<=4) {
        if ($opLesstY(b,0)) type=6;
      }
      else if (type==7) {
        if ($opLesstY(b,0)) type=8;
      }
      else if (type>=9&&type<=10) {
        if ($opLesstY(b,0)) type=11;
      }
      switch (type) {
        case 1:
          return (byte)($pow(a.intValue(),b.intValue()));
        case 2:
          return (short)($pow(a.intValue(),b.intValue()));
        case 3:
          return (int)($pow(a.intValue(),b.intValue()));
        case 4:
          return (long)($pow(a.longValue(),b.longValue()));
        case 5:
          return (float)(Math.pow(a.floatValue(),b.floatValue()));
        case 6:
          return (double)(Math.pow(a.doubleValue(),b.doubleValue()));
        case 7:
          return $cast(BigInteger.class,a).pow(b.intValue());
        case 8:
          return $cast(BigDecimal.class,a).pow(b.intValue(),MATH_CONTEXT);
        case 9:
          return ApintMath.pow($cast(Apint.class,a),b.longValue());
        case 10:
          return ApfloatMath.pow($cast(Apfloat.class,a),$cast(Apfloat.class,b));
        case 11:
          return ApfloatMath.pow($cast(Apfloat.class,a),$cast(Apfloat.class,b));
        case 12:
          return ApcomplexMath.pow($cast(Apcomplex.class,a),$cast(Apcomplex.class,b));
        default:
          throw new IllegalStateException();
      }
    }
  }
  
  // Union
  public static ICollection<?> $opUnionY(Object pOperand1, Object pOperand2) {
    if (((pOperand1 instanceof Set)||(pOperand1 instanceof ISet))&&((pOperand2 instanceof Set)||(pOperand2 instanceof ISet))) {
      ISet<Object> result=$newSet(pOperand1);
      result.union($set((Iterable<?>)pOperand2));
      return result;
    }
    else if ((pOperand1 instanceof IBag)&&(pOperand2 instanceof IBag)) {
      IBag<Object> result=$newBag(pOperand1);
      result.union($bag((Iterable<?>)pOperand2));
      return result;
    }
    else {
      throw new IllegalArgumentException("The operator 'union' must be applied to two sets or two bags.");
    }
  }
  
  // Intersection
  public static ICollection<?> $opInterY(Object pOperand1, Object pOperand2) {
    if (((pOperand1 instanceof Set)||(pOperand1 instanceof ISet))&&((pOperand2 instanceof Set)||(pOperand2 instanceof ISet))) {
      ISet<Object> result=$newSet(pOperand1);
      result.intersection($set((Iterable<?>)pOperand2));
      return result;
    }
    else if ((pOperand1 instanceof IBag)&&(pOperand2 instanceof IBag)) {
      IBag<Object> result=$newBag(pOperand1);
      result.intersection($bag((Iterable<?>)pOperand2));
      return result;
    }
    else {
      throw new IllegalArgumentException("The operator 'intersection' must be applied to two sets or two bags.");
    }
  }
  
  // Difference
  public static ICollection<?> $opDiffeY(Object pOperand1, Object pOperand2) {
    if (((pOperand1 instanceof Set)||(pOperand1 instanceof ISet))&&((pOperand2 instanceof Set)||(pOperand2 instanceof ISet))) {
      ISet<Object> result=$newSet(pOperand1);
      result.difference($set((Iterable<?>)pOperand2));
      return result;
    }
    else if ((pOperand1 instanceof IBag)&&(pOperand2 instanceof IBag)) {
      IBag<Object> result=$newBag(pOperand1);
      result.difference($bag((Iterable<?>)pOperand2));
      return result;
    }
    else {
      throw new IllegalArgumentException("The operator 'difference' must be applied to two sets or two bags.");
    }
  }
  
  // Symmetric difference
  public static ICollection<?> $opSymmdY(Object pOperand1, Object pOperand2) {
    if (((pOperand1 instanceof Set)||(pOperand1 instanceof ISet))&&((pOperand2 instanceof Set)||(pOperand2 instanceof ISet))) {
      ISet<Object> result=$newSet(pOperand1);
      result.symmetricDifference($set((Iterable<?>)pOperand2));
      return result;
    }
    else if ((pOperand1 instanceof IBag)&&(pOperand2 instanceof IBag)) {
      IBag<Object> result=$newBag(pOperand1);
      result.symmetricDifference($bag((Iterable<?>)pOperand2));
      return result;
    }
    else {
      throw new IllegalArgumentException("The operator 'symmetric difference' must be applied to two sets or two bags.");
    }
  }
  
  // Cartesian product
  public static ICollection<?> $opCartsY(Object[] pOperands) {
    IList<ISet<?>> operands=new GArrayList<ISet<?>>(pOperands.length);
    for (Object operand:pOperands) {
      if (!(operand instanceof Iterable)) throw new IllegalArgumentException("The operator 'cartesian product' must be applied over collections.");
      operands.addLast($set((Iterable<?>)operand));
    }
    if (operands.size()==2) {
      return GCollections.<Object,Object>binaryCartesianProduct(operands.get(0),operands.get(1));
    }
    else {
      return GCollections.<Object>cartesianProduct(operands);
    }
  }
  
  // Plus sign (prefix unary)
  public static Number $opPlussY(Object pOperand) {
    return $number(pOperand);
  }
  
  // Minus sign (prefix unary)
  public static Number $opMinusY(Object pOperand) {
    Number a=$number(pOperand);
    switch ($getResultTypeId(a)) {
      case 1:
        return (byte)(-a.intValue());
      case 2:
        return (short)(-a.intValue());
      case 3:
        return (int)(-a.intValue());
      case 4:
        return (long)(-a.longValue());
      case 5:
        return (float)(-a.floatValue());
      case 6:
        return (double)(-a.doubleValue());
      case 7:
        return $cast(BigInteger.class,a).negate();
      case 8:
        return $cast(BigDecimal.class,a).negate(MATH_CONTEXT);
      case 9:
        return $cast(Apint.class,a).negate();
      case 10:
        return $cast(Aprational.class,a).negate();
      case 11:
        return $cast(Apfloat.class,a).negate();
      case 12:
        return $cast(Apcomplex.class,a).negate();
      default:
        throw new IllegalStateException();
    }
  }
  
  // Cardinality (prefix unary)
  public static Integer $opCardiY(Object pOperand) {
    if (pOperand instanceof Iterable) {
      return GCollections.sizeOf((Iterable<?>)pOperand);
    }
    else if (pOperand!=null&&pOperand.getClass().isArray()) {
      return Array.getLength(pOperand);
    }
    else {
      return GCollections.unmodifiableCollection(pOperand).size();
    }
  }
  
  // Complement (prefix unary)
  public static ISet<?> $opComplY(Object pOperand) {
    if ((pOperand instanceof Set)||(pOperand instanceof ISet)) {
      ISet<?> result=$newSet(pOperand);
      result.complement();
      return result;
    }
    else {
      throw new IllegalArgumentException("The operator 'complement' must be applied to a set.");
    }
  }
  
  // Power set (prefix unary)
  public static ISet<?> $opPwsetY(Object pOperand) {
    if ((pOperand instanceof Set)||(pOperand instanceof ISet)) {
      return GCollections.<Object>powerSet($set((Iterable<?>)pOperand));
    }
    else {
      throw new IllegalArgumentException("The operator 'power set' must be applied to a set.");
    }
  }
  
  // Factorial (suffix unary)
  public static Number $opFactrY(Object pOperand) {
    Number a=$number(pOperand);
    switch ($getResultTypeId(a)) {
      case 1:
      case 2:
      case 3:
      case 4:
      case 7:
      case 9:
        return ApintMath.factorial(a.longValue());
      default:
        throw new IllegalArgumentException("The factorial function only can be applied to an integer number.");
    }
  }
  
  // Cardinality (brackets) / Absolute value
  public static Number $opAbsolY(Object pOperand) {
    if ((pOperand instanceof Number)||(pOperand instanceof Character)) {
      Number a=$number(pOperand);
      switch ($getResultTypeId(a)) {
        case 1:
          return (byte)(Math.abs(a.intValue()));
        case 2:
          return (short)(Math.abs(a.intValue()));
        case 3:
          return (int)(Math.abs(a.intValue()));
        case 4:
          return (long)(Math.abs(a.longValue()));
        case 5:
          return (float)(Math.abs(a.floatValue()));
        case 6:
          return (double)(Math.abs(a.doubleValue()));
        case 7:
          return $cast(BigInteger.class,a).abs();
        case 8:
          return $cast(BigDecimal.class,a).abs(MATH_CONTEXT);
        case 9:
          return ApintMath.abs($cast(Apint.class,a));
        case 10:
          return AprationalMath.abs($cast(Aprational.class,a));
        case 11:
          return ApfloatMath.abs($cast(Apfloat.class,a));
        case 12:
          return ApcomplexMath.abs($cast(Apcomplex.class,a));
        default:
          throw new IllegalStateException();
      }
    }
    else {
      return $opCardiY(pOperand);
    }
  }
  
  // Floor (brackets)
  public static Number $opFloorY(Object pOperand) {
    Number a=$number(pOperand);
    switch ($getResultTypeId(a)) {
      case 1:
        return (byte)(a.byteValue());
      case 2:
        return (short)(a.shortValue());
      case 3:
        return (int)(a.intValue());
      case 4:
        return (long)(a.longValue());
      case 5: {
        float x=a.floatValue();
        return (int)(Float.isInfinite(x)?(x>0?Integer.MAX_VALUE:Integer.MIN_VALUE):(Float.isNaN(x)?0:Math.round(Math.floor(x+1E-16))));
      }
      case 6: {
        double x=a.doubleValue();
        return (long)(Double.isInfinite(x)?(x>0?Long.MAX_VALUE:Long.MIN_VALUE):(Double.isNaN(x)?0L:Math.round(Math.floor(x+1E-16))));
      }
      case 7:
        return $cast(BigInteger.class,a);
      case 8:
        return $cast(BigDecimal.class,a).setScale(0,RoundingMode.FLOOR);
      case 9:
        return $cast(Apint.class,a);
      case 10:
        return $cast(Aprational.class,a).floor();
      case 11:
        return $cast(Apfloat.class,a).floor();
      case 12:
        throw new IllegalArgumentException("The operator 'floor' cannot be applied to complex numbers.");
      default:
        throw new IllegalStateException();
    }
  }
  
  // Ceiling (brackets)
  public static Number $opCeilnY(Object pOperand) {
    Number a=$number(pOperand);
    switch ($getResultTypeId(a)) {
      case 1:
        return (byte)(a.byteValue());
      case 2:
        return (short)(a.shortValue());
      case 3:
        return (int)(a.intValue());
      case 4:
        return (long)(a.longValue());
      case 5: {
        float x=a.floatValue();
        return (int)(Float.isInfinite(x)?(x>0?Integer.MAX_VALUE:Integer.MIN_VALUE):(Float.isNaN(x)?0:Math.round(Math.ceil(x-1E-16))));
      }
      case 6: {
        double x=a.doubleValue();
        return (long)(Double.isInfinite(x)?(x>0?Long.MAX_VALUE:Long.MIN_VALUE):(Double.isNaN(x)?0L:Math.round(Math.ceil(x-1E-16))));
      }
      case 7:
        return $cast(BigInteger.class,a);
      case 8:
        return $cast(BigDecimal.class,a).setScale(0,RoundingMode.CEILING);
      case 9:
        return $cast(Apint.class,a);
      case 10:
        return $cast(Aprational.class,a).ceil();
      case 11:
        return $cast(Apfloat.class,a).ceil();
      case 12:
        throw new IllegalArgumentException("The operator 'ceiling' cannot be applied to complex numbers.");
      default:
        throw new IllegalStateException();
    }
  }
  
  // Maximum (max)
  public static Number $fnMax(Object[] pOperands) {
    ICollection<?> collection=pOperands.length==1?GCollections.unmodifiableCollection(pOperands[0]):GCollections.asCollection(pOperands);
    if (collection.isEmpty()) return Double.NEGATIVE_INFINITY;
    Iterator<?> it=collection.iterator();
    Number number=$number(it.next());
    while (it.hasNext()) {
      number=$opMaximY(number,it.next());
    }
    return number;
  }
  
  // Minimum (min)
  public static Number $fnMin(Object[] pOperands) {
    ICollection<?> collection=pOperands.length==1?GCollections.unmodifiableCollection(pOperands[0]):GCollections.asCollection(pOperands);
    if (collection.isEmpty()) return Double.POSITIVE_INFINITY;
    Iterator<?> it=collection.iterator();
    Number number=$number(it.next());
    while (it.hasNext()) {
      number=$opMinimY(number,it.next());
    }
    return number;
  }
  
  // Greatest common divisor (gcd)
  public static Number $fnGcd(Object[] pOperands) {
    return $opGrtcdY($number(pOperands[0]),$number(pOperands[1]));
  }
  
  // Least common multiple (lcm)
  public static Number $fnLcm(Object[] pOperands) {
    return $opLstcmY($number(pOperands[0]),$number(pOperands[1]));
  }
  
  // Absolute value (abs)
  public static Number $fnAbs(Object[] pOperands) {
    return $opAbsolY($number(pOperands[0]));
  }
  
  // Power (pow)
  public static Number $fnPow(Object[] pOperands) {
    return $number($opExponY($number(pOperands[0]),$number(pOperands[1])));
  }
  
  // Square root (sqrt)
  public static Number $fnSqrt(Object[] pOperands) {
    return $invokeUnaryFunction("sqrt",pOperands[0]);
  }
  
  // Cube root (cbrt)
  public static Number $fnCbrt(Object[] pOperands) {
    return $invokeUnaryFunction("cbrt",pOperands[0]);
  }
  
  // Root (root)
  public static Number $fnRoot(Object[] pOperands) {
    return $invokeBinaryFunction("root",pOperands[0],pOperands[1]);
  }
  
  // Natural logarithm (ln)
  public static Number $fnLn(Object[] pOperands) {
    return $invokeUnaryFunction("ln",pOperands[0]);
  }
  
  // Logarithm (log)
  public static Number $fnLog(Object[] pOperands) {
    return $invokeBinaryFunction("log",pOperands[0],pOperands[1]);
  }
  
  // Exponential function (exp)
  public static Number $fnExp(Object[] pOperands) {
    return $invokeUnaryFunction("exp",pOperands[0]);
  }
  
  // Sine function (sin)
  public static Number $fnSin(Object[] pOperands) {
    return $invokeUnaryFunction("sin",pOperands[0]);
  }
  
  // Cosine function (cos)
  public static Number $fnCos(Object[] pOperands) {
    return $invokeUnaryFunction("cos",pOperands[0]);
  }
  
  // Tangent function (tan)
  public static Number $fnTan(Object[] pOperands) {
    return $invokeUnaryFunction("tan",pOperands[0]);
  }
  
  // Hyperbolic sine function (sinh)
  public static Number $fnSinh(Object[] pOperands) {
    return $invokeUnaryFunction("sinh",pOperands[0]);
  }
  
  // Hyperbolic cosine function (cosh)
  public static Number $fnCosh(Object[] pOperands) {
    return $invokeUnaryFunction("cosh",pOperands[0]);
  }
  
  // Hyperbolic tangent function (tanh)
  public static Number $fnTanh(Object[] pOperands) {
    return $invokeUnaryFunction("tanh",pOperands[0]);
  }
  
  // Inverse sine function (asin)
  public static Number $fnAsin(Object[] pOperands) {
    return $invokeUnaryFunction("asin",pOperands[0]);
  }
  
  // Inverse cosine function (acos)
  public static Number $fnAcos(Object[] pOperands) {
    return $invokeUnaryFunction("acos",pOperands[0]);
  }
  
  // Inverse tangent function (atan)
  public static Number $fnAtan(Object[] pOperands) {
    return $invokeUnaryFunction("atan",pOperands[0]);
  }
  
  // Inverse hyperbolic sine function (asinh)
  public static Number $fnAsinh(Object[] pOperands) {
    return $invokeUnaryFunction("asinh",pOperands[0]);
  }
  
  // Inverse hyperbolic cosine function (acosh)
  public static Number $fnAcosh(Object[] pOperands) {
    return $invokeUnaryFunction("acosh",pOperands[0]);
  }
  
  // Inverse hyperbolic tangent function (atanh)
  public static Number $fnAtanh(Object[] pOperands) {
    return $invokeUnaryFunction("atanh",pOperands[0]);
  }
  
  private static Number $invokeUnaryFunction(String pFunction, Object pOperand) {
    Number a=$number(pOperand);
    int t=$getResultTypeId(a);
    if (t<7) {
      double v=a.doubleValue();
      if (pFunction.equals("sqrt")) return Math.sqrt(v);
      if (pFunction.equals("cbrt")) return Math.pow(v,1.0/3);
      if (pFunction.equals("ln")) return Math.log(v);
      if (pFunction.equals("exp")) return Math.exp(v);
      if (pFunction.equals("sin")) return Math.sin(v);
      if (pFunction.equals("cos")) return Math.cos(v);
      if (pFunction.equals("tan")) return Math.tan(v);
      if (pFunction.equals("sinh")) return Math.sinh(v);
      if (pFunction.equals("cosh")) return Math.cosh(v);
      if (pFunction.equals("tanh")) return Math.tanh(v);
      if (pFunction.equals("asin")) return Math.asin(v);
      if (pFunction.equals("acos")) return Math.acos(v);
      if (pFunction.equals("atan")) return Math.atan(v);
      if (pFunction.equals("asinh")) return Math.log(v+Math.sqrt(v*v+1));
      if (pFunction.equals("acosh")) return Math.log(v+Math.sqrt(v+1)*Math.sqrt(v-1));
      if (pFunction.equals("atanh")) return Math.log((1+v)/(1-v))/2;
    }
    else if (t<12) {
      Apfloat v=$precision($cast(Apfloat.class,a));
      if (pFunction.equals("sqrt")) return ApfloatMath.sqrt(v);
      if (pFunction.equals("cbrt")) return ApfloatMath.cbrt(v);
      if (pFunction.equals("ln")) return ApfloatMath.log(v);
      if (pFunction.equals("exp")) return ApfloatMath.exp(v);
      if (pFunction.equals("sin")) return ApfloatMath.sin(v);
      if (pFunction.equals("cos")) return ApfloatMath.cos(v);
      if (pFunction.equals("tan")) return ApfloatMath.tan(v);
      if (pFunction.equals("sinh")) return ApfloatMath.sinh(v);
      if (pFunction.equals("cosh")) return ApfloatMath.cosh(v);
      if (pFunction.equals("tanh")) return ApfloatMath.tanh(v);
      if (pFunction.equals("asin")) return ApfloatMath.asin(v);
      if (pFunction.equals("acos")) return ApfloatMath.acos(v);
      if (pFunction.equals("atan")) return ApfloatMath.atan(v);
      if (pFunction.equals("asinh")) return ApfloatMath.asinh(v);
      if (pFunction.equals("acosh")) return ApfloatMath.acosh(v);
      if (pFunction.equals("atanh")) return ApfloatMath.atanh(v);
    }
    else {
      Apcomplex v=$precision($cast(Apcomplex.class,a));
      if (pFunction.equals("sqrt")) return ApcomplexMath.sqrt(v);
      if (pFunction.equals("cbrt")) return ApcomplexMath.cbrt(v);
      if (pFunction.equals("ln")) return ApcomplexMath.log(v);
      if (pFunction.equals("exp")) return ApcomplexMath.exp(v);
      if (pFunction.equals("sin")) return ApcomplexMath.sin(v);
      if (pFunction.equals("cos")) return ApcomplexMath.cos(v);
      if (pFunction.equals("tan")) return ApcomplexMath.tan(v);
      if (pFunction.equals("sinh")) return ApcomplexMath.sinh(v);
      if (pFunction.equals("cosh")) return ApcomplexMath.cosh(v);
      if (pFunction.equals("tanh")) return ApcomplexMath.tanh(v);
      if (pFunction.equals("asin")) return ApcomplexMath.asin(v);
      if (pFunction.equals("acos")) return ApcomplexMath.acos(v);
      if (pFunction.equals("atan")) return ApcomplexMath.atan(v);
      if (pFunction.equals("asinh")) return ApcomplexMath.asinh(v);
      if (pFunction.equals("acosh")) return ApcomplexMath.acosh(v);
      if (pFunction.equals("atanh")) return ApcomplexMath.atanh(v);
    }
    return null;
  }
  
  private static Number $invokeBinaryFunction(String pFunction, Object pOperand1, Object pOperand2) {
    Number a=$number(pOperand1),b=$number(pOperand2);
    int t=$getResultTypeId(a,b);
    if (t<7) {
      double v=a.doubleValue(),w=b.doubleValue();
      if (pFunction.equals("root")) return Math.pow(v,1.0/w);
      if (pFunction.equals("log")) return Math.log(v)/Math.log(w);
    }
    else if (t<12) {
      Apfloat v=$precision($cast(Apfloat.class,a)),w=$precision($cast(Apfloat.class,b));
      if (pFunction.equals("root")) return ApfloatMath.pow(v,new Apfloat(1.0,w.precision()).divide(w));
      if (pFunction.equals("log")) return ApfloatMath.log(v,w);
    }
    else {
      Apcomplex v=$precision($cast(Apcomplex.class,a)),w=$precision($cast(Apcomplex.class,b));
      if (pFunction.equals("root")) return ApcomplexMath.pow(v,new Apfloat(1.0,w.precision()).divide(w));
      if (pFunction.equals("log")) return ApcomplexMath.log(v,w);
    }
    return null;
  }
  
  /*
   * Collection creation methods
   */
  
  private static <E> IList<E> $list(Iterable<E> pIterable) {
    return (pIterable instanceof IList)?(IList<E>)pIterable:GCollections.unmodifiableList(pIterable);
  }
  
  public static IList<Object> $newList() {
    return new GMutableList();
  }
  
  public static IList<Object> $newList(int pCapacity) {
    return new GMutableList(pCapacity);
  }
  
  public static IList<Object> $newList(Object pObject) {
    if (pObject instanceof GMutableList) {
      return (GMutableList)pObject;
    }
    else if (pObject instanceof Collection) {
      return new GMutableList((Collection<?>)pObject);
    }
    else if (pObject instanceof ICollection) {
      return new GMutableList((ICollection<?>)pObject);
    }
    else {
      return new GMutableList(GCollections.unmodifiableCollection(pObject));
    }
  }
  
  private static <E> ISet<E> $set(Iterable<E> pIterable) {
    return (pIterable instanceof ISet)?(ISet<E>)pIterable:GCollections.unmodifiableSet(pIterable);
  }
  
  public static ISet<Object> $newSet() {
    return new GMutableSet();
  }
  
  public static ISet<Object> $newSet(int pCapacity) {
    return new GMutableSet(pCapacity);
  }
  
  public static ISet<Object> $newSet(Object pObject) {
    if (pObject instanceof GMutableSet) {
      return (GMutableSet)pObject;
    }
    else if (pObject instanceof Collection) {
      return new GMutableSet((Collection<?>)pObject);
    }
    else if (pObject instanceof ICollection) {
      return new GMutableSet((ICollection<?>)pObject);
    }
    else {
      return new GMutableSet(GCollections.unmodifiableCollection(pObject));
    }
  }
  
  private static <E> IBag<E> $bag(Iterable<E> pIterable) {
    return (pIterable instanceof IBag)?(IBag<E>)pIterable:GCollections.unmodifiableBag(pIterable);
  }
  
  public static IBag<Object> $newBag() {
    return new GMutableBag();
  }
  
  public static IBag<Object> $newBag(int pCapacity) {
    return new GMutableBag(pCapacity);
  }
  
  public static IBag<Object> $newBag(Object pObject) {
    if (pObject instanceof GMutableBag) {
      return (GMutableBag)pObject;
    }
    else if (pObject instanceof Collection) {
      return new GMutableBag((Collection<?>)pObject);
    }
    else if (pObject instanceof ICollection) {
      return new GMutableBag((ICollection<?>)pObject);
    }
    else {
      return new GMutableBag(GCollections.unmodifiableCollection(pObject));
    }
  }
  
  /*
   * Method reflection
   */
  
  public static Object $invokeMethod(Method pMethod, Object pObject, Object[] pArguments) {
    try {
      if (!pMethod.isAccessible()&&Modifier.isPublic(pMethod.getModifiers())) pMethod.setAccessible(true);
      return pMethod.invoke(pObject,pArguments);
    }
    catch (Throwable th) {
      if (th instanceof InvocationTargetException) th=((InvocationTargetException)th).getCause();
      if (th instanceof RuntimeException) throw (RuntimeException)th;
      throw new RuntimeException(th);
    }
  }
  
  public static Object $invokeMethod(String pMethodName, Object pObject, Object[] pArguments) {
    return $invokeMethod(pObject.getClass(),pMethodName,false,pObject,pArguments);
  }
  
  public static Object $invokeMethod(Class<?> pDeclaringClass, String pMethodName, boolean pIsStatic, Object pObject, Object[] pArguments) {
    Method method=$searchMethod(pDeclaringClass,pMethodName,pIsStatic,pArguments.length);
    if (method==null) method=$searchMethod(pDeclaringClass,pMethodName,pIsStatic,$getTypes(pArguments));
    return $invokeMethod(method,pObject,$getArguments(method.getParameterTypes(),pArguments));
  }
  
  public static Method $searchMethod(Class<?> pDeclaringClass, String pMethodName, boolean pIsStatic, int pArity) {
    GAddress1 address=new GAddress1(pDeclaringClass,pMethodName,pIsStatic,pArity);
    Method result=(Method)(CACHE_LRU1.get(address));
    if (result!=null) return result;
    int count=0;
    for (Method method:pDeclaringClass.getMethods()) {
      if (Modifier.isStatic(method.getModifiers())==pIsStatic&&Modifier.isPublic(method.getModifiers())&&method.getName().equals(pMethodName)&&(pArity==-1||method.getParameterTypes().length==pArity)) {
        result=method;
        if (++count>=2) return null;
      }
      
    }
    if (result!=null) CACHE_LRU1.put(address,result);
    return result;
  }
  
  public static Method $searchMethod(Class<?> pDeclaringClass, String pMethodName, boolean pIsStatic, Class<?>[] pParameterTypes) {
    if (pDeclaringClass==null) throw new NullPointerException();
    if (pMethodName==null||pMethodName.trim().isEmpty()) throw new IllegalArgumentException("The method name '"+pMethodName+"' is not valid.");
    GAddress2 address=new GAddress2(pDeclaringClass,pMethodName,pIsStatic,pParameterTypes);
    Method answer=(Method)(CACHE_LRU2.get(address));
    if (answer!=null) return answer;
    IList<Method> filteredMethods=new GArrayList<Method>(4);
    IList<Method> allMethods=new GArrayList<Method>(8);
    for (Method method:pDeclaringClass.getMethods()) {
      if ((!pIsStatic||Modifier.isStatic(method.getModifiers()))&&Modifier.isPublic(method.getModifiers())&&method.getName().equals(pMethodName)) {
        if (pParameterTypes==null||$isCompatibleWith(method.getParameterTypes(),pParameterTypes)) {
          filteredMethods.addLast(method);
        }
        allMethods.addLast(method);
      }
    }
    IList<Method> result=new GArrayList<Method>(filteredMethods.size());
    if (pParameterTypes==null) {
      result.concatenate(filteredMethods);
    }
    else {
      cycle:for (Method method1:filteredMethods) {
        for (Method method2:filteredMethods) {
          if (method1!=method2) {
            if (GArrays.equals(method1.getParameterTypes(),method2.getParameterTypes())) {
              if ($isCompatibleWith(method1.getReturnType(),method2.getReturnType())) {
                continue cycle;
              }
            }
            else if ($isCompatibleWith(method1.getParameterTypes(),method2.getParameterTypes())) {
              continue cycle;
            }
          }
        }
        result.addLast(method1);
      }
    }
    if (result.size()>=2||(result.isEmpty()&&filteredMethods.size()>=2)) {
      throw new IllegalArgumentException("The "+(pIsStatic?"static ":"")+"method "+pMethodName+$parametersToString(pParameterTypes)+" is ambiguous for the type "+pDeclaringClass.getCanonicalName()+".");
    }
    else if (result.isEmpty()) {
      if (allMethods.isEmpty()) {
        throw new IllegalArgumentException("The "+(pIsStatic?"static ":"")+"method "+pMethodName+$parametersToString(pParameterTypes)+" is undefined for the type "+pDeclaringClass.getCanonicalName()+".");
      }
      else if (allMethods.size()==1) {
        throw new IllegalArgumentException("The "+(pIsStatic?"static ":"")+"method "+pMethodName+$parametersToString(allMethods.pick().getParameterTypes())+" in the type "+pDeclaringClass.getCanonicalName()+" is not applicable for the arguments "+$parametersToString(pParameterTypes)+".");
      }
      else {
        throw new IllegalArgumentException("The "+(pIsStatic?"static ":"")+"method "+pMethodName+" in the type "+pDeclaringClass.getCanonicalName()+" is not applicable for the arguments "+$parametersToString(pParameterTypes)+".");
      }
    }
    else {
      answer=result.pick();
      CACHE_LRU2.put(address,answer);
      return answer;
    }
  }
  
  /*
   * Constructor reflection
   */
  
  public static <T> T $invokeConstructor(Constructor<T> pConstructor, Object[] pArguments) {
    try {
      if (!pConstructor.isAccessible()&&Modifier.isPublic(pConstructor.getModifiers())) pConstructor.setAccessible(true);
      return pConstructor.newInstance(pArguments);
    }
    catch (Throwable th) {
      if (th instanceof InvocationTargetException) th=((InvocationTargetException)th).getCause();
      if (th instanceof RuntimeException) throw (RuntimeException)th;
      throw new RuntimeException(th);
    }
  }
  
  public static <T> T $invokeConstructor(Class<T> pDeclaringClass, Object[] pArguments) {
    Constructor<T> constructor=$searchConstructor(pDeclaringClass,pArguments.length);
    if (constructor==null) constructor=$searchConstructor(pDeclaringClass,$getTypes(pArguments));
    return $invokeConstructor(constructor,$getArguments(constructor.getParameterTypes(),pArguments));
  }
  
  public static <T> Constructor<T> $searchConstructor(Class<T> pDeclaringClass, int pArity) {
    GAddress1 address=new GAddress1(pDeclaringClass,"~init~",false,pArity);
    Constructor<T> result=(Constructor<T>)(CACHE_LRU1.get(address));
    if (result!=null) return result;
    int count=0;
    for (Constructor<?> constructor:pDeclaringClass.getConstructors()) {
      if (Modifier.isPublic(constructor.getModifiers())&&(pArity==-1||constructor.getParameterTypes().length==pArity)) {
        result=(Constructor<T>)constructor;
        if (++count>=2) return null;
      }
    }
    if (result!=null) CACHE_LRU1.put(address,result);
    return result;
  }
  
  public static <T> Constructor<T> $searchConstructor(Class<T> pDeclaringClass, Class<?>[] pParameterTypes) {
    if (pDeclaringClass==null) throw new NullPointerException();
    GAddress2 address=new GAddress2(pDeclaringClass,"~init~",false,pParameterTypes);
    Constructor<T> answer=(Constructor<T>)(CACHE_LRU2.get(address));
    if (answer!=null) return answer;
    IList<Constructor<?>> filteredConstructors=new GArrayList<Constructor<?>>(4);
    IList<Constructor<?>> allConstructors=new GArrayList<Constructor<?>>(8);
    for (Constructor<?> constructor:pDeclaringClass.getConstructors()) {
      if (Modifier.isPublic(constructor.getModifiers())) {
        if (pParameterTypes==null||$isCompatibleWith(constructor.getParameterTypes(),pParameterTypes)) {
          filteredConstructors.addLast(constructor);
        }
        allConstructors.addLast(constructor);
      }
    }
    IList<Constructor<?>> result=new GArrayList<Constructor<?>>(filteredConstructors.size());
    if (pParameterTypes==null) {
      result.concatenate(filteredConstructors);
    }
    else {
      cycle:for (Constructor<?> constructor1:filteredConstructors) {
        for (Constructor<?> constructor2:filteredConstructors) {
          if (constructor1!=constructor2&&$isCompatibleWith(constructor1.getParameterTypes(),constructor2.getParameterTypes())) {
            continue cycle;
          }
        }
        result.addLast(constructor1);
      }
    }
    if (result.size()>=2||(result.isEmpty()&&filteredConstructors.size()>=2)) {
      throw new IllegalArgumentException("The constructor "+pDeclaringClass.getCanonicalName()+$parametersToString(pParameterTypes)+" is ambiguous.");
    }
    else if (result.isEmpty()) {
      if (allConstructors.size()!=1) {
        throw new IllegalArgumentException("The constructor "+pDeclaringClass.getCanonicalName()+$parametersToString(pParameterTypes)+" is undefined.");
      }
      else {
        throw new IllegalArgumentException("The constructor "+pDeclaringClass.getCanonicalName()+$parametersToString(allConstructors.pick().getParameterTypes())+" in the type "+pDeclaringClass.getCanonicalName()+" is not applicable for the arguments "+$parametersToString(pParameterTypes)+".");
      }
    }
    else {
      answer=(Constructor<T>)(result.pick());
      CACHE_LRU2.put(address,answer);
      return answer;
    }
  }
  
  /*
   * Field reflection
   */
  
  public static Object $invokeField(String pFieldName, Object pObject) {
    return $invokeField(pObject.getClass(),pFieldName,false,pObject);
  }
  
  public static Object $invokeField(Class<?> pDeclaringClass, String pFieldName, boolean pIsStatic, Object pObject) {
    if (pIsStatic&&pFieldName.equals("class")) {
      return pDeclaringClass;
    }
    else if (!pIsStatic&&pFieldName.equals("class")&&(pObject instanceof Class)) {
      return pObject;
    }
    else if (pDeclaringClass.isArray()&&pFieldName.equals("length")) {
      return Array.getLength(pObject);
    }
    else {
      Field field=$searchField(pDeclaringClass,pFieldName,pIsStatic);
      try {
        if (!field.isAccessible()&&Modifier.isPublic(field.getModifiers())) field.setAccessible(true);
        return field.get(pObject);
      }
      catch (Throwable th) {
        if (th instanceof InvocationTargetException) th=((InvocationTargetException)th).getCause();
        if (th instanceof RuntimeException) throw (RuntimeException)th;
        throw new RuntimeException(th);
      }
    }
  }
  
  public static Field $searchField(Class<?> pDeclaringClass, String pFieldName, boolean pIsStatic) {
    if (pFieldName==null||pFieldName.trim().isEmpty()) throw new IllegalArgumentException("The field name '"+pFieldName+"' is not valid.");
    Field field=null;
    try {
      field=pDeclaringClass.getField(pFieldName);
    }
    catch (Throwable th) {
    }
    if (field!=null&&(!pIsStatic||Modifier.isStatic(field.getModifiers()))&&Modifier.isPublic(field.getModifiers())) {
      return field;
    }
    else {
      throw new IllegalArgumentException("The "+(pIsStatic?"static ":"")+"field "+pFieldName+" is undefined for the type "+pDeclaringClass.getCanonicalName()+".");
    }
  }
  
  /*
   * Array reflection
   */
  
  public static Object $getArrayValue(Object pArray, Object[] pIndices) {
    Object object=pArray;
    for (Object index:pIndices) {
      if (object==null||index==null) {
        throw new NullPointerException();
      }
      IPair<Integer> endPoints=!(object instanceof Map)&&!(object instanceof IMap)&&(index instanceof Iterable)?GCollections.endPointsOfIntegerInterval((Iterable<?>)index):null;
      if (endPoints!=null) {
        int fromIndex=endPoints.getFirst(),toIndex=endPoints.getSecond()+1;
        if (object.getClass().isArray()) {
          object=GArrays.copyOfRange(object,fromIndex,toIndex);
        }
        else if (object instanceof List) {
          object=((List<?>)object).subList(fromIndex,toIndex);
        }
        else if (object instanceof IList) {
          object=((IList<?>)object).sublist(fromIndex,toIndex);
        }
        else if (object instanceof ITuple) {
          object=GCollections.unmodifiableList((ITuple<?>)object).sublist(fromIndex,toIndex);
        }
        else if (object instanceof CharSequence) {
          object=((CharSequence)object).subSequence(fromIndex,toIndex);
        }
        else {
          throw new IllegalArgumentException(pArray+" is not an array.");
        }
      }
      else {
        if (object.getClass().isArray()) {
          object=Array.get(object,$int(index));
        }
        else if (object instanceof ITuple) { // ITuple, IList
          object=((ITuple<?>)object).get($int(index));
        }
        else if (object instanceof List) { // List
          object=((List<?>)object).get($int(index));
        }
        else if (object instanceof IMap) { // IMap, IMultiMap
          object=((IMap<?,?>)object).get(index);
        }
        else if (object instanceof Map) { // Map
          object=((Map<?,?>)object).get(index);
        }
        else if (object instanceof Iterable) {
          object=GCollections.unmodifiableList((Iterable<?>)object).get($int(index));
        }
        else if (object instanceof CharSequence) {
          object=((CharSequence)object).charAt($int(index));
        }
        else {
          throw new IllegalArgumentException(pArray+" is not an array.");
        }
      }
    }
    return object;
  }
  
  public static void $setArrayValue(Object pArray, Object pValue, Object[] pIndices) {
    if (pIndices.length==0) throw new IllegalArgumentException("There should be at least one index to access the array.");
    Object object=$getArrayValue(pArray,Arrays.copyOfRange(pIndices,0,pIndices.length-1));
    Object index=pIndices[pIndices.length-1];
    if (object==null||index==null) {
      throw new NullPointerException();
    }
    else if (object.getClass().isArray()) {
      Array.set(object,$int(index),$cast(object.getClass().getComponentType(),pValue));
    }
    else if (object instanceof ITuple) { // ITuple, IList
      $setValue((ITuple<?>)object,$int(index),pValue);
    }
    else if (object instanceof List) { // List
      $setValue((List<?>)object,$int(index),pValue);
    }
    else if (object instanceof IMap) { // IMap, IMultiMap
      $setValue((IMap<?,?>)object,index,pValue);
    }
    else if (object instanceof Map) { // Map
      $setValue((Map<?,?>)object,index,pValue);
    }
    else {
      throw new IllegalArgumentException(pArray+" is not an array.");
    }
  }
  
  private static <E> void $setValue(ITuple<E> pTuple, int pIndex, Object pValue) {
    pTuple.set(pIndex,(E)pValue);
  }
  
  private static <E> void $setValue(List<E> pList, int pIndex, Object pValue) {
    pList.set(pIndex,(E)pValue);
  }
  
  private static <K,V> void $setValue(IMap<K,V> pMap, Object pKey, Object pValue) {
    pMap.put((K)pKey,(V)pValue);
  }
  
  private static <K,V> void $setValue(Map<K,V> pMap, Object pKey, Object pValue) {
    pMap.put((K)pKey,(V)pValue);
  }
  
  /*
   * Type conversion
   */
  
  public static Class<?> $getPrimitiveType(Class<?> pType) {
    // if (pType.isPrimitive()) return pType;
    if (pType==Boolean.class) return boolean.class;
    if (pType==Character.class) return char.class;
    if (pType==Byte.class) return byte.class;
    if (pType==Short.class) return short.class;
    if (pType==Integer.class) return int.class;
    if (pType==Long.class) return long.class;
    if (pType==Float.class) return float.class;
    if (pType==Double.class) return double.class;
    if (pType==Void.class) return void.class;
    return pType;
  }
  
  public static Class<?> $getNonprimitiveType(Class<?> pType) {
    // if (!pType.isPrimitive()) return pType;
    if (pType==long.class) return Long.class;
    if (pType==int.class) return Integer.class;
    if (pType==short.class) return Short.class;
    if (pType==byte.class) return Byte.class;
    if (pType==double.class) return Double.class;
    if (pType==float.class) return Float.class;
    if (pType==char.class) return Character.class;
    if (pType==boolean.class) return Boolean.class;
    if (pType==void.class) return Void.class;
    return pType;
  }
  
  /*
   * Miscellaneous private methods
   */
  
  private static Class<?>[] $getTypes(Object[] pArguments) {
    Class<?>[] argumentTypes=new Class[pArguments.length];
    for (int i=0; i<pArguments.length; i++) {
      argumentTypes[i]=pArguments[i]!=null?pArguments[i].getClass():null;
    }
    return argumentTypes;
  }
  
  private static Object[] $getArguments(Class<?>[] pDeclaredParameterTypes, Object[] pArguments) {
    Object[] arguments=new Object[pDeclaredParameterTypes.length];
    for (int i=0; i<arguments.length; i++) {
      if (i==arguments.length-1&&pDeclaredParameterTypes[i].isArray()&&(arguments.length!=pArguments.length||!$isCompatibleWith(pDeclaredParameterTypes[i],pArguments[i]!=null?pArguments[i].getClass():null))) {
        Object array=Array.newInstance(pDeclaredParameterTypes[i].getComponentType(),pArguments.length-i);
        for (int j=i,k=0; j<pArguments.length; j++,k++) {
          Array.set(array,k,pArguments[j]);
        }
        arguments[i]=array;
      }
      else {
        arguments[i]=pArguments[i];
      }
    }
    return arguments;
  }
  
  private static boolean $isCompatibleWith(Class<?>[] pDeclaredParameterTypes, Class<?>[] pParameterTypes) {
    Class<?>[] array1=pDeclaredParameterTypes,array2=pParameterTypes;
    int size1=array1.length,size2=array2.length;
    if (size1-1>size2) {
      return false;
    }
    else {
      for (int i=0; i<size1-1; i++) {
        if (!$isCompatibleWith(array1[i],array2[i])) return false;
      }
      if (size1==size2&&(size1==0||$isCompatibleWith(array1[size1-1],array2[size1-1]))) {
        return true;
      }
      else if (size1==0||!array1[size1-1].isArray()) {
        return false;
      }
      else {
        Class<?> componentType=array1[size1-1].getComponentType();
        for (int i=size1-1; i<size2; i++) {
          if (!$isCompatibleWith(componentType,array2[i])) return false;
        }
        return true;
      }
    }
  }
  
  private static boolean $isCompatibleWith(Class<?> pDeclaredParameterType, Class<?> pParameterType) {
    if (pParameterType==null) {
      return true;
    }
    else if (pDeclaredParameterType==null) {
      return false;
    }
    else {
      Class<?> type1=$getPrimitiveType(pDeclaredParameterType),type2=$getPrimitiveType(pParameterType);
      if (!type1.isPrimitive()) {
        return $getNonprimitiveType(pDeclaredParameterType).isAssignableFrom($getNonprimitiveType(pParameterType));
      }
      else if (type1==double.class) {
        return type2==double.class||type2==float.class||type2==long.class||type2==int.class||type2==short.class||type2==byte.class||type2==char.class;
      }
      else if (type1==float.class) {
        return type2==float.class||type2==long.class||type2==int.class||type2==short.class||type2==byte.class||type2==char.class;
      }
      else if (type1==long.class) {
        return type2==long.class||type2==int.class||type2==short.class||type2==byte.class||type2==char.class;
      }
      else if (type1==int.class) {
        return type2==int.class||type2==short.class||type2==byte.class||type2==char.class;
      }
      else if (type1==short.class) {
        return type2==short.class||type2==byte.class;
      }
      else {
        return type2==type1;
      }
    }
  }
  
  private static String $parametersToString(Class<?>[] pParameterTypes) {
    if (pParameterTypes==null) return "";
    StringBuilder sb=new StringBuilder(16);
    sb.append('(');
    for (int i=0; i<pParameterTypes.length; i++) {
      if (i>0) sb.append(',');
      sb.append(pParameterTypes[i]!=null?pParameterTypes[i].getCanonicalName():"?");
    }
    sb.append(')');
    return sb.toString();
  }
  
  /*
   * Auxiliary classes
   */
  
  private static final class GAddress1 {
    private final Class<?> declaringClass;
    private final String name;
    private final boolean isStatic;
    private final int arity;
    public GAddress1(Class<?> pDeclaringClass, String pName, boolean pIsStatic, int pArity) {
      declaringClass=pDeclaringClass;
      name=pName;
      isStatic=pIsStatic;
      arity=pArity;
    }
    @Override
    public boolean equals(Object pObject) {
      if (pObject instanceof GAddress1) {
        GAddress1 a1=this,a2=(GAddress1)pObject;
        return a1.isStatic==a2.isStatic&&a1.arity==a2.arity&&a1.declaringClass==a2.declaringClass&&a1.name.equals(a2.name);
      }
      else {
        return false;
      }
    }
    @Override
    public int hashCode() {
      int hash=(declaringClass.hashCode()^name.hashCode())*(arity+2);
      if (isStatic) hash=~hash;
      return hash;
    }
  }
  
  private static final class GAddress2 {
    private final Class<?> declaringClass;
    private final String name;
    private final boolean isStatic;
    private final Class<?>[] parameterTypes;
    public GAddress2(Class<?> pDeclaringClass, String pName, boolean pIsStatic, Class<?>[] pParameterTypes) {
      declaringClass=pDeclaringClass;
      name=pName;
      isStatic=pIsStatic;
      parameterTypes=pParameterTypes;
    }
    @Override
    public boolean equals(Object pObject) {
      if (pObject instanceof GAddress2) {
        GAddress2 a1=this,a2=(GAddress2)pObject;
        return a1.isStatic==a2.isStatic&&a1.declaringClass==a2.declaringClass&&a1.name.equals(a2.name)&&Arrays.equals(a1.parameterTypes,a2.parameterTypes);
      }
      else {
        return false;
      }
    }
    @Override
    public int hashCode() {
      int hash=(declaringClass.hashCode()^name.hashCode())*((parameterTypes==null?0:parameterTypes.length+1));
      if (isStatic) hash=~hash;
      return hash;
    }
  }
  
  private static final class GLRUCache<K,V> extends LinkedHashMap<K,V> {
    private static final long serialVersionUID=4186730578926979132L;
    private final int cacheSize;
    public GLRUCache(int pCacheSize) {
      super((int)(Math.ceil(pCacheSize/0.75+1)),0.75f,true);
      cacheSize=pCacheSize;
    }
    @Override
    protected boolean removeEldestEntry(Map.Entry<K,V> pEldest) {
      return super.size()>cacheSize;
    }
  }
  
  private static class GMutableList extends GArrayList<Object> {
    
    private static final long serialVersionUID=-2473036095479802069L;
    
    public GMutableList() {
      super(16);
    }
    
    public GMutableList(int pCapacity) {
      super(pCapacity);
    }
    
    public GMutableList(Collection<?> pCollection) {
      super(pCollection);
    }
    
    public GMutableList(ICollection<?> pCollection) {
      super(pCollection);
    }
    
  }
  
  private static class GMutableSet extends GLinkedHashTableSet<Object> {
    
    private static final long serialVersionUID=-3621091483324554006L;
    
    public GMutableSet() {
      super(8);
    }
    
    public GMutableSet(int pCapacity) {
      super(pCapacity);
    }
    
    public GMutableSet(Collection<?> pCollection) {
      super(pCollection);
    }
    
    public GMutableSet(ICollection<?> pCollection) {
      super(pCollection);
    }
    
  }
  
  private static class GMutableBag extends GLinkedHashTableBag<Object> {
    
    private static final long serialVersionUID=410154243086826876L;
    
    public GMutableBag() {
      super(8);
    }
    
    public GMutableBag(int pCapacity) {
      super(pCapacity);
    }
    
    public GMutableBag(Collection<?> pCollection) {
      super(pCollection);
    }
    
    public GMutableBag(ICollection<?> pCollection) {
      super(pCollection);
    }
    
  }
  
  private static class GRuntimeException extends RuntimeException {
    
    private static final long serialVersionUID=4060482552843902593L;
    
    public GRuntimeException(Throwable pCause) {
      super(pCause);
    }
    
  }
  
}
