package gold.util;

import java.lang.reflect.*;
import java.util.*;

public final class GToolkit {
  
  private static final Random RANDOM=new Random();
  
  private GToolkit() {
  }
  
  public static boolean isInstanceOf(Class<?> pClass, Object pObject) {
    return pClass!=null&&pObject!=null&&pClass.isInstance(pObject);
  }
  
  public static boolean same(Object pObject1, Object pObject2) {
    return pObject1==pObject2;
  }
  
  public static boolean equals(Object pObject1, Object pObject2) {
    if (pObject1==null&&pObject2==null) return true;
    if (pObject1==null||pObject2==null) return false;
    try {
      if ((pObject1 instanceof Iterable)&&(pObject2 instanceof Iterable)) {
        boolean isEmpty1=GCollections.isEmpty((Iterable<?>)pObject1);
        boolean isEmpty2=GCollections.isEmpty((Iterable<?>)pObject2);
        return isEmpty1||isEmpty2?isEmpty1==isEmpty2:pObject2.equals(pObject1);
      }
      else if (pObject1 instanceof long[]) {
        return GArrays.equals((long[])pObject1,(long[])pObject2);
      }
      else if (pObject1 instanceof int[]) {
        return GArrays.equals((int[])pObject1,(int[])pObject2);
      }
      else if (pObject1 instanceof short[]) {
        return GArrays.equals((short[])pObject1,(short[])pObject2);
      }
      else if (pObject1 instanceof byte[]) {
        return GArrays.equals((byte[])pObject1,(byte[])pObject2);
      }
      else if (pObject1 instanceof double[]) {
        return GArrays.equals((double[])pObject1,(double[])pObject2);
      }
      else if (pObject1 instanceof float[]) {
        return GArrays.equals((float[])pObject1,(float[])pObject2);
      }
      else if (pObject1 instanceof char[]) {
        return GArrays.equals((char[])pObject1,(char[])pObject2);
      }
      else if (pObject1 instanceof boolean[]) {
        return GArrays.equals((boolean[])pObject1,(boolean[])pObject2);
      }
      else if (pObject1 instanceof Object[]) {
        return GArrays.equals((Object[])pObject1,(Object[])pObject2);
      }
      else {
        return pObject2.equals(pObject1);
      }
    }
    catch (Throwable th) {
      return false;
    }
  }
  
  public static String toString(Object pObject) {
    if (pObject==null) return "null";
    Class<?> type=pObject.getClass();
    if (type==long[].class) {
      return GArrays.toString((long[])pObject);
    }
    else if (type==int[].class) {
      return GArrays.toString((int[])pObject);
    }
    else if (type==short[].class) {
      return GArrays.toString((short[])pObject);
    }
    else if (type==byte[].class) {
      return GArrays.toString((byte[])pObject);
    }
    else if (type==double[].class) {
      return GArrays.toString((double[])pObject);
    }
    else if (type==float[].class) {
      return GArrays.toString((float[])pObject);
    }
    else if (type==char[].class) {
      return GArrays.toString((char[])pObject);
    }
    else if (type==boolean[].class) {
      return GArrays.toString((boolean[])pObject);
    }
    else if (pObject instanceof Object[]) {
      return GArrays.toString((Object[])pObject);
    }
    else {
      return pObject.toString();
    }
  }
  
  public static Object clone(Object pObject) {
    if (pObject==null) {
      return null;
    }
    else if (pObject.getClass().isArray()) {
      return GArrays.copyOf(pObject,Array.getLength(pObject));
    }
    else {
      try {
        return GReflection.$invokeMethod("clone",pObject,new Object[0]);
      }
      catch (Throwable th) {
        throw new InternalError("The class "+pObject.getClass().getCanonicalName()+" is not cloneable.");
      }
    }
  }
  
  public static Random getRandomGenerator() {
    return RANDOM;
  }
  
  public static int random(int pMinimum, int pMaximum) {
    int min=Math.min(pMinimum,pMaximum),max=Math.max(pMinimum,pMaximum);
    return min+RANDOM.nextInt(max-min+1);
  }
  
  public static int random(int pMaximum) {
    return pMaximum>0?RANDOM.nextInt(pMaximum):0;
  }
  
  public static long random(long pMinimum, long pMaximum) {
    long min=Math.min(pMinimum,pMaximum),max=Math.max(pMinimum,pMaximum);
    long r=Math.abs(RANDOM.nextLong());
    return min+r%(max-min+1);
  }
  
  public static long random(long pMaximum) {
    return pMaximum>0?random(0,pMaximum):0;
  }
  
  public static double random() {
    return RANDOM.nextDouble();
  }
  
  public static String separate(Object pSeparator, Object...pMessages) {
    StringBuilder sb=new StringBuilder(32);
    String separator=pSeparator.toString();
    for (int index=0; index<pMessages.length; index++) {
      if (index>0&&!separator.isEmpty()) sb.append(separator);
      sb.append(toString(pMessages[index]));
    }
    return sb.toString();
  }
  
  public static String concatenate(Object...pMessages) {
    return separate("",pMessages);
  }
  
  public static String reverse(String pText) {
    return new StringBuilder(pText).reverse().toString();
  }
  
  public static String escape(String pText) {
    if (pText==null) return "";
    boolean found=pText.isEmpty();
    for (int i=0; !found&&i<pText.length(); i++) {
      if ("()<>{}\"-,=\t ".indexOf(pText.charAt(i))!=-1) found=true;
    }
    return found?"\""+pText.replace("\"","\"\"")+"\"":pText;
  }
  
  public static String convert(String pText) {
    if (pText==null) return "";
    return pText.startsWith("\"")&&pText.endsWith("\"")?pText.substring(1,pText.length()-1).replace("\"\"","\""):pText;
  }
  
  // Basado en Apache Commons Lang
  public static String escapeXML(String pText) {
    StringBuilder sb=new StringBuilder((int)(pText.length()*1.5+1));
    try {
      for (int i=0,t=pText.length(); i<t; i++) {
        char c=pText.charAt(i);
        switch (c) {
          case '\0':
            break;
          case '\'':
            sb.append("&apos;");
            break;
          case '"':
            sb.append("&quot;");
            break;
          case '&':
            sb.append("&amp;");
            break;
          case '<':
            sb.append("&lt;");
            break;
          case '>':
            sb.append("&gt;");
            break;
          default:
            if (c>0x7F) {
              sb.append("&#");
              sb.append(Integer.toString(c));
              sb.append(";");
            }
            else {
              sb.append(c);
            }
            break;
        }
      }
    }
    catch (Throwable th) {
    }
    return sb.toString();
  }
  
}
