package gold.visualization.automaton;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.*;
import java.util.concurrent.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.filechooser.*;
import javax.swing.filechooser.FileFilter;
import javax.swing.undo.*;
import gold.structures.automaton.*;
import gold.structures.list.*;
import gold.structures.set.*;
import gold.swing.util.*;
import gold.util.*;
import gold.visualization.util.*;
import edu.uci.ics.jung.algorithms.layout.*;
import edu.uci.ics.jung.visualization.control.*;

@SuppressWarnings("serial")
public class GAutomataFrame extends JFrame {
  
  public static GAutomataFrame show(IAutomaton<?>...pAutomata) {
    GAutomataFrame frame=new GAutomataFrame();
    frame.setVisible(true);
    for (IAutomaton<?> automaton:pAutomata) {
      frame.display(automaton);
    }
    return frame;
  }
  
  protected static Rectangle lastEditAutomatonDialogBounds=null;
  
  protected final GAutomataMenuBar menuBar=new GAutomataMenuBar();
  protected final GDesktop desktop=new GDesktop();
  
  public GAutomataFrame() {
    super("GOLD Automata Viewer");
    setIconImage(GUtilities.getIcon("famfamfam/coins.png").getImage());
    setJMenuBar(menuBar);
    setPreferredSize(new Dimension(800,600));
    setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    setContentPane(desktop);
    pack();
    // La siguiente línea necesita JFrame.setDefaultLookAndFeelDecorated(false)
    // setExtendedState(getExtendedState()|JFrame.MAXIMIZED_BOTH);
    GUtilities.center(this);
  }
  
  public GAutomatonInternalFrame display(File pFile) throws IOException {
    GLayoutConfiguration layoutConfiguration=new GLayoutConfiguration();
    GAutomatonInternalFrame internalFrame=display(GAutomata.read(new FileInputStream(pFile),layoutConfiguration),layoutConfiguration);
    internalFrame.setFile(pFile);
    return internalFrame;
  }
  
  public GAutomatonInternalFrame display(IAutomaton<?> pAutomaton) {
    return display(pAutomaton,null);
  }
  
  public GAutomatonInternalFrame display(IAutomaton<?> pAutomaton, GLayoutConfiguration pLayoutConfiguration) {
    return display(null,pAutomaton,pLayoutConfiguration);
  }
  
  public GAutomatonInternalFrame display(String pTitle, IAutomaton<?> pAutomaton) {
    return display(pTitle,pAutomaton,null);
  }
  
  public GAutomatonInternalFrame display(String pTitle, IAutomaton<?> pAutomaton, GLayoutConfiguration pLayoutConfiguration) {
    GAutomatonInternalFrame internalFrame=new GAutomatonInternalFrame(pTitle!=null?pTitle:"Untitled automaton #"+(desktop.getOpenedFrameCount()+1),pAutomaton,pLayoutConfiguration);
    internalFrame.setFrameIcon(GUtilities.getIcon("gold/automata.png"));
    internalFrame.addInternalFrameListener(new InternalFrameAdapter() {
      @Override
      public void internalFrameActivated(InternalFrameEvent pEvent) {
        menuBar.refresh();
      }
      @Override
      public void internalFrameClosed(InternalFrameEvent pEvent) {
        menuBar.refresh();
      }
      @Override
      public void internalFrameDeactivated(InternalFrameEvent pEvent) {
        menuBar.refresh();
      }
    });
    desktop.addInternalFrame(internalFrame);
    return internalFrame;
  }
  
  public IList<GAutomatonInternalFrame> getInternalFrames() {
    JInternalFrame[] internalFrames=desktop.getAllFrames();
    IList<GAutomatonInternalFrame> list=new GArrayList<GAutomatonInternalFrame>(internalFrames.length);
    for (JInternalFrame internalFrame:internalFrames) {
      if (internalFrame instanceof GAutomatonInternalFrame) list.addLast((GAutomatonInternalFrame)internalFrame);
    }
    Collections.sort(list.toJavaView());
    return list;
  }
  
  public GDesktop getDesktop() {
    return desktop;
  }
  
  @Override
  public void setVisible(boolean pVisible) {
    if (pVisible) menuBar.refresh();
    super.setVisible(pVisible);
  }
  
  protected class GAutomataMenuBar extends JMenuBar implements ActionListener {
    
    protected final JMenuItem buttonNewDFA=GUtilities.getMenuItem(this,"New Deterministic Finite Automaton (DFA)",'D',GUtilities.getIcon("jlfgr/AddD16.gif"),"new:DFA");
    protected final JMenuItem buttonNewNFA=GUtilities.getMenuItem(this,"New Nondeterministic Finite Automaton (NFA)",'N',GUtilities.getIcon("jlfgr/AddN16.gif"),"new:NFA");
    protected final JMenuItem buttonNewDFT=GUtilities.getMenuItem(this,"New Deterministic Finite Transducer (DFT)",'T',GUtilities.getIcon("jlfgr/AddT16.gif"),"new:DFT");
    protected final JMenuItem buttonNewPDA=GUtilities.getMenuItem(this,"New Pushdown Finite Automaton (PDA)",'P',GUtilities.getIcon("jlfgr/AddP16.gif"),"new:PDA");
    protected final JMenuItem buttonNew=GUtilities.getMenu(this,"New",'N',GUtilities.getIcon("jlfgr/New16.gif"),null,new JMenuItem[]{buttonNewDFA,buttonNewNFA,buttonNewDFT,buttonNewPDA});
    protected final JMenuItem buttonOpen=GUtilities.getMenuItem(this,"Open",'O',GUtilities.getIcon("jlfgr/Open16.gif"),"open");
    protected final JMenuItem buttonEdit=GUtilities.getMenuItem(this,"Edit",'E',GUtilities.getIcon("jlfgr/Edit16.gif"),"edit");
    protected final JMenuItem buttonSave=GUtilities.getMenuItem(this,"Save",'S',GUtilities.getIcon("jlfgr/Save16.gif"),"save");
    protected final JMenuItem buttonSaveAs=GUtilities.getMenuItem(this,"Save as...",'A',GUtilities.getIcon("jlfgr/SaveAs16.gif"),"saveAs");
    protected final JMenuItem buttonExportAs=GUtilities.getMenuItem(this,"Export as...",'X',GUtilities.getIcon("jlfgr/Export16.gif"),"exportAs");
    protected final JMenuItem buttonExit=GUtilities.getMenuItem(this,"Exit",'X',GUtilities.getIcon("jlfgr/Close16.gif"),"exit");
    protected final JMenu buttonFile=GUtilities.getMenu(this,"File",'F',null,null,new JMenuItem[]{buttonNew,buttonOpen,null,buttonEdit,null,buttonSave,buttonSaveAs,buttonExportAs,null,buttonExit});
    protected final JMenuItem buttonLayoutCircle=GUtilities.getRadioButtonMenuItem(this,"Circle layout",'C',"layout:CIRCLE");
    protected final JMenuItem buttonLayoutFR=GUtilities.getRadioButtonMenuItem(this,"Fruchterman-Reingold force-directed layout",'F',"layout:FR");
    protected final JMenuItem buttonLayoutFR2=GUtilities.getRadioButtonMenuItem(this,"Fruchterman-Reingold force-directed optimized layout",'R',"layout:FR2");
    protected final JMenuItem buttonLayoutISOM=GUtilities.getRadioButtonMenuItem(this,"Self-organizing map layout",'M',"layout:ISOM");
    protected final JMenuItem buttonLayoutKK=GUtilities.getRadioButtonMenuItem(this,"Kamada-Kawai layout",'K',"layout:KK");
    protected final JMenuItem buttonLayoutSpring=GUtilities.getRadioButtonMenuItem(this,"Spring layout",'S',"layout:SPRING");
    protected final JMenuItem buttonLayoutSpring2=GUtilities.getRadioButtonMenuItem(this,"Spring layout 2",'P',"layout:SPRING2");
    protected final JMenuItem buttonLayout=GUtilities.getMenu(this,"Layout algorithms ",'L',GUtilities.getIcon("gold/layout.png"),null,new JMenuItem[]{buttonLayoutCircle,buttonLayoutFR,buttonLayoutFR2,buttonLayoutISOM,buttonLayoutKK,buttonLayoutSpring,buttonLayoutSpring2});
    protected final JMenuItem buttonModePicking=GUtilities.getRadioButtonMenuItem(this,"Picking",'P',"mode:PICKING");
    protected final JMenuItem buttonModeTransforming=GUtilities.getRadioButtonMenuItem(this,"Transforming",'T',"mode:TRANSFORMING");
    protected final JMenuItem buttonMode=GUtilities.getMenu(this,"Mode ",'M',GUtilities.getIcon("famfamfam/mouse.png"),null,new JMenuItem[]{buttonModePicking,buttonModeTransforming});
    protected final JMenuItem buttonStopLayout=GUtilities.getMenuItem(this,"Stop layout algorithm",'S',GUtilities.getIcon("jlfgr/Stop16.gif"),"stopLayout");
    protected final JMenuItem buttonRunLayout=GUtilities.getMenuItem(this,"Run layout algorithm",'R',GUtilities.getIcon("jlfgr/Play16.gif"),"runLayout");
    protected final JMenuItem buttonShowGrid=GUtilities.getCheckBoxMenuItem(this,"Show grid",'D',"showGrid");
    protected final JMenuItem buttonShowVertexLabels=GUtilities.getCheckBoxMenuItem(this,"Show state labels",'H',"showVertexLabels");
    protected final JMenu buttonView=GUtilities.getMenu(this,"View",'V',null,null,new JMenuItem[]{buttonLayout,buttonMode,null,buttonStopLayout,buttonRunLayout,null,buttonShowGrid,buttonShowVertexLabels});
    protected final JMenuItem buttonBasicInformation=GUtilities.getMenuItem(this,"Basic information",'B',GUtilities.getIcon("jlfgr/Information16.gif"),"basicInformation");
    protected final JMenuItem buttonAcceptedRejected=GUtilities.getMenuItem(this,"Accepted/rejected strings",'A',GUtilities.getIcon("jlfgr/History16.gif"),"acceptedRejected");
    protected final JMenuItem buttonRunAutomaton=GUtilities.getMenuItem(this,"Run automaton",'R',GUtilities.getIcon("famfamfam/accept.png"),"runAutomaton");
    protected final JMenuItem buttonOperations=GUtilities.getMenuItem(this,"Operations",'O',GUtilities.getIcon("famfamfam/cog.png"),"operations");
    protected final JMenu buttonTools=GUtilities.getMenu(this,"Tools",'T',null,null,new JMenuItem[]{buttonBasicInformation,buttonAcceptedRejected,null,buttonRunAutomaton,buttonOperations});
    protected final JMenuItem buttonCascadeWindows=GUtilities.getMenuItem(this,"Cascade windows",'C',GUtilities.getIcon("gold/cascade.png"),"cascadeWindows");
    protected final JMenuItem buttonTileWindows=GUtilities.getMenuItem(this,"Tile windows",'T',GUtilities.getIcon("gold/tile.png"),"tileWindows");
    protected final JMenuItem buttonMaximizeWindow=GUtilities.getMenuItem(this,"Maximize window",'M',GUtilities.getIcon("jlfgr/Maximize16.gif"),"maximizeWindow");
    protected final JMenuItem buttonMinimizeWindow=GUtilities.getMenuItem(this,"Minimize window",'N',GUtilities.getIcon("jlfgr/Minimize16.gif"),"minimizeWindow");
    protected final JMenuItem buttonRestoreWindow=GUtilities.getMenuItem(this,"Restore window",'R',GUtilities.getIcon("jlfgr/Restore16.gif"),"restoreWindow");
    protected final JMenu buttonWindow=GUtilities.getMenu(this,"Window",'W',null,null,new JMenuItem[]{buttonCascadeWindows,buttonTileWindows,null,buttonMaximizeWindow,buttonMinimizeWindow,buttonRestoreWindow});
    protected final JMenuItem buttonAbout=GUtilities.getMenuItem(this,"About GOLD",'A',GUtilities.getIcon("famfamfam/coins.png"),"about");
    protected final JMenu buttonHelp=GUtilities.getMenu(this,"Help",'H',null,null,new JMenuItem[]{buttonAbout});
    protected final GButtonGroup groupLayout=new GButtonGroup(buttonLayoutCircle,buttonLayoutFR,buttonLayoutFR2,buttonLayoutISOM,buttonLayoutKK,buttonLayoutSpring,buttonLayoutSpring2);
    protected final GButtonGroup groupMode=new GButtonGroup(buttonModePicking,buttonModeTransforming);
    protected GAutomatonInternalFrame activeInternalFrame=null;
    
    protected GAutomataMenuBar() {
      buttonLayout.setToolTipText("Configures the layout algorithm used to locate the states of the automaton");
      buttonMode.setToolTipText("Configures the mode used to manipulate the automaton");
      buttonModePicking.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_P,InputEvent.CTRL_MASK));
      buttonModeTransforming.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_T,InputEvent.CTRL_MASK));
      buttonShowGrid.setToolTipText("Show the grid");
      buttonShowVertexLabels.setToolTipText("Show the labels of the states");
      buttonOpen.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_O,InputEvent.CTRL_MASK));
      buttonEdit.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_E,InputEvent.CTRL_MASK));
      buttonSave.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_S,InputEvent.CTRL_MASK));
      buttonRunLayout.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_F5,0));
      buttonRunAutomaton.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_F6,0));
      for (JMenu button:new JMenu[]{buttonFile,buttonView,buttonTools,buttonWindow,buttonHelp}) {
        add(button);
      }
      refresh();
    }
    
    protected void refresh() {
      JInternalFrame internalFrame=desktop!=null?desktop.getSelectedFrame():null;
      activeInternalFrame=internalFrame!=null&&(internalFrame instanceof GAutomatonInternalFrame)?(GAutomatonInternalFrame)internalFrame:null;
      GUtilities.enable(activeInternalFrame!=null,buttonEdit,buttonSave,buttonSaveAs,buttonExportAs,buttonView,buttonTools,buttonMaximizeWindow,buttonMinimizeWindow,buttonRestoreWindow);
      GUtilities.enable(desktop!=null&&desktop.getAllFrames().length>0,buttonWindow);
      if (activeInternalFrame!=null) {
        GAutomatonPainter<?> painter=activeInternalFrame.getPainter();
        groupLayout.setSelectedActionCommand("layout:"+painter.getLayoutType().name(),buttonLayoutFR);
        groupLayout.setSelectedActionCommand("layout:"+painter.getMode().name(),buttonModePicking);
        buttonShowGrid.setSelected(painter.getShowGrid());
        buttonShowVertexLabels.setSelected(painter.getShowVertexLabels());
        buttonBasicInformation.setEnabled(!(activeInternalFrame.getAutomaton() instanceof IPushdownAutomaton));
        buttonAcceptedRejected.setEnabled(!(activeInternalFrame.getAutomaton() instanceof IPushdownAutomaton));
      }
      else {
        groupLayout.clearSelection();
      }
    }
    
    @Override
    public void actionPerformed(ActionEvent pActionEvent) {
      actionPerformed(pActionEvent.getActionCommand());
    }
    
    protected void actionPerformed(String pActionCommand) {
      try {
        String argument=pActionCommand.substring(pActionCommand.indexOf(':')+1);
        if (pActionCommand.startsWith("new:")) {
          new GEditAutomatonDialog(null,pActionCommand.substring("new:".length())).display();
        }
        else if (pActionCommand.equals("open")) {
          new GOpenAutomatonDialog().display();
        }
        else if (pActionCommand.equals("edit")) {
          new GEditAutomatonDialog(activeInternalFrame,null).display();
        }
        else if (pActionCommand.equals("save")) {
          File file=activeInternalFrame.getFile();
          if (file==null) {
            actionPerformed("saveAs");
          }
          else {
            activeInternalFrame.save(file);
          }
        }
        else if (pActionCommand.equals("saveAs")) {
          new GSaveAutomatonDialog().display(activeInternalFrame);
        }
        else if (pActionCommand.equals("exportAs")) {
          new GExportAsAutomatonDialog(activeInternalFrame).display();
        }
        else if (pActionCommand.equals("exit")) {
          GAutomataFrame.this.dispose();
        }
        else if (pActionCommand.startsWith("layout:")) {
          activeInternalFrame.getPainter().setLayoutType(GLayoutType.valueOf(argument));
        }
        else if (pActionCommand.startsWith("mode:")) {
          activeInternalFrame.getPainter().setMode(ModalGraphMouse.Mode.valueOf(argument));
        }
        else if (pActionCommand.equals("stopLayout")) {
          Layout<?,?> layout=activeInternalFrame.getPainter().getDelegateLayout();
          if (layout instanceof AbstractLayout) ((AbstractLayout<?,?>)layout).lock(true);
        }
        else if (pActionCommand.equals("runLayout")) {
          actionPerformed(groupLayout.getSelectedActionCommand(buttonLayoutKK));
        }
        else if (pActionCommand.equals("showGrid")) {
          activeInternalFrame.getPainter().setShowGrid(buttonShowGrid.isSelected());
        }
        else if (pActionCommand.equals("showVertexLabels")) {
          activeInternalFrame.getPainter().setShowVertexLabels(buttonShowVertexLabels.isSelected());
        }
        else if (pActionCommand.equals("basicInformation")) {
          activeInternalFrame.displayBasicInformationDialog();
        }
        else if (pActionCommand.equals("acceptedRejected")) {
          activeInternalFrame.displayAcceptedRejectedDialog();
        }
        else if (pActionCommand.equals("runAutomaton")) {
          activeInternalFrame.displayRunDialog();
        }
        else if (pActionCommand.equals("operations")) {
          Object result=activeInternalFrame.displayOperationsDialog(getInternalFrames());
          try {
            if (result instanceof IAutomaton) {
              GAutomatonInternalFrame internalFrame=display((IAutomaton<?>)result);
              GOptionDialog.showInfo(this,"The automaton was generated successfully with the name '"+internalFrame.getTitle()+"'.");
            }
            else if (result!=null) {
              GOptionDialog.showInfo(this,"The return value of the method invocation was: "+String.valueOf(result)+".");
            }
          }
          catch (Throwable th) {
            GOptionDialog.showError(this,th);
          }
        }
        else if (pActionCommand.equals("cascadeWindows")) {
          desktop.cascade();
        }
        else if (pActionCommand.equals("tileWindows")) {
          desktop.tiles();
        }
        else if (pActionCommand.equals("maximizeWindow")) {
          desktop.maximizeInternalFrame(activeInternalFrame);
        }
        else if (pActionCommand.equals("minimizeWindow")) {
          desktop.minimizeInternalFrame(activeInternalFrame);
        }
        else if (pActionCommand.equals("restoreWindow")) {
          desktop.restoreInternalFrame(activeInternalFrame);
        }
        else if (pActionCommand.equals("about")) {
          GUtilities.show(new GAboutDialog(this),buttonHelp);
        }
        else {
          GOptionDialog.showError(GAutomataMenuBar.this,new IllegalArgumentException("Command not found: "+pActionCommand));
        }
      }
      catch (Throwable th) {
        GOptionDialog.showError(GAutomataMenuBar.this,th);
      }
    }
    
    protected class GOpenAutomatonDialog extends GFileChooser {
      
      protected final static String EXTENSION="txt";
      
      public GOpenAutomatonDialog() {
        super(JFileChooser.OPEN_DIALOG);
        addChoosableFileFilter(new FileNameExtensionFilter("Text File (*."+EXTENSION+")",EXTENSION));
        acceptButton.setText("Open");
        acceptButton.setToolTipText("Open the automaton from the specified file.");
      }
      
      public void display() {
        if (showFileDialog(GAutomataFrame.this,"Open automaton")==JFileChooser.APPROVE_OPTION) {
          try {
            final File file=getSelectedFile();
            if (file==null) throw new IllegalStateException("You must select a file.");
            if (!file.exists()||!file.isFile()) throw new IllegalStateException("You must select a file.");
            GWaitDialog.wait(GAutomataFrame.this,new Callable<Void>() {
              @Override
              public Void call() throws Exception {
                GAutomataFrame.this.display(file);
                return null;
              }
            });
            GOptionDialog.showInfo(GAutomataMenuBar.this,"The automaton was opened successfully.");
          }
          catch (Throwable th) {
            GOptionDialog.showError(GAutomataMenuBar.this,th);
          }
        }
      }
      
    }
    
    public final class GSaveAutomatonDialog extends GFileChooser {
      
      protected final static String EXTENSION="txt";
      
      public GSaveAutomatonDialog() {
        super(JFileChooser.SAVE_DIALOG);
        addChoosableFileFilter(new FileNameExtensionFilter("Text File (*."+EXTENSION+")",EXTENSION));
        acceptButton.setText("Save as");
        acceptButton.setToolTipText("Save the automaton in the specified file.");
      }
      
      public void display(GAutomatonInternalFrame pInternalFrame) {
        if (showFileDialog(GAutomataFrame.this,"Save automaton")==JFileChooser.APPROVE_OPTION) {
          try {
            File file=verifyFileForSaving(GAutomataFrame.this,"."+EXTENSION);
            if (file!=null) pInternalFrame.save(file);
          }
          catch (Throwable th) {
            GOptionDialog.showError(GAutomataMenuBar.this,th);
          }
        }
      }
      
    }
    
    public final class GExportAsAutomatonDialog extends GFileChooser {
      
      protected final GAutomatonInternalFrame internalFrame;
      
      public GExportAsAutomatonDialog(GAutomatonInternalFrame pInternalFrame) {
        super(JFileChooser.SAVE_DIALOG);
        internalFrame=pInternalFrame;
        addChoosableFileFilter(new FileNameExtensionFilter("PNG Image (*.png)","png"));
        addChoosableFileFilter(new FileNameExtensionFilter("JFlap File (*.jff)","jff"));
        acceptButton.setText("Export");
        acceptButton.setToolTipText("Export the automaton in the specified file.");
      }
      
      public void display() {
        setFileFilter(getChoosableFileFilters()[0]);
        if (showFileDialog(GAutomataFrame.this,"Export as...")==JFileChooser.APPROVE_OPTION) {
          FileFilter fileFilter=getFileFilter();
          try {
            String extension=((FileNameExtensionFilter)fileFilter).getExtensions()[0];
            File file=verifyFileForSaving(GAutomataFrame.this,"."+extension);
            if (file!=null&&Arrays.asList("png","jff").contains(extension)) {
              if (extension.equals("png")) {
                internalFrame.getPainter().getImage().crop(new Insets(5,5,5,5)).save(file,"png");
              }
              else if (extension.equals("jff")) {
                GAutomata.write(new FileOutputStream(file),GAutomata.Format.JFLAP,internalFrame.getPainter());
              }
              GOptionDialog.showInfo(this,"The automaton was exported successfully.");
            }
          }
          catch (Throwable th) {
            GOptionDialog.showError(GAutomataMenuBar.this,th);
          }
        }
      }
      
    }
    
    public final class GEditAutomatonDialog extends JDialog {
      
      protected final GAutomatonInternalFrame internalFrame;
      protected final JEditorPane informationLabel=new JEditorPane("text/plain","");
      protected final String type;
      
      public GEditAutomatonDialog(GAutomatonInternalFrame pInternalFrame, String pType) {
        super(GAutomataFrame.this,(pInternalFrame!=null?"Edit":"New")+" automaton",JDialog.ModalityType.TOOLKIT_MODAL);
        internalFrame=pInternalFrame;
        type=pType;
        JButton cancelButton=new JButton("Cancel");
        cancelButton.addActionListener(new ActionListener() {
          @Override
          public void actionPerformed(ActionEvent pEvent) {
            GEditAutomatonDialog.this.dispose();
          }
        });
        JButton acceptButton=new JButton("Accept");
        acceptButton.addActionListener(new ActionListener() {
          @Override
          public void actionPerformed(ActionEvent pEvent) {
            try {
              GLayoutConfiguration map=internalFrame!=null?new GLayoutConfiguration():null;
              IAutomaton<?> automaton=GAutomata.read(new StringReader(informationLabel.getText()),map);
              if (internalFrame==null) {
                GAutomataFrame.this.display(automaton);
              }
              else {
                internalFrame.setAutomaton(automaton,map);
              }
              GEditAutomatonDialog.this.dispose();
            }
            catch (Throwable th) {
              GOptionDialog.showError(GEditAutomatonDialog.this,th);
            }
          }
        });
        informationLabel.setEditable(true);
        informationLabel.setBackground(Color.WHITE);
        informationLabel.setFont(new Font(Font.MONOSPACED,Font.PLAIN,16));
        JScrollPane scrollPane=GUtilities.getScrollPane(informationLabel,ScrollPaneConstants.VERTICAL_SCROLLBAR_ALWAYS,ScrollPaneConstants.HORIZONTAL_SCROLLBAR_AS_NEEDED,800,400);
        GUtilities.configure(this,GUtilities.getBorderPane(5,5,0,8,null,null,GUtilities.getBoxPaneY(0,scrollPane,Box.createVerticalStrut(5)),null,GUtilities.getBoxPaneX(0,Box.createHorizontalGlue(),acceptButton,Box.createHorizontalStrut(5),cancelButton)));
        setResizable(true);
      }
      
      public void display() {
        try {
          IAutomaton<?> automaton=null;
          if (internalFrame!=null) {
            automaton=internalFrame.getAutomaton();
          }
          else if (type.equals("DFA")) {
            automaton=GAutomata.parse("cat","A","D","A-c-B,A-a-A,A-t-A,B-c-B,B-a-C,B-t-A,C-c-B,C-a-A,C-t-D,D-c-D,D-a-D,D-t-D");
          }
          else if (type.equals("NFA")) {
            automaton=GAutomata.parse("0","A","B,E","A-λ-B,B-0-C,C-0-D,D-0-B,A-λ-E,E-0-F,F-0-E");
          }
          else if (type.equals("DFT")) {
            ISet<Object> states=GCollections.<Object>asSet(0,1,2,3,"F");
            String inputAlphabet="0123456789$";
            String outputAlphabet="0123456789R";
            Object initialState=0;
            ISet<Object> acceptStates=GCollections.<Object>asSet("F");
            GDeterministicTransducer<Object> transducer=new GDeterministicTransducer<Object>(states,inputAlphabet,outputAlphabet,initialState,acceptStates);
            for (Object input:states) {
              transducer.setStateOutput(input,"");
              for (char symbol:inputAlphabet.toCharArray()) {
                transducer.setDelta(input,symbol,input.equals("F")||symbol=='$'?"F":(((Integer)input)*10+symbol-'0')%3);
                transducer.setTransitionOutput(input,symbol,input.equals("F")?"":(symbol=='$'?"R"+input:""+(((Integer)input)*10+symbol-'0')/3));
              }
            }
            automaton=transducer;
          }
          else if (type.equals("PDA")) {
            ISet<Object> states=GCollections.<Object>asSet('A','B');
            String inputAlphabet="ab";
            String stackAlphabet="a";
            Object initialState='A';
            ISet<Object> acceptStates=GCollections.<Object>asSet('B');
            GPushdownAutomaton<Object> pushdownAutomaton=new GPushdownAutomaton<Object>(states,inputAlphabet,stackAlphabet,initialState,acceptStates);
            pushdownAutomaton.delta('A','A',"a").push("a");
            pushdownAutomaton.delta('A','B',"").ignore();
            pushdownAutomaton.delta('B','B',"b").pop("a");
            automaton=pushdownAutomaton;
          }
          else {
            automaton=GAutomata.parse("cat","A","D","A-c-B,A-a-A,A-t-A,B-c-B,B-a-C,B-t-A,C-c-B,C-a-A,C-t-D,D-c-D,D-a-D,D-t-D");
          }
          Set<String> strings=new HashSet<String>(automaton.getStates().size());
          for (Object state:automaton.getStates()) {
            if (!strings.add(state.toString())) throw new IllegalArgumentException("The automaton has states with the same textual representation.");
          }
          try {
            if (internalFrame!=null) {
              StringWriter writer=new StringWriter(256);
              GAutomata.write(writer,internalFrame.getPainter());
              informationLabel.setText(writer.toString());
            }
            else {
              informationLabel.setText(automaton.toString());
            }
          }
          catch (IOException ex) {
            informationLabel.setText(automaton.toString());
          }
          informationLabel.setCaretPosition(0);
          final UndoManager undoManager=new UndoManager();
          informationLabel.getDocument().addUndoableEditListener(undoManager);
          Action undoAction=new AbstractAction() {
            @Override
            public void actionPerformed(ActionEvent pEvent) {
              try {
                undoManager.undo();
              }
              catch (Throwable th) {
              }
            }
          };
          Action redoAction=new AbstractAction() {
            @Override
            public void actionPerformed(ActionEvent pEvent) {
              try {
                undoManager.redo();
              }
              catch (Throwable th) {
              }
            }
          };
          informationLabel.registerKeyboardAction(undoAction,KeyStroke.getKeyStroke(KeyEvent.VK_Z,InputEvent.CTRL_MASK),JComponent.WHEN_FOCUSED);
          informationLabel.registerKeyboardAction(redoAction,KeyStroke.getKeyStroke(KeyEvent.VK_Y,InputEvent.CTRL_MASK),JComponent.WHEN_FOCUSED);
          if (lastEditAutomatonDialogBounds!=null) {
            pack();
            setBounds(lastEditAutomatonDialogBounds);
            setVisible(true);
            requestFocus();
          }
          else {
            GUtilities.show(this,getOwner());
          }
          lastEditAutomatonDialogBounds=getBounds();
        }
        catch (Throwable th) {
          GOptionDialog.showError(GAutomataFrame.this,th);
        }
      }
      
    }
    
  }
  
}
