package gold.visualization.automaton;

import java.awt.*;
import java.awt.geom.*;
import java.util.*;
import java.util.List;
import gold.structures.automaton.*;
import gold.structures.graph.*;
import gold.structures.list.*;
import gold.structures.set.*;
import gold.util.*;
import gold.visualization.graph.*;
import gold.visualization.util.*;
import edu.uci.ics.jung.algorithms.layout.*;
import edu.uci.ics.jung.visualization.*;
import edu.uci.ics.jung.visualization.renderers.*;
import edu.uci.ics.jung.visualization.transform.shape.*;
import static gold.visualization.util.GAffineTransforms.*;
import static gold.visualization.util.GShapes.*;

public class GAutomatonPainter<V> extends GGraphPainter<V,IEdge<V,ISet<?>>> {
  
  protected final IAutomaton<V> automaton;
  
  public GAutomatonPainter(IAutomaton<V> pAutomaton) {
    this(pAutomaton,null);
  }
  
  public GAutomatonPainter(IAutomaton<V> pAutomaton, Dimension pPreferredSize) {
    super(pAutomaton.toGraphCopy().toJungView(),pPreferredSize);
    automaton=pAutomaton;
    super.getRenderContext().setVertexLabelTransformer(new GGeneralTransformer<V,String>() {
      @Override
      public String transform(V pVertex) {
        String result=super.transform(pVertex);
        if (result!=null) {
          return result;
        }
        else if (automaton instanceof ITransducer) {
          ITransducer<V> transducer=(ITransducer<V>)automaton;
          String output=transducer.getStateOutput(pVertex);
          return pVertex+"/"+(output!=null&&!output.isEmpty()?output:IAutomaton.LAMBDA);
        }
        else {
          return ""+pVertex;
        }
      }
    });
    super.getRenderContext().setEdgeLabelTransformer(new GGeneralTransformer<IEdge<V,ISet<?>>,String>() {
      @Override
      @SuppressWarnings("unchecked")
      public String transform(IEdge<V,ISet<?>> pEdge) {
        String result=super.transform(pEdge);
        if (result!=null) {
          return result;
        }
        else if (automaton instanceof ITransducer) {
          ISet<Character> data=(ISet<Character>)(pEdge.getData());
          ITransducer<V> transducer=(ITransducer<V>)automaton;
          V state=pEdge.getSource();
          TreeMap<Character,String> map=new TreeMap<Character,String>();
          for (char symbol:data) {
            map.put(symbol,transducer.getTransitionOutput(state,symbol));
          }
          int count=0;
          List<Object> list=new ArrayList<Object>();
          for (Map.Entry<Character,String> entry:map.entrySet()) {
            char symbol=entry.getKey();
            String output=entry.getValue();
            list.add(symbol+"/"+(output!=null&&!output.isEmpty()?output:IAutomaton.LAMBDA)+(((++count)%5)==0?"\n":""));
          }
          return GToolkit.separate("|",list.toArray()).trim();
        }
        else if (automaton instanceof IPushdownAutomaton) {
          ISet<IPushdownTransition<V>> data=(ISet<IPushdownTransition<V>>)(pEdge.getData());
          List<IPushdownTransition<V>> list=new ArrayList<IPushdownTransition<V>>(data.toJavaView());
          Collections.sort(list,new Comparator<IPushdownTransition<V>>() {
            private final Comparator<String> comparator=GComparators.shortlexStringComparator();
            @Override
            public int compare(IPushdownTransition<V> pTransition1, IPushdownTransition<V> pTransition2) {
              int c=comparator.compare(pTransition1.getPrefix(),pTransition2.getPrefix());
              return c!=0?c:comparator.compare(pTransition1.toString(),pTransition2.toString());
            }
          });
          return GToolkit.separate("\n",list.toArray()).trim();
        }
        else {
          ISet<Character> data=(ISet<Character>)(pEdge.getData());
          return GToolkit.separate("|",new GRedBlackTreeSet<Character>(data).toArray());
        }
      }
    });
    super.visualizationViewer.getRenderer().setVertexRenderer(new Renderer.Vertex<V,IEdge<V,ISet<?>>>() {
      @Override
      public void paintVertex(RenderContext<V,IEdge<V,ISet<?>>> pRenderContext, Layout<V,IEdge<V,ISet<?>>> pLayout, V pVertex) {
        boolean isPicked=pRenderContext.getPickedVertexState().isPicked(pVertex);
        Point2D point=pRenderContext.getMultiLayerTransformer().transform(Layer.LAYOUT,pLayout.transform(pVertex));
        Shape shape=pRenderContext.getVertexShapeTransformer().transform(pVertex);
        GraphicsDecorator graphics=pRenderContext.getGraphicsContext();
        Rectangle2D bounds=shape.getBounds2D();
        Stroke oldStroke=graphics.getStroke();
        Paint oldPaint=graphics.getPaint();
        double x=point.getX(),y=point.getY();
        if (pVertex.equals(automaton.getInitialState())) {
          IList<Point2D> points=new GArrayList<Point2D>(automaton.getStates().size());
          Point2D p=pLayout.transform(pVertex);
          for (V neighbor:graph.getNeighbors(pVertex)) {
            if (!neighbor.equals(pVertex)) points.addLast(pLayout.transform(neighbor));
          }
          double bestScore=Double.NEGATIVE_INFINITY,bestAngle=0.0;
          for (Point2D q:points) {
            for (Point2D r:points) {
              double score=GGeometry.angleBetween(p,q,r);
              if (score>bestScore) {
                bestScore=score;
                bestAngle=GGeometry.angle(p,q)+0.5*score*(GGeometry.crossProduct(p,q,r)>0?1:-1);
              }
            }
          }
          double r=bounds.getHeight()/2+2;
          AffineTransform transform=graphics.getTransform();
          Shape arrowShape=getEdgeArrowTransformer().getDefaultValue();
          graphics.setTransform(compose(translate(-r,0),rotate(bestAngle),translate(x,y),transform));
          graphics.setStroke(getEdgeArrowStrokeTransformer().getDefaultValue());
          graphics.setPaint(getEdgeDrawPaintTransformer().getDefaultValue());
          graphics.draw(arrowShape);
          graphics.draw(line(0,0,-17,0));
          graphics.fill(arrowShape);
          graphics.setTransform(transform);
        }
        AffineTransform transform=graphics.getTransform();
        graphics.setTransform(compose(translate(x,y),transform));
        graphics.setStroke(pRenderContext.getVertexStrokeTransformer().transform(pVertex));
        graphics.setPaint(isPicked&&pickedColor!=null?pickedColor:pRenderContext.getVertexFillPaintTransformer().transform(pVertex));
        graphics.fill(shape);
        graphics.setPaint(pRenderContext.getVertexDrawPaintTransformer().transform(pVertex));
        graphics.draw(shape);
        if (automaton.isAcceptState(pVertex)) graphics.draw(transform(shape,scale(0.75,0.75,bounds.getCenterX(),bounds.getCenterY())));
        graphics.setTransform(transform);
        graphics.setStroke(oldStroke);
        graphics.setPaint(oldPaint);
      }
    });
  }
  
  public IAutomaton<V> getAutomaton() {
    return automaton;
  }
  
}
