package gold.visualization.graph;

import java.awt.*;
import java.awt.event.*;
import java.io.*;
import java.util.concurrent.*;
import javax.swing.*;
import javax.swing.filechooser.*;
import gold.structures.graph.*;
import gold.structures.tree.*;
import gold.swing.util.*;
import gold.visualization.util.*;
import edu.uci.ics.jung.algorithms.layout.*;
import edu.uci.ics.jung.graph.*;
import edu.uci.ics.jung.visualization.control.*;

@SuppressWarnings("serial")
public class GGraphFrame extends JFrame {
  
  public static GGraphFrame show(Graph<?,?> pGraph) {
    GGraphFrame frame=new GGraphFrame(pGraph);
    frame.setVisible(true);
    return frame;
  }
  
  public static GGraphFrame show(IGraph<?,?> pGraph) {
    return GGraphFrame.show(pGraph.toJungView());
  }
  
  public static GGraphFrame show(ITree<?> pTree) {
    return GGraphFrame.show(pTree.toJungCopy());
  }
  
  protected final GGraphMenuBar menuBar=new GGraphMenuBar();
  protected GGraphPainter<?,?> painter=null;
  
  public <V,E> GGraphFrame(Graph<V,E> pGraph) {
    super("GOLD Graph Viewer");
    setIconImage(GUtilities.getIcon("famfamfam/coins.png").getImage());
    setJMenuBar(menuBar);
    setPreferredSize(new Dimension(600,400));
    setDefaultCloseOperation(WindowConstants.DISPOSE_ON_CLOSE);
    pack();
    // La siguiente línea necesita JFrame.setDefaultLookAndFeelDecorated(false)
    // setExtendedState(getExtendedState()|JFrame.MAXIMIZED_BOTH);
    GUtilities.center(this);
    setGraph(pGraph);
  }
  
  public GGraphFrame(IGraph<?,?> pGraph) {
    this(pGraph.toJungView());
  }
  
  public GGraphFrame(ITree<?> pTree) {
    this(pTree.toJungCopy());
  }
  
  public GGraphPainter<?,?> getPainter() {
    return painter;
  }
  
  public Graph<?,?> getGraph() {
    return painter.getGraph();
  }
  
  public <V,E> void setGraph(Graph<V,E> pGraph) {
    if (painter!=null) getContentPane().remove(painter.getCanvas());
    painter=new GGraphPainter<V,E>(pGraph,getContentPane().getSize());
    getContentPane().add(painter.getCanvas(),BorderLayout.CENTER);
    painter.getCanvas().revalidate();
    repaint();
    menuBar.refresh();
  }
  
  @Override
  public void setVisible(boolean pVisible) {
    if (pVisible) menuBar.refresh();
    super.setVisible(pVisible);
  }
  
  protected class GGraphMenuBar extends JMenuBar implements ActionListener {
    
    protected final static String EXTENSION="ggf";
    
    protected final JMenuItem buttonOpen=GUtilities.getMenuItem(this,"Open",'O',GUtilities.getIcon("jlfgr/Open16.gif"),"open");
    protected final JMenuItem buttonSaveAs=GUtilities.getMenuItem(this,"Save as...",'A',GUtilities.getIcon("jlfgr/SaveAs16.gif"),"saveAs");
    protected final JMenuItem buttonExportAsImage=GUtilities.getMenuItem(this,"Export as image",'I',GUtilities.getIcon("jlfgr/Export16.gif"),"exportAsImage");
    protected final JMenuItem buttonExit=GUtilities.getMenuItem(this,"Exit",'X',GUtilities.getIcon("jlfgr/Close16.gif"),"exit");
    protected final JMenu buttonFile=GUtilities.getMenu(this,"File",'F',null,null,new JMenuItem[]{buttonOpen,null,buttonSaveAs,buttonExportAsImage,null,buttonExit});
    protected final JMenuItem buttonLayoutCircle=GUtilities.getRadioButtonMenuItem(this,"Circle layout",'C',"layout:CIRCLE");
    protected final JMenuItem buttonLayoutFR=GUtilities.getRadioButtonMenuItem(this,"Fruchterman-Reingold force-directed layout",'F',"layout:FR");
    protected final JMenuItem buttonLayoutFR2=GUtilities.getRadioButtonMenuItem(this,"Fruchterman-Reingold force-directed optimized layout",'R',"layout:FR2");
    protected final JMenuItem buttonLayoutISOM=GUtilities.getRadioButtonMenuItem(this,"Self-organizing map layout",'M',"layout:ISOM");
    protected final JMenuItem buttonLayoutKK=GUtilities.getRadioButtonMenuItem(this,"Kamada-Kawai layout",'K',"layout:KK");
    protected final JMenuItem buttonLayoutSpring=GUtilities.getRadioButtonMenuItem(this,"Spring layout",'S',"layout:SPRING");
    protected final JMenuItem buttonLayoutSpring2=GUtilities.getRadioButtonMenuItem(this,"Spring layout 2",'P',"layout:SPRING2");
    protected final JMenuItem buttonLayoutBalloon=GUtilities.getRadioButtonMenuItem(this,"Balloon forest layout",'B',"layout:BALLOON");
    protected final JMenuItem buttonLayoutRadial=GUtilities.getRadioButtonMenuItem(this,"Radial forest layout",'D',"layout:RADIAL");
    protected final JMenuItem buttonLayoutTree=GUtilities.getRadioButtonMenuItem(this,"Tree forest layout",'T',"layout:TREE");
    protected final JMenuItem buttonLayout=GUtilities.getMenu(this,"Layout algorithms ",'L',GUtilities.getIcon("gold/layout.png"),null,new JMenuItem[]{buttonLayoutCircle,buttonLayoutFR,buttonLayoutFR2,buttonLayoutISOM,buttonLayoutKK,buttonLayoutSpring,buttonLayoutSpring2,null,buttonLayoutBalloon,buttonLayoutRadial,buttonLayoutTree});
    protected final JMenuItem buttonModePicking=GUtilities.getRadioButtonMenuItem(this,"Picking",'P',"mode:PICKING");
    protected final JMenuItem buttonModeTransforming=GUtilities.getRadioButtonMenuItem(this,"Transforming",'T',"mode:TRANSFORMING");
    protected final JMenuItem buttonMode=GUtilities.getMenu(this,"Mode ",'M',GUtilities.getIcon("famfamfam/mouse.png"),null,new JMenuItem[]{buttonModePicking,buttonModeTransforming});
    protected final JMenuItem buttonStopLayout=GUtilities.getMenuItem(this,"Stop layout algorithm",'S',GUtilities.getIcon("jlfgr/Stop16.gif"),"stopLayout");
    protected final JMenuItem buttonRunLayout=GUtilities.getMenuItem(this,"Run layout algorithm",'R',GUtilities.getIcon("jlfgr/Play16.gif"),"runLayout");
    protected final JMenuItem buttonShowGrid=GUtilities.getCheckBoxMenuItem(this,"Show grid",'D',"showGrid");
    protected final JMenuItem buttonShowVertexLabels=GUtilities.getCheckBoxMenuItem(this,"Show vertex labels",'H',"showVertexLabels");
    protected final JMenu buttonView=GUtilities.getMenu(this,"View",'V',null,null,new JMenuItem[]{buttonLayout,buttonMode,null,buttonStopLayout,buttonRunLayout,null,buttonShowGrid,buttonShowVertexLabels});
    protected final JMenuItem buttonAbout=GUtilities.getMenuItem(this,"About GOLD",'A',GUtilities.getIcon("famfamfam/coins.png"),"about");
    protected final JMenu buttonHelp=GUtilities.getMenu(this,"Help",'H',null,null,new JMenuItem[]{buttonAbout});
    protected final GButtonGroup groupLayout=new GButtonGroup(buttonLayoutCircle,buttonLayoutFR,buttonLayoutFR2,buttonLayoutISOM,buttonLayoutKK,buttonLayoutSpring,buttonLayoutSpring2,buttonLayoutBalloon,buttonLayoutRadial,buttonLayoutTree);
    protected final GButtonGroup groupMode=new GButtonGroup(buttonModePicking,buttonModeTransforming);
    
    protected GGraphMenuBar() {
      buttonLayout.setToolTipText("Configures the layout algorithm used to locate the vertices of the graph");
      buttonMode.setToolTipText("Configures the mode used to manipulate the graph");
      buttonModePicking.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_P,InputEvent.CTRL_MASK));
      buttonModeTransforming.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_T,InputEvent.CTRL_MASK));
      buttonShowGrid.setToolTipText("Show the grid");
      buttonShowVertexLabels.setToolTipText("Show/hide the labels of the vertices");
      buttonOpen.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_O,InputEvent.CTRL_MASK));
      buttonRunLayout.setAccelerator(KeyStroke.getKeyStroke(KeyEvent.VK_F5,0));
      for (JMenu button:new JMenu[]{buttonFile,buttonView,buttonHelp}) {
        add(button);
      }
      refresh();
    }
    
    protected void refresh() {
      if (painter!=null) {
        groupLayout.setSelectedActionCommand("layout:"+painter.getLayoutType().name(),buttonLayoutFR);
        groupLayout.setSelectedActionCommand("layout:"+painter.getMode().name(),buttonModePicking);
        buttonShowGrid.setSelected(painter.getShowGrid());
        buttonShowVertexLabels.setSelected(painter.getShowVertexLabels());
        GUtilities.enable(painter.isDisplayingForest(),buttonLayoutBalloon,buttonLayoutRadial,buttonLayoutTree);
      }
    }
    
    @Override
    public void actionPerformed(ActionEvent pActionEvent) {
      actionPerformed(pActionEvent.getActionCommand());
    }
    
    protected void actionPerformed(String pActionCommand) {
      String argument=pActionCommand.substring(pActionCommand.indexOf(':')+1);
      if (pActionCommand.equals("open")) {
        new GOpenGraphDialog().display();
      }
      else if (pActionCommand.equals("saveAs")) {
        new GSaveGraphDialog().display();
      }
      else if (pActionCommand.equals("exportAsImage")) {
        new GExportAsImageGraphDialog().display();
      }
      else if (pActionCommand.equals("exit")) {
        GGraphFrame.this.dispose();
      }
      else if (pActionCommand.startsWith("layout:")) {
        painter.setLayoutType(GLayoutType.valueOf(argument));
      }
      else if (pActionCommand.startsWith("mode:")) {
        painter.setMode(ModalGraphMouse.Mode.valueOf(argument));
      }
      else if (pActionCommand.equals("stopLayout")) {
        Layout<?,?> layout=painter.getDelegateLayout();
        if (layout instanceof AbstractLayout) ((AbstractLayout<?,?>)layout).lock(true);
      }
      else if (pActionCommand.equals("runLayout")) {
        actionPerformed(groupLayout.getSelectedActionCommand(buttonLayoutKK));
      }
      else if (pActionCommand.equals("showGrid")) {
        painter.setShowGrid(buttonShowGrid.isSelected());
      }
      else if (pActionCommand.equals("showVertexLabels")) {
        painter.setShowVertexLabels(buttonShowVertexLabels.isSelected());
      }
      else if (pActionCommand.equals("about")) {
        GUtilities.show(new GAboutDialog(this),buttonHelp);
      }
      else {
        GOptionDialog.showError(GGraphMenuBar.this,new IllegalArgumentException("Command not found: "+pActionCommand));
      }
    }
    
    protected class GOpenGraphDialog extends GFileChooser {
      
      public GOpenGraphDialog() {
        super(JFileChooser.OPEN_DIALOG);
        addChoosableFileFilter(new FileNameExtensionFilter("GOLD Graph File (*."+EXTENSION+")",EXTENSION));
        acceptButton.setText("Open");
        acceptButton.setToolTipText("Open the graph from the specified file.");
      }
      
      public void display() {
        if (showFileDialog(GGraphFrame.this,"Open graph")==JFileChooser.APPROVE_OPTION) {
          try {
            final File file=getSelectedFile();
            if (file==null) throw new IllegalStateException("You must select a file.");
            if (!file.exists()||!file.isFile()) throw new IllegalStateException("You must select a file.");
            GWaitDialog.wait(GGraphFrame.this,new Callable<Void>() {
              @Override
              public Void call() throws Exception {
                ObjectInputStream inputStream=new ObjectInputStream(new FileInputStream(file));
                Graph<?,?> graph=(Graph<?,?>)(inputStream.readObject());
                inputStream.close();
                GGraphFrame.this.setGraph(graph);
                return null;
              }
            });
            GOptionDialog.showInfo(GGraphMenuBar.this,"The graph was opened successfully.");
          }
          catch (Throwable th) {
            GOptionDialog.showError(GGraphMenuBar.this,th);
          }
        }
      }
      
    }
    
    public final class GSaveGraphDialog extends GFileChooser {
      
      public GSaveGraphDialog() {
        super(JFileChooser.SAVE_DIALOG);
        addChoosableFileFilter(new FileNameExtensionFilter("GOLD Graph File (*."+EXTENSION+")",EXTENSION));
        acceptButton.setText("Save as");
        acceptButton.setToolTipText("Save the graph in the specified file.");
      }
      
      public void display() {
        if (showFileDialog(GGraphFrame.this,"Save graph")==JFileChooser.APPROVE_OPTION) {
          try {
            final File file=verifyFileForSaving(GGraphFrame.this,"."+EXTENSION);
            if (file!=null) {
              ObjectOutputStream outputStream=new ObjectOutputStream(new FileOutputStream(file));
              outputStream.writeObject(painter.getGraph());
              outputStream.close();
            }
          }
          catch (Throwable th) {
            GOptionDialog.showError(GGraphMenuBar.this,th);
          }
        }
      }
      
    }
    
    public final class GExportAsImageGraphDialog extends GFileChooser {
      
      public GExportAsImageGraphDialog() {
        super(JFileChooser.SAVE_DIALOG);
        addChoosableFileFilter(new FileNameExtensionFilter("PNG Image (*.png)","png"));
        acceptButton.setText("Export");
        acceptButton.setToolTipText("Export the PNG image in the specified file.");
      }
      
      public void display() {
        if (showFileDialog(GGraphFrame.this,"Export as image")==JFileChooser.APPROVE_OPTION) {
          try {
            File file=verifyFileForSaving(GGraphFrame.this,".png");
            if (file!=null) {
              painter.getImage().crop(new Insets(5,5,5,5)).save(file,"png");
              GOptionDialog.showInfo(this,"The image was exported successfully.");
            }
          }
          catch (Throwable th) {
            GOptionDialog.showError(GGraphMenuBar.this,th);
          }
        }
      }
      
    }
    
  }
  
}
