package gold.visualization.graph;

import java.awt.*;
import java.awt.event.*;
import java.awt.geom.*;
import java.text.*;
import java.util.*;
import java.util.List;
import javax.swing.*;
import javax.swing.border.*;
import gold.structures.graph.*;
import gold.structures.list.*;
import gold.structures.tree.*;
import gold.structures.tree.nary.*;
import gold.structures.tuple.*;
import gold.swing.util.*;
import gold.util.*;
import gold.visualization.util.*;
import edu.uci.ics.jung.algorithms.layout.*;
import edu.uci.ics.jung.graph.*;
import edu.uci.ics.jung.graph.util.*;
import edu.uci.ics.jung.visualization.*;
import edu.uci.ics.jung.visualization.control.*;
import edu.uci.ics.jung.visualization.decorators.*;
import edu.uci.ics.jung.visualization.picking.*;
import edu.uci.ics.jung.visualization.renderers.*;
import edu.uci.ics.jung.visualization.renderers.Renderer;
import edu.uci.ics.jung.visualization.transform.shape.*;
import static gold.visualization.util.GAffineTransforms.*;
import static gold.visualization.util.GShapes.*;

@SuppressWarnings("serial")
public class GGraphPainter<V,E> implements IPainter<Graph<V,E>> {
  
  public static <V,E> GGraphPainter<V,IEdge<V,E>> createGraphPainter(IGraph<V,E> pGraph) {
    return new GGraphPainter<V,IEdge<V,E>>(pGraph.toJungView());
  }
  
  public static <V,E> GGraphPainter<V,IEdge<V,E>> createGraphPainter(IGraph<V,E> pGraph, Dimension pPreferredSize) {
    return new GGraphPainter<V,IEdge<V,E>>(pGraph.toJungView(),pPreferredSize);
  }
  
  public static <E> GGraphPainter<ITree<E>,IPair<ITree<E>>> createTreePainter(ITree<E> pTree) {
    return new GGraphPainter<ITree<E>,IPair<ITree<E>>>(pTree.toJungCopy());
  }
  
  public static <E> GGraphPainter<ITree<E>,IPair<ITree<E>>> createTreePainter(ITree<E> pTree, Dimension pPreferredSize) {
    return new GGraphPainter<ITree<E>,IPair<ITree<E>>>(pTree.toJungCopy(),pPreferredSize);
  }
  
  protected final Graph<V,E> graph;
  protected final GVisualizationViewer visualizationViewer;
  protected final GraphZoomScrollPane scrollPane;
  protected final ITransformer<V,String> vertexToolTipTransformer=new GSpecialVertexTooltipTransformer();
  protected Color pickedColor=new Color(0,255,255);
  protected boolean showVertexLabels=true;
  protected boolean showEdgeData=false;
  protected boolean showEdgeCosts=true;
  protected boolean showGrid=true;
  protected int gridSize=10;
  
  public GGraphPainter(Graph<V,E> pGraph) {
    this(pGraph,null);
  }
  
  public GGraphPainter(Graph<V,E> pGraph, Dimension pPreferredSize) {
    int edgeColor=0x204020;
    graph=pGraph;
    visualizationViewer=new GVisualizationViewer((isDisplayingForest()?GLayoutType.TREE:GLayoutType.KK).getLayout(graph),pPreferredSize!=null?pPreferredSize:new Dimension(800,600));
    scrollPane=new GraphZoomScrollPane(visualizationViewer);
    RenderContext<V,E> renderContext=visualizationViewer.getRenderContext();
    renderContext.setLabelOffset(0);
    renderContext.setVertexDrawPaintTransformer(new GGeneralTransformer<V,Paint>(new Color(0x000000)));
    renderContext.setVertexFillPaintTransformer(new GGeneralTransformer<V,Paint>(new Color(0xEAEAFF)));
    renderContext.setVertexFontTransformer(new GGeneralTransformer<V,Font>(new Font("Verdana",Font.BOLD,11)));
    renderContext.setVertexLabelTransformer(new GSpecialVertexLabelTransformer());
    renderContext.setVertexShapeTransformer(new GGeneralTransformer<V,Shape>(circle(0,0,12.0)));
    renderContext.setVertexStrokeTransformer(new GGeneralTransformer<V,Stroke>(new BasicStroke(1.0f)));
    renderContext.setArrowDrawPaintTransformer(new GGeneralTransformer<E,Paint>(new Color(edgeColor)));
    renderContext.setArrowFillPaintTransformer(new GGeneralTransformer<E,Paint>(new Color(edgeColor)));
    renderContext.setEdgeArrowStrokeTransformer(new GGeneralTransformer<E,Stroke>(new BasicStroke(1.0f)));
    renderContext.setEdgeArrowTransformer(new GGraphContextTransformer<V,E,Shape>(polygon(new double[][]{{0,0},{-7,3},{-5,0},{-7,-3}})));
    renderContext.setEdgeDrawPaintTransformer(new GSpecialEdgeDrawPaintTransformer(new Color(edgeColor)));
    renderContext.setEdgeFillPaintTransformer(new GGeneralTransformer<E,Paint>(new Color(0x00000000,true)));
    renderContext.setEdgeFontTransformer(new GGeneralTransformer<E,Font>(new Font("Verdana",Font.PLAIN,11)));
    renderContext.setEdgeLabelClosenessTransformer(new GGraphContextTransformer<V,E,Number>(0.5));
    renderContext.setEdgeLabelTransformer(new GSpecialEdgeLabelTransformer());
    renderContext.setEdgeShapeTransformer(new GGraphContextTransformer<V,E,Shape>(new GSpecialEdgeShapeTransformer()));
    renderContext.setEdgeStrokeTransformer(new GGeneralTransformer<E,Stroke>(new BasicStroke(1.0f)));
    renderContext.setEdgeLabelRenderer(new GEdgeLabelRenderer(true));
    GSpecialGraphMouse graphMouse=new GSpecialGraphMouse();
    visualizationViewer.addMouseListener(graphMouse);
    visualizationViewer.addMouseMotionListener(graphMouse);
    visualizationViewer.getRenderer().setEdgeRenderer(new BasicEdgeRenderer<V,E>() {
      @Override
      public void paintEdge(RenderContext<V,E> pRenderContext, Layout<V,E> pLayout, E pEdge) {
        boolean isPicked=pickedColor!=null&&pRenderContext.getPickedEdgeState().isPicked(pEdge);
        Shape shape=getEdgeShapeTransformer().transform(pEdge);
        if ((shape instanceof GeneralPath)||(shape instanceof Line2D)) {
          GraphicsDecorator graphics=pRenderContext.getGraphicsContext();
          Pair<V> pair=pLayout.getGraph().getEndpoints(pEdge);
          Point2D a=pRenderContext.getMultiLayerTransformer().transform(Layer.LAYOUT,pLayout.transform(pair.getFirst()));
          Point2D b=pRenderContext.getMultiLayerTransformer().transform(Layer.LAYOUT,pLayout.transform(pair.getSecond()));
          double dx=b.getX()-a.getX(),dy=b.getY()-a.getY();
          GeneralPath path=new GeneralPath(Path2D.WIND_EVEN_ODD);
          List<Point2D> points=new ArrayList<Point2D>();
          for (double[] g:points(shape)) {
            double gx=g[0],gy=g[1],px=a.getX()+dx*gx-dy*gy,py=a.getY()+dy*gx+dx*gy;
            if (points.isEmpty()) {
              path.moveTo(px,py);
            }
            else {
              path.lineTo(px,py);
            }
            points.add(new Point2D.Double(px,py));
          }
          graphics.setStroke(pRenderContext.getEdgeStrokeTransformer().transform(pEdge));
          graphics.setPaint(getEdgeDrawPaintTransformer().transform(pEdge));
          graphics.draw(path);
          if (graph.getEdgeType(pEdge)!=EdgeType.UNDIRECTED) {
            @SuppressWarnings("unchecked")
            AffineTransform affineTransform=edgeArrowRenderingSupport.getArrowTransform(pRenderContext,path,translate(b.getX(),b.getY()).createTransformedShape(pRenderContext.getVertexShapeTransformer().transform(pair.getSecond())));
            if (affineTransform!=null) {
              Shape arrow=affineTransform.createTransformedShape(pRenderContext.getEdgeArrowTransformer().transform(Context.<Graph<V,E>,E>getInstance(graph,pEdge)));
              graphics.setPaint(pRenderContext.getArrowFillPaintTransformer().transform(pEdge));
              graphics.fill(arrow);
              graphics.setPaint(pRenderContext.getArrowDrawPaintTransformer().transform(pEdge));
              graphics.draw(arrow);
            }
          }
          if (isPicked) {
            graphics.setStroke(new BasicStroke(1.0f));
            for (int i=1; i<points.size()-1; i++) {
              Point2D p=points.get(i);
              graphics.setPaint(Color.GRAY);
              graphics.draw(ellipse(p.getX(),p.getY(),4,4));
              graphics.setPaint(pickedColor.darker());
              graphics.fill(ellipse(p.getX(),p.getY(),4,4));
            }
          }
        }
        else {
          super.paintEdge(pRenderContext,pLayout,pEdge);
        }
      }
    });
    visualizationViewer.getRenderer().setEdgeLabelRenderer(new BasicEdgeLabelRenderer<V,E>() {
      @Override
      public Component prepareRenderer(RenderContext<V,E> pRenderContext, EdgeLabelRenderer pEdgeLabelRenderer, Object pValue, boolean pIsSelected, E pEdge) {
        Component component=super.prepareRenderer(pRenderContext,pEdgeLabelRenderer,pValue,pIsSelected,pEdge);
        if (component instanceof JLabel) {
          JLabel label=(JLabel)component;
          Pair<V> pair=graph.getEndpoints(pEdge);
          IList<V> vertices=GCollections.unmodifiableList(graph.getVertices());
          boolean isReversed=graph.findEdge(pair.getSecond(),pair.getFirst())!=null;
          boolean isLoop=pair.getFirst().equals(pair.getSecond());
          boolean isVisible=isLoop||GGraphPainter.this.isDisplayingDirectedGraph()||!isReversed||vertices.firstIndexOf(pair.getFirst())<=vertices.firstIndexOf(pair.getSecond());
          String value=isVisible&&pValue!=null?pValue.toString():"";
          label.setVisible(isVisible);
          label.setText(value.toLowerCase().startsWith("<html>")?value:"<html><center>"+GUtilities.escapeHTML(value)+"</center></html>");
        }
        return component;
      }
      @Override
      public void labelEdge(RenderContext<V,E> pRenderContext, Layout<V,E> pLayout, E pEdge, String pLabel) {
        if (pLabel==null||pLabel.isEmpty()) return;
        Graph<V,E> graph=pLayout.getGraph();
        Pair<V> pair=graph.getEndpoints(pEdge);
        GraphicsDecorator graphics=pRenderContext.getGraphicsContext();
        Shape shape=getEdgeShapeTransformer().transform(pEdge);
        if ((shape instanceof GeneralPath)||(shape instanceof Line2D)) {
          Point2D a=pRenderContext.getMultiLayerTransformer().transform(Layer.LAYOUT,pLayout.transform(pair.getFirst()));
          Point2D b=pRenderContext.getMultiLayerTransformer().transform(Layer.LAYOUT,pLayout.transform(pair.getSecond()));
          List<double[]> points=new ArrayList<double[]>(Arrays.asList(points(shape)));
          double d=a.distance(b),dx=b.getX()-a.getX(),dy=b.getY()-a.getY();
          double p[]=points.get(points.size()/2-1),q[]=points.get(points.size()/2);
          double g[]=p,gx=g[0]*d,gy=g[1]*d,px=a.getX()+dx*gx/d-dy*gy/d,py=a.getY()+dy*gx/d+dx*gy/d;
          double h[]=q,hx=h[0]*d,hy=h[1]*d,qx=a.getX()+dx*hx/d-dy*hy/d,qy=a.getY()+dy*hx/d+dx*hy/d;
          boolean isRotateEdgeLabels=pRenderContext.getEdgeLabelRenderer().isRotateEdgeLabels();
          double theta=Math.atan2(qy-py,qx-px);
          Component component=prepareRenderer(pRenderContext,pRenderContext.getEdgeLabelRenderer(),pLabel,pRenderContext.getPickedEdgeState().isPicked(pEdge),pEdge);
          Dimension dimension=component.getPreferredSize();
          AffineTransform oldTransform=graphics.getTransform();
          graphics.setTransform(compose(translate(-dimension.width*0.5,0.0),isRotateEdgeLabels?rotate(qx-px<0?theta+Math.PI:theta):null,translate((px+qx)/2,(py+qy)/2),oldTransform));
          graphics.draw(component,pRenderContext.getRendererPane(),0,0,dimension.width,dimension.height,true);
          graphics.setTransform(oldTransform);
        }
        else if (pair.getFirst().equals(pair.getSecond())) {
          Point2D point=pRenderContext.getMultiLayerTransformer().transform(Layer.LAYOUT,pLayout.transform(pair.getFirst()));
          double height=pRenderContext.getVertexShapeTransformer().transform(pair.getFirst()).getBounds2D().getHeight();
          Component component=prepareRenderer(pRenderContext,pRenderContext.getEdgeLabelRenderer(),pLabel,pRenderContext.getPickedEdgeState().isPicked(pEdge),pEdge);
          Dimension dimension=component.getPreferredSize();
          AffineTransform oldTransform=graphics.getTransform();
          graphics.setTransform(compose(translate(-dimension.width*0.5,-dimension.height-height*0.5-12),translate(point.getX(),point.getY()),oldTransform));
          graphics.draw(component,pRenderContext.getRendererPane(),0,0,dimension.width,dimension.height,true);
          graphics.setTransform(oldTransform);
        }
        else {
          super.labelEdge(pRenderContext,pLayout,pEdge,pLabel);
        }
      }
    });
    visualizationViewer.getRenderer().setVertexRenderer(new Renderer.Vertex<V,E>() {
      @Override
      public void paintVertex(RenderContext<V,E> pRenderContext, Layout<V,E> pLayout, V pVertex) {
        boolean isPicked=pRenderContext.getPickedVertexState().isPicked(pVertex);
        Point2D point=pRenderContext.getMultiLayerTransformer().transform(Layer.LAYOUT,pLayout.transform(pVertex));
        Shape shape=pRenderContext.getVertexShapeTransformer().transform(pVertex);
        GraphicsDecorator graphics=pRenderContext.getGraphicsContext();
        Stroke oldStroke=graphics.getStroke();
        Paint oldPaint=graphics.getPaint();
        double x=point.getX(),y=point.getY();
        AffineTransform transform=graphics.getTransform();
        graphics.setTransform(compose(translate(x,y),transform));
        graphics.setStroke(pRenderContext.getVertexStrokeTransformer().transform(pVertex));
        graphics.setPaint(isPicked&&pickedColor!=null?pickedColor:pRenderContext.getVertexFillPaintTransformer().transform(pVertex));
        graphics.fill(shape);
        graphics.setPaint(pRenderContext.getVertexDrawPaintTransformer().transform(pVertex));
        graphics.draw(shape);
        graphics.setTransform(transform);
        graphics.setStroke(oldStroke);
        graphics.setPaint(oldPaint);
      }
    });
    visualizationViewer.getRenderer().setVertexLabelRenderer(new BasicVertexLabelRenderer<V,E>() {
      @Override
      public void labelVertex(RenderContext<V,E> pRenderContext, Layout<V,E> pLayout, V pVertex, String pLabel) {
        if (!getShowVertexLabels()) return;
        Point2D point=pRenderContext.getMultiLayerTransformer().transform(Layer.LAYOUT,pLayout.transform(pVertex));
        GraphicsDecorator graphics=pRenderContext.getGraphicsContext();
        graphics.setFont(pRenderContext.getVertexFontTransformer().transform(pVertex));
        FontMetrics fontMetrics=graphics.getFontMetrics();
        Rectangle2D textBounds=fontMetrics.getStringBounds(pLabel,graphics.getDelegate());
        double textWidth=textBounds.getWidth(),textHeight=fontMetrics.getAscent()-fontMetrics.getDescent();
        Paint paint=pRenderContext.getVertexFillPaintTransformer().transform(pVertex);
        Color color=(paint instanceof Color)?(Color)paint:Color.WHITE;
        graphics.setPaint((color.getRed()+color.getGreen()+color.getBlue())/3>20?new Color(0,0,0):new Color(255,255,255));
        graphics.drawString(pLabel,(float)(point.getX()-textWidth*0.5),(float)(point.getY()+textHeight*0.5));
      }
    });
    if (isDisplayingForest()) {
      ITransformer<V,Paint> vertexFillPaintTransformer=getVertexFillPaintTransformer();
      for (V vertex:graph.getVertices()) {
        if (vertex instanceof GTrie) {
          GTrie trie=(GTrie)vertex;
          if (trie.isMarked()) vertexFillPaintTransformer.set(vertex,new Color(181,230,29));
          else if (trie.isRoot()) vertexFillPaintTransformer.set(vertex,new Color(200,191,231));
        }
        else if (vertex instanceof GQuadtree) {
          GQuadtree quadtree=(GQuadtree)vertex;
          vertexFillPaintTransformer.set(vertex,quadtree.getRoot().getColor());
        }
      }
    }
  }
  
  public Graph<V,E> getGraph() {
    return graph;
  }
  
  public VisualizationViewer<V,E> getVisualizationViewer() {
    return visualizationViewer;
  }
  
  public RenderContext<V,E> getRenderContext() {
    return visualizationViewer.getRenderContext();
  }
  
  public Paint getPickedColor() {
    return pickedColor;
  }
  
  public boolean getShowVertexLabels() {
    return showVertexLabels;
  }
  
  public boolean getShowEdgeData() {
    return showEdgeData;
  }
  
  public boolean getShowEdgeCosts() {
    return showEdgeCosts;
  }
  
  public boolean getShowGrid() {
    return showGrid;
  }
  
  public int getGridSize() {
    return gridSize;
  }
  
  public boolean isDisplayingDirectedGraph() {
    boolean flag=(graph instanceof DirectedGraph);
    if (graph instanceof GViewJungGraph) flag=!((GViewJungGraph<?,?>)graph).getAdaptee().isUndirected();
    return flag;
  }
  
  public boolean isDisplayingForest() {
    return graph instanceof Forest;
  }
  
  public Layout<V,E> getLayout() {
    return visualizationViewer.getGraphLayout();
  }
  
  public GLayoutConfiguration getLayoutConfiguration() {
    Layout<V,E> layout=getLayout();
    GLayoutConfiguration layoutConfiguration=new GLayoutConfiguration();
    for (V vertex:graph.getVertices()) {
      layoutConfiguration.getVertexLayout().put(vertex.toString(),layout.transform(vertex));
    }
    ITransformer<E,Shape> edgeShapeTransformer=getEdgeShapeTransformer();
    for (E edge:graph.getEdges()) {
      Shape shape=edgeShapeTransformer.transform(edge);
      if (shape instanceof GeneralPath) {
        List<Point2D> points=new ArrayList<Point2D>();
        for (double[] point:GShapes.points(shape)) {
          points.add(new Point2D.Double(point[0],point[1]));
        }
        layoutConfiguration.getEdgeLayout().put(new GPair<String>(graph.getSource(edge).toString(),graph.getDest(edge).toString()),points.toArray(new Point2D[0]));
      }
    }
    return layoutConfiguration;
  }
  
  public void setLayoutConfiguration(GLayoutConfiguration pLayoutConfiguration) {
    Layout<V,?> newLayout=getLayout();
    for (V vertex:graph.getVertices()) {
      try {
        Point2D point=pLayoutConfiguration.getVertexLayout().get(vertex.toString());
        if (point!=null) newLayout.setLocation(vertex,point);
      }
      catch (Throwable th) {
      }
    }
    ITransformer<E,Shape> edgeShapeTransformer=getEdgeShapeTransformer();
    for (E edge:graph.getEdges()) {
      Point2D[] array=pLayoutConfiguration.getEdgeLayout().get(new GPair<String>(graph.getSource(edge).toString(),graph.getDest(edge).toString()));
      if (array!=null) {
        List<double[]> points=new ArrayList<double[]>();
        for (Point2D point:array) {
          points.add(new double[]{point.getX(),point.getY()});
        }
        edgeShapeTransformer.set(edge,polygonalChain(points.toArray(new double[0][])));
      }
    }
  }
  
  public Layout<V,E> getDelegateLayout() {
    Layout<V,E> layout=getLayout();
    while (layout instanceof LayoutDecorator) {
      layout=((LayoutDecorator<V,E>)layout).getDelegate();
    }
    return layout;
  }
  
  public GLayoutType getLayoutType() {
    return GLayoutType.getLayoutType(getDelegateLayout());
  }
  
  public ModalGraphMouse.Mode getMode() {
    try {
      return (ModalGraphMouse.Mode)(((AbstractModalGraphMouse)(visualizationViewer.getGraphMouse())).getModeComboBox().getSelectedItem());
    }
    catch (Throwable th) {
      return ModalGraphMouse.Mode.PICKING;
    }
  }
  
  public void clearSelection() {
    RenderContext<V,E> renderContext=getRenderContext();
    renderContext.getPickedVertexState().clear();
    renderContext.getPickedEdgeState().clear();
    repaint();
  }
  
  public void setLayoutType(GLayoutType pLayoutType) {
    Layout<V,E> layout=pLayoutType.getLayout(getGraph());
    if (pLayoutType!=GLayoutType.TREE) layout.setSize(visualizationViewer.getSize());
    visualizationViewer.setGraphLayout(layout);
    repaint();
  }
  
  public void setMode(ModalGraphMouse.Mode pMode) {
    ((AbstractModalGraphMouse)(visualizationViewer.getGraphMouse())).setMode(pMode);
    repaint();
  }
  
  public void setPickedColor(Color pColor) {
    pickedColor=pColor;
    repaint();
  }
  
  public void setShowVertexLabels(boolean pShowVertexLabels) {
    showVertexLabels=pShowVertexLabels;
    repaint();
  }
  
  public void setShowEdgeData(boolean pShowEdgeData) {
    showEdgeData=pShowEdgeData;
    repaint();
  }
  
  public void setShowEdgeCosts(boolean pShowEdgeCosts) {
    showEdgeCosts=pShowEdgeCosts;
    repaint();
  }
  
  public void setShowGrid(boolean pShowGrid) {
    showGrid=pShowGrid;
    repaint();
  }
  
  public void setGridSize(int pGridSize) {
    gridSize=Math.max(1,pGridSize);
    repaint();
  }
  
  public void repaint() {
    visualizationViewer.repaint();
  }
  
  public ITransformer<V,String> getVertexToolTipTransformer() {
    return vertexToolTipTransformer;
  }
  
  public ITransformer<V,Paint> getVertexDrawPaintTransformer() {
    return (ITransformer<V,Paint>)(getRenderContext().getVertexDrawPaintTransformer());
  }
  
  public ITransformer<V,Paint> getVertexFillPaintTransformer() {
    return (ITransformer<V,Paint>)(getRenderContext().getVertexFillPaintTransformer());
  }
  
  public ITransformer<V,Font> getVertexFontTransformer() {
    return (ITransformer<V,Font>)(getRenderContext().getVertexFontTransformer());
  }
  
  public ITransformer<V,String> getVertexLabelTransformer() {
    return (ITransformer<V,String>)(getRenderContext().getVertexLabelTransformer());
  }
  
  public ITransformer<V,Shape> getVertexShapeTransformer() {
    return (ITransformer<V,Shape>)(getRenderContext().getVertexShapeTransformer());
  }
  
  public ITransformer<V,Stroke> getVertexStrokeTransformer() {
    return (ITransformer<V,Stroke>)(getRenderContext().getVertexStrokeTransformer());
  }
  
  public ITransformer<E,Paint> getArrowDrawPaintTransformer() {
    return (ITransformer<E,Paint>)(getRenderContext().getArrowDrawPaintTransformer());
  }
  
  public ITransformer<E,Paint> getArrowFillPaintTransformer() {
    return (ITransformer<E,Paint>)(getRenderContext().getArrowFillPaintTransformer());
  }
  
  public ITransformer<E,Stroke> getEdgeArrowStrokeTransformer() {
    return (ITransformer<E,Stroke>)(getRenderContext().getEdgeArrowStrokeTransformer());
  }
  
  @SuppressWarnings("unchecked")
  public ITransformer<E,Shape> getEdgeArrowTransformer() {
    return ((GGraphContextTransformer<V,E,Shape>)(getRenderContext().getEdgeArrowTransformer())).getDelegate();
  }
  
  public ITransformer<E,Paint> getEdgeDrawPaintTransformer() {
    return (ITransformer<E,Paint>)(getRenderContext().getEdgeDrawPaintTransformer());
  }
  
  public ITransformer<E,Paint> getEdgeFillPaintTransformer() {
    return (ITransformer<E,Paint>)(getRenderContext().getEdgeFillPaintTransformer());
  }
  
  public ITransformer<E,Font> getEdgeFontTransformer() {
    return (ITransformer<E,Font>)(getRenderContext().getEdgeFontTransformer());
  }
  
  @SuppressWarnings("unchecked")
  public ITransformer<E,Number> getEdgeLabelClosenessTransformer() {
    return ((GGraphContextTransformer<V,E,Number>)(getRenderContext().getEdgeLabelClosenessTransformer())).getDelegate();
  }
  
  public ITransformer<E,String> getEdgeLabelTransformer() {
    return (ITransformer<E,String>)(getRenderContext().getEdgeLabelTransformer());
  }
  
  @SuppressWarnings("unchecked")
  public ITransformer<E,Shape> getEdgeShapeTransformer() {
    return ((GGraphContextTransformer<V,E,Shape>)(getRenderContext().getEdgeShapeTransformer())).getDelegate();
  }
  
  public ITransformer<E,Stroke> getEdgeStrokeTransformer() {
    return (ITransformer<E,Stroke>)(getRenderContext().getEdgeStrokeTransformer());
  }
  
  @Override
  public GImage getImage() {
    Dimension dimension=visualizationViewer.getSize();
    GImage image=new GImage(dimension.width,dimension.height,visualizationViewer.getBackground());
    visualizationViewer.exportImage(image.createQualityGraphics());
    return image;
  }
  
  @Override
  public GraphZoomScrollPane getCanvas() {
    return scrollPane;
  }
  
  protected class GVisualizationViewer extends VisualizationViewer<V,E> {
    
    private Paint background=Color.WHITE;
    
    public GVisualizationViewer(Layout<V,E> pLayout, Dimension pPreferredSize) {
      super(pLayout,pPreferredSize);
      super.setBackground(new Color(0,0,0,0));
      setGraphMouse(new GModalGraphMouse());
      setRenderingHints(GImage.createRenderingHints());
      setVertexToolTipTransformer(vertexToolTipTransformer);
      setPickSupport(new GSpecialShapePickSupport(this));
    }
    
    public void exportImage(Graphics2D pGraphics) {
      paintComponent(pGraphics);
    }
    
    public void setBackground(Paint pBackground) {
      background=pBackground;
      repaint();
    }
    
    @Override
    public void paint(Graphics pGraphics) {
      Graphics2D graphics=(Graphics2D)pGraphics;
      int w=getWidth(),h=getHeight();
      graphics.setPaint(background);
      graphics.fillRect(0,0,w,h);
      graphics.setRenderingHint(RenderingHints.KEY_ANTIALIASING,RenderingHints.VALUE_ANTIALIAS_ON);
      if (showGrid) {
        graphics.setStroke(new BasicStroke(1.0f));
        graphics.setColor(new Color(220,220,220));
        MultiLayerTransformer transformer=visualizationViewer.getRenderContext().getMultiLayerTransformer();
        Rectangle2D r=transformer.inverseTransform(new Rectangle2D.Double(0,0,w,h)).getBounds2D();
        for (int j=(int)(Math.round(Math.floor(r.getMinY()/gridSize))*gridSize); j<=r.getMaxY(); j+=gridSize) {
          graphics.draw(transformer.transform(new Line2D.Double(r.getMinX(),j,r.getMaxX(),j)));
        }
        for (int i=(int)(Math.round(Math.floor(r.getMinX()/gridSize))*gridSize); i<=r.getMaxX(); i+=gridSize) {
          graphics.draw(transformer.transform(new Line2D.Double(i,r.getMinY(),i,r.getMaxY())));
        }
      }
      super.paint(graphics);
    }
    
  }
  
  protected class GSpecialVertexLabelTransformer extends GGeneralTransformer<V,String> {
    
    @Override
    public String transform(V pVertex) {
      String result=super.transform(pVertex);
      if (result!=null) return result;
      Object data=pVertex;
      if (pVertex instanceof ITree) data=((ITree<?>)pVertex).getRoot();
      if (pVertex instanceof GTrie) data=((GTrie)pVertex).isRoot()?"":data;
      return data!=null?data.toString():"";
    }
    
  }
  
  protected class GSpecialVertexTooltipTransformer extends GGeneralTransformer<V,String> {
    
    @Override
    public String transform(V pVertex) {
      String result=super.transform(pVertex);
      if (result!=null) return result;
      return visualizationViewer.getRenderContext().getVertexLabelTransformer().transform(pVertex);
    }
    
  }
  
  protected class GSpecialEdgeLabelTransformer extends GGeneralTransformer<E,String> {
    
    protected final DecimalFormat DECIMAL_FORMAT=GUtilities.getDecimalFormat("0.########");
    
    @Override
    public String transform(E pEdge) {
      String result=super.transform(pEdge);
      if (result!=null) return result;
      if (GGraphPainter.this.isDisplayingForest()) {
        return "";
      }
      else {
        Object data=pEdge;
        if (pEdge instanceof IEdge) {
          if (GGraphPainter.this.getShowEdgeData()) {
            data=((IEdge<?,?>)pEdge).getData();
          }
          else if (GGraphPainter.this.getShowEdgeCosts()) {
            data=DECIMAL_FORMAT.format(((IEdge<?,?>)pEdge).getCost());
          }
          else {
            data=null;
          }
        }
        return data!=null?data.toString():"";
      }
    }
    
  }
  
  protected class GSpecialEdgeShapeTransformer extends GGeneralTransformer<E,Shape> {
    
    protected final Shape loop=ellipse(0.00,-0.35,0.30,0.40);
    protected final EdgeShape.QuadCurve<V,E> quadCurve=new EdgeShape.QuadCurve<V,E>();
    protected final EdgeShape.Line<V,E> line=new EdgeShape.Line<V,E>();
    
    public GSpecialEdgeShapeTransformer() {
      quadCurve.setControlOffsetIncrement(10.0f);
    }
    
    @Override
    public Shape transform(E pEdge) {
      Shape result=super.transform(pEdge);
      if (result!=null) return result;
      Pair<V> pair=graph.getEndpoints(pEdge);
      IList<V> vertices=GCollections.unmodifiableList(graph.getVertices());
      boolean isReversed=graph.findEdge(pair.getSecond(),pair.getFirst())!=null;
      boolean isLoop=pair.getFirst().equals(pair.getSecond());
      if (isLoop) {
        return loop;
      }
      else if (GGraphPainter.this.isDisplayingDirectedGraph()&&isReversed) {
        return quadCurve.transform(Context.getInstance(graph,pEdge));
      }
      else if (GGraphPainter.this.isDisplayingDirectedGraph()||!isReversed||vertices.firstIndexOf(pair.getFirst())<=vertices.firstIndexOf(pair.getSecond())) {
        return line.transform(Context.getInstance(graph,pEdge));
      }
      else {
        return new Area();
      }
    }
    
  }
  
  protected class GSpecialEdgeDrawPaintTransformer extends GGeneralTransformer<E,Paint> {
    
    protected final Paint defaultPaint;
    
    public GSpecialEdgeDrawPaintTransformer(Paint pDefaultPaint) {
      defaultPaint=pDefaultPaint;
    }
    
    @Override
    public Paint transform(E pEdge) {
      if (visualizationViewer.getPickedEdgeState().isPicked(pEdge)) {
        return pickedColor!=null?pickedColor:defaultPaint;
      }
      else {
        return defaultPaint;
      }
    }
    
  }
  
  protected class GEdgeLabelRenderer extends DefaultEdgeLabelRenderer {
    
    public GEdgeLabelRenderer() {
      this(true);
    }
    
    public GEdgeLabelRenderer(boolean pRotateEdgeLabels) {
      super(pickedColor,pRotateEdgeLabels);
    }
    
    @Override
    public <F> Component getEdgeLabelRendererComponent(JComponent pVisualizationViewer, Object pValue, Font pFont, boolean pIsSelected, F pEdge) {
      GEdgeLabelRenderer component=new GEdgeLabelRenderer(super.isRotateEdgeLabels());
      component.setForeground(pIsSelected&&pickedColor!=null?pickedColor.darker():pVisualizationViewer.getForeground());
      component.setBackground(pVisualizationViewer.getBackground());
      component.setFont(pFont!=null?pFont:pVisualizationViewer.getFont());
      component.setIcon(null);
      component.setBorder(new EmptyBorder(0,0,0,0));
      component.setValue(pValue);
      return component;
    }
    
  }
  
  protected class GModalGraphMouse extends DefaultModalGraphMouse<V,E> {
    
    protected V vertex=null;
    protected Point2D point=null;
    
    protected GModalGraphMouse() {
      setMode(ModalGraphMouse.Mode.PICKING);
    }
    
    @Override
    public void mousePressed(MouseEvent pEvent) {
      super.mousePressed(pEvent);
      Layout<V,E> layout=visualizationViewer.getGraphLayout();
      RenderContext<V,E> renderContext=visualizationViewer.getRenderContext();
      PickedState<V> pickedState=visualizationViewer.getPickedVertexState();
      vertex=visualizationViewer.getPickSupport().getVertex(layout,pEvent.getX(),pEvent.getY());
      if (vertex!=null&&pickedState.getPicked().size()==1&&pickedState.isPicked(vertex)&&GGraphPainter.this.getMode()==ModalGraphMouse.Mode.PICKING) {
        point=GGeometry.subtract(layout.transform(vertex),renderContext.getMultiLayerTransformer().inverseTransform(Layer.LAYOUT,pEvent.getPoint()));
      }
      else {
        point=null;
      }
    }
    
    @Override
    public void mouseDragged(MouseEvent pEvent) {
      super.mouseDragged(pEvent);
      if (vertex!=null&&point!=null) {
        RenderContext<V,E> renderContext=visualizationViewer.getRenderContext();
        Point2D p=GGeometry.add(point,renderContext.getMultiLayerTransformer().inverseTransform(Layer.LAYOUT,pEvent.getPoint()));
        Point2D q=new Point2D.Double(gridSize*Math.floor(p.getX()/gridSize),gridSize*Math.floor(p.getY()/gridSize));
        visualizationViewer.getGraphLayout().setLocation(vertex,q);
      }
    }
    
    @Override
    public void mouseEntered(MouseEvent pEvent) {
      try {
        super.mouseEntered(pEvent);
      }
      catch (Throwable th) {
        /**
         * <code>
         *   Oculta un par de fallas en JUNG:
         *   Exception in thread "AWT-EventQueue-0" java.util.ConcurrentModificationException
         *     at java.util.LinkedHashMap$LinkedHashIterator.nextEntry(Unknown Source)
         *     at java.util.LinkedHashMap$KeyIterator.next(Unknown Source)
         *     at java.util.AbstractCollection.toArray(Unknown Source)
         *     at edu.uci.ics.jung.visualization.control.PluggableGraphMouse.checkMouseListeners(PluggableGraphMouse.java:71)
         *     at edu.uci.ics.jung.visualization.control.PluggableGraphMouse.mouseEntered(PluggableGraphMouse.java:115)
         *   Exception in thread "AWT-EventQueue-0" java.lang.NullPointerException
         *     at edu.uci.ics.jung.visualization.control.PluggableGraphMouse.mouseEntered(PluggableGraphMouse.java:116)
         * </code>
         */
      }
    }
    
  }
  
  protected class GSpecialGraphMouse extends MouseAdapter implements VisualizationViewer.GraphMouse {
    
    protected static final double R=5.0;
    protected E selectedEdge=null;
    protected Line2D selectedSegment=null;
    protected double[][] selectedRoute=null;
    protected int selectedIndex=-1;
    
    protected Point2D getEdgePoint(Line2D pSegment, Point2D pPoint) {
      double d=pSegment.getP1().distance(pSegment.getP2());
      Point2D u=GGeometry.subtract(pSegment.getP2(),pSegment.getP1()),v=GGeometry.subtract(pPoint,pSegment.getP1());
      return new Point2D.Double(GGeometry.pointProduct(u,v)/d,GGeometry.crossProduct(u,v)/d);
    }
    
    protected void updateEdge(E pEdge, double[][] pPoints) {
      getEdgeShapeTransformer().set(pEdge,polygonalChain(pPoints));
      repaint();
    }
    
    @Override
    public void mousePressed(MouseEvent pEvent) {
      if (SwingUtilities.isLeftMouseButton(pEvent)&&pEvent.getClickCount()==1) {
        Layout<V,E> layout=visualizationViewer.getGraphLayout();
        RenderContext<V,E> renderContext=visualizationViewer.getRenderContext();
        E edge=visualizationViewer.getPickSupport().getEdge(layout,pEvent.getX(),pEvent.getY());
        if (edge!=null&&renderContext.getPickedEdgeState().isPicked(edge)&&GGraphPainter.this.getMode()==ModalGraphMouse.Mode.PICKING) {
          Pair<V> pair=graph.getEndpoints(edge);
          Line2D segment=new Line2D.Double(layout.transform(pair.getFirst()),layout.transform(pair.getSecond()));
          double d=segment.getP1().distance(segment.getP2());
          Point2D q=getEdgePoint(segment,renderContext.getMultiLayerTransformer().inverseTransform(Layer.LAYOUT,pEvent.getPoint()));
          Shape shape=getEdgeShapeTransformer().transform(edge);
          if ((shape instanceof GeneralPath)||(shape instanceof Line2D)) {
            List<double[]> points=new ArrayList<double[]>(Arrays.asList(points(shape)));
            for (int i=1; i<points.size()-1; i++) {
              double[] h=points.get(i);
              Point2D f=new Point2D.Double(h[0]*d,h[1]*d);
              if (f.distance(q)<R) {
                if (pEvent.isControlDown()) {
                  points.remove(i);
                  updateEdge(edge,points.toArray(new double[0][]));
                }
                else {
                  selectedEdge=edge;
                  selectedSegment=segment;
                  selectedRoute=points.toArray(new double[0][]);
                  selectedIndex=i;
                }
                return;
              }
            }
            if (pEvent.isControlDown()) {
              for (int i=1; i<points.size(); i++) {
                double[] g=points.get(i-1),h=points.get(i);
                Point2D e=new Point2D.Double(g[0]*d,g[1]*d),f=new Point2D.Double(h[0]*d,h[1]*d);
                if (new Line2D.Double(e,f).ptSegDist(q)<R&&e.distance(q)>=R&&f.distance(q)>=R) {
                  points.add(i,new double[]{q.getX()/d,q.getY()/d});
                  updateEdge(edge,points.toArray(new double[0][]));
                }
              }
            }
          }
        }
      }
    }
    
    @Override
    public void mouseReleased(MouseEvent pEvent) {
      selectedEdge=null;
      selectedSegment=null;
      selectedRoute=null;
      selectedIndex=-1;
    }
    
    @Override
    public void mouseDragged(MouseEvent pEvent) {
      if (selectedEdge!=null) {
        RenderContext<V,E> renderContext=visualizationViewer.getRenderContext();
        Line2D line=selectedSegment;
        double d=line.getP1().distance(line.getP2());
        Point2D z=renderContext.getMultiLayerTransformer().inverseTransform(Layer.LAYOUT,pEvent.getPoint());
        Point2D y=getEdgePoint(line,z);
        Point2D q=new Point2D.Double(y.getX(),y.getY());
        boolean moved=false;
        /**
         * <code>
         * double e=E*1;
         * Point2D p=new Point2D.Double(selectedRoute[selectedIndex-1][0]*d,selectedRoute[selectedIndex-1][1]*d);
         * Point2D r=new Point2D.Double(selectedRoute[selectedIndex+1][0]*d,selectedRoute[selectedIndex+1][1]*d);
         * if (Math.abs(q.getX()-(p.getX()+r.getX())/2)<e&&selectedRoute.length==3) {
         *   q.setLocation((p.getX()+r.getX())/2,q.getY());
         *   moved=true;
         * }
         * if (Math.abs(q.getX()-r.getX())<e||Math.abs(q.getX()-p.getX())<e) {
         *   q.setLocation(Math.abs(q.getX()-p.getX())<Math.abs(q.getX()-r.getX())?p.getX():r.getX(),q.getY());
         *   moved=true;
         * }
         * if (Math.abs(q.getY()-r.getY())<e||Math.abs(q.getY()-p.getY())<e) {
         *   q.setLocation(q.getX(),Math.abs(q.getY()-p.getY())<Math.abs(q.getY()-r.getY())?p.getY():r.getY());
         *   moved=true;
         * }
         * </code>
         */
        if (!moved) {
          q.setLocation(getEdgePoint(line,new Point2D.Double(gridSize*Math.floor(z.getX()/gridSize),gridSize*Math.floor(z.getY()/gridSize))));
          moved=true;
        }
        selectedRoute[selectedIndex][0]=q.getX()/d;
        selectedRoute[selectedIndex][1]=q.getY()/d;
        updateEdge(selectedEdge,selectedRoute);
      }
    }
    
    @Override
    public void mouseMoved(MouseEvent pEvent) {
    }
    
    @Override
    public void mouseWheelMoved(MouseWheelEvent pEvent) {
    }
    
  }
  
  protected class GSpecialShapePickSupport extends ShapePickSupport<V,E> {
    
    public GSpecialShapePickSupport(VisualizationViewer<V,E> pVisualizationViewer) {
      super(pVisualizationViewer,8.0f);
    }
    
    // Método adaptado de edu.uci.ics.jung.visualization.picking.ShapePickSupport
    // Gran parte del código es de JUNG
    @Override
    public E getEdge(Layout<V,E> pLayout, double pX, double pY) {
      Point2D point=super.vv.getRenderContext().getMultiLayerTransformer().inverseTransform(Layer.VIEW,new Point2D.Double(pX,pY));
      double x=point.getX(),y=point.getY(),minDist=Double.POSITIVE_INFINITY;
      Rectangle2D pickArea=new Rectangle2D.Double(x-pickSize/2,y-pickSize/2,pickSize,pickSize);
      E closest=null;
      try {
        for (E edge:super.getFilteredEdges(pLayout)) {
          Shape shape=getTransformedEdgeShape(pLayout,edge);
          if (shape!=null&&shape.intersects(pickArea)) {
            List<double[]> points=new ArrayList<double[]>();
            double[] c=new double[6];
            for (PathIterator it=new GeneralPath(shape).getPathIterator(null); !it.isDone(); it.next()) {
              int type=it.currentSegment(c);
              points.add(new double[]{c[0],c[1]});
              if (type==PathIterator.SEG_QUADTO||type==PathIterator.SEG_CUBICTO) points.add(new double[]{c[2],c[3]});
              if (type==PathIterator.SEG_CUBICTO) points.add(new double[]{c[4],c[5]});
            }
            for (int i=1; i<points.size(); i++) {
              double dist=Line2D.ptSegDist(points.get(i-1)[0],points.get(i-1)[1],points.get(i)[0],points.get(i)[1],x,y);
              if (dist<minDist&&dist<pickSize) {
                minDist=dist;
                closest=edge;
              }
            }
          }
        }
      }
      catch (Throwable th) {
      }
      return closest;
    }
    
    // Método adaptado de edu.uci.ics.jung.visualization.picking.ShapePickSupport
    // Gran parte del código es de JUNG
    private Shape getTransformedEdgeShape(Layout<V,E> pLayout, E pEdge) {
      Pair<V> pair=pLayout.getGraph().getEndpoints(pEdge);
      V v1=pair.getFirst(),v2=pair.getSecond();
      Point2D p1=vv.getRenderContext().getMultiLayerTransformer().transform(Layer.LAYOUT,pLayout.transform(v1));
      Point2D p2=vv.getRenderContext().getMultiLayerTransformer().transform(Layer.LAYOUT,pLayout.transform(v2));
      if (p1==null||p2==null) return null;
      double x1=p1.getX(),y1=p1.getY(),x2=p2.getX(),y2=p2.getY();
      AffineTransform xform=AffineTransform.getTranslateInstance(x1,y1);
      Shape shape=vv.getRenderContext().getEdgeShapeTransformer().transform(Context.<Graph<V,E>,E>getInstance(vv.getGraphLayout().getGraph(),pEdge));
      if (v1.equals(v2)) {
        Rectangle2D s2Bounds=vv.getRenderContext().getVertexShapeTransformer().transform(v2).getBounds2D();
        xform.scale(s2Bounds.getWidth(),s2Bounds.getHeight());
        xform.translate(0,-shape.getBounds2D().getHeight()/2);
      }
      else {
        double dx=x2-x1,dy=y2-y1,dist=Math.sqrt(dx*dx+dy*dy);
        xform.rotate(Math.atan2(dy,dx));
        // En JUNG estaba xform.scale(dist,1.0);
        xform.scale(dist,(shape instanceof GeneralPath)||(shape instanceof Line2D)?dist:1.0);
      }
      return xform.createTransformedShape(shape);
    }
    
  }
  
}
