package gold.visualization.quadtree;

import java.awt.*;
import javax.swing.*;
import gold.structures.tree.nary.*;
import gold.visualization.util.*;

public class GQuadtreePainter implements IPainter<GQuadtree> {
  
  protected final GQuadtree quadtree;
  protected final int imageSize;
  protected final int pixelSize;
  protected final boolean showGrid;
  
  public GQuadtreePainter(GQuadtree pQuadtree, int pImageSize) {
    this(pQuadtree,pImageSize,4,false);
  }
  
  public GQuadtreePainter(GQuadtree pQuadtree, int pImageSize, int pPixelSize, boolean pShowGrid) {
    if (pImageSize!=Integer.highestOneBit(pImageSize)) throw new IllegalArgumentException("The image size must be a power of two.");
    if (pPixelSize<=0) throw new IllegalArgumentException("The pixel size must be a positive integer.");
    quadtree=pQuadtree;
    imageSize=pImageSize;
    pixelSize=pPixelSize;
    showGrid=pShowGrid&&pPixelSize>=3;
  }
  
  public GQuadtree getQuadtree() {
    return quadtree;
  }
  
  protected void paint(GQuadtree pQuadtree, int pSize, Graphics2D pGraphics, Point pPoint) {
    int half=pSize>>>1;
    if (pSize<=pixelSize||!pQuadtree.getRoot().isGray()) {
      pGraphics.setColor(pQuadtree.getRoot().getColor());
      pGraphics.fillRect(pPoint.x,pPoint.y,pSize,pSize);
    }
    else {
      paint(pQuadtree.getSubtree(0),half,pGraphics,new Point(pPoint.x,pPoint.y));
      paint(pQuadtree.getSubtree(1),half,pGraphics,new Point(pPoint.x+half,pPoint.y));
      paint(pQuadtree.getSubtree(2),half,pGraphics,new Point(pPoint.x,pPoint.y+half));
      paint(pQuadtree.getSubtree(3),half,pGraphics,new Point(pPoint.x+half,pPoint.y+half));
    }
  }
  
  @Override
  public GImage getImage() {
    int side=imageSize*pixelSize;
    GImage image=new GImage(side+(showGrid?1:0),side+(showGrid?1:0));
    Graphics2D graphics=image.createGraphics();
    paint(quadtree,side,graphics,new Point(0,0));
    if (showGrid) {
      graphics.setColor(new Color(0,162,232));
      for (int i=0; i<=side; i+=pixelSize) {
        graphics.drawLine(i,0,i,side);
        graphics.drawLine(0,i,side,i);
      }
    }
    return image;
  }
  
  @Override
  public JLabel getCanvas() {
    return new JLabel(new ImageIcon(getImage()));
  }
  
}
