package gold.visualization.util;

import java.awt.geom.*;

public final class GAffineTransforms {
  
  private GAffineTransforms() {
  }
  
  public static AffineTransform translate(double pDeltaX, double pDeltaY) {
    return AffineTransform.getTranslateInstance(pDeltaX,pDeltaY);
  }
  
  public static AffineTransform rotate(double pAngle) {
    return AffineTransform.getRotateInstance(pAngle);
  }
  
  public static AffineTransform rotate(double pCenterX, double pCenterY, double pAngle) {
    return AffineTransform.getRotateInstance(pAngle,pCenterX,pCenterY);
  }
  
  public static AffineTransform scale(double pFactorX, double pFactorY) {
    return AffineTransform.getScaleInstance(pFactorX,pFactorY);
  }
  
  public static AffineTransform scale(double pFactorX, double pFactorY, double pCenterX, double pCenterY) {
    return compose(translate(-pCenterX,-pCenterY),scale(pFactorX,pFactorY),translate(pCenterX,pCenterY));
  }
  
  public static AffineTransform shear(double pMultiplierX, double pMultiplierY) {
    return AffineTransform.getShearInstance(pMultiplierX,pMultiplierY);
  }
  
  public static AffineTransform compose(AffineTransform...pAffineTransforms) {
    AffineTransform result=new AffineTransform();
    for (AffineTransform affineTransform:pAffineTransforms) {
      if (affineTransform!=null) result.preConcatenate(affineTransform);
    }
    return result;
  }
  
  public static AffineTransform precompose(AffineTransform...pAffineTransforms) {
    AffineTransform result=new AffineTransform();
    for (AffineTransform affineTransform:pAffineTransforms) {
      if (affineTransform!=null) result.concatenate(affineTransform);
    }
    return result;
  }
  
  public static AffineTransform invert(AffineTransform pAffineTransform) throws NoninvertibleTransformException {
    return pAffineTransform.createInverse();
  }
  
}
