package gold.visualization.util;

import gold.structures.map.*;

public class GGeneralTransformer<K,V> implements ITransformer<K,V> {
  
  protected V defaultValue=null;
  protected IMap<K,V> map=null;
  
  public GGeneralTransformer() {
    this(null);
  }
  
  public GGeneralTransformer(V pDefaultValue) {
    this(pDefaultValue,32);
  }
  
  public GGeneralTransformer(V pDefaultValue, int pInitialCapacity) {
    map=new GHashTableMap<K,V>(pInitialCapacity);
    defaultValue=pDefaultValue;
  }
  
  @Override
  public V getDefaultValue() {
    return defaultValue;
  }
  
  @Override
  public V get(K pKey) {
    return map.get(pKey);
  }
  
  @Override
  public void setDefaultValue(V pDefaultValue) {
    defaultValue=pDefaultValue;
  }
  
  @Override
  public void set(K pKey, V pValue) {
    if (pValue==null) {
      clear(pKey);
    }
    else {
      map.put(pKey,pValue);
    }
  }
  
  @Override
  public void setAll(V pValue) {
    defaultValue=pValue;
    map.clear();
  }
  
  @Override
  public void setAll(Iterable<? extends K> pKeys, V pValue) {
    if (pValue==null) {
      clearAll(pKeys);
    }
    else {
      for (K key:pKeys) {
        map.put(key,pValue);
      }
    }
  }
  
  @Override
  public void clear(K pKey) {
    map.remove(pKey);
  }
  
  @Override
  public void clearAll() {
    map.clear();
  }
  
  @Override
  public void clearAll(Iterable<? extends K> pKeys) {
    for (K key:pKeys) {
      map.remove(key);
    }
  }
  
  @Override
  public V transform(K pKey) {
    V value=map.get(pKey);
    return value!=null?value:defaultValue;
  }
  
}
