package gold.visualization.util;

import java.awt.*;
import java.awt.image.*;
import java.io.*;
import java.util.*;
import javax.imageio.*;

public class GImage extends BufferedImage {
  
  public GImage(int pWidth, int pHeight, Color pBackgroundColor) {
    super(pWidth,pHeight,BufferedImage.TYPE_INT_ARGB);
    clean(pBackgroundColor);
  }
  
  public GImage(int pWidth, int pHeight) {
    this(pWidth,pHeight,new Color(0xFFFFFFFF,true));
  }
  
  public GImage(Image pImage) {
    this(pImage.getWidth(null),pImage.getHeight(null));
    draw(pImage,0,0);
  }
  
  public Graphics2D createQualityGraphics() {
    Graphics2D graphics=createGraphics();
    graphics.setRenderingHints(GImage.createRenderingHints());
    return graphics;
  }
  
  public void clean(Color pBackgroundColor) {
    Graphics graphics=getGraphics();
    graphics.setColor(pBackgroundColor);
    graphics.fillRect(0,0,getWidth(),getHeight());
  }
  
  public void draw(Image pImage, int pX, int pY) {
    getGraphics().drawImage(pImage,pX,pY,null);
  }
  
  public GImage crop(Insets pInsets) {
    int color=getRGB(0,0);
    int w=getWidth(),h=getHeight(),minX=w-1,maxX=0,minY=h-1,maxY=0;
    for (int i=0; i<w; i++) {
      for (int j=0; j<h; j++) {
        int c=getRGB(i,j);
        if (c!=color) {
          minX=Math.min(minX,i);
          maxX=Math.max(maxX,i);
          minY=Math.min(minY,j);
          maxY=Math.max(maxY,j);
        }
      }
    }
    if (minX>maxX||minY>maxY) {
      return new GImage(this);
    }
    else {
      GImage image=new GImage(maxX-minX+1+pInsets.left+pInsets.right,maxY-minY+1+pInsets.top+pInsets.bottom,new Color(color,true));
      image.draw(getSubimage(minX,minY,maxX-minX+1,maxY-minY+1),pInsets.left,pInsets.top);
      return image;
    }
  }
  
  public GImage scale(double pFactor) {
    int w=(int)(Math.round(getWidth()*pFactor));
    int h=(int)(Math.round(getHeight()*pFactor));
    GImage image=new GImage(w,h);
    image.draw(this.getScaledInstance(w,h,Image.SCALE_SMOOTH),0,0);
    return image;
  }
  
  public void save(OutputStream pOutputStream, String pFormat) throws IOException {
    ImageIO.write(this,pFormat,pOutputStream);
  }
  
  public void save(File pFile, String pFormat) throws IOException {
    ImageIO.write(this,pFormat,pFile);
  }
  
  public void save(File pFile) throws IOException {
    save(pFile,"png");
  }
  
  public static GImage load(File pFile) throws IOException {
    return new GImage(ImageIO.read(pFile));
  }
  
  public static GImage load(InputStream pInputStream) throws IOException {
    return new GImage(ImageIO.read(pInputStream));
  }
  
  public static Map<RenderingHints.Key,Object> createRenderingHints() {
    Map<RenderingHints.Key,Object> result=new HashMap<RenderingHints.Key,Object>(16);
    result.put(RenderingHints.KEY_ALPHA_INTERPOLATION,RenderingHints.VALUE_ALPHA_INTERPOLATION_QUALITY);
    result.put(RenderingHints.KEY_ANTIALIASING,RenderingHints.VALUE_ANTIALIAS_ON);
    result.put(RenderingHints.KEY_COLOR_RENDERING,RenderingHints.VALUE_COLOR_RENDER_QUALITY);
    result.put(RenderingHints.KEY_DITHERING,RenderingHints.VALUE_DITHER_ENABLE);
    result.put(RenderingHints.KEY_FRACTIONALMETRICS,RenderingHints.VALUE_FRACTIONALMETRICS_DEFAULT);
    result.put(RenderingHints.KEY_INTERPOLATION,RenderingHints.VALUE_INTERPOLATION_BICUBIC);
    result.put(RenderingHints.KEY_RENDERING,RenderingHints.VALUE_RENDER_QUALITY);
    result.put(RenderingHints.KEY_STROKE_CONTROL,RenderingHints.VALUE_STROKE_PURE);
    // result.put(RenderingHints.KEY_TEXT_ANTIALIASING,RenderingHints.VALUE_TEXT_ANTIALIAS_LCD_HRGB);
    return result;
  }
  
}
