package gold.visualization.util;

import edu.uci.ics.jung.algorithms.layout.*;
import edu.uci.ics.jung.graph.*;

public enum GLayoutType {
  
  /**
   * Graphs
   */
  CIRCLE,FR,FR2,ISOM,KK,SPRING,SPRING2,
  /**
   * Forests
   */
  BALLOON,RADIAL,TREE;
  
  public boolean isTreeLayout() {
    return this==GLayoutType.BALLOON||this==GLayoutType.RADIAL||this==GLayoutType.TREE;
  }
  
  public <V,E> Layout<V,E> getLayout(Graph<V,E> pGraph) {
    switch (this) {
      case CIRCLE:
        return new CircleLayout<V,E>(pGraph);
      case FR:
        return new FRLayout<V,E>(pGraph);
      case FR2:
        return new FRLayout2<V,E>(pGraph);
      case ISOM:
        return new ISOMLayout<V,E>(pGraph);
      case KK:
        return new KKLayout<V,E>(pGraph);
      case SPRING:
        SpringLayout<V,E> springLayout=new SpringLayout<V,E>(pGraph,new GConstantTransformer<E,Integer>(120));
        springLayout.setForceMultiplier(0.1);
        return springLayout;
      case SPRING2:
        return new SpringLayout2<V,E>(pGraph,new GConstantTransformer<E,Integer>(70));
      case BALLOON:
        return new BalloonLayout<V,E>((Forest<V,E>)pGraph);
      case RADIAL:
        return new RadialTreeLayout<V,E>((Forest<V,E>)pGraph);
      case TREE:
        return new TreeLayout<V,E>((Forest<V,E>)pGraph);
    }
    return null;
  }
  
  public static <V,E> GLayoutType getLayoutType(Layout<V,E> pLayout) {
    if (pLayout instanceof LayoutDecorator) {
      return getLayoutType(((LayoutDecorator<V,E>)pLayout).getDelegate());
    }
    else if (pLayout instanceof CircleLayout) {
      return GLayoutType.CIRCLE;
    }
    else if (pLayout instanceof FRLayout2) {
      return GLayoutType.FR2;
    }
    else if (pLayout instanceof FRLayout) {
      return GLayoutType.FR;
    }
    else if (pLayout instanceof ISOMLayout) {
      return GLayoutType.ISOM;
    }
    else if (pLayout instanceof KKLayout) {
      return GLayoutType.KK;
    }
    else if (pLayout instanceof SpringLayout2) {
      return GLayoutType.SPRING2;
    }
    else if (pLayout instanceof SpringLayout) {
      return GLayoutType.SPRING;
    }
    else if (pLayout instanceof BalloonLayout) {
      return GLayoutType.BALLOON;
    }
    else if (pLayout instanceof RadialTreeLayout) {
      return GLayoutType.RADIAL;
    }
    else if (pLayout instanceof TreeLayout) {
      return GLayoutType.TREE;
    }
    else {
      return GLayoutType.KK;
    }
  }
  
}
